//Written for www.HomoFaciens.de
//Coded by Norbert Heinz in 2014
//
//The code was written to create some animation sequences for a video about electrostatics:
//http://www.homofaciens.de/technics-charge_en_navion.htm
//
//This is a mess of code!!!
//
//FieldLines.c is just a fraction of the total code and so probably better to understand how this one works.
//
//Created and tested with Ubuntu Linux:
//1.) Get a live and installation DVD of Ubuntu at www.ubuntu.com (if not already installed on your hard disk)
//2.) Open Terminal and install additional components:
//    sudo apt-get install freeglut3 freeglut3-dev
//    To create picture sequences:
//    sudo apt-get install imagemagick
//
//3.) Compile with: gcc ElectrostaticField.c -lglut -lGLU -lGL -o ElectrostaticField
//4.) Run with: ./ElectrostaticField
//
//      Example of usage: Electrostatics 21 10 0
//      First integer dials the animation sequence (1-29)
//      Second integer sets slow down factor of animation in milliseconds(0-10000)
//      If third integer is greater than 0, pictures are stored in folder PicSequence, starting with picture number given.
//      
//      When animation is running:
//      Space: Stop/Start
//      +: Zoom in
//      -: Zoom out
//      u: move up
//      d: move down
//      cursor: rotate
//      up/down key: rotate
//      F2: take snapshot
//      F1: change between window mode and fullscreen
//
//This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free
//Software Foundation version 3 of the License.
//
//This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
//FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License along with this program; if not, see http://www.gnu.org/licenses/.
//
 
#include <stdio.h>
#include <stdlib.h>
#include <GL/glx.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <X11/extensions/xf86vmode.h>
#include <X11/keysym.h>
#include <math.h>           /* for the sin() function */
#include <string.h>

#define CHARACTERS 256            //Number of characters
#define PI 3.1415927f
#define PATHMAX 20


GLfloat SchriftFarbe[6];
GLfloat HGFarbe[3]={0.0f, 0.0f, 0.0f};


int gridX = 64;
int gridY = 32;
GLfloat GRIDDIVISOR = 5.0f;
GLfloat SCALE = 0.1f;

int slowDownMS = 0;
int animationPause = 0;
int temp = 0;
int animation = 0;
int savePics = 0;
int picNumber = 0;
int firstStart = 10;
int paintLastPic = 0;
GLfloat minDistance = 0.35f;
GLfloat particleCharge = 1.0f, testCharge = 1.0f;
GLfloat coordScale;
GLfloat xMax = 8.0, yMax;
GLfloat fieldLineEndUp = 0.1f;
GLfloat fieldLineEndDown = 0.1f;
GLfloat particles[1000][7];
GLfloat path[20][30000][3];
int pathEnd[20];
int pathColor[20];
GLfloat potentialPoints[10][3];
double k1 = 1.0 / (4.0 * PI * 8.85419E-12);
double alpha;
double doubleTemp = 0.0;
GLfloat GLfloatTemp = 0.0f;
int particleNumber = 1;
int potentialEnd = 1;
GLfloat scalePlaneHeigth = 200.0f;
GLfloat planeOffset = 0.0f;
GLfloat moveAlpha = 0.0f;
GLfloat moveBeta = 0.0f;
GLfloat minValue, maxValue, minValue1, maxValue1, minValue2, maxValue2;
GLfloat totalCharge = 0.0f;
GLfloat minVectorDistance = 0.15f;
GLfloat pathWidth[PATHMAX];
GLfloat potentialSphereRadius = 0.16f;
GLfloat protonRadius = 0.85f;
GLfloat particleSpeed = 0.9f;
GLfloat dipoleRadius = 0.05f;

int enableLight1 = 1, enableLight2 = 1, enableLight3 = 1;
int enablePotential = 0;
int enablePotentialPoints = 0;
int enablePotentialDiff = 0;
int enablePotentialPlane = 0;
int enableParticles = 0;
int enableMinDisc = 0;
int enableFieldVectors = 0;
int enableLablesFieldVectors = 1;
int enableGrid = 1;
int enableSecondEquipotential = 1;
int enablePotentialSphere = 0;
int startPicNumber = 0;
int enableTransLucentSphere = 0;

int pauseFrames = 0;
GLfloat potentialSphereX = 0.0f, potentialSphereY = 0.0f, potentialSphereZ = 0.0f;

GLuint base;            /* base display list for the font */

GLfloat planeColor[] = {1.0, 1.0, 1.0}; 
GLfloat axeXColor[] = {1.0, 0.0, 1.0}; 
GLfloat axeYColor[] = {0.0, 1.0, 1.0}; 
GLfloat axeZColor[] = {1.0, 1.0, 0.0}; 


GLfloat redMaterial[] = {1.0, 0.0, 0.0}; 
GLfloat blueMaterial[] = {0.0, 0.0, 1.0};
GLfloat greenMaterial[] = {0.0, 1.0, 0.0}; 
GLfloat orangeMaterial[] = {1.0, 0.0, 1.0}; 

GLfloat lightWhite1[] = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightRed1[] = { 1.0f, 0.0f, 0.0f, 1.0f };
GLfloat lightRed2[] = { 0.7f, 0.0f, 0.0f, 0.50f };
GLfloat lightRed3[] = { 1.0f, 0.6f, 0.6f, 0.50f };
GLfloat lightGreen1[] = { 0.0f, 1.0f, 0.0f, 1.0f };
GLfloat lightBlue1[] = { 0.0f, 0.7f, 1.0f, 1.0f };
GLfloat lightBlue2[] = { 0.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightBlue3[] = { 0.0f, 0.4f, 1.0f, 1.0f };
GLfloat lightOrange1[] = { 1.0f, 0.7f, 0.0f, 1.0f };
GLfloat lightOrange2[] = { 0.7f, 0.5f, 0.0f, 0.50f };
GLfloat lightSpecularOrange1[] = { 0.5f, 0.35f, 0.0f, 0.3f };

GLfloat lightTemp[] = { 1.0f, 0.0f, 0.0f, 1.0f };

GLfloat lightPosition0[] = { -5.0f, -2.0f, 5.0f, 1.0f };

GLfloat specular1[] = { 1.0f, 1.0f, 1.0f, 1.0f}; 
GLfloat lightSpot1[] = { 0.3f, 0.3f, 0.3f, 0.3f}; 
GLfloat lightAmbient1[] = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightDiffuse1[] = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightPosition1[] = { -5.0f, 2.0f, 5.0f, 1.0f };

GLfloat specular2[] = { 1.0f, 1.0f, 1.0f, 0.3f}; 
GLfloat lightSpot2[] = { 0.3f, 0.3f, 0.3f, 0.3f}; 
GLfloat lightAmbient2[] = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightDiffuse2[] = { 0.3f, 0.3f, 0.3f, 0.5f };
GLfloat lightPosition2[] = { 0.0f, 2.0f, 10.0f, 1.0f };

GLfloat specular3[] = { 1.0f, 1.0f, 1.0f, 0.3f}; 
GLfloat lightSpot3[] = { 0.3f, 0.3f, 0.3f, 0.3f}; 
GLfloat lightAmbient3[] = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightDiffuse3[] = { 0.3f, 0.3f, 0.3f, 0.5f };
GLfloat lightPosition3[] = { 2.0f, 2.0f, 10.0f, 1.0f };

GLfloat white[] = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat red[] = { 1.0f, 0.0f, 0.0f, 1.0f };

GLfloat normalVector[] = { 1.0f, 1.0f, 1.0f };
GLfloat p1[] = { 1.0f, 1.0f, 1.0f };
GLfloat p2[] = { 1.0f, 1.0f, 1.0f };
GLfloat p3[] = { 1.0f, 1.0f, 1.0f };

/* stuff about our window grouped together */
typedef struct {
    Display *dpy;
    int screen;
    Window win;
    GLXContext ctx;
    XSetWindowAttributes attr;
    Bool fs;
    XF86VidModeModeInfo deskMode;
    int x, y;
    unsigned int width, height;
    unsigned int depth;    
} GLWindow;

typedef struct {
    int width;
    int height;
    unsigned char *data;
} textureImage;

/* attributes for a single buffered visual in RGBA format with at least
 * 4 bits per color and a 16 bit depth buffer */
static int attrListSgl[] = {GLX_RGBA, GLX_RED_SIZE, 4, 
    GLX_GREEN_SIZE, 4, 
    GLX_BLUE_SIZE, 4, 
    GLX_DEPTH_SIZE, 16,
    None};

/* attributes for a double buffered visual in RGBA format with at least
 * 4 bits per color and a 16 bit depth buffer */
static int attrListDbl[] = { GLX_RGBA, GLX_DOUBLEBUFFER, 
    GLX_RED_SIZE, 4, 
    GLX_GREEN_SIZE, 4, 
    GLX_BLUE_SIZE, 4, 
    GLX_DEPTH_SIZE, 16,
    None };

GLWindow GLWin;
Bool done;
GLfloat hold;               /* temporary variable */
GLfloat rotX;  /* X Rotation */
GLfloat rotY;  /* Y Rotation */
GLfloat rotZ;  /* Z Rotation */

GLfloat translateZ = 0.0f;
GLfloat translateX = 0.0f;
GLfloat translateY = 0.0f;

GLfloat rotXdiff;  /* X Rotation */
GLfloat rotYdiff;  /* Y Rotation */
GLfloat rotZdiff;  /* Z Rotation */
GLfloat translateZdiff = 0.0f;
GLfloat translateXdiff = 0.0f;
GLfloat translateYdiff = 0.0f;

GLfloat rotXtarget;
GLfloat rotYtarget;
GLfloat rotZtarget;
GLfloat translateZtarget = 0.0f;
GLfloat translateXtarget = 0.0f;
GLfloat translateYtarget = 0.0f;



GLuint  textureFont[512 + 10];  /* Storage For One Texture */
int TextureImageX, TextureImageY;


//+++++++++++++ Loads A Bitmap Image +++++++++++++++++++++++++++++++
unsigned char *LoadBMP(char *Filename){

  FILE *File=NULL;                                // File Handle
  char FilenameLanguage[1000];
  unsigned char *ImageData;
  unsigned short int bfType;
  long int bfOffBits;
  short int biPlanes;
  short int biBitCount;
  long int biSizeImage;
  int i;
  unsigned char temp;

  if (!Filename)                                  // Make Sure A Filename Was Given
  {
    return NULL;                                  // If Not Return NULL
  }


  /* make sure the file is there and open it read-only (binary) */
  if ((File = fopen(Filename, "rb")) == NULL){
    printf("File not found : %s\n", Filename);
    return NULL;
  }
  if(!fread(&bfType, sizeof(short int), 1, File)){
    printf("Error reading : %s\n", Filename);
    return NULL;
  }
  /* check if file is a bitmap */
  if (bfType != 19778){
    printf("No bitmap-format! : %s\n", Filename);
    return NULL;
  }
  /* get the file size */
  /* skip file size and reserved fields of bitmap file header */
  fseek(File, 8, SEEK_CUR);
  /* get the position of the actual bitmap data */
  if (!fread(&bfOffBits, sizeof(long int), 1, File)){
    printf("Error reading OffBits : %s\n", Filename);
    return NULL;
  }
  //printf("Data at Offset: %ld\n", bfOffBits);
  /* skip size of bitmap info header */
  fseek(File, 4, SEEK_CUR);
  /* get the width of the bitmap */
  fread(&TextureImageX, sizeof(int), 1, File);
  //printf("Width of Bitmap: %d\n", texture->width);
  /* get the height of the bitmap */
  fread(&TextureImageY, sizeof(int), 1, File);
  //printf("Height of Bitmap: %d\n", texture->height);
  /* get the number of planes (must be set to 1) */
  fread(&biPlanes, sizeof(short int), 1, File);
  if (biPlanes != 1){
    printf("Wrong BiPlane : %s\n", Filename);
    return NULL;
  }
  /* get the number of bits per pixel */
  if (!fread(&biBitCount, sizeof(short int), 1, File)){
    printf("Wrong BitCount : %s\n", Filename);
    return NULL;
  }
  //printf("Bits per Pixel: %d\n", biBitCount);
  if (biBitCount != 24){
    printf("Wrong depth : %s\n", Filename);
    return NULL;
  }
  /* calculate the size of the image in bytes */
  biSizeImage = TextureImageX * TextureImageY * 3;
  //printf("Size of the image data: %ld\n", biSizeImage);
  if((ImageData = (unsigned char*)malloc(biSizeImage))==NULL){
    printf("Could not allocate memory for %s\n", Filename);
    return NULL;
  }
  /* seek to the actual data */
  fseek(File, bfOffBits, SEEK_SET);
  if (!fread(ImageData, biSizeImage, 1, File)){
    printf("Error reading data : %s\n", Filename);
    return NULL;
  }
  /* swap red and blue (bgr -> rgb) */
  for (i = 0; i < biSizeImage; i += 3){
    temp = ImageData[i];
    ImageData[i] = ImageData[i + 2];
    ImageData[i + 2] = temp;
  }

  fclose(File);

  return ImageData;   // If Load Failed Return 0
}//-------------------------------------------------------------------------- Loads A Bitmap Image -------------------------


//++++++++++++++++++++++++++++++++++++++++ Load Fonts And Convert To Textures +++++++++++++++++++++++++++++++++++++
int LoadFonts(){
  int i, i2, i3;
  int FarbInvers;
  float StartZeit, EndZeit;
  int SchwarzerSchirm=False;
  int AnzeigeEnde=False;
  char Taste;
  char DateiName[1000];
  unsigned char *TextureImageData;


  for(i = 0; i < CHARACTERS; i++){// Loop Through All CHARACTERS Textures
    sprintf(DateiName, "Data/font_%03d.bmp", i);
    if ((TextureImageData = LoadBMP(DateiName))==NULL){
      printf("Could not load %s", DateiName);
      done=True;
      return False;
    }

    printf("Loading %s, TextureImageX=%d, TextureImageY=%d\n", DateiName, TextureImageX, TextureImageX);

    for(i2 = 0; i2 < TextureImageX * TextureImageY * 3; i2++){
      FarbInvers = TextureImageData[i2];
      if(FarbInvers<0){
        printf("FarbInvers=%d", FarbInvers);
      }
      FarbInvers-=255;
      FarbInvers=-FarbInvers;
      TextureImageData[i2]=(char)FarbInvers;
    }

    glGenTextures(1, &textureFont[i]);
    glBindTexture(GL_TEXTURE_2D, textureFont[i]);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    glTexImage2D(GL_TEXTURE_2D, 0, 3, TextureImageX, TextureImageY, 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImageData);

    //Zugehoerige Maske generieren
    for(i2 = 0; i2 < TextureImageX * TextureImageY * 3; i2++){
      FarbInvers = TextureImageData[i2];
      if(FarbInvers < 0){
        printf("FarbInvers=%d", FarbInvers);
      }
      FarbInvers -= 255;
      FarbInvers = -FarbInvers;
      TextureImageData[i2]=(unsigned char)FarbInvers;
    }

    glGenTextures(1, &textureFont[256 + i]);
    glBindTexture(GL_TEXTURE_2D, textureFont[256 + i]);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    glTexImage2D(GL_TEXTURE_2D, 0, 3, TextureImageX, TextureImageY, 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImageData);

    if (TextureImageData!=NULL){    // If Texture Image Exists
      free(TextureImageData); // Free The Texture Image Memory
    }

    if(done)return False;
  }//for(i = 0; i < CHARACTERS; i++)


  return True;                                // Return The Status
}//----------------------------------------------------------------- Ende Load Fonts ------------------------------------------



//+++++++++++++++++++++++++++++++++ 2D text with OpenGL +++++++++++++++++++
void printGLf(GLfloat x, GLfloat y, const char *stringToPrint, int center, int fontColor, GLfloat Size){
GLfloat BreiteF, HoeheF;
unsigned int i, b;


  glPushMatrix();

  BreiteF = Size;
  HoeheF = 0.09f / 0.055f * Size;
  
  glDisable(GL_LIGHTING);
  
  //printf("printGLf: >%s< BreiteF=%lf, HoeheF=%lf\n", stringToPrint, BreiteF, HoeheF);

  if(center == 1){
    x=-1.0f * strlen(stringToPrint) * BreiteF;
  }
  if (strlen(stringToPrint) > 0){
    for(i=0;i<strlen(stringToPrint);i++){
      b=(unsigned int)(stringToPrint[i]);
      //printf("b=%d\n", b);
      //if(string[i]=='Ä')b=196;
      //if(string[i]=='Ö')b=214;
      //if(string[i]=='Ü')b=220;
      //if(string[i]=='ä')b=228;
      //if(string[i]=='ö')b=246;
      //if(string[i]=='ü')b=252;
      //if(string[i]=='ß')b=223;
      //if(string[i]=='´')b=146;
      if(b > 0 && b < 256){
        if(fontColor < 0){//Black Background color if fontcolor < 0
          glColor3f(0.0f, 0.0f, 0.0f);
        }
        else{
          glColor3f(1.0f, 1.0f, 1.0f);
        }
        glEnable(GL_TEXTURE_2D);
        glEnable(GL_BLEND);
        glBlendFunc(GL_DST_COLOR, GL_ZERO);
        glLoadIdentity();
        glTranslatef((float)(x + i * BreiteF * 2.0f), y, -6.0f);
        glBindTexture(GL_TEXTURE_2D, textureFont[256 + b]);
        
        glBegin(GL_QUADS);
            glTexCoord2f(0.0f, 1.0f);
            glVertex3f(-BreiteF, HoeheF, 0.0f);
            glTexCoord2f(1.0f, 1.0f);
            glVertex3f(BreiteF, HoeheF, 0.0f);
            glTexCoord2f(1.0f, 0.0f);
            glVertex3f(BreiteF, -HoeheF, 0.0f);
            glTexCoord2f(0.0f, 0.0f);
            glVertex3f(-BreiteF, -HoeheF, 0.0f);
        glEnd();


        glColor3f(1.0f, 1.0f, 0.0f);//Yellow is the standard
        if(fontColor == 1 || fontColor == -1){
          glColor3f(0.0f, 1.0f, 0.0f);
        }
        if(fontColor == 2 || fontColor == -2){//Orange
          glColor3f(1.0f, 0.75f, 0.0f);
        }
        if(fontColor == 3 || fontColor == -3){//Red
          glColor3f(1.0f, 0.0f, 0.0f);
        }
        if(fontColor == 4 || fontColor == -4){
          glColor3f(0.0f, 1.0f, 1.0f);
        }
        if(fontColor == 5 || fontColor == -5){//Yellow
          glColor3f(1.0f, 1.0f, 0.0f);
        }
        if(fontColor == 6 || fontColor == -6){//Light blue
          glColor3f(0.0f, 0.7f, 1.0f);
        }

        glBlendFunc(GL_ONE, GL_ONE);
        glBindTexture(GL_TEXTURE_2D, textureFont[b]);
        glBegin(GL_QUADS);
            glTexCoord2f(0.0f, 1.0f);
            glVertex3f(-BreiteF, HoeheF, 0.0f);
            glTexCoord2f(1.0f, 1.0f);
            glVertex3f(BreiteF, HoeheF, 0.0f);
            glTexCoord2f(1.0f, 0.0f);
            glVertex3f(BreiteF, -HoeheF, 0.0f);
            glTexCoord2f(0.0f, 0.0f);
            glVertex3f(-BreiteF, -HoeheF, 0.0f);
        glEnd();

      }
    }
    glColor3f(1.0f, 1.0f, 1.0f);
  }
  
  glPopMatrix();


}//----------------------------------------------------------------------------- Ende Textausgabe mit OpenGL ---------------------

/* function called when our window is resized (should only happen in window mode) */
void resizeGLScene(unsigned int width, unsigned int height){
    if (height == 0)    /* Prevent A Divide By Zero If The Window Is Too Small */
        height = 1;
    /* Reset The Current Viewport And Perspective Transformation */
    glViewport(0, 0, width, height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45.0f, (GLfloat)width / (GLfloat)height, 0.1f, 100.0f);
    //glFrustum (-1.0, 1.0, -1.0, 1.0, 1.5, 20.0);
    //glOrtho(-4.0, 4.0, -4.0, 4.0, 5, 100);
    glMatrixMode(GL_MODELVIEW);
}



int screenshotTest(){
  int Xres = 1920, Yres = 1080;
  unsigned int i;
  long result=0;
  char fileName[1000];
  char picNumberChar[1000];
  
	static unsigned char header[54] = {
		0x42, 0x4D, 0x36, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x36, 0x00, 0x00, 0x00, 0x28, 0x00, 
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x18, 0x00, 0x00, 0x00, 
		0x00, 0x00, 0x00, 0x00, 0x03, 0x00, 0xC4, 0x0E, 0x00, 0x00, 0xC4, 0x0E, 0x00, 0x00, 0x00, 0x00, 
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
 
	unsigned char *pixels = (unsigned char *) malloc(Xres * Yres * 3);
	((uint16_t *) header)[ 9] = Xres;
	((uint16_t *) header)[11] = Yres;
 
	glReadPixels(0,0,Xres,Yres,GL_RGB,GL_UNSIGNED_BYTE,pixels);
 
	unsigned char temp;
	for ( i = 0; i < Xres * Yres * 3; i += 3){
		temp = pixels[i];
		pixels[i] = pixels[i + 2];
		pixels[i + 2] = temp;
	}
 
	FILE *saveFile = NULL;
	unsigned long Size;
 
  strcpy(fileName, "./PicSequence/pic_");
  sprintf(picNumberChar, "%05d.bmp", picNumber);
  strcat(fileName, picNumberChar);
 
  if ((saveFile = fopen(fileName, "w+b")) == NULL){
      printf("File not opened for writing!\n");
      return 1;
  }
 
 
  result = fwrite(header, 54, 1, saveFile);
  if(result < 1){
    printf("Error writing header to file ImageData.bmp! Bytes written = %ld\n", result);
    return 1;
  }    
 
  result = fwrite(pixels, 3 * Xres * Yres, 1, saveFile);
  
  if(result < 1){
    printf("Error writing file ImageData.bmp! Bytes written = %ld\n", result);
    return 1;
  }    
  
  fclose(saveFile);
  
  free(pixels);
 
  picNumber++;
 
  return 0;
}



int ScreenShot(){  
  char fileName[1000];
  char fileName2[1000];
  char picNumberChar[1000];
  char commandLine[1000];

   
  // GLubyte * bits; //RGB bits  
  GLint viewport[4]; //current viewport  
    
  //get current viewport  
  glGetIntegerv(GL_VIEWPORT, viewport);  
  
  uint32_t screenWidth = viewport[2];  
  uint32_t screenHeight = viewport[3];  
  int i, j;
  FILE *saveFile;
  uint32_t BMsize;
  uint32_t tempLong = 0;
  uint16_t tempInt = 0;
  uint32_t BMoffset;
  uint32_t BMheaderSize;
  unsigned char tempChar;
  unsigned char *buffer = (unsigned char *) malloc(screenWidth * screenHeight * 3);

  printf("Starting screenshot.\n", fileName);
   
//  GLubyte bits[3*w*h];
  
  //read pixel from frame buffer  
  glFinish(); //finish all commands of OpenGL  
  glPixelStorei(GL_PACK_ALIGNMENT,1); //or glPixelStorei(GL_PACK_ALIGNMENT,4);  
  glPixelStorei(GL_PACK_ROW_LENGTH, 0);  
  glPixelStorei(GL_PACK_SKIP_ROWS, 0);  
  glPixelStorei(GL_PACK_SKIP_PIXELS, 0);  
    
  //buffer = (GLubyte *)malloc(screenWidth * screenHeight * 3);  
  
  glReadPixels(0, 0, screenWidth, screenHeight, GL_RGB, GL_UNSIGNED_BYTE, buffer);  
  
  printf("Width=%d, height=%d\n", screenWidth, screenHeight);
  
	for ( i = 0; i < screenWidth * screenHeight * 3; i += 3){
		tempChar = buffer[i];
		buffer[i] = buffer[i + 2];
		buffer[i + 2] = tempChar;
	}
  
  strcpy(fileName, "./PicSequence/pic_");
  sprintf(picNumberChar, "%05d.bmp", picNumber);
  strcat(fileName, picNumberChar);

  strcpy(fileName2, "./PicSequence/pic_");
  sprintf(picNumberChar, "%05d.jpg", picNumber);
  strcat(fileName2, picNumberChar);
  
  if ((saveFile = fopen(fileName, "w+b")) == NULL){
      //printf("Highscores not opened for writing: %s\n", filename);
      return 1;
  }
  else{
    printf("%s opened.\n", fileName);
  }
  
  //IplImage * capImg = cvCreateImage( cvSize(w,h), IPL_DEPTH_8U, 3);  
  
  fwrite("BM", 2, 1, saveFile);
  BMsize = screenWidth * screenHeight * 3 + 54;
  fwrite(&BMsize, 4, 1, saveFile);
  tempLong = 0;
  fwrite(&tempLong, 4, 1, saveFile);
  BMoffset = 54;
  fwrite(&BMoffset, 4, 1, saveFile);
  BMheaderSize = 40;
  fwrite(&BMheaderSize, 4, 1, saveFile);
  fwrite(&screenWidth, 4, 1, saveFile);
  fwrite(&screenHeight, 4, 1, saveFile);
  tempInt = 1;
  fwrite(&tempInt, 2, 1, saveFile);
  tempInt = 3 * 8;
  fwrite(&tempInt, 2, 1, saveFile);
  tempLong = 0;
  fwrite(&tempLong, 4, 1, saveFile);
  tempLong = 3 * screenWidth * screenHeight;
  fwrite(&tempLong, 4, 1, saveFile);
  tempLong = 0;
  fwrite(&tempLong, 4, 1, saveFile);
  tempLong = 0;
  fwrite(&tempLong, 4, 1, saveFile);
  tempLong = 0;
  fwrite(&tempLong, 4, 1, saveFile);
  tempLong = 0;
  fwrite(&tempInt, 4, 1, saveFile);
  
  printf("Writing buffer to file.\n");
  
  if(!fwrite(buffer, 3 * screenWidth * screenHeight, 1, saveFile)){
    printf("Error writing file ImageData.bmp!\n");
    return 1;
  }    
  
  fclose(saveFile);
  
  free(buffer);
  
  printf("%s saved.\n", fileName);
  
  strcpy(commandLine, "convert ");
  strcat(commandLine, fileName);
  strcat(commandLine, " -quality 90 ");
  strcat(commandLine, fileName2);
  printf("executing %s...\n", commandLine);
  system(commandLine);
  strcpy(commandLine, "rm ");
  strcat(commandLine, fileName);
  system(commandLine);
  printf("...done.\n");

  picNumber++;
  
  return 0;
   
}  


/* general OpenGL initialization function */
int initGL(GLvoid){
    int x, y;
    float chargeX = gridX / 2.0f, chargeY = gridY / 2.0f;
    float totalDistance = 0.0f, totalForce = 0.0f;

    printf("Loading fonts...\n");
    if (!LoadFonts())                // Jump To Texture Loading Routine
    {
      return False;                        // If Texture Didn't Load Return False
    }
    else{
      printf("Fonts OK.\n");
    }


    glEnable(GL_TEXTURE_2D);       /* Enable Texture Mapping */
    glShadeModel(GL_SMOOTH);
    
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    glClearDepth(1.0f);
    glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
    glEnable(GL_LINE_SMOOTH);
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE);
    glShadeModel(GL_SMOOTH);
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
    //glPolygonMode(GL_BACK, GL_FILL);
    glPolygonMode(GL_BACK, GL_LINE);
    glPolygonMode(GL_FRONT, GL_LINE);

    //glColor4f(1.0f, 1.0f, 1.0f, 0.9f);  /* full brightness, 50% Alpha */


    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, white);
    glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 80.0f);

    glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient1);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse1);
    glLightfv(GL_LIGHT0, GL_POSITION, lightPosition0);
    //glLightfv(GL_LIGHT0,GL_SPECULAR,specular1);
//    glEnable(GL_LIGHT0);


    glLightfv(GL_LIGHT1, GL_AMBIENT, lightAmbient1);
    glLightfv(GL_LIGHT1, GL_DIFFUSE, lightDiffuse1);
    glLightfv(GL_LIGHT1, GL_POSITION, lightPosition1);
    glLightfv(GL_LIGHT1,GL_SPECULAR,specular1);
    glEnable(GL_LIGHT1);

    //glLightfv(GL_LIGHT2, GL_AMBIENT, lightAmbient2);
    glLightfv(GL_LIGHT2, GL_DIFFUSE, lightDiffuse2);
    //glLightfv(GL_LIGHT2, GL_POSITION, lightPosition2);
    //glLightfv(GL_LIGHT2,GL_SPECULAR,specular2);
//    glEnable(GL_LIGHT2);
    
    //glLightfv(GL_LIGHT3, GL_AMBIENT, lightAmbient3);
    glLightfv(GL_LIGHT3, GL_DIFFUSE, lightDiffuse3);
    //glLightfv(GL_LIGHT3, GL_POSITION, lightPosition3);
    //glLightfv(GL_LIGHT3,GL_SPECULAR,specular3);
//    glEnable(GL_LIGHT3);
    
    glEnable(GL_LIGHTING);
    
    //glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    
    //glEnable(GL_BLEND);
    
    /* we use resizeGLScene once to set up our initial perspective */
    resizeGLScene(GLWin.width, GLWin.height);
    glFlush();
    return True;
}


void calculateNormalVector(){
  GLfloat length;
  
  GLfloat p2minp1[3];
  GLfloat p3minp1[3];
 
  p2minp1[0] = p2[0] - p1[0];
  p2minp1[1] = p2[1] - p1[1];
  p2minp1[2] = p2[2] - p1[2];
  
  p3minp1[0] = -(p3[0] - p1[0]);
  p3minp1[1] = -(p3[1] - p1[1]);
  p3minp1[2] = -(p3[2] - p1[2]);
  
  
  //Normalize Z-component to 1.0f for maximum reflection
  
  normalVector[2] = p2minp1[0] * p3minp1[1] - p2minp1[1] * p3minp1[0];
  normalVector[0] = (p2minp1[1] * p3minp1[2] - p2minp1[2] * p3minp1[1]);
  normalVector[1] = (p2minp1[2] * p3minp1[0] - p2minp1[0] * p3minp1[2]);
  
  length = sqrt(pow(normalVector[0], 2.0) + pow(normalVector[1], 2.0) + pow(normalVector[2], 2.0));

  normalVector[0] = normalVector[0] / length;
  normalVector[1] = normalVector[1] / length;
  normalVector[2] = normalVector[2] / length;
  
  //normalVector[2] = 1.0;
  
//  if(normalVector[2] < 0.0f){
//    normalVector[2] = -normalVector[2];
//  }
  
}

//++++++++++++++++++++++++++++++++++ Draw 3D equipotential plane ++++++++++++++++++++++++++++++++++
void drawEquipotentialPlane(int planeNumber, int fillMode){
  int i, iEnd, layers;
  GLfloat x1, x2, x3, x4, y1, y2, y3, y4, z1, z2, z3, z4, GLfloatTemp;
  GLfloat alphaStep = 80.0f;
  
  x1 = path[planeNumber][0][0];
  y1 = path[planeNumber][0][1];
  
    
  for (i = 10; i < pathEnd[planeNumber]; i++){
    if(path[planeNumber][0][0] < path[planeNumber][1][0] && path[planeNumber][i+1][0] < path[planeNumber][i][0]){
      iEnd = i + 1;
      break;
    }
    if(path[planeNumber][0][0] > path[planeNumber][1][0] && path[planeNumber][i+1][0] > path[planeNumber][i][0]){
      iEnd = i + 1;
      break;
    }
  }

  if(planeNumber == 0 && path[planeNumber][iEnd][0] > 1.0f){
    enableSecondEquipotential = 0;
  }

  //printf("iEnd = %d\n", iEnd);


  glLightfv(GL_LIGHT1, GL_AMBIENT, lightRed1);
  glLightfv(GL_LIGHT1, GL_DIFFUSE, lightRed1);
  glLightfv(GL_LIGHT1, GL_SPECULAR, lightOrange1);

  if(fillMode == 0){//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1
    glPolygonMode(GL_FRONT, GL_LINE);
    glPolygonMode(GL_BACK, GL_LINE);
    glLineWidth(2.5);
  }

  //glBlendEquationSeparate(GL_FUNC_ADD, GL_FUNC_ADD);
  //glBlendFuncSeparate(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA, GL_ONE, GL_ZERO);

  //glBlendEquationSeparate(GL_FUNC_ADD, GL_FUNC_ADD);
  //glBlendFuncSeparate(GL_ONE, GL_ONE_MINUS_SRC_ALPHA, GL_ONE, GL_ZERO);

  //glBlendFunc(GL_SRC_ALPHA,GL_ONE);
  //glBlendFunc(GL_ONE ,GL_SRC_ALPHA);
  //glBlendFunc(GL_DST_COLOR, GL_ZERO);
  //glBlendFunc(GL_DST_COLOR, GL_ONE);
  //glBlendFunc(GL_ONE, GL_ONE);
  //glBlendFunc(GL_SRC_ALPHA,GL_ZERO);
  //glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  //glBlendFunc(GL_SRC_COLOR, GL_DST_ALPHA);
  
  //glEnable(GL_BLEND);
  //glDisable(GL_DEPTH_TEST);
  //glEnable(GL_COLOR_MATERIAL);
  //glDisable(GL_LIGHTING);
  //glColor4f(0.2f,0.2f,0.2f,0.7f); 
  
  for(layers = 0; layers < 1; layers++){
    if(layers == 1){
      glEnable(GL_LIGHTING);
    }
    glBegin(GL_QUADS);
    for (i = 0; i < iEnd; i++){
          
      x1 = path[planeNumber][i][0];
      x3 = path[planeNumber][i+1][0];
      
      for(alpha = 0.0f; alpha < 2.0f * PI; alpha += 2.0f * PI / alphaStep){

        y1 = path[planeNumber][i][1] * cos(alpha);
        y2 = path[planeNumber][i][1] * cos(alpha + 2.0f * PI / alphaStep);

        z1 = path[planeNumber][i][1] * sin(alpha);
        z2 = path[planeNumber][i][1] * sin(alpha + 2.0f * PI / alphaStep);
        
        x2 = x1;
        x4 = x3;
        
        y3 = path[planeNumber][i+1][1] * cos(alpha + 2.0f * PI / alphaStep);;
        y4 = path[planeNumber][i+1][1] * cos(alpha);

        z3 = path[planeNumber][i+1][1] * sin(alpha + 2.0f * PI / alphaStep);
        z4 = path[planeNumber][i+1][1] * sin(alpha);
        
        if(path[planeNumber][0][0] > path[planeNumber][10][0]){
  //        x1 = x3;
  //        x3 = x2;
  //        x2 = x1;
  //        x4 = x3;
          GLfloatTemp = y1;
          y1 = y2;
          y2 = GLfloatTemp;
          GLfloatTemp = y3;
          y3 = y4;
          y4 = GLfloatTemp;

          GLfloatTemp = z1;
          z1 = z2;
          z2 = GLfloatTemp;
          GLfloatTemp = z3;
          z3 = z4;
          z4 = GLfloatTemp;
        }

        p1[0] = x2;
        p1[1] = y2;
        p1[2] = z2;
        
        p2[0] = x1;
        p2[1] = y1;
        p2[2] = z1;
        
        p3[0] = x4;
        p3[1] = y4;
        p3[2] = z4;
        
        calculateNormalVector();

        glNormal3f(normalVector[0], normalVector[1], normalVector[2]);
        //glTexCoord2f(float_x, float_y);
        glVertex3f(x1, y1, z1); 
        //glTexCoord2f(float_x, float_yb);
        glVertex3f(x2, y2, z2); 
        //glTexCoord2f(float_xb, float_yb);
        glVertex3f(x3, y3, z3); 
        //glTexCoord2f(float_xb, float_y);
        glVertex3f(x4, y4, z4); 
      }
    }
    glEnd();
  }

  glPolygonMode(GL_BACK, GL_FILL);
  glPolygonMode(GL_FRONT, GL_FILL);


  glColor4f(1.0f,1.0f,1.0f,1.0f);
  glDisable(GL_BLEND);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_LIGHTING);
  //glDisable(GL_COLOR_MATERIAL);

  
  
}

//++++++++++++++++++++++++++++++++++ Draw sphere ++++++++++++++++++++++++++++++++++++++++++++++++++
void drawSphere(GLfloat x1, GLfloat y1, GLfloat z1, double radius, int nSlice, int nStack, int enableTranslucentSphere){
    int i, j;
    double phi; //
    double theta; //long
    float sp[129][129][3]; //Vertex
    float *sp1,*sp2,*sp3,*sp4;
    float spn[129][129][3]; //Vertex
    float *spn1,*spn2,*spn3,*spn4;




  glPushMatrix();
  glTranslatef( x1,y1,z1 );  

  if(enableTranslucentSphere == 1){
//    glPolygonMode(GL_FRONT, GL_LINE);
//    glPolygonMode(GL_BACK, GL_LINE);
//    glRotatef(90.0f, 0.0f, 1.0f, 0.0f);  /* rotate on the X axis */

    glBlendFunc(GL_SRC_ALPHA,GL_ONE);
    //glBlendFunc(GL_ONE ,GL_SRC_ALPHA);
    //glBlendFunc(GL_DST_COLOR, GL_ZERO);
    //glBlendFunc(GL_DST_COLOR, GL_ONE);
    //glBlendFunc(GL_ONE, GL_ONE);
    //glBlendFunc(GL_SRC_ALPHA,GL_ZERO);
    //glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    //glBlendFunc(GL_SRC_COLOR, GL_DST_ALPHA);
    
    glEnable(GL_BLEND);
    glDisable(GL_DEPTH_TEST);
    //glEnable(GL_COLOR_MATERIAL);
    //glDisable(GL_LIGHTING);
    glColor4f(0.2f,0.2f,0.2f,0.1f); 

  }

  

    if(nSlice > 128) nSlice = 128;
    if(nStack > 128) nStack = 128;

    //Vertex
    for(i = 0;i <= nSlice;i++){   
        phi = 2.0 * M_PI * (double)i / (double)nSlice;
        for(j = 0;j <= nStack;j++){   
            theta = M_PI * (double)j / (double)nStack;
            sp[i][j][0] = (float)(radius * sin(theta) * cos(phi));//x
            sp[i][j][1] = (float)(radius * sin(theta) * sin(phi));//y
            sp[i][j][2] = (float)(radius * cos(theta));           //z
        }
    }

    //Normals (radius = 1);
    for(i = 0;i <= nSlice;i++){   
        phi = 2.0 * M_PI * (double)i / (double)nSlice;
        for(j = 0;j <= nStack;j++){   
            theta = M_PI * (double)j / (double)nStack;
            spn[i][j][0] = (float)(1.0 * sin(theta) * cos(phi));//x
            spn[i][j][1] = (float)(1.0 * sin(theta) * sin(phi));//y
            spn[i][j][2] = (float)(1.0 * cos(theta));           //z
        }
    }


    //Top(j=0)
    for(i = 0;i < nSlice; i++){
        sp1 = sp[i][0];
        sp2 = sp[i][1];
        sp3 = sp[i+1][1]; 

        spn1 = spn[i][0];
        spn2 = spn[i][1];
        spn3 = spn[i+1][1]; 

        glBegin(GL_TRIANGLES);            
            glNormal3fv(spn1); glVertex3fv(sp1);
            glNormal3fv(spn2); glVertex3fv(sp2);
            glNormal3fv(spn3); glVertex3fv(sp3);
        glEnd();
    }
    //Bottom
    j=nStack-1;
    for(i = 0;i < nSlice; i++){
        sp1 = sp[i][j];
        sp2 = sp[i][j+1];
        sp3 = sp[i+1][j]; 

        spn1 = spn[i][j];
        spn2 = spn[i][j+1];
        spn3 = spn[i+1][j]; 

        glBegin(GL_TRIANGLES);
            glNormal3fv(spn1); glVertex3fv(sp1);
            glNormal3fv(spn2); glVertex3fv(sp2);
            glNormal3fv(spn3); glVertex3fv(sp3);
        glEnd();
    }

    for(i = 0;i < nSlice;i++){
        for(j = 1;j < nStack-1; j++){
            sp1 = sp[i][j];
            sp2 = sp[i][j+1];
            sp3 = sp[i+1][j+1];
            sp4 = sp[i+1][j];

            spn1 = spn[i][j];
            spn2 = spn[i][j+1];
            spn3 = spn[i+1][j+1];
            spn4 = spn[i+1][j];

            glBegin(GL_QUADS);
                glNormal3fv(spn1); glVertex3fv(sp1);
                glNormal3fv(spn2); glVertex3fv(sp2);
                glNormal3fv(spn3); glVertex3fv(sp3);
                glNormal3fv(spn4); glVertex3fv(sp4);
            glEnd();
        }
    }
    
    glPopMatrix();

    glPolygonMode(GL_BACK, GL_FILL);
    glPolygonMode(GL_FRONT, GL_FILL);

    glDisable(GL_BLEND);
    glEnable(GL_DEPTH_TEST);
    glColor4f(1.0f,1.0f,1.0f,1.0f); 


}

//++++++++++++++++++++++++++++++++++ Draw cylinder ++++++++++++++++++++++++++++++++++++++++++++++++++
void drawCylinder(float x1, float y1, float z1, float x2,float y2, float z2, float radius1, float radius2, int subdivisions){

/*
  if(x1 == 0.0 && x2 == 0.0){
    x1 = 0.001;
  }
  if(y1 == 0.0 && y2 == 0.0){
    y1 = 0.001;
  }
  if(z1 == 0.0 && z2 == 0.0){
    z1 = 0.001;
  }
*/

  GLfloat vx = x2-x1;
  GLfloat vy = y2-y1;
  GLfloat vz = z2-z1;

  //handle the degenerate case with an approximation
/*
  if(vx < 0.001 && vx > 0.0){
    vx = 0.001;
  }
  if(vx > -0.001 && vx < 0.0){
    vx = -0.001;
  }
  if(vx == 0.0){
    vx = 0.001;
  }

  if(vy < 0.001 && vy > 0.0){
    vy = 0.001;
  }
  if(vy > -0.001 && vy < 0.0){
    vy = -0.001;
  }
  if(vy == 0.0){
    vy = 0.001;
  }
*/

  if(vz < 0.001 && vz > 0.0){
    vz = 0.001;
  }
  if(vz > -0.001 && vz < 0.0){
    vz = -0.001;
  }
  if(vz == 0.0){
    vz = 0.001;
  }
  
  
  GLfloat v = sqrt( vx*vx + vy*vy + vz*vz );
//  if(v<0.001)v=0.001;
  GLfloat ax = 57.2957795*acos( vz * 1000.0 / (v * 1000.0f) );//multiply with 1000.0f to avoid wrong result at small angles
  if ( vz < 0.0 ){
    ax = -ax;
  }
  GLfloat rx = -vy*vz * 1000.0f;//multiply with 1000.0f to avoid wrong result at small angles
  GLfloat ry = vx*vz * 1000.0f;//multiply with 1000.0f to avoid wrong result at small angles

  GLUquadricObj *quadric=gluNewQuadric();
  gluQuadricNormals(quadric, GLU_SMOOTH);

/*
  if(rx < 0.0001 && rx > 0.0){
    rx = 0.0001;
  }
  if(rx > -0.0001 && rx < 0.0){
    rx = -0.0001;
  }
  if(ry < 0.0001 && ry > 0.0){
    ry = 0.0001;
  }
  if(ry > -0.0001 && ry < 0.0){
    ry = -0.0001;
  }
*/
//ax = 89.5;//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  glPushMatrix();
  glTranslatef( x1,y1,z1 );
  glRotatef(ax, rx, ry, 0.000001);
  
  //printf("vx = %lf, vy = %lf, vz = %lf, ax = %lf, rx = %lf, ry = %lf, v=%lf\n", vx, vy, vz, ax, rx, ry, v);
  
  //draw the cylinder
  gluCylinder(quadric, radius1, radius2, v, 32, 1);
  gluQuadricOrientation(quadric,GLU_INSIDE);
  //draw the first cap
  gluDisk( quadric, 0.0, radius1, 32, 1);
  glTranslatef( 0,0,v );
  //draw the second cap
  gluQuadricOrientation(quadric,GLU_OUTSIDE);
  gluDisk( quadric, 0.0, radius2, 32, 1);
  

  glPopMatrix();

  gluDeleteQuadric(quadric);
}




void paintAxes(GLfloat xMax, GLfloat yMax, GLfloat zMax, int orientation){
    GLfloat peak, peakLength = 0.5f;

  
    glDisable(GL_LIGHTING);
    glDisable(GL_BLEND);
    glEnable(GL_DEPTH_TEST);
    
    
    glLineWidth(5.5);
    
    //X-axis
    glColor3f(axeXColor[0], axeXColor[1], axeXColor[2]);

    peak = xMax + peakLength * 1.1f;
    glBegin(GL_LINES);
    glVertex3f(0.0, 0.0, 0.0);
    glVertex3f(peak, 0.0f, 0.0f);
    glEnd();
    glBegin(GL_LINES);
    glVertex3f(peak - peakLength, peakLength, 0.0f);
    glVertex3f(peak, 0.0f, 0.0f);
    glEnd();
    glBegin(GL_LINES);
    glVertex3f(peak - peakLength, -peakLength, 0.0f);
    glVertex3f(peak, 0.0f, 0.0f);
    glEnd();

    //Paint X
    glBegin(GL_LINES);
    glVertex3f(peak + peakLength * 0.5, -peakLength/2.0f, 0.0f);
    glVertex3f(peak + peakLength * 1.5, peakLength/2.0f, 0.0f);
    glEnd();
    glBegin(GL_LINES);
    glVertex3f(peak + peakLength * 1.5, -peakLength/2.0f, 0.0f);
    glVertex3f(peak + peakLength * 0.5, peakLength/2.0f, 0.0f);
    glEnd();


    //Z-axis
    peak = zMax + peakLength * 1.1f;
    if(orientation == 1 || orientation == 2){
      glColor3f(axeYColor[0], axeYColor[1], axeYColor[2]);
    }
    else{
      glColor3f(axeZColor[0], axeZColor[1], axeZColor[2]);
    }
    
    glBegin(GL_LINES);
    glVertex3f(0.0, 0.0, 0.0);
    glVertex3f(0.0f, 0.0f, -peak);
    glEnd();
    if(zMax > 0.0f){
      glBegin(GL_LINES);
      glVertex3f(peakLength, 0.0f, -(peak - peakLength));
      glVertex3f(0.0f, 0.0f, -peak);
      glEnd();
      glBegin(GL_LINES);
      glVertex3f(-peakLength, 0.0f, -(peak - peakLength));
      glVertex3f(0.0f, 0.0f, -peak);
      glEnd();
    }
    else{
      glBegin(GL_LINES);
      glVertex3f(peakLength, 0.0f, -peak - peakLength);
      glVertex3f(0.0f, 0.0f, -peak);
      glEnd();
      glBegin(GL_LINES);
      glVertex3f(-peakLength, 0.0f, -peak - peakLength);
      glVertex3f(0.0f, 0.0f, -peak);
      glEnd();
    }
    if(orientation == 1 || orientation == 2){
      //Paint Y at Z-axis when orientation is swapped
      glBegin(GL_LINES);
      glVertex3f( -peakLength/2.0f, 0.0f, -(peak + peakLength * 1.5));
      glVertex3f( 0.0f, 0.0f, -(peak + peakLength * 0.8));
      glEnd();
      glBegin(GL_LINES);
      glVertex3f( peakLength/2.0f, 0.0f, -(peak + peakLength * 1.5));
      glVertex3f( 0.0f, 0.0f, -(peak + peakLength * 0.8));
      glEnd();
      glBegin(GL_LINES);
      glVertex3f( 0.0f, 0.0f, -(peak + peakLength * 0.8));
      glVertex3f( 0.0f, 0.0f, -(peak + peakLength * 0.5));
      glEnd();
    }
    else{
      //Paint Z
      if(zMax > 0.0f){
        glBegin(GL_LINES);
        glVertex3f( -peakLength/2.0f, 0.0f, -(peak + peakLength * 1.5));
        glVertex3f( peakLength/2.0f, 0.0f, -(peak + peakLength * 1.5));
        glEnd();
        if(orientation == 3){
          glBegin(GL_LINES);
          glVertex3f( -peakLength/2.0f, 0.0f, -(peak + peakLength * 1.5));
          glVertex3f( peakLength/2.0f, 0.0f, -(peak + peakLength * 0.5));
          glEnd();
        }
        else{
          glBegin(GL_LINES);
          glVertex3f( peakLength/2.0f, 0.0f, -(peak + peakLength * 1.5));
          glVertex3f( -peakLength/2.0f, 0.0f, -(peak + peakLength * 0.5));
          glEnd();
        }
        glBegin(GL_LINES);
        glVertex3f( -peakLength/2.0f, 0.0f, -(peak + peakLength * 0.5));
        glVertex3f( peakLength/2.0f, 0.0f, -(peak + peakLength * 0.5));
        glEnd();
      }
      else{
        glBegin(GL_LINES);
        glVertex3f( -peakLength/2.0f, 0.0f, -peak + peakLength * 1.5);
        glVertex3f( peakLength/2.0f, 0.0f, -peak + peakLength * 1.5);
        glEnd();
        if(orientation == 3){
          glBegin(GL_LINES);
          glVertex3f( peakLength/2.0f, 0.0f, -peak + peakLength * 1.5);
          glVertex3f( -peakLength/2.0f, 0.0f, -peak + peakLength * 0.5);
          glEnd();
        }
        else{
          glBegin(GL_LINES);
          glVertex3f( -peakLength/2.0f, 0.0f, -peak + peakLength * 1.5);
          glVertex3f( peakLength/2.0f, 0.0f, -peak + peakLength * 0.5);
          glEnd();
        }
        glBegin(GL_LINES);
        glVertex3f( -peakLength/2.0f, 0.0f, -peak + peakLength * 0.5);
        glVertex3f( peakLength/2.0f, 0.0f, -peak + peakLength * 0.5);
        glEnd();
      }
    }


    if(axeYColor[0] > 0.0f || axeYColor[1] > 0.0f || axeYColor[2] > 0.0f){//Dont paint if color is black
      //Paint Y-axis
      peak = yMax + peakLength * 1.1f;
      if(orientation == 1 || orientation == 2){
        glColor3f(axeZColor[0], axeZColor[1], axeZColor[2]);
      }
      else{
        glColor3f(axeYColor[0], axeYColor[1], axeYColor[2]);
      }
      

      glBegin(GL_LINES);
      glVertex3f(0.0, 0.0, 0.0);
      glVertex3f(0.0f, peak, 0.0f);
      glEnd();
      glBegin(GL_LINES);
      glVertex3f(peakLength, peak - peakLength, 0.0f);
      glVertex3f(0.0f, peak, 0.0f);
      glEnd();
      glBegin(GL_LINES);
      glVertex3f(-peakLength, peak - peakLength, 0.0f);
      glVertex3f(0.0f, peak, 0.0f);
      glEnd();
      
      if(orientation == 1 || orientation == 2){
        //Paint Z at Y-axis when orientation is swapped
        if(zMax > 0.0f){
          glBegin(GL_LINES);
          glVertex3f( -peakLength/2.0f, (peak + peakLength * 1.5), 0.0f);
          glVertex3f( peakLength/2.0f, (peak + peakLength * 1.5), 0.0f);
          glEnd();
          if(orientation == 1){
            glBegin(GL_LINES);
            glVertex3f( peakLength/2.0f, (peak + peakLength * 1.5), 0.0f);
            glVertex3f( -peakLength/2.0f, (peak + peakLength * 0.5), 0.0f);
            glEnd();
          }
          else{
            glBegin(GL_LINES);
            glVertex3f( -peakLength/2.0f, (peak + peakLength * 1.5), 0.0f);
            glVertex3f( peakLength/2.0f, (peak + peakLength * 0.5), 0.0f);
            glEnd();
          }
          glBegin(GL_LINES);
          glVertex3f( -peakLength/2.0f, (peak + peakLength * 0.5), 0.0f);
          glVertex3f( peakLength/2.0f, (peak + peakLength * 0.5), 0.0f);
          glEnd();
        }
        else{
          glBegin(GL_LINES);
          glVertex3f( -peakLength/2.0f, peak + peakLength * 1.5, 0.0f);
          glVertex3f( peakLength/2.0f, peak + peakLength * 1.5, 0.0f);
          glEnd();
          glBegin(GL_LINES);
          glVertex3f( peakLength/2.0f, peak + peakLength * 1.5, 0.0f);
          glVertex3f( -peakLength/2.0f, peak + peakLength * 0.5, 0.0f);
          glEnd();
          glBegin(GL_LINES);
          glVertex3f( -peakLength/2.0f, peak + peakLength * 0.5, 0.0f);
          glVertex3f( peakLength/2.0f, peak + peakLength * 0.5, 0.0f);
          glEnd();
        }
      }
      else{
        //Paint Y
        glBegin(GL_LINES);
        glVertex3f( -peakLength/2.0f, peak + peakLength * 1.5, 0.0f);
        glVertex3f( 0.0f, peak + peakLength * 0.8, 0.0f);
        glEnd();
        glBegin(GL_LINES);
        glVertex3f( peakLength/2.0f, peak + peakLength * 1.5, 0.0f);
        glVertex3f( 0.0f, peak + peakLength * 0.8, 0.0f);
        glEnd();
        glBegin(GL_LINES);
        glVertex3f( 0.0f, peak + peakLength * 0.8, 0.0f);
        glVertex3f( 0.0f, peak + peakLength * 0.5, 0.0f);
        glEnd();
      }
    }
    

}

/* Here goes our drawing code */
int drawGLScene(GLvoid){
    int x, z, i, i2;
    GLfloat x1, x2, x3, x4, y1, y2, y3, y4, z1, z2, z3, z4;
    GLfloat chargeX = 0.0f, chargeY = 0.0f;
    GLfloat totalDistance;
    GLfloat totalForce, potential;
    GLfloat steps = 200.0f;
    char textLine[1000];
    float float_x, float_z, float_xb, float_zb;
    GLfloat k = 1.0 / (4.0 * PI * 8.85419E-12);
    GLfloat forceX, forceY, forceZ;
    double fieldLineStep = 0.1;
    GLfloat arrowRadius = 3.0;
    GLfloat peak = 0.0;
    
    usleep(slowDownMS);
    
    yMax = xMax / 16.0f * 9.0f;
    coordScale = 2.0 * xMax / gridX;
    
    glColor4f(1.0f,1.0f,1.0f,1.0f);         // Full Brightness, 50% Alpha ( NEW )
    glBlendFunc(GL_SRC_ALPHA,GL_ONE);       // Blending Function For Translucency Based On Source Alpha 
        
    
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    
    glLightfv(GL_LIGHT1, GL_POSITION, lightPosition1);
    
    glLoadIdentity();
    glTranslatef(translateX, translateY, translateZ);
    glRotatef(rotX, 1.0f, 0.0f, 0.0f);  /* rotate on the X axis */
    glRotatef(rotY, 0.0f, 1.0f, 0.0f);  /* rotate on the Y axis */
    glRotatef(rotZ, 0.0f, 0.0f, 1.0f);  /* rotate on the Z axis */
    //glColor3f(1.0f, 1.0f, 0.0f);
    
    totalForce = particleCharge * testCharge / pow(minDistance, 2.0);
    if(totalForce < 4.0f){
      totalForce = 4.0f;
    }
    if(animation < 100){
      paintAxes(gridX / 2.0 * coordScale, totalForce, -(gridY / 2.0 * coordScale + 1.2f), 0);
    }
    if(animation >= 100 && animation < 200){
      paintAxes(4.0, 4.0, -4.0, 3);    
    }
    if(animation >= 200 && animation < 300){
      paintAxes(5.5, 4.0, -4.0, 0);
    }
    if(animation >= 300 && animation < 400){
      paintAxes(8.5 * xMax / 8.0, 5.5, -5.0 * xMax / 8.0, 0);
    }
    if(animation >= 400 && animation < 500){
      paintAxes(8.5 * xMax / 8.0, 5.5, -5.0 * xMax / 8.0, 0);
    }
    if(animation >= 522 && animation < 525){
      paintAxes(5.5, 4.0, -3.5, 3);
    }
    if(animation >= 1500 && animation < 1600){
      paintAxes(8.0, 5.0, -5.5, 0);
    }
    if(animation >= 1700 && animation < 1800){
      paintAxes(8.0, 3.0, -5.5, 0);
    }
    
    if(animation >= 1800 && animation < 1900){
      paintAxes(8.0, 0.3, -5.5, 0);
    }

    if(animation >= 1901 && animation < 2000){
      paintAxes(6.0, 4.0, 2.0, 1);
    }

    if(animation >= 2300 && animation < 2400){
      paintAxes(33.0, 8.0, -19.0, 0);
    }


    //glEnable(GL_BLEND);
    //glDisable(GL_DEPTH_TEST);
    glEnable(GL_LIGHTING);
    glLineWidth(4.5);
    glPolygonMode(GL_BACK, GL_LINE);
    glPolygonMode(GL_FRONT, GL_LINE);
    

    if(enableParticles == 1){
      //Draw particles
      glEnable(GL_LIGHTING);
      glPolygonMode(GL_BACK, GL_FILL);
      glPolygonMode(GL_FRONT, GL_FILL);
      for(i=0; i< particleNumber; i++){
        enableTransLucentSphere = 0;
        if(particles[i][4] < 0.0f){
          glLightfv(GL_LIGHT1, GL_AMBIENT, lightBlue3);
          glLightfv(GL_LIGHT1, GL_DIFFUSE, lightBlue3);
          glLightfv(GL_LIGHT1, GL_SPECULAR, lightBlue3);
        }
        else{
          glLightfv(GL_LIGHT1, GL_AMBIENT, lightRed1);
          glLightfv(GL_LIGHT1, GL_DIFFUSE, lightRed1);
          glLightfv(GL_LIGHT1, GL_SPECULAR, lightOrange1);
        }
        if(particles[i][5] == 1.0f){//"Neutral" particle
          glLightfv(GL_LIGHT1, GL_AMBIENT, lightWhite1);
          glLightfv(GL_LIGHT1, GL_DIFFUSE, lightWhite1);
          glLightfv(GL_LIGHT1, GL_SPECULAR, lightWhite1);
        }
        if(particles[i][5] == 2.0f){//Almost "neutral" particle
          glLightfv(GL_LIGHT1, GL_AMBIENT, lightRed3);
          glLightfv(GL_LIGHT1, GL_DIFFUSE, lightRed3);
          glLightfv(GL_LIGHT1, GL_SPECULAR, lightRed3);
        }
        if(particles[i][5] == 3.0f){//Translucent blue particle
          glLightfv(GL_LIGHT1, GL_AMBIENT, lightBlue3);
          glLightfv(GL_LIGHT1, GL_DIFFUSE, lightBlue3);
          glLightfv(GL_LIGHT1, GL_SPECULAR, lightBlue3);
          enableTransLucentSphere = 1;
        }
        if(particles[i][5] != -1.0f){//Invisible particle
          drawSphere(particles[i][0],particles[i][1], particles[i][2], particles[i][3], 40, 40, enableTransLucentSphere);
        }
      }
    }

    if(animation >= 1000 && animation < 1100){
      glLightfv(GL_LIGHT1, GL_AMBIENT, lightWhite1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightWhite1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightWhite1);
      
      drawCylinder(particles[0][0], particles[0][1], -0.051f, particles[1][0], particles[1][1], -0.051f, 0.05f, 0.05f, 32);//"Bar" between charges
      drawCylinder(particles[0][0], particles[0][1], -0.051f, particles[2][0], particles[2][1], -0.051f, 0.05f, 0.05f, 32);//"Bar" between charges
      
      drawCylinder(particles[3][0], particles[3][1], -0.051f, particles[4][0], particles[4][1], -0.051f, 0.05f, 0.05f, 32);//"Bar" between charges
      drawCylinder(particles[3][0], particles[3][1], -0.051f, particles[5][0], particles[5][1], -0.051f, 0.05f, 0.05f, 32);//"Bar" between charges
      
      drawCylinder(particles[6][0], particles[6][1], -0.051f, particles[7][0], particles[7][1], -0.051f, 0.05f, 0.05f, 32);//"Bar" between charges
      drawCylinder(particles[6][0], particles[6][1], -0.051f, particles[8][0], particles[8][1], -0.051f, 0.05f, 0.05f, 32);//"Bar" between charges

      drawCylinder(particles[9][0], particles[9][1], -0.051f, particles[10][0], particles[10][1], -0.051f, 0.05f, 0.05f, 32);//"Bar" between charges
      drawCylinder(particles[9][0], particles[9][1], -0.051f, particles[11][0], particles[11][1], -0.051f, 0.05f, 0.05f, 32);//"Bar" between charges
    }
    

    
    //Paint force vectors at dipole or molecules
    if(enableFieldVectors == 20 || enableFieldVectors == 21){
      glLightfv(GL_LIGHT1, GL_AMBIENT, lightGreen1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightGreen1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightGreen1);

      testCharge = 0.000001;
      GLfloat beta;
      GLfloat vectorDiameter = 0.03f;
      GLfloat vectorLength;
      int iEnd = 2;
      GLfloat forceFactor = 12.51f;
      GLfloat particleForceX = 0.0f;
      GLfloat particleForceY = 0.0f;

      if(enableFieldVectors == 21){
        iEnd = 12;
        forceFactor = 100.0f;
      }

      for(i2 = 0; i2 < iEnd; i2++){
        forceX = 0.0;
        forceY = 0.0;
        totalForce = 0.0;
        totalDistance = 0.0;
        if(i2 == 0 || i2 == 3 || i2 == 6 || i2 == 9){
          particleForceX = 0.0f;
          particleForceY = 0.0f;
        }
        for(i = 0; i < particleNumber; i++){
          if(enableFieldVectors == 20){
            if(i != i2){
              totalDistance = sqrt(pow(particles[i2][0] - particles[i][0], 2.0) + pow(particles[i2][1] - particles[i][1], 2.0) + pow(particles[i2][2] - particles[i][2], 2.0));
              totalForce = k1 * particles[i][4] * particles[i2][4] / pow(totalDistance, 2.0);
              beta = atan2(particles[i2][1] - particles[i][1], particles[i2][0] - particles[i][0]);
              forceX += totalForce * cos(beta);
              forceY += totalForce * sin(beta);
            }
          }
          if(enableFieldVectors == 21){
            if((i2 < 3 && i > 2 && i < 6) || (i2 > 2 && i2 < 6 && i < 3) || (i2 > 5 && i2 < 9 && i > 8) || (i2 > 8 && i < 9 && i2 > 5)){//Don't calculate intermolecular fores
              totalDistance = sqrt(pow(particles[i2][0] - particles[i][0], 2.0) + pow(particles[i2][1] - particles[i][1], 2.0) + pow(particles[i2][2] - particles[i][2], 2.0));
              totalForce = k1 * particles[i][4] * particles[i2][4] / pow(totalDistance, 2.0);
              beta = atan2(particles[i2][1] - particles[i][1], particles[i2][0] - particles[i][0]);
              forceX += totalForce * cos(beta);
              forceY += totalForce * sin(beta);
              //particleForceX += totalForce * cos(beta);
              particleForceY += totalForce * sin(beta);
            }
          }
        }//for(i = 0; i < particleNumber; i++){


        if(enableFieldVectors == 20){
          totalForce = sqrt(pow(forceX, 2.0) + pow(forceY, 2.0));
          x2 = forceX * sqrt(totalForce)/totalForce * forceFactor;
          y2 = forceY * sqrt(totalForce)/totalForce * forceFactor;
          vectorLength = sqrt(pow(x2, 2.0) + pow(y2, 2.0) + pow(z2, 2.0));
          vectorDiameter = 0.05 * vectorLength;

          beta = atan2(forceY, forceX);
          x1 = particles[i2][0];
          y1 = particles[i2][1];
          x2 = x1 + x2;
          y2 = y1 + y2;
          z1 = 0.0f;
          z2 = 0.0f;
                        
          drawCylinder(x1, y1, z1, x2, y2, z2, vectorDiameter, vectorDiameter, 32);
          x1 += 0.7 * (x2 - x1);
          y1 += 0.7 * (y2 - y1);
          drawCylinder(x1, y1, z1, x2, y2, z2, vectorDiameter * 3.33f, vectorDiameter, 32);
        }

        if(enableFieldVectors == 21){//Total force between two "molecules"
          glDisable(GL_DEPTH_TEST);
          glDisable(GL_LIGHTING);
          if(i2 == 2 || i2 == 5 || i2 == 8 || i2 == 11){
            
            totalForce = sqrt(pow(particleForceX, 2.0) + pow(particleForceY, 2.0));
            x2 = particleForceX * sqrt(totalForce)/totalForce * forceFactor;
            y2 = particleForceY * sqrt(totalForce)/totalForce * forceFactor;
            vectorLength = sqrt(pow(x2, 2.0) + pow(y2, 2.0) + pow(z2, 2.0));
            vectorDiameter = 0.05 * vectorLength;

            beta = atan2(particleForceY, particleForceX);
            x1 = particles[i2 - 2][0];
            y1 = particles[i2 - 2][1];
            x2 = x1 + x2;
            y2 = y1 + y2;
            z1 = 0.0f;
            z2 = 0.0f;
            
//            glBegin(GL_LINES);
//            glVertex3f(x1, y1, z1);
//            glVertex3f(x2, y2, z2);
//            glEnd();        
            

            glColor3f(1.0f, 1.0f, 0.0f);
            drawCylinder(x1, y1, z1, x2, y2, z2, vectorDiameter, vectorDiameter, 32);
            x1 += 0.7 * (x2 - x1);
            y1 += 0.7 * (y2 - y1);
            drawCylinder(x1, y1, z1, x2, y2, z2, vectorDiameter * 3.33f, vectorDiameter, 32);
            
          }
          glEnable(GL_DEPTH_TEST);
          glEnable(GL_LIGHTING);
        }


      }//for(i2 = 0; i2 < iEnd; i2++){
       

    }//if(enableFieldVectors == 20 || enableFieldVectors == 21){
    
    //Paint single charge with force vector moving around
    if(enableFieldVectors == 10 || enableFieldVectors == 11){
      if(animation > 1600 && animation < 1700){
        glLightfv(GL_LIGHT1, GL_AMBIENT, lightOrange1);
        glLightfv(GL_LIGHT1, GL_DIFFUSE, lightOrange1);
        glLightfv(GL_LIGHT1, GL_SPECULAR, lightOrange1);
      }
      else{
        glLightfv(GL_LIGHT1, GL_AMBIENT, lightGreen1);
        glLightfv(GL_LIGHT1, GL_DIFFUSE, lightGreen1);
        glLightfv(GL_LIGHT1, GL_SPECULAR, lightGreen1);
      }

      testCharge = 0.000001;
      GLfloat vectors[300][3];
      int pointerDrawn = 0;
      GLfloat beta;
      GLfloat vectorDiameter = 0.03f;
      GLfloat vectorLength;


      forceX = 0.0;
      forceY = 0.0;
      totalForce = 0.0;
      totalDistance = 0.0;
      pointerDrawn = 0;
      testCharge = 0.000001;
      for(i = 0; i < particleNumber-1; i++){
        totalDistance = sqrt(pow(particles[particleNumber-1][0] - particles[i][0], 2.0) + pow(particles[particleNumber-1][1] - particles[i][1], 2.0) + pow(particles[particleNumber-1][2] - particles[i][2], 2.0));
        totalForce = k1 * particles[i][4] * testCharge / pow(totalDistance, 2.0);
        beta = atan2(particles[particleNumber-1][1] - particles[i][1], particles[particleNumber-1][0] - particles[i][0]);
        forceX += totalForce * cos(beta);
        forceY += totalForce * sin(beta);
      }

      totalForce = sqrt(pow(forceX, 2.0) + pow(forceY, 2.0));
      if(enableFieldVectors == 10){
        x2 = forceX * sqrt(totalForce)/totalForce * 4.51;
        y2 = forceY * sqrt(totalForce)/totalForce * 4.51;
      }
      else{
        x2 = forceX * sqrt(totalForce)/totalForce * 1.51;
        y2 = forceY * sqrt(totalForce)/totalForce * 1.51;
      }
      vectorLength = sqrt(pow(x2, 2.0) + pow(y2, 2.0) + pow(z2, 2.0));
      vectorDiameter = 0.05 * vectorLength;

      beta = atan2(forceY, forceX);
      x1 = particles[particleNumber-1][0];
      y1 = particles[particleNumber-1][1];
      x2 = x1 + x2;
      y2 = y1 + y2;
      z1 = 0.20f;
      z2 = 0.20f;
            
      drawCylinder(x1, y1, z1, x2, y2, z2, vectorDiameter, vectorDiameter, 32);
      x1 += 0.7 * (x2 - x1);
      y1 += 0.7 * (y2 - y1);
      drawCylinder(x1, y1, z1, x2, y2, z2, vectorDiameter * 3.33f, vectorDiameter, 32);
      if(enableLablesFieldVectors == 1){
        glEnable(GL_TEXTURE_2D);
        glEnable(GL_BLEND);
        glDisable(GL_LIGHTING);
        glEnable(GL_DEPTH_TEST);
        sprintf(textLine, "F = %+4.0fmN", totalForce * 1000.0f);
        printGLf(-4.2, 2.2, textLine, 0, -1, 0.055f);
        if(minValue2 > 9999.9f){
          if(totalForce < minValue1){
            minValue1 = totalForce;
          }
          if(totalForce > maxValue1){
            maxValue1 = totalForce;
          }
          sprintf(textLine, "F(min) = %+4.0fmN", minValue1 * 1000.0f);
          printGLf(-4.2, 2.0, textLine, 0, -1, 0.055f);
          sprintf(textLine, "F(max) = %+4.0fmN", maxValue1 * 1000.0f);
          printGLf(-4.2, 1.8, textLine, 0, -1, 0.055f);
        }
        else{
          if(totalForce < minValue2){
            minValue2 = totalForce;
          }
          if(totalForce > maxValue2){
            maxValue2 = totalForce;
          }
          sprintf(textLine, "F(min) = %+4.0fmN", minValue2 * 1000.0f);
          printGLf(-4.2, 2.0, textLine, 0, -5, 0.055f);
          sprintf(textLine, "F(max) = %+4.0fmN", maxValue2 * 1000.0f);
          printGLf(-4.2, 1.8, textLine, 0, -5, 0.055f);
          sprintf(textLine, "Relative change = %3.0f%%", 100.0f * (maxValue2 - minValue2) / maxValue2);
          printGLf(-4.2, 1.4, textLine, 0, -5, 0.055f);
        }
        sprintf(textLine, "Relative change = %3.0f%%", 100.0f * (maxValue1 - minValue1) / maxValue1);
        printGLf(-4.2, 1.6, textLine, 0, -1, 0.055f);
      }
      
      glDisable(GL_TEXTURE_2D);       /* Enable Texture Mapping */
      glDisable(GL_BLEND);

    }//if(enableFieldVectors == 10){
    
    
    //Paint grid of electric field vectors
    if(enableFieldVectors == 1 || enableFieldVectors == 2 || enableFieldVectors == 20 || enableFieldVectors == 21){
      glLightfv(GL_LIGHT1, GL_AMBIENT, lightBlue2);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightBlue2);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightBlue2);


      testCharge = 0.000001;
      GLfloat vectors[300][3];
      int v;
      int pointerDrawn = 0;
      GLfloat beta;
      GLfloat x1a, y1a, x2a, y2a;
      GLfloat vectorDiameter = 0.03f;
      GLfloat vectorLength;

      for(i = 0; i < 54; i++){//Define coords of the grid
        for(i2 = 0; i2 < 36; i2++){
          if(enableFieldVectors == 1 || enableFieldVectors == 20 || enableFieldVectors == 21){
            vectors[i * 36 + i2][0] = (double)(i) / 2.0 - 14.0;
            vectors[i * 36 + i2][1] = (double)(i2) / 2.0 - 8.0;
          }
          else{
            vectors[i * 36 + i2][0] = (double)(i) / 1.0 - 14.3;
            vectors[i * 36 + i2][1] = (double)(i2) / 1.0 - 8.0;
          }
          vectors[i * 36 + i2][2] = 0.0;
        }
      }

      for(v = 0; v < 36 * 54; v++){
        forceX = 0.0;
        forceY = 0.0;
        totalForce = 0.0;
        totalDistance = 0.0;
        pointerDrawn = 0;
        for(i = 0; i < particleNumber; i++){
          totalDistance = sqrt(pow(vectors[v][0] - particles[i][0], 2.0) + pow(vectors[v][1] - particles[i][1], 2.0) + pow(vectors[v][2] - particles[i][2], 2.0));
          if(totalDistance < particles[i][3] + minVectorDistance){
            pointerDrawn = 1;
            break;
          }
          totalForce = k1 * particles[i][4] * testCharge / pow(totalDistance, 2.0);
          beta = atan2(vectors[v][1] - particles[i][1], vectors[v][0] - particles[i][0]);
          //printf("totalForce=%lf, beta=%lf\n", totalForce, beta);
          forceX += totalForce * cos(beta);
          forceY += totalForce * sin(beta);
        }

        totalForce = sqrt(pow(forceX, 2.0) + pow(forceY, 2.0));
        x2 = forceX * sqrt(totalForce)/totalForce * 1.71;
        y2 = forceY * sqrt(totalForce)/totalForce * 1.71;
        vectorLength = sqrt(pow(x2, 2.0) + pow(y2, 2.0) + pow(z2, 2.0));
        vectorDiameter = 0.1 * vectorLength;

        for(i = 0; i < particleNumber; i++){
          totalDistance = sqrt(pow(vectors[v][0] + x2 - particles[i][0], 2.0) + pow(vectors[v][1] + y2 - particles[i][1], 2.0) + pow(vectors[v][2] - particles[i][2], 2.0));
          if(totalDistance < particles[i][3] + minVectorDistance){
            pointerDrawn = 1;
            break;
          }
          totalDistance = sqrt(pow(vectors[v][0] + x2 * 0.33 - particles[i][0], 2.0) + pow(vectors[v][1] + y2 * 0.33 - particles[i][1], 2.0) + pow(vectors[v][2] - particles[i][2], 2.0));
          if(totalDistance < particles[i][3] + minVectorDistance){
            pointerDrawn = 1;
            break;
          }
          totalDistance = sqrt(pow(vectors[v][0] + x2 * 0.66 - particles[i][0], 2.0) + pow(vectors[v][1] + y2 * 0.66 - particles[i][1], 2.0) + pow(vectors[v][2] - particles[i][2], 2.0));
          if(totalDistance < particles[i][3] + minVectorDistance){
            pointerDrawn = 1;
            break;
          }
        }

        if(pointerDrawn == 0){          
          beta = atan2(forceY, forceX);
          x1 = vectors[v][0];
          y1 = vectors[v][1];
          x2 = x1 + x2;
          y2 = y1 + y2;
          z1 = 0.0f;
          z2 = 0.0f;
          //x2 += 10 * s * Math.cos(beta);
          //y2 += 10 * s * Math.sin(beta);
          
          drawCylinder(x1, y1, z1, x2, y2, z2, vectorDiameter, vectorDiameter, 32);
          x1 += 0.7 * (x2 - x1);
          y1 += 0.7 * (y2 - y1);
          drawCylinder(x1, y1, z1, x2, y2, z2, vectorDiameter * 3.33f, vectorDiameter, 32);

          //x1a = ((-0.3) * cos(beta) - (0.3) * sin(beta));
          //y1a = ((-0.3) * sin(beta) + (0.3) * cos(beta));
          //x2a = ((-0.3) * cos(beta) - (-0.3) * sin(beta));
          //y2a = ((-0.3) * sin(beta) + (-0.3) * cos(beta));
          
          //printf("x1=%lf, y1=%lf, z1=%lf, x2=%lf, y2=%lf, z2=%lf\n", x1, y1, z1, x2, y2, z2);

          //bufferImageSurface.drawLine((int)(x2), (int)(y2), (int)(x2 + x1a), (int)(y2 + y1a));
          //bufferImageSurface.drawLine((int)(x2), (int)(y2), (int)(x2 + x2a), (int)(y2 + y2a));          

        }

      }//for(vector = 0; vector < 10; vector++){
 
    }//if(enableFieldVectors == 1){
      

    
    //Draw potential plane
    if(enablePotentialPlane == 1){
      //Draw particles
      glEnable(GL_LIGHTING);
      glPolygonMode(GL_BACK, GL_FILL);
      glPolygonMode(GL_FRONT, GL_FILL);
      for(i=0; i< particleNumber; i++){
        if(particles[i][4] < 0.0f){
          glLightfv(GL_LIGHT1, GL_AMBIENT, lightBlue1);
          glLightfv(GL_LIGHT1, GL_DIFFUSE, lightBlue1);
          glLightfv(GL_LIGHT1, GL_SPECULAR, lightBlue1);
        }
        else{
          glLightfv(GL_LIGHT1, GL_AMBIENT, lightRed1);
          glLightfv(GL_LIGHT1, GL_DIFFUSE, lightRed1);
          glLightfv(GL_LIGHT1, GL_SPECULAR, lightOrange1);
        }
        drawSphere(particles[i][0], 0.0, particles[i][2], particles[i][3], 40, 40, 0);
      }
      
      if(animation > 323 && animation < 400){
        glLightfv(GL_LIGHT1, GL_AMBIENT, lightTemp);
        glLightfv(GL_LIGHT1, GL_DIFFUSE, lightTemp);
        glLightfv(GL_LIGHT1,GL_SPECULAR,lightTemp);
        //glLightfv(GL_LIGHT2, GL_AMBIENT, lightTemp);
        glLightfv(GL_LIGHT2, GL_DIFFUSE, lightTemp);
        //glLightfv(GL_LIGHT2,GL_SPECULAR,lightTemp);
        //glLightfv(GL_LIGHT3, GL_AMBIENT, lightTemp);
        glLightfv(GL_LIGHT3, GL_DIFFUSE, lightTemp);
        //glLightfv(GL_LIGHT3,GL_SPECULAR,lightTemp);
      }
      else{
        glLightfv(GL_LIGHT1, GL_AMBIENT, lightAmbient1);
        glLightfv(GL_LIGHT1, GL_DIFFUSE, lightDiffuse1);
        glLightfv(GL_LIGHT1,GL_SPECULAR,specular1);
        //glLightfv(GL_LIGHT2, GL_AMBIENT, lightAmbient2);
        glLightfv(GL_LIGHT2, GL_DIFFUSE, lightDiffuse2);
        //glLightfv(GL_LIGHT2,GL_SPECULAR,specular2);
        //glLightfv(GL_LIGHT3, GL_AMBIENT, lightAmbient3);
        glLightfv(GL_LIGHT3, GL_DIFFUSE, lightDiffuse3);
        //glLightfv(GL_LIGHT3,GL_SPECULAR,specular3);
      }
      
      if(enableLight2 == 1){
        glEnable(GL_LIGHT2);
      }
      if(enableLight3 == 1){
        glEnable(GL_LIGHT3);
      }
      glColor3f(planeColor[0], planeColor[1], planeColor[2]);
      
      if(animation > 330 && animation < 400){
        glDisable(GL_LIGHTING);
        glColor3f(1.0f, 1.0f, 1.0f);
      }
      glLineWidth(4.5);
      if(enableGrid == 1){
        glPolygonMode(GL_FRONT, GL_LINE);
        glPolygonMode(GL_BACK, GL_LINE);
      }
      glBegin(GL_QUADS);
        for (x = 0; x < gridX - 1; x++){
          for (z = 0; z < gridY - 1; z++){
            
            x1 = x * xMax / (gridX / 2.0) - xMax;
            x2 = x * xMax / (gridX / 2.0) - xMax;
            x3 = (x + 1.0) * xMax / (gridX / 2.0) - xMax;
            x4 = (x + 1.0) * xMax / (gridX / 2.0) - xMax;
            z1 = z * yMax / (gridY / 2.0) - yMax;
            z2 = (z + 1.0) * yMax / (gridY / 2.0) - yMax;
            z3 = (z + 1.0) * yMax / (gridY / 2.0) - yMax;
            z4 = z * yMax / (gridY / 2.0) - yMax;

            //Calculating potential plane. particle[potentialEnd][4] represents test charge !!!
            potential = 0.0;
            for(i=0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(z1-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
//                potential = k1 * particles[i][4] * particles[potentialEnd][4] / minDistance;
//                break;
              }
              potential += k1 * particles[i][4] * particles[potentialEnd][4] / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            y1 = potential * scalePlaneHeigth + planeOffset;
            
            potential = 0.0;
            for(i=0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x2-particles[i][0], 2.0) + pow(z2-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
//                potential = k1 * particles[i][4] * particles[potentialEnd][4] / minDistance;
//                break;
              }
              potential += k1 * particles[i][4] * particles[potentialEnd][4] / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            y2 = potential * scalePlaneHeigth + planeOffset;

            potential = 0.0;
            for(i=0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x3-particles[i][0], 2.0) + pow(z3-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
//                potential = k1 * particles[i][4] * particles[potentialEnd][4] / minDistance;
//                break;
              }
              potential += k1 * particles[i][4] * particles[potentialEnd][4] / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            y3 = potential * scalePlaneHeigth + planeOffset;

            potential = 0.0;
            for(i=0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x4-particles[i][0], 2.0) + pow(z4-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
//                potential = k1 * particles[i][4] * particles[potentialEnd][4] / minDistance;
//                break;
              }
              potential += k1 * particles[i][4] * particles[potentialEnd][4] / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            y4 = potential * scalePlaneHeigth + planeOffset;
            
            float_x = (float) x / (float)(gridX - 1) / GRIDDIVISOR;
            float_z = (float) z / (float)(gridY - 1) / GRIDDIVISOR;
            float_xb = (float) (x + 1) / (float)(gridX - 1) / GRIDDIVISOR;
            float_zb = (float) (z + 1) / (float)(gridY - 1) / GRIDDIVISOR;


            p1[0] = x2;
            p1[1] = y2;
            p1[2] = z2;
            
            p2[0] = x1;
            p2[1] = y1;
            p2[2] = z1;
            
            p3[0] = x4;
            p3[1] = y4;
            p3[2] = z4;
            
            calculateNormalVector();

            glNormal3f(normalVector[0], normalVector[1], normalVector[2]);
            //glNormal3f(0.0f, 0.0f, 1.4f);//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            
            //if(temp==0)printf("normalVector[0]=%lf, normalVector[1]=%lf, normalVector[2]=%lf\n", normalVector[0], normalVector[1], normalVector[2]);
            
  //        if(x == 10 || x == 20 || x == 30){
  //          glNormal3f( 0.0f, 1.0f, 1.0f);
  //        }
            //glTexCoord2f(float_x, float_y);
            glVertex3f(x1, y1, z1); 
            //glTexCoord2f(float_x, float_yb);
            glVertex3f(x2, y2, z2); 
            //glTexCoord2f(float_xb, float_yb);
            glVertex3f(x3, y3, z3); 
            //glTexCoord2f(float_xb, float_y);
            glVertex3f(x4, y4, z4); 
          }
        } 
      glEnd();

      glDisable(GL_LIGHT2);
      glDisable(GL_LIGHT3);
      
      glEnable(GL_LIGHTING);
      
      //Draw potential sphere
      if(enablePotentialSphere == 1){
        //printf("painting potential sphere at %lf, %lf, %lf\n", potentialSphereX, potentialSphereY, potentialSphereZ);
        glPolygonMode(GL_BACK, GL_FILL);
        glPolygonMode(GL_FRONT, GL_FILL);
    
        glLightfv(GL_LIGHT1, GL_AMBIENT, lightOrange1);
        glLightfv(GL_LIGHT1, GL_DIFFUSE, lightOrange1);
        glLightfv(GL_LIGHT1, GL_SPECULAR, lightSpecularOrange1);
        //glEnable(GL_COLOR_MATERIAL);
        //glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, orangeMaterial);
        //glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, orangeMaterial);
        //glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, orangeMaterial);
        //glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, orangeMaterial);
        //glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, orangeMaterial);
        //glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 5.0f);
        //glColor3f(1.0f, 0.7f, 0.0f);
        drawSphere(potentialSphereX, potentialSphereY + potentialSphereRadius + 0.05f, potentialSphereZ, potentialSphereRadius, 40, 40, 0);
        //glDisable(GL_COLOR_MATERIAL);
        glLightfv(GL_LIGHT1, GL_AMBIENT, lightAmbient1);
        glLightfv(GL_LIGHT1, GL_DIFFUSE, lightDiffuse1);
        glLightfv(GL_LIGHT1,GL_SPECULAR,specular1);
        
      }
      if(enablePotentialPoints == 1 || enablePotentialDiff){
        for(i = 0; i < 10; i++){
//          glDisable(GL_LIGHTING);
//          if(potentialPoints[i][1] < 0.0f){
//            glColor3f(0.0f, 0.7f, 1.0f);
//          }
//          else{
//            glColor3f(1.0f, 0.0f, 0.0f);
//          }
//          glBegin(GL_LINES);
//          glVertex3f( potentialPoints[i][0], potentialPoints[i][1] + 0.2f, potentialPoints[i][2]);
//          glVertex3f( potentialPoints[i][0], potentialPoints[i][1] + 2.0, potentialPoints[i][2]);
//          glEnd();        
//          glEnable(GL_LIGHTING);
          glPolygonMode(GL_BACK, GL_FILL);
          glPolygonMode(GL_FRONT, GL_FILL);
          glLightfv(GL_LIGHT1, GL_AMBIENT, lightOrange1);
          glLightfv(GL_LIGHT1, GL_DIFFUSE, lightOrange1);
          glLightfv(GL_LIGHT1, GL_SPECULAR, lightSpecularOrange1);
          drawSphere(potentialPoints[i][0], potentialPoints[i][1] + 0.0, potentialPoints[i][2], 0.16, 40, 40, 0);
          glLightfv(GL_LIGHT1, GL_AMBIENT, lightAmbient1);
          glLightfv(GL_LIGHT1, GL_DIFFUSE, lightDiffuse1);
          glLightfv(GL_LIGHT1,GL_SPECULAR,specular1);
        }        
      }
      glLineWidth(7.5);

    }
    
    //Draw paths      
    int pathNumber;
    glEnable(GL_LINE_SMOOTH);

    for(pathNumber = 0; pathNumber < PATHMAX; pathNumber++){
      if(pathEnd[pathNumber] > 1){
        glLineWidth(pathWidth[pathNumber]);
        glDisable(GL_LIGHTING);
        for(i = 0; i < pathEnd[pathNumber] - 1; i++){
          if(pathColor[pathNumber] == 0){
            if(path[pathNumber][i][1] < 0.0){
              glColor3f(0.0f, 0.7f, 1.0f);
            }
            else{
              glColor3f(1.0f, 0.0f, 0.0f);
            }
          }
          if(pathColor[pathNumber] == 1){
            glColor3f(0.0f, 1.0f, 0.0f);
          }
          if(pathColor[pathNumber] == 2){
            glColor3f(1.0f, 0.8f, 0.0f);
          }
          if(pathColor[pathNumber] == 3){
            glColor3f(1.0f, 0.0f, 0.0f);
          }
          if(pathColor[pathNumber] == 4){
            glColor3f(0.0f, 0.7f, 1.0f);
          }
          glBegin(GL_LINES);
          glVertex3f( path[pathNumber][i][0], path[pathNumber][i][1] * scalePlaneHeigth / 200.0f, path[pathNumber][i][2]);
          glVertex3f( path[pathNumber][i+1][0], path[pathNumber][i+1][1] * scalePlaneHeigth / 200.0f, path[pathNumber][i+1][2]);
          glEnd();        
        }
      }
    }//for(pathNumber = 0; pathNumber < 10; pathNumber++){        
    glDisable(GL_LINE_SMOOTH);
    
    
    //Draw force plane
    if(animation < 100){
      glPolygonMode(GL_FRONT, GL_LINE);
      glPolygonMode(GL_BACK, GL_LINE);
      glLightfv(GL_LIGHT1, GL_AMBIENT, lightAmbient1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightDiffuse1);
      glLightfv(GL_LIGHT1,GL_SPECULAR,specular1);
      
      glLineWidth(4.5);
      glBegin(GL_QUADS);
        for (x = 0; x < gridX - 1; x++){
          for (z = 0; z < gridY - 1; z++){
            
            x1 = (x - gridX / 2.0) * coordScale;
            x2 = (x - gridX / 2.0) * coordScale;
            x3 = (x + 1.0 - gridX / 2.0) * coordScale;
            x4 = (x + 1.0 - gridX / 2.0) * coordScale;
            z1 = (z - gridY / 2.0) * coordScale;
            z2 = (z + 1.0 - gridY / 2.0) * coordScale;
            z3 = (z + 1.0 - gridY / 2.0) * coordScale;
            z4 = (z - gridY / 2.0) * coordScale;

            totalDistance = sqrt(pow(x1-chargeX, 2.0) + pow(z1-chargeY, 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
            }
            totalForce = particleCharge * testCharge / pow(totalDistance, 2.0);
            y1 = totalForce * 1.0f;

            totalDistance = sqrt(pow(x2-chargeX, 2.0) + pow(z2-chargeY, 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
            }
            totalForce = particleCharge * testCharge / pow(totalDistance, 2.0);
            y2 = totalForce * 1.0f;

            totalDistance = sqrt(pow(x3-chargeX, 2.0) + pow(z3-chargeY, 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
            }
            totalForce = particleCharge * testCharge / pow(totalDistance, 2.0);
            y3 = totalForce * 1.0f;

            totalDistance = sqrt(pow(x4-chargeX, 2.0) + pow(z4-chargeY, 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
            }
            totalForce = particleCharge * testCharge / pow(totalDistance, 2.0);
            y4 = totalForce * 1.0f;
            
            float_x = (float) x / (float)(gridX - 1) / GRIDDIVISOR;
            float_z = (float) z / (float)(gridY - 1) / GRIDDIVISOR;
            float_xb = (float) (x + 1) / (float)(gridX - 1) / GRIDDIVISOR;
            float_zb = (float) (z + 1) / (float)(gridY - 1) / GRIDDIVISOR;


            p1[0] = x2;
            p1[1] = y2;
            p1[2] = z2;
            
            p2[0] = x1;
            p2[1] = y1;
            p2[2] = z1;
            
            p3[0] = x4;
            p3[1] = y4;
            p3[2] = z4;
            
            calculateNormalVector();

            glNormal3f(normalVector[0], normalVector[1], normalVector[2]);
            //glNormal3f(0.0f, 0.0f, 1.4f);//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            
            //if(temp==0)printf("normalVector[0]=%lf, normalVector[1]=%lf, normalVector[2]=%lf\n", normalVector[0], normalVector[1], normalVector[2]);
            
  //        if(x == 10 || x == 20 || x == 30){
  //          glNormal3f( 0.0f, 1.0f, 1.0f);
  //        }
            //glTexCoord2f(float_x, float_y);
            glVertex3f(x1, y1, z1); 
            //glTexCoord2f(float_x, float_yb);
            glVertex3f(x2, y2, z2); 
            //glTexCoord2f(float_xb, float_yb);
            glVertex3f(x3, y3, z3); 
            //glTexCoord2f(float_xb, float_y);
            glVertex3f(x4, y4, z4); 
          }
        } 
      glEnd();

    }//if(animation < 100){

    //Draw particles disc with minimal distance
    //if(animation < 100 || animation >= 300){
    if(enableMinDisc == 1){
      glPolygonMode(GL_BACK, GL_FILL);
      glPolygonMode(GL_FRONT, GL_FILL);
      glDisable(GL_LIGHTING);
      glEnable(GL_DEPTH_TEST);
      glDisable(GL_BLEND);
      for(i = 0; i < potentialEnd; i++){
        if(particles[i][4] > 0.0f){
          glColor3f(1.0f, 0.0f, 0.0f);
        }
        else{
          glColor3f(0.0f, 0.0f, 1.0f);
        }
        glBegin(GL_TRIANGLES);
        GLfloat degInRad;
        GLfloat circleStep = 1.0f;
        if(animation >= 300){
          
          
            peak = 0.0;
            for(i2=0; i2 < potentialEnd; i2++){
              totalDistance = sqrt(pow(particles[i][0]-particles[i2][0], 2.0) + pow(particles[i][2]-particles[i2][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
              }
              peak += k1 * particles[i2][4] * particles[potentialEnd][4] / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            peak = peak * scalePlaneHeigth + planeOffset;
          
          
          
          
          
//          peak = k1 * particles[i][4] * particles[potentialEnd][4] / minDistance * scalePlaneHeigth;
//          peak += 0.05f;
        }
        else{
          peak = particleCharge * testCharge / pow(minDistance, 2.0);
        }

        peak += 0.05f;
        
        for (x=0; x < 360; x += circleStep){
           degInRad = x * 3.1415927 / 180.0;
           glVertex3f(particles[i][0], peak, particles[i][2]);
           glVertex3f(particles[i][0] + cos(degInRad) * minDistance, peak, particles[i][2] + sin(degInRad) * minDistance);
           degInRad = (x + circleStep) * 3.1415927 / 180.0;
           glVertex3f(particles[i][0] + cos(degInRad) * minDistance, peak, particles[i][2] + sin(degInRad) * minDistance);
        }
        glEnd();    
      }//for(i = 0; i < potentialEnd; i++){
    }
    
    temp=1;
    




//####################################################################################
//############# Coulomb constant #####################################################
//####################################################################################


    if(animation >= 2800 && animation < 2900 && fieldLineEndUp > 0.0f){//Draw force vectors

      glDisable(GL_DEPTH_TEST);
      glDisable(GL_LIGHTING);
      
      glColor3f(1.0f, 0.7f, 0.0f);
      
      GLfloat beta, forceFactor = 100.0f;
      
      if(particles[1][4] > 0.0f){
        forceFactor = 230.0f;
      }
      
      //particleNumber = 6;// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      
      for(i2 = 0; i2 < 2; i2++){
        forceX = 0.0f;
        forceY = 0.0f;
        for(i = 0; i < particleNumber; i++){
          if(i != i2){
            totalDistance = sqrt(pow(particles[i][0] - particles[i2][0], 2.0) + pow(particles[i][1] - particles[i2][1], 2.0));
            totalForce = k * particles[i][4] * particles[i2][4] / pow(totalDistance, 2.0);

            beta = atan2(particles[i2][1] - particles[i][1], particles[i2][0] - particles[i][0]);
            forceX += totalForce * cos(beta);
            forceY += totalForce * sin(beta);

          }
        }
        x1 = particles[i2][0];
        y1 = particles[i2][1];
        z1 = particles[i2][2];
        x2 = particles[i2][0] + forceX * forceFactor * 0.7;
        y2 = particles[i2][1] + forceY * forceFactor * 0.7;
        z2 = particles[i2][2];
        drawCylinder(x1, y1, z1, x2, y2, z2, 0.15, 0.15, 32);

        x1 = x2;
        y1 = y2;
        z1 = z2;
        x2 = particles[i2][0] + forceX * forceFactor;
        y2 = particles[i2][1] + forceY * forceFactor;
        z2 = particles[i2][2];
        drawCylinder(x1, y1, z1, x2, y2, z2, 0.35, 0.15, 32);
      }
      //printf("forceX=%lf, forceY=%lf\n", forceX, forceY);

      glEnable(GL_DEPTH_TEST);
      glEnable(GL_LIGHTING);
    }

    if(animation >= 2801 && animation < 2802 && fieldLineEndUp > 0.0f){//Draw field lines
      GLfloat fieldLineCoords[10000][4];
      GLfloat fieldLineLength = 40.0;
      double alpha, beta;
      int fieldLineNumber = 18;
      
      fieldLineStep = 0.15f;
      x1 = 0.0f;
      
      glEnable(GL_LIGHTING);
      glPolygonMode(GL_BACK, GL_FILL);
      glPolygonMode(GL_FRONT, GL_FILL);


      glLightfv(GL_LIGHT1, GL_AMBIENT, lightGreen1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightGreen1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightGreen1);

      if(particles[1][4] < 0.0f){
        for(i = 0; i < 18; i++){
          alpha = (GLfloat)(i) * 2.0 * PI / 18.0f;
          fieldLineCoords[i][0] = particles[0][0] + cos(alpha) * particles[0][3];
          fieldLineCoords[i][1] = particles[0][1] + sin(alpha) * particles[0][3];
          fieldLineCoords[i][2] = 0.2;
          fieldLineCoords[i][3] = 0.0;
        }
      }
      else{
        for(i = 0; i < 18; i++){
          alpha = (GLfloat)(i) * 2.0 * PI / 18.0f;
          fieldLineCoords[i][0] = particles[0][0] + cos(alpha) * particles[0][3];
          fieldLineCoords[i][1] = particles[0][1] + sin(alpha) * particles[0][3];
          fieldLineCoords[i][2] = 0.2;
          fieldLineCoords[i][3] = 0.0;
        }
        for(i = 0; i < 18; i++){
          alpha = (GLfloat)(i) * 2.0 * PI / 18.0f;
          fieldLineCoords[i+18][0] = particles[1][0] + cos(alpha) * particles[1][3];
          fieldLineCoords[i+18][1] = particles[1][1] + sin(alpha) * particles[1][3];
          fieldLineCoords[i+18][2] = 0.2;
          fieldLineCoords[i+18][3] = 0.0;
        }
        fieldLineNumber = 36;
      }


      for(i2 = 0; i2 < fieldLineNumber; i2++){
        for(i = 0; i < fieldLineEndUp; i++){
          totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[0][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[0][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[0][2], 2.0));
          totalForce = k * particles[0][4] / pow(totalDistance, 2.0);
          forceX = (fieldLineCoords[i2][0] - particles[0][0]) / totalDistance * totalForce;
          forceY = (fieldLineCoords[i2][1] - particles[0][1]) / totalDistance * totalForce;
          forceZ = (fieldLineCoords[i2][2] - particles[0][2]) / totalDistance * totalForce;

          totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
          totalForce = k * particles[1][4] / pow(totalDistance, 2.0);
          forceX += (fieldLineCoords[i2][0] - particles[1][0]) / totalDistance * totalForce;
          forceY += (fieldLineCoords[i2][1] - particles[1][1]) / totalDistance * totalForce;
          forceZ += (fieldLineCoords[i2][2] - particles[1][2]) / totalDistance * totalForce;

          totalForce = sqrt(pow(forceX, 2.0) + pow(forceY, 2.0) + pow(forceZ, 2.0));
          x2 = fieldLineCoords[i2][0] + forceX / totalForce * fieldLineStep;
          y2 = fieldLineCoords[i2][1] + forceY / totalForce * fieldLineStep;
          z2 = fieldLineCoords[i2][2] + forceZ / totalForce * fieldLineStep;
          z2 = 0.2f;
                    
          drawCylinder(fieldLineCoords[i2][0], fieldLineCoords[i2][1], fieldLineCoords[i2][2], x2, y2, z2, 0.05, 0.05, 32);            
          
          if(particles[1][4] < 0.0f){
            if(fieldLineCoords[i2][3] == 0.0){
              if(fieldLineCoords[i2][0] > 0.0f && (i2 == 0 || i2 == 1 || i2 == 2 || i2 == 16 || i2 == 17)){
                drawCylinder(x1, y1, z1, x2, y2, z2, 0.15, 0.05, 32);
                fieldLineCoords[i2][3] = 1.0;
              }
            }
          }
          else{
            if(fieldLineCoords[i2][3] == 0.0){
              if(i2<18){
                totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[0][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[0][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[0][2], 2.0));
              }
              else{
                totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
              }
              if(totalDistance > 3.0f){
                drawCylinder(x1, y1, z1, x2, y2, z2, 0.15, 0.05, 32);
                fieldLineCoords[i2][3] = 1.0;
              }
            }
          }

          x1 = fieldLineCoords[i2][0];
          y1 = fieldLineCoords[i2][1];
          z1 = fieldLineCoords[i2][2];
          z1 = 0.2f;

          if(!(fabs(x1) < 0.1f && fabs(y1) < 0.1f)){
            fieldLineCoords[i2][0] = x2;
            fieldLineCoords[i2][1] = y2;
            fieldLineCoords[i2][2] = z2;
          }
          
//        totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
        }//for(i = 0; i < fieldLineEndUp; i++){
      }
      drawCylinder(particles[1][0], particles[1][1], z1, 10.0f, 0.0f, z2, 0.05, 0.05, 32);
    }

    if(animation >= 2802 && animation < 2900){
      GLfloat x, y;
      for(i = 2; i < particleNumber; i += 2){
        x = particles[i][0] + (particles[i+1][0] - particles[i][0])/2.0f;
        y = particles[i][1] + (particles[i+1][1] - particles[i][1])/2.0f;
        moveAlpha = particles[i][6];
        particles[i][0] = x + dipoleRadius * cos(moveAlpha);
        particles[i][1] = y + dipoleRadius * sin(moveAlpha);

        particles[i+1][0] = x - dipoleRadius * cos(moveAlpha);
        particles[i+1][1] = y - dipoleRadius * sin(moveAlpha);
        
        if(dipoleRadius < 0.00055f){//Particle color to white
          particles[i][5] = 1.0f;
          particles[i+1][5] = 1.0f;
        }
        else{
          particles[i][5] = 0.0f;
          particles[i+1][5] = 0.0f;
        }
      }
    }



    if(pauseFrames > 10 && animationPause == 0){

      if(animation == 2806){
        savePics = 0;
        printf("...un ferddisch!\n");
        done = True;         // Exit programm !!!        
      }

      if(animation == 2805){
        printf("hier! particles[1][4]=%lf\n", particles[1][4]);
        if(particles[1][4] < 0.0f){
          printf("...un hier!\n");
          GLfloat x, y;
          particles[1][4] = 0.00001f;
          i = 2;
          for(x = -4.0f; x <= 4.0f; x += 0.5){
            for(y = -4.0; y <= 4.0; y += 0.5){
              moveAlpha += 1.03f;
              moveAlpha = (double)(rand()) * 2.0f * PI / (double)(RAND_MAX);
              while(moveAlpha > PI * 2.0f){
                moveAlpha -= PI * 2.0f;
              }
              if(sqrt(pow(particles[0][0] - x, 2.0) + pow(particles[0][1] - y, 2.0)) > 1.15 &&
                 sqrt(pow(particles[1][0] - x, 2.0) + pow(particles[1][1] - y, 2.0)) > 1.15){
                particles[i][0] = x + dipoleRadius * cos(moveAlpha);
                particles[i][1] = y + dipoleRadius * sin(moveAlpha);
                particles[i][2] = 0.0f;
                particles[i][3] = 0.15f;
                particles[i][4] = 0.00000003f;
                particles[i][5] = 0.0f;
                particles[i][6] = moveAlpha;
                i++;
                particles[i][0] = x - dipoleRadius * cos(moveAlpha);
                particles[i][1] = y - dipoleRadius * sin(moveAlpha);
                particles[i][2] = 0.0f;
                particles[i][3] = 0.15f;
                particles[i][4] = -0.00000003f;
                particles[i][5] = 0.0f;
                particles[i][6] = moveAlpha;
                i++;
              }
            }
          }
          animation = 2800;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;
        }
      }

      if(animation == 2804){
        if(dipoleRadius + 0.0005f < 0.05f){
          dipoleRadius += 0.0005f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;
          
          dipoleRadius = 0.05f;
        }        
      }

      if(animation == 2803){
        animation++;
        printf("Animation set to %d\n", animation);
        pauseFrames = 0;
      }

      if(animation == 2802){
        if(dipoleRadius - 0.0005f > 0.0f){
          dipoleRadius -= 0.0005f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;
          
          dipoleRadius = 0.0f;
        }        
      }
      
      if(animation == 2801){
        animation++;
        printf("Animation set to %d\n", animation);
        pauseFrames = 0;
      }

      if(animation == 2800){
        GLfloat x, y;
        int rotationFinished = 0;
        GLfloat beta;
        
        for(i = 2; i < particleNumber; i+=2){
          forceX = 0.0f;
          forceY = 0.0f;
          x = particles[i][0] + (particles[i+1][0] - particles[i][0])/2.0f;
          y = particles[i][1] + (particles[i+1][1] - particles[i][1])/2.0f;

          totalDistance = sqrt(pow(x - particles[0][0], 2.0) + pow(y - particles[0][1], 2.0));
          totalForce = k1 * particles[i][4] * particles[0][4] / pow(totalDistance, 2.0);
          beta = atan2(y - particles[0][1], x - particles[0][0]);
          forceX += totalForce * cos(beta);
          forceY += totalForce * sin(beta);

          totalDistance = sqrt(pow(x - particles[1][0], 2.0) + pow(y - particles[1][1], 2.0));
          totalForce = k1 * particles[i][4] * particles[1][4] / pow(totalDistance, 2.0);
          beta = atan2(y - particles[1][1], x - particles[1][0]);
          forceX += totalForce * cos(beta);
          forceY += totalForce * sin(beta);

          beta = atan2(forceY, forceX);
          if(beta - particles[i][6] > PI){
            beta -= PI;
          }
          if(beta - particles[i][6] < PI){
            beta += PI;
          }
          if(beta > particles[i][6]){
            if(beta - particles[i][6] > PI){
              particles[i][6] += 0.0091f;
              particles[i+1][6] += 0.0091f;
            }
            else{
              particles[i][6] -= 0.0091f;
              particles[i+1][6] -= 0.0091f;
            }
          }
          if(beta < particles[i][6]){
            if(beta - particles[i][6] < -PI){
              particles[i][6] -= 0.0091f;
              particles[i+1][6] -= 0.0091f;
            }
            else{
              particles[i][6] += 0.0091f;
              particles[i+1][6] += 0.0091f;
            }
          }

          moveAlpha = particles[i][6];
//          particles[i+1][6] += 0.001f;
          particles[i][0] = x + dipoleRadius * cos(moveAlpha);
          particles[i][1] = y + dipoleRadius * sin(moveAlpha);

          particles[i+1][0] = x - dipoleRadius * cos(moveAlpha);
          particles[i+1][1] = y - dipoleRadius * sin(moveAlpha);
        } 

        doubleTemp++;

        if(doubleTemp > 400){//400 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
          animation++;
          printf("Animation set to %d\n", animation);
          //pauseFrames = 0;
          
          doubleTemp = 0.0f;
          fieldLineEndUp = 1000;
        }
      }
      

    }//if(pauseFrames > 10){


//####################################################################################
//############# Influence on metal sphere ############################################
//####################################################################################


    if(animation >= 2701 && animation < 2800){
      
//      glDisable(GL_DEPTH_TEST);
      

      testCharge = 0.000001;
      GLfloat beta;
      GLfloat vectorDiameter = 0.1f;
      GLfloat vectorLength;
      GLfloat stepWidth = 0.02f;
      GLfloat sphereRadius = 2.50f;
      GLfloat particleForceX = 0.0f;
      GLfloat particleForceY = 0.0f;
      GLfloat minDistance = 0.1f;
      GLfloat totalX = 0.0f, totalY = 0.0f;


      for(i2 = 0; i2 < particleNumber; i2++){
        if(particles[i2][4] < 0.0f){//Only negative charges can move
          forceX = 0.0;
          forceY = 0.0;
          totalForce = 0.0;
          totalDistance = 0.0;
          for(i = 0; i < particleNumber; i++){
            if(i != i2 && !(particles[i][4] > 0.0f && i > 0)){
              totalDistance = sqrt(pow(particles[i2][0] - particles[i][0], 2.0) + pow(particles[i2][1] - particles[i][1], 2.0) + pow(particles[i2][2] - particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
              }
              totalForce = k1 * particles[i][4] * particles[i2][4] / pow(totalDistance, 2.0);
              beta = atan2(particles[i2][1] - particles[i][1], particles[i2][0] - particles[i][0]);
              forceX += totalForce * cos(beta);
              forceY += totalForce * sin(beta);
            }
          }
          totalForce = sqrt(pow(forceX, 2.0) + pow(forceY, 2.0));
          particles[i2][0] += forceX / totalForce * stepWidth;
          particles[i2][1] += forceY / totalForce * stepWidth;
          totalDistance = sqrt(pow(particles[i2][0] - 3.0f, 2.0) + pow(particles[i2][1] - 0.0f, 2.0) + pow(particles[i2][2] - 0.0f, 2.0));
          if(totalDistance > sphereRadius){
            beta = atan2(particles[i2][1], particles[i2][0] + 3.0f);
            //printf("beta = %lf, particles[i2][0] = %lf, particles[i2][1] = %lf\n", beta, particles[i2][0] - 3.0f, particles[i2][1]);
            //printf("cos(beta) * sphereRadius = %lf, sin(beta) * sphereRadius = %lf\n", cos(beta) * sphereRadius, sin(beta) * sphereRadius);
            particles[i2][0] = cos(beta) * sphereRadius - 3.0f;
            particles[i2][1] = sin(beta) * sphereRadius;
            //printf("particles[i2][0] = %lf, particles[i2][1] = %lf\n", particles[i2][0], particles[i2][1]);
            //printf("beta (neu) = %lf\n", atan2(particles[i2][1], particles[i2][0] - 3.0f));
          }
          totalX += particles[i2][0];
          totalY += particles[i2][1];
        }
      }//for(i2 = 0; i2 < particleNumber; i2++){

      totalX = totalX / 12.0f;
      totalY = totalY / 12.0f;


      for(i=1; i< particleNumber; i+=2){
        enableTransLucentSphere = 0;
        glLightfv(GL_LIGHT1, GL_AMBIENT, lightRed1);
        glLightfv(GL_LIGHT1, GL_DIFFUSE, lightRed1);
        glLightfv(GL_LIGHT1, GL_SPECULAR, lightOrange1);

        drawSphere(particles[i][0],particles[i][1], particles[i][2], particles[i][3], 40, 40, 0);
      }

      for(i=2; i< particleNumber; i+=2){
        glLightfv(GL_LIGHT1, GL_AMBIENT, lightBlue3);
        glLightfv(GL_LIGHT1, GL_DIFFUSE, lightBlue3);
        glLightfv(GL_LIGHT1, GL_SPECULAR, lightBlue3);
        drawSphere(particles[i][0],particles[i][1], particles[i][2], particles[i][3], 40, 40, 1);
      }

      glLightfv(GL_LIGHT1, GL_AMBIENT, lightRed1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightRed1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightOrange1);
      drawSphere(particles[0][0],particles[0][1], particles[0][2], particles[0][3], 40, 40, 1);


      
      glLightfv(GL_LIGHT1, GL_AMBIENT, lightBlue1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightBlue1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightBlue1);

      drawCylinder(totalX - 0.25f, totalY + 0.25f, 0.0f, totalX + 0.25f, totalY - 0.25f, 0.0f, 0.06f, 0.06f, 32);
      drawCylinder(totalX + 0.25f, totalY + 0.25f, 0.0f, totalX - 0.25f, totalY - 0.25f, 0.0f, 0.06f, 0.06f, 32);

      glLightfv(GL_LIGHT1, GL_AMBIENT, lightRed1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightRed1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightRed1);

      drawCylinder(-3.0f - 0.25f, 0.25f, 0.0f, -3.0f + 0.25f, -0.25f, 0.0f, 0.06f, 0.06f, 32);
      drawCylinder(-3.0f + 0.25f, 0.25f, 0.0f, -3.0f - 0.25f, -0.25f, 0.0f, 0.06f, 0.06f, 32);
                    
      glLightfv(GL_LIGHT1, GL_AMBIENT, lightOrange1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightOrange1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightOrange1);

      drawCylinder(-3.0f, 0.0f, 0.0f, totalX, 0.0f, 0.0f, 0.06f, 0.06f, 32);

//      glEnable(GL_DEPTH_TEST);


    }//if(animation >= 2700 && animation < 2800){






    if(pauseFrames > 10 && animationPause == 0){

      if(animation == 2702){
        if(doubleTemp < 200){
          doubleTemp++;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;

          savePics = 0;
          printf("...un ferddisch!\n");
          done = True;         // Exit programm !!!        
        }
      }


      if(animation == 2701){
        if(particles[0][0] > 3.0f){
          particles[0][0] -= 0.03f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;
         
        }
      }

      if(animation == 2700){
        if(particles[1][3] > 0.11f){
          for(i = 1; i < 24; i += 2){
            particles[i][3] -= 0.003f;
          }
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;
         
          for(i = 1; i < 24; i += 2){
            particles[i][3] = 0.11f;
            particles[i][5] = 2.0f;
          }
          enableParticles = 0;
        }
      }

      
    }//if(pauseFrames > 10){

    

//####################################################################################
//############# Influence on "molecules" #############################################
//####################################################################################


    if(animation > 2600 && animation < 2700){
      
      /*
//      glDisable(GL_DEPTH_TEST);
      
      glLightfv(GL_LIGHT1, GL_AMBIENT, lightGreen1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightGreen1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightGreen1);

      testCharge = 0.000001;
      GLfloat beta;
      GLfloat vectorDiameter = 0.03f;
      GLfloat vectorLength;
      GLfloat forceFactor = 10.00f;
      GLfloat particleForceX = 0.0f;
      GLfloat particleForceY = 0.0f;


      forceX = 0.0;
      forceY = 0.0;
      totalForce = 0.0;
      totalDistance = 0.0;
      for(i = 1; i < 3; i++){
        totalDistance = sqrt(pow(particles[0][0] - particles[i][0], 2.0) + pow(particles[0][1] - particles[i][1], 2.0) + pow(particles[0][2] - particles[i][2], 2.0));
        totalForce = k1 * particles[i][4] * particles[i2][4] / pow(totalDistance, 2.0);
        beta = atan2(particles[i2][1] - particles[i][1], particles[i2][0] - particles[i][0]);
        forceX += totalForce * cos(beta);
        forceY += totalForce * sin(beta);
      }


      totalForce = sqrt(pow(forceX, 2.0) + pow(forceY, 2.0));
      x2 = forceX * sqrt(totalForce)/totalForce * forceFactor;
      y2 = forceY * sqrt(totalForce)/totalForce * forceFactor;
      vectorLength = sqrt(pow(x2, 2.0) + pow(y2, 2.0) + pow(z2, 2.0));
      vectorDiameter = 0.05 * vectorLength;

      beta = atan2(forceY, forceX);
      x1 = particles[1][0] - (particles[1][0] - particles[2][0]) / 2.0f;
      y1 = particles[1][1];
      x2 = x1 + x2;
      y2 = y1 + y2;
      z1 = 1.5f;
      z2 = 1.5f;
                    
      drawCylinder(x1, y1, z1, x2, y2, z2, vectorDiameter, vectorDiameter, 32);
      x1 += 0.7 * (x2 - x1);
      y1 += 0.7 * (y2 - y1);
      drawCylinder(x1, y1, z1, x2, y2, z2, vectorDiameter * 3.33f, vectorDiameter, 32);

//      glEnable(GL_DEPTH_TEST);
*/

      z1 = 1.5f;
      z2 = 1.5f;
      
      glLightfv(GL_LIGHT1, GL_AMBIENT, lightRed1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightRed1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightRed1);

      drawCylinder(particles[1][0] - 0.25f, particles[1][1] + 0.25f, z1, particles[1][0] + 0.25f, particles[1][1] - 0.25f, z2, 0.06f, 0.06f, 32);
      drawCylinder(particles[1][0] + 0.25f, particles[1][1] + 0.25f, z1, particles[1][0] - 0.25f, particles[1][1] - 0.25f, z2, 0.06f, 0.06f, 32);

      glLightfv(GL_LIGHT1, GL_AMBIENT, lightBlue1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightBlue1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightBlue1);

      drawCylinder(particles[2][0] - 0.25f, particles[2][1] + 0.25f, z1, particles[2][0] + 0.25f, particles[2][1] - 0.25f, z2, 0.06f, 0.06f, 32);
      drawCylinder(particles[2][0] + 0.25f, particles[2][1] + 0.25f, z1, particles[2][0] - 0.25f, particles[2][1] - 0.25f, z2, 0.06f, 0.06f, 32);
                    
      glLightfv(GL_LIGHT1, GL_AMBIENT, lightOrange1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightOrange1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightOrange1);

      drawCylinder(particles[1][0], 0.0f, z1, particles[2][0], 0.0f, z2, 0.06f, 0.06f, 32);


    }//if(animation >= 2600 && animation < 2700){






    if(pauseFrames > 10 && animationPause == 0){

      if(animation == 2601){
        if(particles[2][0] < 0.0f){
          particles[2][0] += 0.01f;
          particles[1][0] = particles[2][0] - 40.0f / pow((particles[0][0] - particles[1][0]), 2.0);
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);

          savePics = 0;
          printf("...un ferddisch!\n");
          done = True;         // Exit programm !!!        
        }
      }

      if(animation == 2600){
        if(particles[0][0] > 5.2f){
          particles[0][0] -= 0.01f;
          particles[1][0] = particles[2][0] - 40.0f / pow((particles[0][0] - particles[1][0]), 2.0);
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;
          
          particles[1][5] = 0.0f;
          particles[2][5] = 0.0f;
        }
      }

      
    }//if(pauseFrames > 10){


//####################################################################################
//############# Moving negative charge ###############################################
//####################################################################################


    //Paint grid of electric field vectors
    if(animation >= 2500 && animation < 2600){
      glLightfv(GL_LIGHT1, GL_AMBIENT, lightBlue2);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightBlue2);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightBlue2);


      testCharge = 0.000001;
      GLfloat vectors[300][3];
      int v;
      int pointerDrawn = 0;
      GLfloat beta;
      GLfloat x1a, y1a, x2a, y2a;
      GLfloat vectorDiameter = 0.03f;
      GLfloat vectorLength;
      GLfloat lightSpeed = 1.0f;
      GLfloat xOld;
      
      minVectorDistance = 0.4f;

      for(i = 0; i < 54; i++){//Define coords of the grid
        for(i2 = 0; i2 < 36; i2++){
          vectors[i * 36 + i2][0] = (double)(i) / 2.0 - 14.0;
          vectors[i * 36 + i2][1] = (double)(i2) / 2.0 - 8.0;
          vectors[i * 36 + i2][2] = 0.0;
        }
      }

      for(v = 0; v < 36 * 54; v++){
        forceX = 0.0;
        forceY = 0.0;
        totalForce = 0.0;
        totalDistance = 0.0;
        pointerDrawn = 0;

        totalDistance = sqrt(pow(vectors[v][0] - particles[0][0], 2.0) + pow(vectors[v][1] - 0.0f, 2.0));
        for(i = 0; i < 10; i++){
          xOld = particles[0][0] + (totalDistance / lightSpeed) * particleSpeed;
          totalDistance = sqrt(pow(vectors[v][0] - xOld, 2.0) + pow(vectors[v][1] - 0.0f, 2.0));
        }
        
        if(totalDistance < particles[0][3] + minVectorDistance){
          pointerDrawn = 1;
        }
        totalForce = k1 * particles[0][4] * testCharge / pow(totalDistance, 2.0);
        beta = atan2(vectors[v][1] - particles[0][1], vectors[v][0] - xOld);
        //printf("totalForce=%lf, beta=%lf\n", totalForce, beta);
        forceX += totalForce * cos(beta);
        forceY += totalForce * sin(beta);

        totalForce = sqrt(pow(forceX, 2.0) + pow(forceY, 2.0));
        x2 = forceX * sqrt(totalForce)/totalForce * 1.71;
        y2 = forceY * sqrt(totalForce)/totalForce * 1.71;
        vectorLength = sqrt(pow(x2, 2.0) + pow(y2, 2.0) + pow(z2, 2.0));
        vectorDiameter = 0.1 * vectorLength;

        totalDistance = sqrt(pow(vectors[v][0] + x2 - particles[0][0], 2.0) + pow(vectors[v][1] + y2 - particles[0][1], 2.0) + pow(vectors[v][2] - particles[0][2], 2.0));
        if(totalDistance < particles[0][3] + minVectorDistance){
          pointerDrawn = 1;
        }
        totalDistance = sqrt(pow(vectors[v][0] + x2 * 0.33 - particles[0][0], 2.0) + pow(vectors[v][1] + y2 * 0.33 - particles[0][1], 2.0) + pow(vectors[v][2] - particles[0][2], 2.0));
        if(totalDistance < particles[0][3] + minVectorDistance){
          pointerDrawn = 1;
        }
        totalDistance = sqrt(pow(vectors[v][0] + x2 * 0.66 - particles[0][0], 2.0) + pow(vectors[v][1] + y2 * 0.66 - particles[0][1], 2.0) + pow(vectors[v][2] - particles[0][2], 2.0));
        if(totalDistance < particles[0][3] + minVectorDistance){
          pointerDrawn = 1;
        }

        if(pointerDrawn == 0){          
          beta = atan2(forceY, forceX);
          x1 = vectors[v][0];
          y1 = vectors[v][1];
          x2 = x1 + x2;
          y2 = y1 + y2;
          z1 = 0.0f;
          z2 = 0.0f;
          
          drawCylinder(x1, y1, z1, x2, y2, z2, vectorDiameter, vectorDiameter, 32);
          x1 += 0.7 * (x2 - x1);
          y1 += 0.7 * (y2 - y1);
          drawCylinder(x1, y1, z1, x2, y2, z2, vectorDiameter * 3.33f, vectorDiameter, 32);


        }

      }//for(vector = 0; vector < 10; vector++){
 
      sprintf(textLine, "v = %4.2lfc", fabs(particleSpeed));
      printGLf(-4.0, 2.2, textLine, 0, -5, 0.09f);

    }



    if(pauseFrames > 10 && animationPause == 0){
      

      if(animation == 2502){
        particleSpeed = 0.9f;
        if(particles[0][0] > -30.0f){
          particles[0][0] -= particleSpeed * 0.1f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);     

          savePics = 0;
          printf("...un ferddisch!\n");
          done = True;         // Exit programm !!!        
        }
      }
      


      if(animation == 2501){
        if(particles[0][0] > -10.0f){
          particles[0][0] -= particleSpeed * 0.1f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);     
          
          particles[0][0] = 10.0f;
        }
      }
      

      if(animation == 2500){
        if(particleSpeed < 0.1){
          particleSpeed += 0.003;
          particles[0][0] -= particleSpeed * 0.1f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          particleSpeed = 0.1f;
        }
      }

      
    }//if(pauseFrames > 10){



//####################################################################################
//############# Potential between two protons composed of quarks #####################
//####################################################################################

    if(animation >= 2400 && animation < 2500){
      glLightfv(GL_LIGHT1, GL_AMBIENT, lightOrange1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightOrange1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightOrange1);
      drawCylinder(particles[1][0], particles[1][1], particles[1][2], particles[4][0], particles[4][1], particles[4][2], 0.03f, 0.03f, 32);

      glLightfv(GL_LIGHT1, GL_AMBIENT, lightRed2);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightRed2);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightOrange2);
      
      drawSphere(particles[1][0], particles[1][1], particles[1][2], protonRadius, 64, 64, 1);
      drawSphere(particles[4][0], particles[4][1], particles[4][2], protonRadius, 64, 64, 1);
      
      potential = 0.0f;
      for(i2 = 0; i2 < 3; i2++){
        for(i = 3; i < 6; i++){
          totalDistance = sqrt(pow(particles[i2][0]-particles[i][0], 2.0) + pow(particles[i2][1]-particles[i][1], 2.0) + pow(particles[i2][2]-particles[i][2], 2.0));
          potential += k1 * particles[i][4] * particles[i2][4] / totalDistance;
        }//for(i=0; i < potentialEnd; i++){
      }
      

      
      sprintf(textLine, "Q1 = Q2 = 1.602E-19C");
      printGLf(-4.3, 2.3, textLine, 0, -3, 0.09f);
      sprintf(textLine, "r = %5.2lfE10-15m", particles[4][0] - particles[1][0]);
      printGLf(-4.3, 1.8, textLine, 0, -2, 0.09f);
      sprintf(textLine, "E = %5.2lfE-15J = %5.3lfMeV", potential * 10000.0f, potential * 100.0f / 1.602);
      printGLf(-4.3, 1.3, textLine, 0, -5, 0.09f);
    }

    if(pauseFrames > 10 && animationPause == 0){
      

      if(animation == 2407){
        savePics = 0;
        printf("...un ferddisch!\n");
        done = True;         // Exit programm !!!        
      }


      if(animation == 2406){
        if(moveAlpha < 2.0f * PI){
          particles[5][0] = particles[4][0] + cos(moveAlpha) * 0.6f;
          particles[5][1] = sin(moveAlpha) * 0.6f;
          particles[3][0] = particles[4][0] - cos(moveAlpha) * 0.6f;
          particles[3][1] = -sin(moveAlpha) * 0.6f;
          moveAlpha += 0.01f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);     
          moveAlpha = 0.0f;   
          
          pauseFrames = 0;

          particles[5][0] = particles[4][0] + 0.6f;
          particles[5][1] = 0.0f;
          particles[3][0] = particles[4][0] - 0.6f;
          particles[3][1] = 0.0f;
        }        
      }
      
      if(animation == 2405){
        if(doubleTemp < 0.6){
          particles[5][0] = particles[4][0] + cos(moveAlpha) * doubleTemp;
          particles[5][1] = sin(moveAlpha) * doubleTemp;
          particles[3][0] = particles[4][0] - cos(moveAlpha) * doubleTemp;
          particles[3][1] = -sin(moveAlpha) * doubleTemp;

          particles[2][0] = particles[1][0];
          particles[2][1] = doubleTemp;
          particles[0][0] = particles[1][0];
          particles[0][1] = -doubleTemp;
          
          protonRadius = doubleTemp + particles[0][3];
          
          moveAlpha += 0.01f;
          doubleTemp += 0.001;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          while(moveAlpha > PI){
            moveAlpha -= PI;
          }
          
          particles[2][0] = particles[1][0];
          particles[2][1] = 0.6f;
          particles[0][0] = particles[1][0];
          particles[0][1] = -0.6f;
          
          protonRadius = 0.85f;
        }        
      }
      
      if(animation == 2404){
        if(doubleTemp > 0.0){
          particles[5][0] = particles[4][0] + cos(moveAlpha) * doubleTemp;
          particles[5][1] = sin(moveAlpha) * doubleTemp;
          particles[3][0] = particles[4][0] - cos(moveAlpha) * doubleTemp;
          particles[3][1] = -sin(moveAlpha) * doubleTemp;

          particles[2][0] = particles[1][0];
          particles[2][1] = doubleTemp;
          particles[0][0] = particles[1][0];
          particles[0][1] = -doubleTemp;
          
          protonRadius = doubleTemp + particles[0][3];
          
          moveAlpha += 0.01f;
          doubleTemp -= 0.001;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);     
          moveAlpha = 0.0f;   
          doubleTemp = 0.0;
          
          pauseFrames = 0;

          particles[5][0] = particles[4][0];
          particles[5][1] = 0.0f;
          particles[3][0] = particles[4][0];
          particles[3][1] = 0.0f;

          particles[2][0] = particles[1][0];
          particles[2][1] = 0.0f;
          particles[0][0] = particles[1][0];
          particles[0][1] = 0.0f;
        }        
      }
      
      
      

      if(animation == 2403){
        if(moveAlpha < 2.0f * PI){
          particles[2][0] = particles[1][0] + cos(moveAlpha + PI / 2.0f) * 0.6f;
          particles[2][1] = sin(moveAlpha + PI / 2.0f) * 0.6f;
          particles[0][0] = particles[1][0] - cos(moveAlpha + PI / 2.0f) * 0.6f;
          particles[0][1] = -sin(moveAlpha + PI / 2.0f) * 0.6f;
          moveAlpha += 0.01f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);     
          moveAlpha = 0.0f;   
          doubleTemp = 0.6;

          particles[2][0] = particles[1][0];
          particles[2][1] = 0.6f;
          particles[0][0] = particles[1][0];
          particles[0][1] = -0.6f;
          
          
        }
        
      }

      if(animation == 2402){
        if(moveAlpha < 2.0f * PI){
          particles[5][0] = particles[4][0] + cos(moveAlpha) * 0.6f;
          particles[5][1] = sin(moveAlpha) * 0.6f;
          particles[3][0] = particles[4][0] - cos(moveAlpha) * 0.6f;
          particles[3][1] = -sin(moveAlpha) * 0.6f;
          moveAlpha += 0.01f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);     
          moveAlpha = 0.0f;   

          particles[5][0] = particles[4][0] + 0.6f;
          particles[5][1] = 0.0f;
          particles[3][0] = particles[4][0] - 0.6f;
          particles[3][1] = 0.0f;
        }        
      }

      if(animation == 2401){
        if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
          animation = 2402;
        }
        if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
          animation = 2402;
        }
        if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
          animation = 2402;
        }
        if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
          animation = 2402;
        }
        if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
          animation = 2402;
        }
        if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
          animation = 2402;
        }
        if(animation == 2402){
          printf("Animation set to %d\n", animation);

          moveAlpha = 0.0f;

          translateXdiff = 0.0f;
          translateYdiff = 0.0f;
          translateZdiff = 0.0f;
          rotXdiff = 0.0f;
          rotYdiff = 0.0f;
          rotZdiff = 0.0f;
                    
        }
      }

      
      if(animation == 2400){
        if(particles[4][0] - particles[1][0] > 3.4f){
          for(i=3; i<6; i++){
            particles[i][0] -= 0.01f;
          }
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);        

          particles[3][0] = 3.4f + particles[1][0] + 0.6f;
          particles[4][0] = 3.4f + particles[1][0];
          particles[5][0] = 3.4f + particles[1][0] - 0.6f;

          
          translateXtarget = 3.85f;
          translateYtarget = -0.11f;
          translateZtarget = -4.55f;
          
          rotXtarget = 0.0f;
          rotYtarget = 0.0f;
          rotZtarget = 0.0f;

          translateXdiff = (translateXtarget - translateX) / steps;
          translateYdiff = (translateYtarget - translateY) / steps;
          translateZdiff = (translateZtarget - translateZ) / steps;
          rotXdiff = (rotXtarget - rotX) / steps;
          rotYdiff = (rotYtarget - rotY) / steps;
          rotZdiff = (rotZtarget - rotZ) / steps;
        }
        
      }
      
      
    }//if(pauseFrames > 10){



//####################################################################################
//############# Potential plane of proton composed of quarks #########################
//####################################################################################

    if(pauseFrames > 10 && animationPause == 0){


      if(animation == 2309){
        if(particles[3][2] > 2.0f){
          particles[3][2] -= 0.1f;
          pathEnd[12]++;
        }
        else{
        
          animation++;
          printf("Animation set to %d\n", animation);
          
          pauseFrames = 0;
          
          savePics = 0;
          printf("...un ferddisch!\n");
          done = True;         // Exit programm !!!        
        }
      }



      if(animation == 2308){
        if(translateZ > -26.20f){
          translateZ -= 0.1f;
        }
        else{
          enableGrid = 0;
          gridX = gridX * 4;
          gridY = gridY * 4;

          enablePotentialSphere = 1;

          particles[3][0] = 0.0f;
          particles[3][1] = 0.0f;
          particles[3][2] = yMax;
          particles[3][3] = 0.15f;
          
          animation++;
          printf("Animation set to %d\n", animation);        
        }
      }


      if(animation == 2307){
        gridX = 128;
        gridY = 64;
        enableGrid = 1;
        if(translateZ < -10.0){
          translateZ += 0.1f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          
          pauseFrames = 0;
        }        
      }



      if(animation == 2306){//Convert to single positive charge
        pathEnd[12] = 0;
        enablePotentialSphere = 0;



        potentialEnd = 3;
        for(i=0; i<PATHMAX;i++){
          path[i][0][2] = -1000.0f;
          pathEnd[i] = 0;
          pathColor[i] = 3;
        }
        for(x1 = particles[0][0] - 0.1f; x1 > -25.0; x1 -= 0.0001){//Find staring points of equipotential lines
          potential = 0.0;
          for(i = 0; i < potentialEnd; i++){
            totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
              potential = k1 * particles[i][4] * particles[3][4] / totalDistance;
              break;
            }
            potential += k1 * particles[i][4] * particles[3][4] / totalDistance;
          }//for(i=0; i < potentialEnd; i++){
          potential = potential * scalePlaneHeigth;
          //printf("potential=%lf\n", potential);
          if(potential < 9.00 && path[0][0][2] == -1000.0f){
            printf("potential %lf at %lf\n", potential, x1);
            path[0][0][0] = x1;
            path[0][0][1] = potential;
            path[0][0][2] = 0.0;
          }
          if(potential < 8.00 && path[1][0][2] == -1000.0f){
            printf("potential %lf at %lf\n", potential, x1);
            path[1][0][0] = x1;
            path[1][0][1] = potential;
            path[1][0][2] = 0.0;
          }
          if(potential < 7.00 && path[2][0][2] == -1000.0f){
            printf("potential %lf at %lf\n", potential, x1);
            path[2][0][0] = x1;
            path[2][0][1] = potential;
            path[2][0][2] = 0.0;
          }
          if(potential < 6.00 && path[3][0][2] == -1000.0f){
            printf("potential %lf at %lf\n", potential, x1);
            path[3][0][0] = x1;
            path[3][0][1] = potential;
            path[3][0][2] = 0.0;
          }
          if(potential < 5.00 && path[4][0][2] == -1000.0f){
            printf("potential %lf at %lf\n", potential, x1);
            path[4][0][0] = x1;
            path[4][0][1] = potential;
            path[4][0][2] = 0.0;
          }
          if(potential < 4.00 && path[5][0][2] == -1000.0f){
            printf("potential %lf at %lf\n", potential, x1);
            path[5][0][0] = x1;
            path[5][0][1] = potential;
            path[5][0][2] = 0.0;
          }
          if(potential < 3.00 && path[6][0][2] == -1000.0f){
            printf("potential %lf at %lf\n", potential, x1);
            path[6][0][0] = x1;
            path[6][0][1] = potential;
            path[6][0][2] = 0.0;
          }
          if(potential < 2.00 && path[7][0][2] == -1000.0f){
            printf("potential %lf at %lf\n", potential, x1);
            path[7][0][0] = x1;
            path[7][0][1] = potential;
            path[7][0][2] = 0.0;
          }
          if(potential < 1.00 && path[8][0][2] == -1000.0f){
            printf("potential %lf at %lf\n", potential, x1);
            path[8][0][0] = x1;
            path[8][0][1] = potential;
            path[8][0][2] = 0.0;
          }
          if(potential < 1.00 && path[9][0][2] == -1000.0f){
            printf("potential %lf at %lf\n", potential, x1);
            path[9][0][0] = x1;
            path[9][0][1] = potential;
            path[9][0][2] = 0.0;
          }
        }
        
        for(x1 = particles[1][0] + 0.1; x1 < 8.0; x1 += 0.0001){//Find staring points of equipotential lines
          potential = 0.0;
          for(i = 0; i < potentialEnd; i++){
            totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
              potential = k1 * particles[i][4] * particles[3][4] / totalDistance;
              break;
            }
            potential += k1 * particles[i][4] * particles[3][4] / totalDistance;
          }//for(i=0; i < potentialEnd; i++){
          potential = potential * scalePlaneHeigth;
          //printf("potential=%lf\n", potential);
          if(potential < 9.00 && path[10][0][2] == -1000.0f){
            printf("potential %lf at %lf\n", potential, x1);
            path[10][0][0] = x1;
            path[10][0][1] = potential;
            path[10][0][2] = 0.0;
          }
          if(potential < 8.00 && path[11][0][2] == -1000.0f){
            printf("potential %lf at %lf\n", potential, x1);
            path[11][0][0] = x1;
            path[11][0][1] = potential;
            path[11][0][2] = 0.0;
          }
        }



        int pathPoint;
        GLfloat startPotential;
        GLfloat alphaStart;
        GLfloat alphaEnd;
        GLfloat beta;
        GLfloat newPotential;
        GLfloat pathStep = 0.2f;
        
        for(pathPoint=0; pathPoint<550; pathPoint++){
          for(pathNumber = 0; pathNumber < 12; pathNumber++){
            potential = 0.0;
            for(i=0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(path[pathNumber][0][0]-particles[i][0], 2.0) + pow(path[pathNumber][0][2]-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * particles[3][4] / totalDistance;
                break;
              }
              potential += k1 * particles[i][4] * particles[3][4] / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            startPotential = potential * scalePlaneHeigth;
            
            if(pathPoint < 1){
              alphaStart = 0.0;
              alphaEnd = 3.1415927;
            }
            else{
              beta = atan2(path[pathNumber][pathPoint][2] - path[pathNumber][pathPoint-1][2], path[pathNumber][pathPoint][0] - path[pathNumber][pathPoint-1][0]);
              alphaStart = beta - 3.1415927 / 2.0;
              alphaEnd = beta + 3.1415927 / 2.0;
            }
            newPotential = 1000000;
            for(alpha = alphaStart; alpha < alphaEnd; alpha += 0.1){
              x1 = cos(alpha) * pathStep + path[pathNumber][pathPoint][0];
              z1 = sin(alpha) * pathStep + path[pathNumber][pathPoint][2];
              potential = 0.0;
              for(i=0; i < potentialEnd; i++){
                totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(z1-particles[i][2], 2.0));
                if(totalDistance < minDistance){
                  totalDistance = minDistance;
                  potential = k1 * particles[i][4] * particles[3][4] / minDistance;
                  break;
                }
                potential += k1 * particles[i][4] * particles[3][4] / totalDistance;
              }//for(i=0; i < potentialEnd; i++){
              potential = potential * scalePlaneHeigth;
              if(fabs(startPotential - potential) < fabs(startPotential - newPotential)){
                newPotential = potential;
                beta = alpha;
                path[pathNumber][pathPoint+1][0] = x1;
                path[pathNumber][pathPoint+1][1] = potential + 0.07f;
                path[pathNumber][pathPoint+1][2] = z1;
                //printf("newPotential = %lf\n", newPotential);
              }    
            }
            newPotential = 1000000;
            for(alpha = beta - 0.1; alpha < beta + 0.1; alpha += 0.01){//Calculate again with fine adjustment
              x1 = cos(alpha) * pathStep + path[pathNumber][pathPoint][0];
              z1 = sin(alpha) * pathStep + path[pathNumber][pathPoint][2];
              potential = 0.0;
              for(i=0; i < potentialEnd; i++){
                totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(z1-particles[i][2], 2.0));
                if(totalDistance < minDistance){
                  totalDistance = minDistance;
                  potential = k1 * particles[i][4] * particles[3][4] / minDistance;
                  break;
                }
                potential += k1 * particles[i][4] * particles[3][4] / totalDistance;
              }//for(i=0; i < potentialEnd; i++){
              potential = potential * scalePlaneHeigth;
              if(fabs(startPotential - potential) < fabs(startPotential - newPotential)){
                newPotential = potential;
                path[pathNumber][pathPoint+1][0] = x1;
                path[pathNumber][pathPoint+1][1] = potential + 0.07f;
                path[pathNumber][pathPoint+1][2] = z1;
                //printf("newPotential = %lf\n", newPotential);
              }    
            }
            for(i = 0; i < 12; i++){
              pathEnd[i]=pathPoint;
            }
            path[pathNumber][0][1] = startPotential + 0.07f;
            path[pathNumber][1][1] = startPotential + 0.07f;
            path[pathNumber][2][1] = startPotential + 0.07f;
            path[pathNumber][3][1] = startPotential + 0.07f;
          }//for(pathNumber = 0; pathNumber < PATHMAX; pathNumber++){
        }//for(pathPoint=0; pathPoint<460; pathPoint++){

        minDistance = 0.2f;
        if(particles[0][0] < 0.0f){
          particles[0][0] += 0.01f;
          particles[1][0] -= 0.01f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          
          pauseFrames = 0;

          doubleTemp = 0.0f;
          particles[0][0] = 0.0f;
          particles[1][0] = 0.0f;
        }

      }


      if(animation == 2305){
        if(particles[3][2] > 2.0f){
          particles[3][2] -= 0.1f;
          pathEnd[12]++;
        }
        else{
        
          animation++;
          printf("Animation set to %d\n", animation);
          
          pauseFrames = 0;
          
          //savePics = 0;
          //printf("...un ferddisch!\n");
          //done = True;         // Exit programm !!!        
        }
      }

      if(animation == 2304){
        if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
          animation = 2305;
        }
        if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
          animation = 2305;
        }
        if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
          animation = 2305;
        }
        if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
          animation = 2305;
        }
        if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
          animation = 2305;
        }
        if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
          animation = 2305;
        }
        if(animation == 2305){
          printf("Animation set to %d\n", animation);
                    
          translateXdiff = 0.0f;
          translateYdiff = 0.0f;
          translateZdiff = 0.0f;
          rotXdiff = 0.0f;
          rotYdiff = 0.0f;
          rotZdiff = 0.0f;
                    
        }
      }


      if(animation == 2303){
        if(translateZ > -29.35f){
          translateZ -= 0.1f;
        }
        else{
          enableGrid = 0;
          gridX = gridX * 4;
          gridY = gridY * 4;

          enablePotentialSphere = 1;
          particles[3][0] = 0.0f;
          particles[3][1] = 0.0f;
          particles[3][2] = yMax;
          particles[3][3] = 0.15f;
          
          animation++;
          printf("Animation set to %d\n", animation);


          
          translateXtarget = 0.0f;
          translateYtarget = -2.64f;
          translateZtarget = -26.20f;
          
          rotXtarget = 8.50f;
          rotYtarget = -44.25f;
          rotZtarget = 0.0f;

          translateXdiff = (translateXtarget - translateX) / steps;
          translateYdiff = (translateYtarget - translateY) / steps;
          translateZdiff = (translateZtarget - translateZ) / steps;
          rotXdiff = (rotXtarget - rotX) / steps;
          rotYdiff = (rotYtarget - rotY) / steps;
          rotZdiff = (rotZtarget - rotZ) / steps;
        }        
      }


      if(animation == 2302){
        gridX = 128;
        gridY = 64;
        enableGrid = 1;
        if(translateZ < -5.0){
          translateZ += 0.1f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);

          pauseFrames = 0;
        }        
      }


      if(animation == 2301){//Draw equipotential lines
        GLfloat pathStep = 0.2;
        GLfloat startPotential, newPotential;
        GLfloat alphaStart, alphaEnd;
        GLfloat beta;
        int pathNumber;
        
        for(pathNumber = 0; pathNumber < PATHMAX; pathNumber++){
          potential = 0.0;
          for(i=0; i < potentialEnd; i++){
            totalDistance = sqrt(pow(path[pathNumber][0][0]-particles[i][0], 2.0) + pow(path[pathNumber][0][2]-particles[i][2], 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
              potential = k1 * particles[i][4] * particles[3][4] / totalDistance;
              break;
            }
            potential += k1 * particles[i][4] * particles[3][4] / totalDistance;
          }//for(i=0; i < potentialEnd; i++){
          startPotential = potential * scalePlaneHeigth;
          
          if(pathEnd[pathNumber] < 1){
            alphaStart = 0.0;
            alphaEnd = 3.1415927;
          }
          else{
            beta = atan2(path[pathNumber][pathEnd[pathNumber]][2] - path[pathNumber][pathEnd[pathNumber]-1][2], path[pathNumber][pathEnd[pathNumber]][0] - path[pathNumber][pathEnd[pathNumber]-1][0]);
            alphaStart = beta - 3.1415927 / 2.0;
            alphaEnd = beta + 3.1415927 / 2.0;
          }
          newPotential = 1000000;
          for(alpha = alphaStart; alpha < alphaEnd; alpha += 0.1){
            x1 = cos(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][0];
            z1 = sin(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][2];
            potential = 0.0;
            for(i=0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(z1-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * particles[3][4] / minDistance;
                break;
              }
              potential += k1 * particles[i][4] * particles[3][4] / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * scalePlaneHeigth;
            if(fabs(startPotential - potential) < fabs(startPotential - newPotential)){
              newPotential = potential;
              beta = alpha;
              path[pathNumber][pathEnd[pathNumber]+1][0] = x1;
              path[pathNumber][pathEnd[pathNumber]+1][1] = potential + 0.07f;
              path[pathNumber][pathEnd[pathNumber]+1][2] = z1;
              //printf("newPotential = %lf\n", newPotential);
            }    
          }
          newPotential = 1000000;
          for(alpha = beta - 0.1; alpha < beta + 0.1; alpha += 0.01){//Calculate again with fine adjustment
            x1 = cos(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][0];
            z1 = sin(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][2];
            potential = 0.0;
            for(i=0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(z1-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * particles[3][4] / minDistance;
                break;
              }
              potential += k1 * particles[i][4] * particles[3][4] / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * scalePlaneHeigth;
            if(fabs(startPotential - potential) < fabs(startPotential - newPotential)){
              newPotential = potential;
              path[pathNumber][pathEnd[pathNumber]+1][0] = x1;
              path[pathNumber][pathEnd[pathNumber]+1][1] = potential + 0.07f;
              path[pathNumber][pathEnd[pathNumber]+1][2] = z1;
              //printf("newPotential = %lf\n", newPotential);
            }    
          }


          //path[pathNumber][0][1] = path[pathNumber][1][1];//Approximation for initial potential
        }//for(pathNumber = 1, pathNumber < 3; pathNumber++){


        if(pathEnd[1] < 550){//Return to 460 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
          for(pathNumber = 0; pathNumber < 12; pathNumber++){
            pathEnd[pathNumber]++;
          }
          GLfloat lightX, lightZ;
          alpha = 2.0f * PI / 550 * pathEnd[0] + PI * 3.0f / 4.0f;
          lightPosition1[0] = cos(alpha) * 7.071067812f;
          lightPosition1[1] = sin(alpha) * 7.071067812f;
          //lightPosition1[0] = -5.0f;
          //lightPosition1[2] = 5.0f;
          //printf("lightPosition1[0] = %lf, lightPosition1[1] = %lf, lightPosition1[2] = %lf, lightPosition1[3] = %lf\n", lightPosition1[0], lightPosition1[1], lightPosition1[2], lightPosition1[3]);
          //GLfloat lightPosition1[] = { lightX, 2.0f, lightZ, 1.0f };
          
          
          
//          if(pathEnd[1] > 150 && translateZ > -35.4f){
//            translateZ -= 0.1f;
//          }
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          
          doubleTemp = 0.0;
          pauseFrames = 0;
          
        }
      }      




      if(animation == 2300){
        if(particles[0][0] < -1.5f){
          particles[0][0] += 0.01f;
          particles[1][0] -= 0.01f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);

          particles[0][0] = -1.5f;
          particles[1][0] = 1.5f;

          minDistance = 0.01f;
          potentialEnd = 3;
          for(i=0; i<PATHMAX;i++){
            path[i][0][2] = -1000.0f;
            pathEnd[i] = 0;
            pathColor[i] = 3;
          }
          for(x1 = -2.1; x1 > -25.0; x1 -= 0.0001){//Find staring points of equipotential lines
            potential = 0.0;
            for(i = 0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * particles[3][4] / totalDistance;
                break;
              }
              potential += k1 * particles[i][4] * particles[3][4] / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * scalePlaneHeigth;
            //printf("potential=%lf\n", potential);
            if(potential < 9.00 && path[0][0][2] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[0][0][0] = x1;
              path[0][0][1] = potential;
              path[0][0][2] = 0.0;
            }
            if(potential < 8.00 && path[1][0][2] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[1][0][0] = x1;
              path[1][0][1] = potential;
              path[1][0][2] = 0.0;
            }
            if(potential < 7.00 && path[2][0][2] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[2][0][0] = x1;
              path[2][0][1] = potential;
              path[2][0][2] = 0.0;
            }
            if(potential < 6.00 && path[3][0][2] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[3][0][0] = x1;
              path[3][0][1] = potential;
              path[3][0][2] = 0.0;
            }
            if(potential < 5.00 && path[4][0][2] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[4][0][0] = x1;
              path[4][0][1] = potential;
              path[4][0][2] = 0.0;
            }
            if(potential < 4.00 && path[5][0][2] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[5][0][0] = x1;
              path[5][0][1] = potential;
              path[5][0][2] = 0.0;
            }
            if(potential < 3.00 && path[6][0][2] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[6][0][0] = x1;
              path[6][0][1] = potential;
              path[6][0][2] = 0.0;
            }
            if(potential < 2.00 && path[7][0][2] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[7][0][0] = x1;
              path[7][0][1] = potential;
              path[7][0][2] = 0.0;
            }
            if(potential < 1.00 && path[8][0][2] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[8][0][0] = x1;
              path[8][0][1] = potential;
              path[8][0][2] = 0.0;
            }
            if(potential < 1.00 && path[9][0][2] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[9][0][0] = x1;
              path[9][0][1] = potential;
              path[9][0][2] = 0.0;
            }
          }
          
          for(x1 = 2.1; x1 < 8.0; x1 += 0.0001){//Find staring points of equipotential lines
            potential = 0.0;
            for(i = 0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * particles[3][4] / totalDistance;
                break;
              }
              potential += k1 * particles[i][4] * particles[3][4] / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * scalePlaneHeigth;
            //printf("potential=%lf\n", potential);
            if(potential < 9.00 && path[10][0][2] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[10][0][0] = x1;
              path[10][0][1] = potential;
              path[10][0][2] = 0.0;
            }
            if(potential < 8.00 && path[11][0][2] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[11][0][0] = x1;
              path[11][0][1] = potential;
              path[11][0][2] = 0.0;
            }
          }

        }
      }
      
      
      
    }//if(pauseFrames > 10){


    if(animation >= 2300 && animation < 2400){//Calculate height of potential sphere / path

      potential = 0.0;
      
      potential = 0.0;
      for(i = 0; i < 3; i++){
        totalDistance = sqrt(pow(particles[3][0]-particles[i][0], 2.0) + pow(particles[3][2]-particles[i][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
          potential = k1 * particles[i][4] * particles[3][4] / totalDistance;
          break;
        }
        potential += k1 * particles[i][4] * particles[3][4] / totalDistance;
      }//for(i=0; i < potentialEnd; i++){
      potentialSphereY = potential * scalePlaneHeigth;
      
      potentialSphereX = particles[3][0];
      potentialSphereZ = particles[3][2];
            
      if(animation > 2303){
        path[12][pathEnd[12]][0] = potentialSphereX;
        path[12][pathEnd[12]][1] = potentialSphereY + 0.07f;
        path[12][pathEnd[12]][2] = potentialSphereZ;
        pathColor[12] = 1;
      }
      
    }




//####################################################################################
//############# Proton composed of Quarks ############################################
//####################################################################################

    if(animation >= 2200 && animation < 2300 && fieldLineEndUp > 0.0f){
      GLfloat fieldLineCoords[10000][4];
      GLfloat fieldLineLength = 40.0;
      double alpha, beta;
      int fieldLineNumber = 36;
      GLfloat arrowRadius_02;
      
      
      glEnable(GL_LIGHTING);
      glPolygonMode(GL_BACK, GL_FILL);
      glPolygonMode(GL_FRONT, GL_FILL);


      glLightfv(GL_LIGHT1, GL_AMBIENT, lightGreen1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightGreen1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightGreen1);

      for(i = 0; i < 18; i++){
        alpha = (GLfloat)(i) * 2.0 * PI / 18.0f;
        fieldLineCoords[i][0] = particles[0][0] + cos(alpha) * particles[0][3];
        fieldLineCoords[i][1] = particles[0][1] + sin(alpha) * particles[0][3];
        fieldLineCoords[i][2] = 0.0;
        fieldLineCoords[i][3] = 0.0;
      }

      for(i = 0; i < 18; i++){
        alpha = (GLfloat)(i) * 2.0 * PI / 18.0f + PI;
        fieldLineCoords[i + 18][0] = particles[1][0] + cos(alpha) * particles[1][3];
        fieldLineCoords[i + 18][1] = particles[1][1] + sin(alpha) * particles[1][3];
        fieldLineCoords[i + 18][2] = 0.0;
        fieldLineCoords[i + 18][3] = 0.0;
      }


      for(i2 = 0; i2 < fieldLineNumber; i2++){
        for(i = 0; i < fieldLineEndUp; i++){
          totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[0][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[0][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[0][2], 2.0));
          totalForce = k * particles[0][4] / pow(totalDistance, 2.0);
          forceX = (fieldLineCoords[i2][0] - particles[0][0]) / totalDistance * totalForce;
          forceY = (fieldLineCoords[i2][1] - particles[0][1]) / totalDistance * totalForce;
          forceZ = (fieldLineCoords[i2][2] - particles[0][2]) / totalDistance * totalForce;

          totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
          totalForce = k * particles[1][4] / pow(totalDistance, 2.0);
          forceX += (fieldLineCoords[i2][0] - particles[1][0]) / totalDistance * totalForce;
          forceY += (fieldLineCoords[i2][1] - particles[1][1]) / totalDistance * totalForce;
          forceZ += (fieldLineCoords[i2][2] - particles[1][2]) / totalDistance * totalForce;

          totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[2][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[2][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[2][2], 2.0));
          totalForce = k * particles[2][4] / pow(totalDistance, 2.0);
          forceX += (fieldLineCoords[i2][0] - particles[2][0]) / totalDistance * totalForce;
          forceY += (fieldLineCoords[i2][1] - particles[2][1]) / totalDistance * totalForce;
          forceZ += (fieldLineCoords[i2][2] - particles[2][2]) / totalDistance * totalForce;

          totalForce = sqrt(pow(forceX, 2.0) + pow(forceY, 2.0) + pow(forceZ, 2.0));
          x2 = fieldLineCoords[i2][0] + forceX / totalForce * fieldLineStep;
          y2 = fieldLineCoords[i2][1] + forceY / totalForce * fieldLineStep;
          z2 = fieldLineCoords[i2][2] + forceZ / totalForce * fieldLineStep;
                    

          if(fieldLineCoords[i2][3] != -10.0){
            drawCylinder(fieldLineCoords[i2][0], fieldLineCoords[i2][1], fieldLineCoords[i2][2], x2, y2, z2, 0.05, 0.05, 32);
            
            if(fieldLineCoords[i2][3] == 0.0){
              arrowRadius_02 = 3.0f;
              //Half of field lines at both particles
              if(i2 < fieldLineNumber / 2){
                totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[0][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[0][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[0][2], 2.0));
              }
              else{
                totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
              }
              if(totalDistance > arrowRadius_02){
                drawCylinder(x1, y1, z1, x2, y2, z2, 0.15, 0.05, 32);
                fieldLineCoords[i2][3] = 1.0;
              }
            }
            if(fieldLineCoords[i2][3] == 0.0){//Arrow at middle between charges while approaching
              if((i2 < fieldLineNumber / 2 && x2 > particles[0][0] / 2.0f) || (i2 >= fieldLineNumber / 2 && x2 < particles[1][0] / 2.0f)){
                GLfloat x3;
                if(i2 < fieldLineNumber / 2 && x2 > particles[0][0] / 2.0f){
                  x3 = particles[0][0] / 2.0f;
                }
                else{
                  x3 = particles[1][0] / 2.0f;
                }
                drawCylinder( x3 - 1.00f * (x2 - x1), y1, z1, x3, y2, z2, 0.15, 0.05, 32);
                fieldLineCoords[i2][3] = 1.0;
              }            
            }
          }

          x1 = fieldLineCoords[i2][0];
          y1 = fieldLineCoords[i2][1];
          z1 = fieldLineCoords[i2][2];

          if(!(fabs(x1) < 0.1f && fabs(y1) < 0.1f && fabs(z1) < 0.1f)){
            fieldLineCoords[i2][0] = x2;
            fieldLineCoords[i2][1] = y2;
            fieldLineCoords[i2][2] = z2;
          }
          
//        totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
        }//for(i = 0; i < fieldLineEndUp; i++){
      }//for(i2 = 0; i2 < 2; i2++){
      
    }



    if(pauseFrames > 10 && animationPause == 0){




      if(animation == 2201){//Draw equipotential lines
        GLfloat pathStep = 0.2;
        GLfloat startPotential, newPotential;
        GLfloat alphaStart, alphaEnd;
        GLfloat beta;
        int pathNumber;
        
        for(pathNumber = 0; pathNumber < PATHMAX; pathNumber++){
          potential = 0.0;
          for(i=0; i < potentialEnd; i++){
            totalDistance = sqrt(pow(path[pathNumber][0][0]-particles[i][0], 2.0) + pow(path[pathNumber][0][1]-particles[i][1], 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
              potential = k1 * particles[i][4] * 0.000001f / totalDistance;
              break;
            }
            potential += k1 * particles[i][4] * 0.000001f / totalDistance;
          }//for(i=0; i < potentialEnd; i++){
          startPotential = potential * 200.0f;
          
          if(pathEnd[pathNumber] < 1){
            alphaStart = 0.0;
            alphaEnd = 3.1415927;
          }
          else{
            beta = atan2(path[pathNumber][pathEnd[pathNumber]][1] - path[pathNumber][pathEnd[pathNumber]-1][1], path[pathNumber][pathEnd[pathNumber]][0] - path[pathNumber][pathEnd[pathNumber]-1][0]);
            alphaStart = beta - 3.1415927 / 2.0;
            alphaEnd = beta + 3.1415927 / 2.0;
          }
          newPotential = 1000000;
          for(alpha = alphaStart; alpha < alphaEnd; alpha += 0.1){
            x1 = cos(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][0];
            z1 = sin(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][1];
            potential = 0.0;
            for(i=0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(z1-particles[i][1], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / minDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * 200.0f;
            if(fabs(startPotential - potential) < fabs(startPotential - newPotential)){
              newPotential = potential;
              path[pathNumber][pathEnd[pathNumber]+1][0] = x1;
              path[pathNumber][pathEnd[pathNumber]+1][1] = z1;
              path[pathNumber][pathEnd[pathNumber]+1][2] = 0.0;
              //printf("newPotential = %lf\n", newPotential);
            }    
          }
          //path[pathNumber][0][1] = path[pathNumber][1][1];//Approximation for initial potential
        }//for(pathNumber = 1, pathNumber < 3; pathNumber++){


        if(pathEnd[1] < 460){//Return to 460 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
          for(pathNumber = 0; pathNumber < PATHMAX; pathNumber++){
            pathEnd[pathNumber]++;
          }
          if(pathEnd[1] > 150 && translateZ > -35.4f){
            translateZ -= 0.1f;
          }
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          
          savePics = 0;
          printf("...un ferddisch!\n");
          done = True;         // Exit programm !!!
        }
      }      



      if(animation == 2200){//Draw field lines
        if(particles[0][0] < -6.0f){
          particles[0][0] += 0.1f;
          particles[1][0] -= 0.1f;
        }
        if(particles[0][0] < -2.0f){
          particles[0][0] += 0.02f;
          particles[1][0] -= 0.02f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);

          particles[0][0] = -2.0f;
          particles[1][0] = 2.0f;

          minDistance = 0.01f;
          potentialEnd = 3;
          for(i=0; i<10;i++){
            path[i][0][1] = -1000.0f;
            pathEnd[i] = 0;
            pathColor[i] = 3;
          }
          for(x1 = -2.1; x1 > -15.0; x1 -= 0.0001){//Find staring points of equipotential lines
            potential = 0.0;
            for(i = 0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / totalDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * scalePlaneHeigth;
            //printf("potential=%lf\n", potential);
            if(potential < 1.50 && path[0][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[0][0][0] = x1;
              path[0][0][1] = 0.0;
              path[0][0][2] = 0.0;
            }
            if(potential < 1.25 && path[1][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[1][0][0] = x1;
              path[1][0][1] = 0.0;
              path[1][0][2] = 0.0;
            }
            if(potential < 1.00 && path[2][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[2][0][0] = x1;
              path[2][0][1] = 0.0;
              path[2][0][2] = 0.0;
            }
            if(potential < 0.75 && path[3][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[3][0][0] = x1;
              path[3][0][1] = 0.0;
              path[3][0][2] = 0.0;
            }
            if(potential < 0.50 && path[4][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[4][0][0] = x1;
              path[4][0][1] = 0.0;
              path[4][0][2] = 0.0;
            }
            if(potential < 0.25 && path[5][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[5][0][0] = x1;
              path[5][0][1] = 0.0;
              path[5][0][2] = 0.0;
            }
          }
          
          for(x1 = 2.1; x1 < 6.0; x1 += 0.0001){//Find staring points of equipotential lines
            potential = 0.0;
            for(i = 0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / totalDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * scalePlaneHeigth;
            //printf("potential=%lf\n", potential);
            if(potential < 1.50 && path[7][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[7][0][0] = x1;
              path[7][0][1] = 0.0;
              path[7][0][2] = 0.0;
            }
            if(potential < 1.25 && path[8][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[8][0][0] = x1;
              path[8][0][1] = 0.0;
              path[8][0][2] = 0.0;
            }
            if(potential < 1.00 && path[9][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[9][0][0] = x1;
              path[9][0][1] = 0.0;
              path[9][0][2] = 0.0;
            }
          }

        }
      }
      
      
      
    }//if(pauseFrames > 10){




//####################################################################################
//############# 3D equipotential plane ###############################################
//####################################################################################

    if(animation >= 2100 && animation < 2200 && fieldLineEndUp > 0.0f){
      GLfloat fieldLineCoords[10000][4];
      GLfloat fieldLineLength = 40.0;
      double alpha, beta;
      int fieldLineNumber = 36;
      GLfloat arrowRadius_02;
      
      
      glEnable(GL_LIGHTING);
      glPolygonMode(GL_BACK, GL_FILL);
      glPolygonMode(GL_FRONT, GL_FILL);


      glLightfv(GL_LIGHT1, GL_AMBIENT, lightGreen1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightGreen1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightGreen1);

      for(i = 0; i < 18; i++){
        alpha = (GLfloat)(i) * 2.0 * PI / 18.0f;
        fieldLineCoords[i][0] = particles[0][0] + cos(alpha) * particles[0][3];
        fieldLineCoords[i][1] = particles[0][1] + sin(alpha) * particles[0][3];
        fieldLineCoords[i][2] = 0.0;
        fieldLineCoords[i][3] = 0.0;
      }

      for(i = 0; i < 18; i++){
        alpha = (GLfloat)(i) * 2.0 * PI / 18.0f + PI;
        fieldLineCoords[i + 18][0] = particles[1][0] + cos(alpha) * particles[1][3];
        fieldLineCoords[i + 18][1] = particles[1][1] + sin(alpha) * particles[1][3];
        fieldLineCoords[i + 18][2] = 0.0;
        fieldLineCoords[i + 18][3] = 0.0;
      }


      for(i2 = 0; i2 < fieldLineNumber; i2++){
        for(i = 0; i < fieldLineEndUp; i++){
          totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[0][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[0][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[0][2], 2.0));
          totalForce = k * particles[0][4] / pow(totalDistance, 2.0);
          forceX = (fieldLineCoords[i2][0] - particles[0][0]) / totalDistance * totalForce;
          forceY = (fieldLineCoords[i2][1] - particles[0][1]) / totalDistance * totalForce;
          forceZ = (fieldLineCoords[i2][2] - particles[0][2]) / totalDistance * totalForce;

          totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
          totalForce = k * particles[1][4] / pow(totalDistance, 2.0);
          forceX += (fieldLineCoords[i2][0] - particles[1][0]) / totalDistance * totalForce;
          forceY += (fieldLineCoords[i2][1] - particles[1][1]) / totalDistance * totalForce;
          forceZ += (fieldLineCoords[i2][2] - particles[1][2]) / totalDistance * totalForce;

          totalForce = sqrt(pow(forceX, 2.0) + pow(forceY, 2.0) + pow(forceZ, 2.0));
          x2 = fieldLineCoords[i2][0] + forceX / totalForce * fieldLineStep;
          y2 = fieldLineCoords[i2][1] + forceY / totalForce * fieldLineStep;
          z2 = fieldLineCoords[i2][2] + forceZ / totalForce * fieldLineStep;
                    

          if(fieldLineCoords[i2][3] != -10.0){
            drawCylinder(fieldLineCoords[i2][0], fieldLineCoords[i2][1], fieldLineCoords[i2][2], x2, y2, z2, 0.03, 0.03, 32);
            
            if(fieldLineCoords[i2][3] == 0.0){
              arrowRadius_02 = 3.0f;
              //Half of field lines at both particles
              if(i2 < fieldLineNumber / 2){
                totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[0][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[0][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[0][2], 2.0));
              }
              else{
                totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
              }
              if(i2 == 0 || i2 == 18){//Field line betwwen the two charges
                arrowRadius_02 = 1.5f;
              }
              if(totalDistance > arrowRadius_02){
                drawCylinder(x1, y1, z1, x2, y2, z2, 0.1, 0.03, 32);
                fieldLineCoords[i2][3] = 1.0;
              }
            }
          }

          x1 = fieldLineCoords[i2][0];
          y1 = fieldLineCoords[i2][1];
          z1 = fieldLineCoords[i2][2];

          if(!(fabs(x1) < 0.1f && fabs(y1) < 0.1f && fabs(z1) < 0.1f)){
            fieldLineCoords[i2][0] = x2;
            fieldLineCoords[i2][1] = y2;
            fieldLineCoords[i2][2] = z2;
          }
          
          totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
        }//for(i = 0; i < fieldLineEndUp; i++){
      }//for(i2 = 0; i2 < 2; i2++){
      
    }



    if(pauseFrames > 10 && animationPause == 0){

      if(animation == 2102){//Draw equipotential planes
        GLfloat animationStep = 0.0025f;

//savePics = 1;//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

        if(translateZ < -19.8f){
          animationStep = 0.025f;
        }
      
        if(path[0][0][0] < -4.0f){
          drawEquipotentialPlane(0, 1);
        }
        else{
          drawEquipotentialPlane(0, 0);
        }
        if(enableSecondEquipotential == 1){
          drawEquipotentialPlane(7, 0);
        }
                
        path[0][0][0] -= animationStep;
        if(path[7][0][0] < 4.0f){
          path[7][0][0] += animationStep;
        }

        if(path[0][0][0] < -4.0f && translateZ > -20.0f){
          translateZ -= 0.1f;
        }

        GLfloat pathStep = 0.05;
        GLfloat startPotential, newPotential;
        GLfloat alphaStart, alphaEnd;
        GLfloat beta;
        int pathNumber;
        int i3;
        
        for(pathNumber = 0; pathNumber < 10; pathNumber += 7){
          potential = 0.0;
          for(i=0; i < 2; i++){
            totalDistance = sqrt(pow(path[pathNumber][0][0]-particles[i][0], 2.0) + pow(path[pathNumber][0][1]-particles[i][1], 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
              potential = k1 * particles[i][4] * 0.000001f / totalDistance;
              break;
            }
            potential += k1 * particles[i][4] * 0.000001f / totalDistance;
          }//for(i=0; i < potentialEnd; i++){
          startPotential = potential * 200.0f;
          //printf("startPotential = %lf, pathNumber = %d\n", startPotential, pathNumber);
          
          for(i3 = 0; i3 < 6000; i3++){
            pathEnd[pathNumber] = i3;
            if(pathEnd[pathNumber] < 1){
              alphaStart = 0.0;
              alphaEnd = 3.1415927;
            }
            else{
              beta = atan2(path[pathNumber][pathEnd[pathNumber]][1] - path[pathNumber][pathEnd[pathNumber]-1][1], path[pathNumber][pathEnd[pathNumber]][0] - path[pathNumber][pathEnd[pathNumber]-1][0]);
              alphaStart = beta - 3.1415927 / 2.0;
              alphaEnd = beta + 3.1415927 / 2.0;
            }
            newPotential = 1000000;
            for(alpha = alphaStart; alpha < alphaEnd; alpha += 0.1){
              x1 = cos(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][0];
              z1 = sin(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][1];
              potential = 0.0;
              for(i=0; i < 2; i++){
                totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(z1-particles[i][1], 2.0));
                if(totalDistance < minDistance){
                  totalDistance = minDistance;
                  potential = k1 * particles[i][4] * 0.000001f / minDistance;
                  break;
                }
                potential += k1 * particles[i][4] * 0.000001f / totalDistance;
              }//for(i=0; i < potentialEnd; i++){
              potential = potential * 200.0f;
              if(fabs(startPotential - potential) < fabs(startPotential - newPotential)){
                newPotential = potential;
                beta = alpha;
                path[pathNumber][pathEnd[pathNumber]+1][0] = x1;
                path[pathNumber][pathEnd[pathNumber]+1][1] = z1;
                path[pathNumber][pathEnd[pathNumber]+1][2] = 0.0;
                //printf("newPotential = %lf\n", newPotential);
              }    
            }
            newPotential = 1000000;
            for(alpha = beta - 0.1; alpha < beta + 0.1; alpha += 0.01){//Calculate again with fine adjustment
              x1 = cos(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][0];
              z1 = sin(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][1];
              potential = 0.0;
              for(i=0; i < 2; i++){
                totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(z1-particles[i][1], 2.0));
                if(totalDistance < minDistance){
                  totalDistance = minDistance;
                  potential = k1 * particles[i][4] * 0.000001f / minDistance;
                  break;
                }
                potential += k1 * particles[i][4] * 0.000001f / totalDistance;
              }//for(i=0; i < potentialEnd; i++){
              potential = potential * 200.0f;
              if(fabs(startPotential - potential) < fabs(startPotential - newPotential)){
                newPotential = potential;
                beta = alpha;
                path[pathNumber][pathEnd[pathNumber]+1][0] = x1;
                path[pathNumber][pathEnd[pathNumber]+1][1] = z1;
                path[pathNumber][pathEnd[pathNumber]+1][2] = 0.0;
                //printf("newPotential = %lf\n", newPotential);
              }    
            }
          }

        }//for(pathNumber = 1, pathNumber < 3; pathNumber++){


        if(path[0][0][0] < -7.708976f){
          savePics = 0;
          printf("...un ferddisch!\n");
          done = True;         // Exit programm !!!
        }


      }


      if(animation == 2101){//Draw equipotential lines
        GLfloat pathStep = 0.2;
        GLfloat startPotential, newPotential;
        GLfloat alphaStart, alphaEnd;
        GLfloat beta;
        int pathNumber;
        
        for(pathNumber = 0; pathNumber < 10; pathNumber++){
          potential = 0.0;
          for(i=0; i < 2; i++){
            totalDistance = sqrt(pow(path[pathNumber][0][0]-particles[i][0], 2.0) + pow(path[pathNumber][0][1]-particles[i][1], 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
              potential = k1 * particles[i][4] * 0.000001f / totalDistance;
              break;
            }
            potential += k1 * particles[i][4] * 0.000001f / totalDistance;
          }//for(i=0; i < potentialEnd; i++){
          startPotential = potential * 200.0f;
          
          if(pathEnd[pathNumber] < 1){
            alphaStart = 0.0;
            alphaEnd = 3.1415927;
          }
          else{
            beta = atan2(path[pathNumber][pathEnd[pathNumber]][1] - path[pathNumber][pathEnd[pathNumber]-1][1], path[pathNumber][pathEnd[pathNumber]][0] - path[pathNumber][pathEnd[pathNumber]-1][0]);
            alphaStart = beta - 3.1415927 / 2.0;
            alphaEnd = beta + 3.1415927 / 2.0;
          }
          newPotential = 1000000;
          for(alpha = alphaStart; alpha < alphaEnd; alpha += 0.1){
            x1 = cos(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][0];
            z1 = sin(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][1];
            potential = 0.0;
            for(i=0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(z1-particles[i][1], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / minDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * 200.0f;
            if(fabs(startPotential - potential) < fabs(startPotential - newPotential)){
              newPotential = potential;
              path[pathNumber][pathEnd[pathNumber]+1][0] = x1;
              path[pathNumber][pathEnd[pathNumber]+1][1] = z1;
              path[pathNumber][pathEnd[pathNumber]+1][2] = 0.0;
              //printf("newPotential = %lf\n", newPotential);
            }    
          }
          //path[pathNumber][0][1] = path[pathNumber][1][1];//Approximation for initial potential
        }//for(pathNumber = 1, pathNumber < 3; pathNumber++){


        if(pathEnd[1] < 460){//Return to 460 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
          for(pathNumber = 0; pathNumber < 10; pathNumber++){
            pathEnd[pathNumber]++;
          }
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
        }
      }      


      if(animation == 2100){//Paint field lines
        
        if(fieldLineEndUp < 70){//Return to 70 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
          fieldLineEndUp++;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          fieldLineEndUp = 150;
          pauseFrames = 0;

          potentialEnd = 2;
          for(i=0; i<10;i++){
            path[i][0][1] = -1000.0f;
            pathEnd[i] = 0;
            pathColor[i] = 3;
          }
          for(x1 = -2.0; x1 > -15.0; x1 -= 0.0001){//Find staring points of equipotential lines
            potential = 0.0;
            for(i = 0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / totalDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * scalePlaneHeigth;
            //printf("potential=%lf\n", potential);
            if(potential < 3.00 && path[0][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[0][0][0] = x1;
              path[0][0][1] = 0.0;
              path[0][0][2] = 0.0;
            }
            if(potential < 2.50 && path[1][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[1][0][0] = x1;
              path[1][0][1] = 0.0;
              path[1][0][2] = 0.0;
            }
            if(potential < 2.00 && path[2][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[2][0][0] = x1;
              path[2][0][1] = 0.0;
              path[2][0][2] = 0.0;
            }
            if(potential < 1.50 && path[3][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[3][0][0] = x1;
              path[3][0][1] = 0.0;
              path[3][0][2] = 0.0;
            }
            if(potential < 1.00 && path[4][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[4][0][0] = x1;
              path[4][0][1] = 0.0;
              path[4][0][2] = 0.0;
            }
            if(potential < 0.50 && path[5][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[5][0][0] = x1;
              path[5][0][1] = 0.0;
              path[5][0][2] = 0.0;
            }
            if(potential < 0.25 && path[6][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[6][0][0] = x1;
              path[6][0][1] = 0.0;
              path[6][0][2] = 0.0;
            }
          }
          
          for(x1 = 2.0; x1 < 4.0; x1 += 0.0001){//Find staring points of equipotential lines
            potential = 0.0;
            for(i = 0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / totalDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * scalePlaneHeigth;
            //printf("potential=%lf\n", potential);
            if(potential < 3.00 && path[7][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[7][0][0] = x1;
              path[7][0][1] = 0.0;
              path[7][0][2] = 0.0;
            }
            if(potential < 2.50 && path[8][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[8][0][0] = x1;
              path[8][0][1] = 0.0;
              path[8][0][2] = 0.0;
            }
            if(potential < 2.00 && path[9][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[9][0][0] = x1;
              path[9][0][1] = 0.0;
              path[9][0][2] = 0.0;
            }
          }

        }
      }//if(animation == 2100){
      
      
    }//if(pauseFrames > 10){
















//####################################################################################
//############# Field lines of two positive charges with distant view ################
//####################################################################################

    if(animation >= 2000 && animation < 2100 && fieldLineEndUp > 0.0f){
      GLfloat fieldLineCoords[10000][4];
      GLfloat fieldLineLength = 40.0;
      double alpha, beta;
      int fieldLineNumber = 36;
      GLfloat arrowRadius_02;
      
      fieldLineStep = 0.15f;
      
      glEnable(GL_LIGHTING);
      glPolygonMode(GL_BACK, GL_FILL);
      glPolygonMode(GL_FRONT, GL_FILL);


      glLightfv(GL_LIGHT1, GL_AMBIENT, lightGreen1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightGreen1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightGreen1);

      if(animation < 2005){
        for(i = 0; i < 18; i++){
          alpha = (GLfloat)(i) * 2.0 * PI / 18.0f;
          fieldLineCoords[i][0] = particles[0][0] + cos(alpha) * particles[0][3];
          fieldLineCoords[i][1] = particles[0][1] + sin(alpha) * particles[0][3];
          fieldLineCoords[i][2] = 0.0;
          fieldLineCoords[i][3] = 0.0;
        }

        for(i = 0; i < 18; i++){
          alpha = (GLfloat)(i) * 2.0 * PI / 18.0f + PI;
          fieldLineCoords[i + 18][0] = particles[1][0] + cos(alpha) * particles[1][3];
          fieldLineCoords[i + 18][1] = particles[1][1] + sin(alpha) * particles[1][3];
          fieldLineCoords[i + 18][2] = 0.0;
          fieldLineCoords[i + 18][3] = 0.0;
        }
      }
      else{
        for(i = 0; i < 36; i++){//Single charge at center
          alpha = (GLfloat)(i) * 2.0 * PI / 36.0f;
          fieldLineCoords[i][0] = particles[0][0] + cos(alpha) * particles[0][3];
          fieldLineCoords[i][1] = particles[0][1] + sin(alpha) * particles[0][3];
          fieldLineCoords[i][2] = 0.0;
          fieldLineCoords[i][3] = 0.0;
        }
      }

      if(animation > 2001 && animation < 2005){//Remove some field lines
         fieldLineCoords[1][3] = -10.0;
         fieldLineCoords[17][3] = -10.0;
         fieldLineCoords[19][3] = -10.0;
         fieldLineCoords[35][3] = -10.0;
      }
      if(animation > 2003 && animation < 2005){//Remove some field lines         
         fieldLineCoords[2][3] = -10.0;
         fieldLineCoords[16][3] = -10.0;
         fieldLineCoords[20][3] = -10.0;
         fieldLineCoords[34][3] = -10.0;         
      }
      if(animation > 2002 && animation < 2005){//Remove some field lines         
         //Replace field lines between particles by a line
         fieldLineCoords[0][3] = -10.0;
         fieldLineCoords[18][3] = -10.0;     
         drawCylinder(particles[0][0], particles[0][1], particles[0][2], particles[1][0], particles[1][1], particles[1][2], 0.05, 0.05, 32);
       }


      for(i2 = 0; i2 < fieldLineNumber; i2++){
        for(i = 0; i < fieldLineEndUp; i++){
          totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[0][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[0][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[0][2], 2.0));
          totalForce = k * particles[0][4] / pow(totalDistance, 2.0);
          forceX = (fieldLineCoords[i2][0] - particles[0][0]) / totalDistance * totalForce;
          forceY = (fieldLineCoords[i2][1] - particles[0][1]) / totalDistance * totalForce;
          forceZ = (fieldLineCoords[i2][2] - particles[0][2]) / totalDistance * totalForce;
          if(animation < 2005){
            totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
            totalForce = k * particles[1][4] / pow(totalDistance, 2.0);
            forceX += (fieldLineCoords[i2][0] - particles[1][0]) / totalDistance * totalForce;
            forceY += (fieldLineCoords[i2][1] - particles[1][1]) / totalDistance * totalForce;
            forceZ += (fieldLineCoords[i2][2] - particles[1][2]) / totalDistance * totalForce;
          }

          totalForce = sqrt(pow(forceX, 2.0) + pow(forceY, 2.0) + pow(forceZ, 2.0));
          x2 = fieldLineCoords[i2][0] + forceX / totalForce * fieldLineStep;
          y2 = fieldLineCoords[i2][1] + forceY / totalForce * fieldLineStep;
          z2 = fieldLineCoords[i2][2] + forceZ / totalForce * fieldLineStep;
                    

          if(fieldLineCoords[i2][3] != -10.0){
            drawCylinder(fieldLineCoords[i2][0], fieldLineCoords[i2][1], fieldLineCoords[i2][2], x2, y2, z2, 0.05, 0.05, 32);
            
            if(fieldLineCoords[i2][3] == 0.0){
              arrowRadius_02 = 3.0f;
              if(animation < 2005){
                //Half of field lines at both particles
                if(i2 < fieldLineNumber / 2){
                  totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[0][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[0][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[0][2], 2.0));
                }
                else{
                  totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
                }
                if(i2 == 0 || i2 == 18){//Field line betwwen the two charges
                  arrowRadius_02 = 1.5f;
                }
              }
              else{
                totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[0][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[0][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[0][2], 2.0));
              }
              if(totalDistance > arrowRadius_02){
                drawCylinder(x1, y1, z1, x1 - 1.20f * (x1 - x2), y1 - 1.20f * (y1 - y2), z1 - 1.20f * (z1 - z2), 0.2, 0.05, 32);
                fieldLineCoords[i2][3] = 1.0;
              }
            }
          }

          x1 = fieldLineCoords[i2][0];
          y1 = fieldLineCoords[i2][1];
          z1 = fieldLineCoords[i2][2];

          if(!(fabs(x1) < 0.2f && fabs(y1) < 0.2f && fabs(z1) < 0.2f) || animation > 2004){
            fieldLineCoords[i2][0] = x2;
            fieldLineCoords[i2][1] = y2;
            fieldLineCoords[i2][2] = z2;
          }
          
          totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
        }//for(i = 0; i < fieldLineEndUp; i++){
      }//for(i2 = 0; i2 < 2; i2++){
      
    }



    if(pauseFrames > 10 && animationPause == 0){
      

      if(animation == 2005){
        
        GLfloat pathStep = 0.2;
        GLfloat startPotential, newPotential;
        GLfloat alphaStart, alphaEnd;
        GLfloat beta;
        int pathNumber;
        
        for(pathNumber = 0; pathNumber < 10; pathNumber++){
          potential = 0.0;
          for(i=0; i < 1; i++){
            totalDistance = sqrt(pow(path[pathNumber][0][0]-particles[i][0], 2.0) + pow(path[pathNumber][0][1]-particles[i][1], 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
              potential = k1 * particles[i][4] * 0.000001f / totalDistance;
              break;
            }
            potential += k1 * particles[i][4] * 0.000001f / totalDistance;
          }//for(i=0; i < potentialEnd; i++){
          startPotential = potential * 200.0f;
          
          if(pathEnd[pathNumber] < 1){
            alphaStart = 0.0;
            alphaEnd = 3.1415927;
          }
          else{
            beta = atan2(path[pathNumber][pathEnd[pathNumber]][1] - path[pathNumber][pathEnd[pathNumber]-1][1], path[pathNumber][pathEnd[pathNumber]][0] - path[pathNumber][pathEnd[pathNumber]-1][0]);
            alphaStart = beta - 3.1415927 / 2.0;
            alphaEnd = beta + 3.1415927 / 2.0;
          }
          newPotential = 1000000;
          for(alpha = alphaStart; alpha < alphaEnd; alpha += 0.1){
            x1 = cos(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][0];
            z1 = sin(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][1];
            potential = 0.0;
            for(i=0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(z1-particles[i][1], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / minDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * 200.0f;
            if(fabs(startPotential - potential) < fabs(startPotential - newPotential)){
              newPotential = potential;
              path[pathNumber][pathEnd[pathNumber]+1][0] = x1;
              path[pathNumber][pathEnd[pathNumber]+1][1] = z1;
              path[pathNumber][pathEnd[pathNumber]+1][2] = 0.0;
              //printf("newPotential = %lf\n", newPotential);
            }    
          }
          //path[pathNumber][0][1] = path[pathNumber][1][1];//Approximation for initial potential
        }//for(pathNumber = 1, pathNumber < 3; pathNumber++){


        if(pathEnd[1] < 460){
          for(pathNumber = 0; pathNumber < 7; pathNumber++){
            pathEnd[pathNumber]++;
          }
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);

          savePics = 0;
          printf("...un ferddisch!\n");
          done = True;         // Exit programm !!!
        }


      }
      

      if(animation == 2004){//Draw equipotential lines
        GLfloat pathStep = 0.2;
        GLfloat startPotential, newPotential;
        GLfloat alphaStart, alphaEnd;
        GLfloat beta;
        int pathNumber;
        
        for(pathNumber = 0; pathNumber < 10; pathNumber++){
          potential = 0.0;
          for(i=0; i < 2; i++){
            totalDistance = sqrt(pow(path[pathNumber][0][0]-particles[i][0], 2.0) + pow(path[pathNumber][0][1]-particles[i][1], 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
              potential = k1 * particles[i][4] * 0.000001f / totalDistance;
              break;
            }
            potential += k1 * particles[i][4] * 0.000001f / totalDistance;
          }//for(i=0; i < potentialEnd; i++){
          startPotential = potential * 200.0f;
          
          if(pathEnd[pathNumber] < 1){
            alphaStart = 0.0;
            alphaEnd = 3.1415927;
          }
          else{
            beta = atan2(path[pathNumber][pathEnd[pathNumber]][1] - path[pathNumber][pathEnd[pathNumber]-1][1], path[pathNumber][pathEnd[pathNumber]][0] - path[pathNumber][pathEnd[pathNumber]-1][0]);
            alphaStart = beta - 3.1415927 / 2.0;
            alphaEnd = beta + 3.1415927 / 2.0;
          }
          newPotential = 1000000;
          for(alpha = alphaStart; alpha < alphaEnd; alpha += 0.1){
            x1 = cos(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][0];
            z1 = sin(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][1];
            potential = 0.0;
            for(i=0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(z1-particles[i][1], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / minDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * 200.0f;
            if(fabs(startPotential - potential) < fabs(startPotential - newPotential)){
              newPotential = potential;
              path[pathNumber][pathEnd[pathNumber]+1][0] = x1;
              path[pathNumber][pathEnd[pathNumber]+1][1] = z1;
              path[pathNumber][pathEnd[pathNumber]+1][2] = 0.0;
              //printf("newPotential = %lf\n", newPotential);
            }    
          }
          //path[pathNumber][0][1] = path[pathNumber][1][1];//Approximation for initial potential
        }//for(pathNumber = 1, pathNumber < 3; pathNumber++){


        if(pathEnd[1] < 460){
          for(pathNumber = 0; pathNumber < 10; pathNumber++){
            pathEnd[pathNumber]++;
          }
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);


          particleNumber = 1;
          particles[0][0] = 0.0f;
          particles[1][4] = 0.0f;
          potentialEnd = 1;
          
          for(i = 0; i < 10; i++){
            pathEnd[i] = 0;
          }
          for(x1 = 0.1; x1 < 15.0; x1 += 0.0001){//Find staring points of equipotential lines
            potential = 0.0;
            for(i = 0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / totalDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * scalePlaneHeigth;
            //printf("potential=%lf\n", potential);
            if(potential < 3.00 && path[0][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[0][0][0] = x1;
              path[0][0][1] = 0.0;
              path[0][0][2] = 0.0;
            }
            if(potential < 2.50 && path[1][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[1][0][0] = x1;
              path[1][0][1] = 0.0;
              path[1][0][2] = 0.0;
            }
            if(potential < 2.00 && path[2][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[2][0][0] = x1;
              path[2][0][1] = 0.0;
              path[2][0][2] = 0.0;
            }
            if(potential < 1.50 && path[3][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[3][0][0] = x1;
              path[3][0][1] = 0.0;
              path[3][0][2] = 0.0;
            }
            if(potential < 1.00 && path[4][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[4][0][0] = x1;
              path[4][0][1] = 0.0;
              path[4][0][2] = 0.0;
            }
            if(potential < 0.50 && path[5][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[5][0][0] = x1;
              path[5][0][1] = 0.0;
              path[5][0][2] = 0.0;
            }
            if(potential < 0.25 && path[6][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[6][0][0] = x1;
              path[6][0][1] = 0.0;
              path[6][0][2] = 0.0;
            }
          }

        }
      }      


      
      
      
      if(animation == 2003){
        if(particles[0][0] < -0.35f){
          particles[0][0] += 0.01f;
          particles[1][0] -= 0.01f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          particles[0][0] = -0.35f;
          particles[1][0] = 0.35f;
          pauseFrames = 0;
          

          for(i=0; i<10;i++){
            path[i][0][1] = -1000.0f;
            pathEnd[i] = 0;
            pathColor[i] = 3;
          }
          for(x1 = -2.0; x1 > -15.0; x1 -= 0.0001){//Find staring points of equipotential lines
            potential = 0.0;
            for(i = 0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / totalDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * scalePlaneHeigth;
            //printf("potential=%lf\n", potential);
            if(potential < 3.00 && path[0][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[0][0][0] = x1;
              path[0][0][1] = 0.0;
              path[0][0][2] = 0.0;
            }
            if(potential < 2.50 && path[1][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[1][0][0] = x1;
              path[1][0][1] = 0.0;
              path[1][0][2] = 0.0;
            }
            if(potential < 2.00 && path[2][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[2][0][0] = x1;
              path[2][0][1] = 0.0;
              path[2][0][2] = 0.0;
            }
            if(potential < 1.50 && path[3][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[3][0][0] = x1;
              path[3][0][1] = 0.0;
              path[3][0][2] = 0.0;
            }
            if(potential < 1.00 && path[4][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[4][0][0] = x1;
              path[4][0][1] = 0.0;
              path[4][0][2] = 0.0;
            }
            if(potential < 0.50 && path[5][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[5][0][0] = x1;
              path[5][0][1] = 0.0;
              path[5][0][2] = 0.0;
            }
            if(potential < 0.25 && path[6][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[6][0][0] = x1;
              path[6][0][1] = 0.0;
              path[6][0][2] = 0.0;
            }
          }
          
          for(x1 = 2.0; x1 < 4.0; x1 += 0.0001){//Find staring points of equipotential lines
            potential = 0.0;
            for(i = 0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / totalDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * scalePlaneHeigth;
            //printf("potential=%lf\n", potential);
            if(potential < 3.00 && path[7][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[7][0][0] = x1;
              path[7][0][1] = 0.0;
              path[7][0][2] = 0.0;
            }
            if(potential < 2.50 && path[8][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[8][0][0] = x1;
              path[8][0][1] = 0.0;
              path[8][0][2] = 0.0;
            }
            if(potential < 2.00 && path[9][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[9][0][0] = x1;
              path[9][0][1] = 0.0;
              path[9][0][2] = 0.0;
            }
          }
          
        }
      }


      if(animation == 2002){
        if(translateZ < -20.0f){
          translateZ += 0.1f;
          //printf("translateZ = %lf\n", translateZ);
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          for(i = 0; i < 10; i++){
            pathEnd[i] = 0;
          }
          pauseFrames = 0;
        }
      }

      if(animation == 2001){//Draw equipotential lines
        GLfloat pathStep = 0.2;
        GLfloat startPotential, newPotential;
        GLfloat alphaStart, alphaEnd;
        GLfloat beta;
        int pathNumber;
        
        for(pathNumber = 0; pathNumber < 10; pathNumber++){
          potential = 0.0;
          for(i=0; i < 2; i++){
            totalDistance = sqrt(pow(path[pathNumber][0][0]-particles[i][0], 2.0) + pow(path[pathNumber][0][1]-particles[i][1], 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
              potential = k1 * particles[i][4] * 0.000001f / totalDistance;
              break;
            }
            potential += k1 * particles[i][4] * 0.000001f / totalDistance;
          }//for(i=0; i < potentialEnd; i++){
          startPotential = potential * 200.0f;
          
          if(pathEnd[pathNumber] < 1){
            alphaStart = 0.0;
            alphaEnd = 3.1415927;
          }
          else{
            beta = atan2(path[pathNumber][pathEnd[pathNumber]][1] - path[pathNumber][pathEnd[pathNumber]-1][1], path[pathNumber][pathEnd[pathNumber]][0] - path[pathNumber][pathEnd[pathNumber]-1][0]);
            alphaStart = beta - 3.1415927 / 2.0;
            alphaEnd = beta + 3.1415927 / 2.0;
          }
          newPotential = 1000000;
          for(alpha = alphaStart; alpha < alphaEnd; alpha += 0.1){
            x1 = cos(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][0];
            z1 = sin(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][1];
            potential = 0.0;
            for(i=0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(z1-particles[i][1], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / minDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * 200.0f;
            if(fabs(startPotential - potential) < fabs(startPotential - newPotential)){
              newPotential = potential;
              path[pathNumber][pathEnd[pathNumber]+1][0] = x1;
              path[pathNumber][pathEnd[pathNumber]+1][1] = z1;
              path[pathNumber][pathEnd[pathNumber]+1][2] = 0.0;
              //printf("newPotential = %lf\n", newPotential);
            }    
          }
          //path[pathNumber][0][1] = path[pathNumber][1][1];//Approximation for initial potential
        }//for(pathNumber = 1, pathNumber < 3; pathNumber++){


        if(translateZ > -46.0f){
          translateZ -= 0.1f;
        }
        if(pathEnd[1] < 460){
          for(pathNumber = 0; pathNumber < 10; pathNumber++){
            pathEnd[pathNumber]++;
          }
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
        }
      }      


      if(animation == 2000){//Paint field lines
        
        if(fieldLineEndUp < 70){
          fieldLineEndUp++;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          fieldLineEndUp = 260;
          pauseFrames = 0;

          potentialEnd = 2;
          for(i=0; i<10;i++){
            path[i][0][1] = -1000.0f;
            pathEnd[i] = 0;
            pathColor[i] = 3;
          }
          for(x1 = -2.0; x1 > -15.0; x1 -= 0.0001){//Find staring points of equipotential lines
            potential = 0.0;
            for(i = 0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / totalDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * scalePlaneHeigth;
            //printf("potential=%lf\n", potential);
            if(potential < 3.00 && path[0][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[0][0][0] = x1;
              path[0][0][1] = 0.0;
              path[0][0][2] = 0.0;
            }
            if(potential < 2.50 && path[1][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[1][0][0] = x1;
              path[1][0][1] = 0.0;
              path[1][0][2] = 0.0;
            }
            if(potential < 2.00 && path[2][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[2][0][0] = x1;
              path[2][0][1] = 0.0;
              path[2][0][2] = 0.0;
            }
            if(potential < 1.50 && path[3][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[3][0][0] = x1;
              path[3][0][1] = 0.0;
              path[3][0][2] = 0.0;
            }
            if(potential < 1.00 && path[4][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[4][0][0] = x1;
              path[4][0][1] = 0.0;
              path[4][0][2] = 0.0;
            }
            if(potential < 0.50 && path[5][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[5][0][0] = x1;
              path[5][0][1] = 0.0;
              path[5][0][2] = 0.0;
            }
            if(potential < 0.25 && path[6][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[6][0][0] = x1;
              path[6][0][1] = 0.0;
              path[6][0][2] = 0.0;
            }
          }
          
          for(x1 = 2.0; x1 < 4.0; x1 += 0.0001){//Find staring points of equipotential lines
            potential = 0.0;
            for(i = 0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / totalDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * scalePlaneHeigth;
            //printf("potential=%lf\n", potential);
            if(potential < 3.00 && path[7][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[7][0][0] = x1;
              path[7][0][1] = 0.0;
              path[7][0][2] = 0.0;
            }
            if(potential < 2.50 && path[8][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[8][0][0] = x1;
              path[8][0][1] = 0.0;
              path[8][0][2] = 0.0;
            }
            if(potential < 2.00 && path[9][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[9][0][0] = x1;
              path[9][0][1] = 0.0;
              path[9][0][2] = 0.0;
            }
          }

        }
      }//if(animation == 2000){
      
      
    }//if(pauseFrames > 10){









//####################################################################################
//############# Field lines of three positive charges 3D Model #######################
//####################################################################################

    if(animation >= 1900 && animation < 2000){
      GLfloat fieldLineCoords[10000][4];
      GLfloat fieldLineLength = 40.0;
      double alpha, beta;
      int fieldLineNumber = 9 * 9;
      int i, i2, i3;
      
      arrowRadius = 2.5f;
      fieldLineStep = 0.05f;
      
      glEnable(GL_LIGHTING);
      glPolygonMode(GL_BACK, GL_FILL);
      glPolygonMode(GL_FRONT, GL_FILL);


      glLightfv(GL_LIGHT1, GL_AMBIENT, lightGreen1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightGreen1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightGreen1);


      if(animation < 1902){
        beta = 0.0f;
      }
      if(animation == 1902){
        beta = PI / 180.0f * 2.0f;
      }
      if(animation == 1903){
        beta = -PI / 180.0f * 2.0f;
      }

      for(i2 = 0; i2 < 9; i2++){
        for(i = 0; i < 9; i++){
          alpha = (GLfloat)(i) * 2.0 * PI / 9.0f + 2.0f * PI / 18;// + 0.0560
          fieldLineCoords[i + i2 * 9][0] = particles[i2][0] + cos(alpha) * particles[i2][3] * cos(beta);
          fieldLineCoords[i + i2 * 9][1] = particles[i2][1] + sin(alpha) * particles[i2][3] * cos(beta);
          fieldLineCoords[i + i2 * 9][2] = sin(beta);
          fieldLineCoords[i + i2 * 9][3] = 0.0f;
        }
      }


      int iEnd = fieldLineEndUp;
      
      for(i2 = 0; i2 < fieldLineNumber; i2++){
        iEnd = fieldLineEndUp;
        if(fieldLineCoords[i2][3] == -1.0f){
          iEnd = 200;
        }
        if(fieldLineCoords[i2][3] == -2.0f){
          iEnd = 0;
        }
        for(i = 0; i < iEnd; i++){
          forceX = 0.0f;
          forceY = 0.0f;
          forceZ = 0.0f;
          
          for(i3 = 0; i3 < 9; i3++){
            totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[i3][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[i3][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[i3][2], 2.0));
            totalForce = k * particles[0][4] / pow(totalDistance, 2.0);
            forceX += (fieldLineCoords[i2][0] - particles[i3][0]) / totalDistance * totalForce;
            forceY += (fieldLineCoords[i2][1] - particles[i3][1]) / totalDistance * totalForce;
            forceZ += (fieldLineCoords[i2][2] - particles[i3][2]) / totalDistance * totalForce;
          }


          totalForce = sqrt(pow(forceX, 2.0) + pow(forceY, 2.0) + pow(forceZ, 2.0));
          x2 = fieldLineCoords[i2][0] + forceX / totalForce * fieldLineStep;
          y2 = fieldLineCoords[i2][1] + forceY / totalForce * fieldLineStep;
          z2 = fieldLineCoords[i2][2] + forceZ / totalForce * fieldLineStep;
          
          

          drawCylinder(fieldLineCoords[i2][0], fieldLineCoords[i2][1], fieldLineCoords[i2][2], x2, y2, z2, 0.02, 0.02, 32);
          
          if(fieldLineCoords[i2][3] == 0.0){
            //One third of field lines at each particle
            for(i3 = 0; i3 < 9; i3++){
              if(i2 >= i3 * fieldLineNumber / 9 && i2 < (i3 + 1) * fieldLineNumber / 9){
                totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[i3][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[i3][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[i3][2], 2.0));
              }
            }
            if(totalDistance > arrowRadius){
              drawCylinder(x1, y1, z1, x1 + 2.0f * (x1 - x2), y1 + 2.0f * (y1 - y2), z1 + 2.0f * (z1 - z2), 0.02, 0.1, 32);
              fieldLineCoords[i2][3] = 1.0;
            }
          }

          
          //Stop field line if center of arrangement is reached
          y1 = -2.598076211 + 3.0f * tan(3.1415927 / 180.0 * 30.0);
//          if(!(fabs(x2) < 0.05f && fabs(y2-y1) < 0.05f && fabs(z2-z1) < 0.05f)){
            x1 = fieldLineCoords[i2][0];
            y1 = fieldLineCoords[i2][1];
            z1 = fieldLineCoords[i2][2];

            fieldLineCoords[i2][0] = x2;
            fieldLineCoords[i2][1] = y2;
            fieldLineCoords[i2][2] = z2;
//          }
          
          
          
          totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
        }//for(i = 0; i < fieldLineEndUp; i++){
      }//for(i2 = 0; i2 < 2; i2++){
      
    }



    if(pauseFrames > 10 && animationPause == 0){


      
      if(animation == 1914){
        GLfloat newRadiusFactor;
        
        if(particles[0][0] > 0.3f){
          newRadiusFactor = 0.1f / particles[0][0];
          for(i = 0; i < particleNumber; i++){
            particles[i][0] -= particles[i][0] * newRadiusFactor;
            particles[i][1] -= particles[i][1] * newRadiusFactor;
            particles[i][2] -= particles[i][2] * newRadiusFactor;
          }
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);          
          pauseFrames = 0;

          savePics = 0;
          printf("...un ferddisch!\n");
          done = True;         // Exit programm !!!
        } 
      }


      if(animation == 1913){
        if(particles[101][3] < 0.25f){
          for(i = 0; i < particleNumber; i++){
            if(particles[i][2] != 0.0f){
              particles[i][3] += 0.005f;
            }
          }
          particleNumber++;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);          
          pauseFrames = 0;

          for(i = 0; i < particleNumber; i++){
            particles[i][3] = 0.25f;
          }
        } 
      }


      if(animation == 1912){
        if(particles[101][3] > 0.001f){
          for(i = 0; i < particleNumber; i++){
            if(particles[i][2] != 0.0f){
              particles[i][3] -= 0.005f;
            }
          }
          particleNumber++;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);          
          pauseFrames = 0;

          for(i = 0; i < particleNumber; i++){
            if(particles[i][2] != 0.0f){
              particles[i][3] = 0.0f;
            }
          }
        } 
      }

      if(animation == 1911){
        if(particleNumber < 433){
          particleNumber++;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);          
          pauseFrames = 0;
        } 
      }

      if(animation == 1910){
        GLfloat angleStep, angleStepBeta;
        GLfloat radius;
        GLfloat beta;
        int number = 0;
        int particlesPerLayer;
        GLfloat particleRadius = 0.25f;

        i2 = 0;
        radius = 3.0f;
        particlesPerLayer = 2.0 * PI * radius / (particleRadius * 2.0f);
        angleStep = 2.0f * PI / (GLfloat)(particlesPerLayer);
        angleStepBeta = angleStep;
        
        for(i = 0; i < particlesPerLayer; i++){
          alpha = (GLfloat)(i) * angleStep;
          particles[number][0] = cos(alpha) * 3.0f;
          particles[number][1] = sin(alpha) * 3.0f;
          particles[number][2] = 0.0f;
          particles[number][3] = 0.25f;
          particles[number][4] = 0.0000003f;
          number++;
        }
      
        for(i2 = 1; i2 < 9; i2++){
          beta = -angleStepBeta * i2;
          radius = cos(beta) * 3.0f;          
          particlesPerLayer = 2.0 * PI * radius / (particleRadius * 2.0f);
          angleStep = 2.0f * PI / (GLfloat)(particlesPerLayer);
          for(i=0; i<particlesPerLayer; i++){
            alpha = (GLfloat)(i) * angleStep;
            particles[number][0] = cos(alpha) * 3.0f * cos(beta);
            particles[number][1] = sin(alpha) * 3.0f * cos(beta);
            particles[number][2] = sin(beta) * 3.0f;
            particles[number][3] = 0.25f;
            particles[number][4] = 0.0000003f;
            number++;
          }
        }

        particles[number][0] = 0.0f;
        particles[number][1] = 0.0f;
        particles[number][2] = -3.0f;
        particles[number][3] = 0.25f;
        particles[number][4] = 0.0000003f;
        number++;


        for(i2 = 1; i2 < 9; i2++){
          beta = angleStepBeta * i2;
          radius = cos(beta) * 3.0f;          
          particlesPerLayer = 2.0 * PI * radius / (particleRadius * 2.0f);
          angleStep = 2.0f * PI / (GLfloat)(particlesPerLayer);
          for(i=0; i<particlesPerLayer; i++){
            alpha = (GLfloat)(i) * angleStep;
            particles[number][0] = cos(alpha) * 3.0f * cos(beta);
            particles[number][1] = sin(alpha) * 3.0f * cos(beta);
            particles[number][2] = sin(beta) * 3.0f;
            particles[number][3] = 0.25f;
            particles[number][4] = 0.0000003f;
            number++;
          }
        }

        particles[number][0] = 0.0f;
        particles[number][1] = 0.0f;
        particles[number][2] = 3.0f;
        particles[number][3] = 0.25f;
        particles[number][4] = 0.0000003f;
        number++;
        
        
        for(i = 0; i < number; i++){
          particles[i][4] = 0.0000012f / number * 29.0f;
        }
        
        animation++;
        printf("Animation set to %d\n", animation);
        particleNumber = 0;
        printf("number = %d\n", number);

      }

      if(animation == 1909){
        if(rotX > 0.0f){
          rotX -= 0.1f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);          
          rotX = 0.0f;
          
          pauseFrames = 0;
        }
      
      }
      

      if(animation == 1908){
        if(rotX < 90.0f){
          rotX += 0.1f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);          
        }
      
      }
      
      if(animation == 1907){
        GLfloat beta;
        
        animation++;
        printf("Animation set to %d\n", animation);          
        pauseFrames = 0;
        

        i2 = 0;
        for(i=0; i<9; i++){
          alpha = (GLfloat)(i) * 2.0f * PI / 9.0f;
          particles[i2][0] = cos(alpha) * 3.0f;
          particles[i2][1] = sin(alpha) * 3.0f;
          particles[i2][2] = 0.0f;
          particles[i2][3] = 0.25f;
          particles[i2][4] = 0.0000012f;
          i2++;
        }
      
        beta = PI / 6.0f;
        for(i=0; i<6; i++){
          alpha = (GLfloat)(i) * 2.0f * PI / 6.0f;
          particles[i2][0] = cos(alpha) * 3.0f * cos(beta);
          particles[i2][1] = sin(alpha) * 3.0f * cos(beta);
          particles[i2][2] = sin(beta) * 3.0f;
          particles[i2][3] = 0.25f;
          particles[i2][4] = 0.0000012f;
          i2++;
        }

        beta = PI / 6.0f * 2.0f;
        for(i=0; i<3; i++){
          alpha = (GLfloat)(i) * 2.0f * PI / 3.0f;
          particles[i2][0] = cos(alpha) * 3.0f * cos(beta);
          particles[i2][1] = sin(alpha) * 3.0f * cos(beta);
          particles[i2][2] = sin(beta) * 3.0f;
          particles[i2][3] = 0.25f;
          particles[i2][4] = 0.0000012f;
          i2++;
        }

        particles[i2][0] = 0.0f;
        particles[i2][1] = 0.0f;
        particles[i2][2] = 3.0f;
        particles[i2][3] = 0.25f;
        particles[i2][4] = 0.0000012f;
        i2++;
        
      
        beta = -PI / 6.0f;
        for(i=0; i<6; i++){
          alpha = (GLfloat)(i) * 2.0f * PI / 6.0f;
          particles[i2][0] = cos(alpha) * 3.0f * cos(beta);
          particles[i2][1] = sin(alpha) * 3.0f * cos(beta);
          particles[i2][2] = sin(beta) * 3.0f;
          particles[i2][3] = 0.25f;
          particles[i2][4] = 0.0000012f;
          i2++;
        }

        beta = -PI / 6.0f * 2.0f;
        for(i=0; i<3; i++){
          alpha = (GLfloat)(i) * 2.0f * PI / 3.0f;
          particles[i2][0] = cos(alpha) * 3.0f * cos(beta);
          particles[i2][1] = sin(alpha) * 3.0f * cos(beta);
          particles[i2][2] = sin(beta) * 3.0f;
          particles[i2][3] = 0.25f;
          particles[i2][4] = 0.0000012f;
          i2++;
        }

        particles[i2][0] = 0.0f;
        particles[i2][1] = 0.0f;
        particles[i2][2] = -3.0f;
        particles[i2][3] = 0.25f;
        particles[i2][4] = 0.0000012f;
        i2++;
        

        particleNumber = i2;


      }

      if(animation == 1906){//Paint upper field lines        
        animation++;
        printf("Animation set to %d\n", animation);          
        
        enableFieldVectors = 1;
        pauseFrames = 0;
      }


      if(animation == 1905){//Paint upper field lines
        
        if(fieldLineEndUp > 0){
          fieldLineEndUp--;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);          
          
          fieldLineEndUp = 0;
          pauseFrames = 0;
        }
      }


      if(animation == 1904){
        if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
          animation = 1905;
        }
        if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
          animation = 1905;
        }
        if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
          animation = 1905;
        }
        if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
          animation = 1905;
        }
        if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
          animation = 1905;
        }
        if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
          animation = 1905;
        }
        if(animation == 1905){
          printf("Animation set to %d\n", animation);
                    
          translateXdiff = 0.0f;
          translateYdiff = 0.0f;
          translateZdiff = 0.0f;
          rotXdiff = 0.0f;
          rotYdiff = 0.0f;
          rotZdiff = 0.0f;
                    
        }
      }



      if(animation == 1903){
        
        if(fieldLineEndUp < 200){
          fieldLineEndUp++;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);          
          
          translateXtarget = 0.0f;
          translateYtarget = 0.0f;
          translateZtarget = -10.0f;
          
          rotXtarget = 0.0f;
          rotYtarget = 0.0f;
          rotZtarget = 0.0f;

          translateXdiff = (translateXtarget - translateX) / steps;
          translateYdiff = (translateYtarget - translateY) / steps;
          translateZdiff = (translateZtarget - translateZ) / steps;
          rotXdiff = (rotXtarget - rotX) / steps;
          rotYdiff = (rotYtarget - rotY) / steps;
          rotZdiff = (rotZtarget - rotZ) / steps;
        }
      }


      
      if(animation == 1902){//Paint upper field lines
        
        if(fieldLineEndUp < 200){
          fieldLineEndUp++;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);          
          
          fieldLineEndUp = 0;
        }
      }
      
      
      if(animation == 1901){
        if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
          animation = 1902;
        }
        if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
          animation = 1902;
        }
        if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
          animation = 1902;
        }
        if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
          animation = 1902;
        }
        if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
          animation = 1902;
        }
        if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
          animation = 1902;
        }
        if(animation == 1902){
          printf("Animation set to %d\n", animation);
          
          fieldLineEndUp = 0;
          
          translateXdiff = 0.0f;
          translateYdiff = 0.0f;
          translateZdiff = 0.0f;
          rotXdiff = 0.0f;
          rotYdiff = 0.0f;
          rotZdiff = 0.0f;
                    
        }
      }


      if(animation == 1900){//Paint field lines
        
        if(fieldLineEndUp < 200){
          fieldLineEndUp++;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          
          translateXtarget = 0.0f;
          translateYtarget = 0.0f;
          translateZtarget = -10.0f;
          
          rotXtarget = 90.0f;
          rotYtarget = 0.0f;
          rotZtarget = -43.0f;

          translateXdiff = (translateXtarget - translateX) / steps;
          translateYdiff = (translateYtarget - translateY) / steps;
          translateZdiff = (translateZtarget - translateZ) / steps;
          rotXdiff = (rotXtarget - rotX) / steps;
          rotYdiff = (rotYtarget - rotY) / steps;
          rotZdiff = (rotZtarget - rotZ) / steps;

        }
      }


      
      
    }//if(pauseFrames > 10){









//####################################################################################
//############# Potential plane of 9 positive charges ################################
//####################################################################################



    if(pauseFrames > 10 && animationPause == 0){
      
      
      
      
      if(animation == 1807){
        if(moveAlpha < 8.0f * PI){
          particles[9][0] = cos(moveAlpha) * 2.0f;
          particles[9][2] = sin(moveAlpha) * 1.0f;
          moveAlpha += 0.05f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          savePics = 0;
          printf("...un ferddisch!\n");
          done = True;         // Exit programm !!!
        }
      }

      
      if(animation == 1806){

        particles[9][0] = cos(moveAlpha) * 2.0f;
        particles[9][2] = sin(moveAlpha) * 1.0f;
        moveAlpha += 0.05f;

        
        if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
          animation = 1807;
        }
        if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
          animation = 1807;
        }
        if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
          animation = 1807;
        }
        if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
          animation = 1807;
        }
        if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
          animation = 1807;
        }
        if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
          animation = 1807;
        }
        if(animation == 1807){
          printf("Animation set to %d\n", animation);
          
          translateXdiff = 0.0f;
          translateYdiff = 0.0f;
          translateZdiff = 0.0f;
          rotXdiff = 0.0f;
          rotYdiff = 0.0f;
          rotZdiff = 0.0f;
                    
        }
      }
      
      
      
      
      
      
      
      if(animation == 1805){
        if(moveAlpha < 2.0f * PI){
          particles[9][0] = cos(moveAlpha) * 2.0f;
          particles[9][2] = sin(moveAlpha) * 1.0f;
          moveAlpha += 0.05f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);        
        
          translateXtarget = 0.0f;
          translateYtarget = 0.0f;
          translateZtarget = -10.0f;
          
          rotXtarget = -90.0f;
          rotYtarget = 0.0f;
          rotZtarget = 0.0f;
          
          translateXdiff = (translateXtarget - translateX) / steps;
          translateYdiff = (translateYtarget - translateY) / steps;
          translateZdiff = (translateZtarget - translateZ) / steps;
          rotXdiff = (rotXtarget - rotX) / steps;
          rotYdiff = (rotYtarget - rotY) / steps;
          rotZdiff = (rotZtarget - rotZ) / steps;


        }
      }

      if(animation == 1804){
        if(particles[9][0] < 2.0f){
          particles[9][0] += 0.03f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          moveAlpha = 0.0f;
        }
      }

      if(animation == 1803){
        if(particles[9][0] < 0.0f){
          particles[9][0] += 0.01f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;
        }
      }
      
      
      if(animation == 1802){
        if(rotY > -278.75f){
          rotY -= 0.5f;
          particles[9][0] += 0.01f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          rotYdiff = 0.0f;          
        }
      }
      
      
      
      if(animation == 1801){//Draw equipotential lines
        GLfloat pathStep = 0.1;
        GLfloat startPotential, newPotential;
        GLfloat alphaStart, alphaEnd;
        GLfloat beta;
        int pathNumber;
        
        for(pathNumber = 0; pathNumber < 8; pathNumber++){
          potential = 0.0;
          for(i=0; i < 9; i++){
            //printf("path[1][0][0] = %lf, path[1][0][2] = %lf, particles[i][0] = %lf, particles[i][2] = %lf\n", path[1][0][0], path[1][0][2], particles[i][0], particles[i][2]);
            totalDistance = sqrt(pow(path[pathNumber][0][0]-particles[i][0], 2.0) + pow(path[pathNumber][0][2]-particles[i][2], 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
              potential = k1 * particles[i][4] * 0.000001f / totalDistance;
              break;
            }
            potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            //printf("totalDistance = %lf, potential = %lf\n", totalDistance, potential);
          }//for(i=0; i < potentialEnd; i++){
          startPotential = potential * 200.0f;
          
          if(pathEnd[pathNumber] < 1){
            alphaStart = 0.0;
            alphaEnd = 3.1415927;
          }
          else{
            beta = atan2(path[pathNumber][pathEnd[pathNumber]][2] - path[pathNumber][pathEnd[pathNumber]-1][2], path[pathNumber][pathEnd[pathNumber]][0] - path[pathNumber][pathEnd[pathNumber]-1][0]);
            //printf("%lf, %lf\n", path[1][pathEnd[1]][2] - path[1][pathEnd[1]-1][2], path[1][pathEnd[1]][0] - path[1][pathEnd[1]-1][0]);
            alphaStart = beta - 3.1415927 / 2.0;
            alphaEnd = beta + 3.1415927 / 2.0;
            //printf("startPotential = %lf, path[1][pathEnd[1]][0] = %lf, path[1][pathEnd[1]][2] = %lf, beta = %lf, alphaStart = %lf, alphaEnd = %lf\n", startPotential, path[1][pathEnd[1]][0], path[1][pathEnd[1]][2], beta, alphaStart, alphaEnd);
          }
          newPotential = 1000000;
          for(alpha = alphaStart; alpha < alphaEnd; alpha += 0.1){
            x1 = cos(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][0];
            z1 = sin(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][2];
            potential = 0.0;
            for(i=0; i < 9; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(z1-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / minDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * 200.0f;
            //printf("x1 = %lf, z1 = %lf, totalDistance = %lf, potential = %lf\n", x1, z1, totalDistance, potential);
            //printf("abs(startPotential - potential) < abs(startPotential - newPotential): %lf < %lf\n", fabs(startPotential - potential), fabs(startPotential - newPotential));
            if(fabs(startPotential - potential) < fabs(startPotential - newPotential)){
              newPotential = potential;
              path[pathNumber][pathEnd[pathNumber]+1][0] = x1;
              path[pathNumber][pathEnd[pathNumber]+1][1] = potential + 0.03f;
              path[pathNumber][pathEnd[pathNumber]+1][2] = z1;
              //printf("newPotential = %lf\n", newPotential);
            }    
          }
          //path[pathNumber][0][1] = path[pathNumber][1][1];//Approximation for initial potential
        }//for(pathNumber = 1, pathNumber < 3; pathNumber++){

        if(pathEnd[1] < 350){
          for(pathNumber = 0; pathNumber < 7; pathNumber++){
            pathEnd[pathNumber]++;
          }
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          particles[9][0] = -8.0f;
          particleNumber = 10;
          enablePotentialSphere = 1;
        }
      }
      
      
      if(animation == 1800){
        
        if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
          animation = 1801;
        }
        if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
          animation = 1801;
        }
        if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
          animation = 1801;
        }
        if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
          animation = 1801;
        }
        if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
          animation = 1801;
        }
        if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
          animation = 1801;
        }
        if(animation == 1801){
          printf("Animation set to %d\n", animation);
          
          translateXdiff = 0.0f;
          translateYdiff = 0.0f;
          translateZdiff = 0.0f;
          rotXdiff = 0.0f;
          rotYdiff = 0.0f;
          rotZdiff = 0.0f;
                    
          pauseFrames = 0;


          for(i=0; i<10; i++){
            path[i][0][1] = -1000.0f;
            pathEnd[i] = 0;
          }
          
          GLfloat potential_01, potential_02;

          x1 = 1.0f;

          potential = 0.0f;
          for(i=0; i < 9; i++){
            totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
            }
            potential += k1 * particles[i][4] * 0.000001f / totalDistance;
          }//for(i=0; i < potentialEnd; i++){
          potential_01 = potential * scalePlaneHeigth;

          path[0][0][0] = x1;
          path[0][0][1] = potential_01;
          path[0][0][2] = 0.0;

          x1 = 2.0f;

          potential = 0.0f;
          for(i=0; i < 9; i++){
            totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
            }
            potential += k1 * particles[i][4] * 0.000001f / totalDistance;
          }//for(i=0; i < potentialEnd; i++){
          potential_02 = potential * scalePlaneHeigth;

          path[1][0][0] = x1;
          path[1][0][1] = potential_02;
          path[1][0][2] = 0.0;


          for(x1 = 2.1; x1 < 8.0; x1 += 0.0001){//Find staring points of equipotential lines
            potential = 0.0f;
            for(i=0; i < 9; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * scalePlaneHeigth;
            //printf("potential=%lf\n", potential);
            if(potential < potential_01 && path[2][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[2][0][0] = x1;
              path[2][0][1] = potential;
              path[2][0][2] = 0.0;
            }
            if(potential < potential_02 && path[3][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[3][0][0] = x1;
              path[3][0][1] = potential;
              path[3][0][2] = 0.0;
            }
            if(potential < potential_01 - (potential_02 - potential_01) && path[4][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[4][0][0] = x1;
              path[4][0][1] = potential;
              path[4][0][2] = 0.0;
            }
            if(potential < potential_01 - 2.0f * (potential_02 - potential_01)  && path[5][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[5][0][0] = x1;
              path[5][0][1] = potential;
              path[5][0][2] = 0.0;
            }
            if(potential < potential_01 - 3.0f * (potential_02 - potential_01)  && path[6][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[6][0][0] = x1;
              path[6][0][1] = potential;
              path[6][0][2] = 0.0;
            }
          }
        }
      }
      
      
    }//(pauseFrames > 10){



    if(animation >= 1800 && animation < 1900){//Calculate height of potential sphere / path

      potential = 0.0;
      
      for(i=0; i < 9; i++){
        totalDistance = sqrt(pow( particles[9][0]-particles[i][0], 2.0) + pow( particles[9][2]-particles[i][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential += k1 * particles[i][4] * particles[potentialEnd][4] / totalDistance;
      }//for(i=0; i < potentialEnd; i++){
      potentialSphereY = potential * scalePlaneHeigth;
      //potentialSphereY = 1.0f;// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      
      potentialSphereX = particles[9][0];
      potentialSphereZ = particles[9][2];
      
      if(animation > 1801 && animation < 1804){
        path[7][pathEnd[7]][0] = potentialSphereX;
        path[7][pathEnd[7]][1] = potentialSphereY + 0.03f;
        path[7][pathEnd[7]][2] = potentialSphereZ;
        pathColor[7] = 2;
        pathEnd[7]++;
      }
      
      if(animation > 1803){
        path[8][pathEnd[8]][0] = potentialSphereX;
        path[8][pathEnd[8]][1] = potentialSphereY + 0.03f;
        path[8][pathEnd[8]][2] = potentialSphereZ;
        pathColor[8] = 1;
        pathEnd[8]++;
      }
      
      if(animation > 1804){
        path[9][pathEnd[9]][0] = potentialSphereX;
        path[9][pathEnd[9]][1] = 0.0f;
        path[9][pathEnd[9]][2] = potentialSphereZ;
        pathColor[9] = 1;
        pathEnd[9]++;
      }
      
    }

      

//####################################################################################
//############# Potential plane of three positive charges ############################
//####################################################################################



    if(pauseFrames > 10 && animationPause == 0){
      
      
      
      if(animation == 1702){
        if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
          animation = 1703;
        }
        if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
          animation = 1703;
        }
        if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
          animation = 1703;
        }
        if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
          animation = 1703;
        }
        if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
          animation = 1703;
        }
        if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
          animation = 1703;
        }
        if(animation == 1703){
          printf("Animation set to %d\n", animation);

          translateXdiff = 0.0f;
          translateYdiff = 0.0f;
          translateZdiff = 0.0f;
          rotXdiff = 0.0f;
          rotYdiff = 0.0f;
          rotZdiff = 0.0f;
                    
          savePics = 0;
          printf("...un ferddisch!\n");
          done = True;         // Exit programm !!!

        }
      }
      
      
      
      if(animation == 1701){//Draw equipotential lines
        GLfloat pathStep = 0.1;
        GLfloat startPotential, newPotential;
        GLfloat alphaStart, alphaEnd;
        GLfloat beta;
        int pathNumber;
        
        for(pathNumber = 0; pathNumber < 8; pathNumber++){
          potential = 0.0;
          for(i=0; i < 3; i++){
            //printf("path[1][0][0] = %lf, path[1][0][2] = %lf, particles[i][0] = %lf, particles[i][2] = %lf\n", path[1][0][0], path[1][0][2], particles[i][0], particles[i][2]);
            totalDistance = sqrt(pow(path[pathNumber][0][0]-particles[i][0], 2.0) + pow(path[pathNumber][0][2]-particles[i][2], 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
              potential = k1 * particles[i][4] * 0.000001f / totalDistance;
              break;
            }
            potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            //printf("totalDistance = %lf, potential = %lf\n", totalDistance, potential);
          }//for(i=0; i < potentialEnd; i++){
          startPotential = potential * 200.0f;
          
          if(pathEnd[pathNumber] < 1){
            alphaStart = 0.0;
            alphaEnd = 3.1415927;
          }
          else{
            beta = atan2(path[pathNumber][pathEnd[pathNumber]][2] - path[pathNumber][pathEnd[pathNumber]-1][2], path[pathNumber][pathEnd[pathNumber]][0] - path[pathNumber][pathEnd[pathNumber]-1][0]);
            //printf("%lf, %lf\n", path[1][pathEnd[1]][2] - path[1][pathEnd[1]-1][2], path[1][pathEnd[1]][0] - path[1][pathEnd[1]-1][0]);
            alphaStart = beta - 3.1415927 / 2.0;
            alphaEnd = beta + 3.1415927 / 2.0;
            //printf("startPotential = %lf, path[1][pathEnd[1]][0] = %lf, path[1][pathEnd[1]][2] = %lf, beta = %lf, alphaStart = %lf, alphaEnd = %lf\n", startPotential, path[1][pathEnd[1]][0], path[1][pathEnd[1]][2], beta, alphaStart, alphaEnd);
          }
          newPotential = 1000000;
          for(alpha = alphaStart; alpha < alphaEnd; alpha += 0.1){
            x1 = cos(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][0];
            z1 = sin(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][2];
            potential = 0.0;
            for(i=0; i < 3; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(z1-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / minDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * 200.0f;
            //printf("x1 = %lf, z1 = %lf, totalDistance = %lf, potential = %lf\n", x1, z1, totalDistance, potential);
            //printf("abs(startPotential - potential) < abs(startPotential - newPotential): %lf < %lf\n", fabs(startPotential - potential), fabs(startPotential - newPotential));
            if(fabs(startPotential - potential) < fabs(startPotential - newPotential)){
              newPotential = potential;
              path[pathNumber][pathEnd[pathNumber]+1][0] = x1;
              path[pathNumber][pathEnd[pathNumber]+1][1] = potential + 0.03f;
              path[pathNumber][pathEnd[pathNumber]+1][2] = z1;
              //printf("newPotential = %lf\n", newPotential);
            }    
          }
          //path[pathNumber][0][1] = path[pathNumber][1][1];//Approximation for initial potential
        }//for(pathNumber = 1, pathNumber < 3; pathNumber++){

        if(pathEnd[1] < 350){
          for(pathNumber = 0; pathNumber < 7; pathNumber++){
            pathEnd[pathNumber]++;
          }
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          
          translateXtarget = 0.0f;
          translateYtarget = -1.11f;
          translateZtarget = -9.3f;
          
          rotXtarget = 2.75f;
          rotYtarget = 65.0f;
          rotZtarget = 0.0f;

          translateXdiff = (translateXtarget - translateX) / steps * 2.0;
          translateYdiff = (translateYtarget - translateY) / steps * 2.0;
          translateZdiff = (translateZtarget - translateZ) / steps * 2.0;
          rotXdiff = (rotXtarget - rotX) / steps * 2.0;
          rotYdiff = (rotYtarget - rotY) / steps * 2.0;
          rotZdiff = (rotZtarget - rotZ) / steps * 2.0;

        }
      }
      
      
      if(animation == 1700){
        
        if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
          animation = 1701;
        }
        if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
          animation = 1701;
        }
        if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
          animation = 1701;
        }
        if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
          animation = 1701;
        }
        if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
          animation = 1701;
        }
        if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
          animation = 1701;
        }
        if(animation == 1701){
          printf("Animation set to %d\n", animation);
          
          translateXdiff = 0.0f;
          translateYdiff = 0.0f;
          translateZdiff = 0.0f;
          rotXdiff = 0.0f;
          rotYdiff = 0.0f;
          rotZdiff = 0.0f;
                    
          pauseFrames = 0;


          for(i=0; i<10; i++){
            path[i][0][1] = -1000.0f;
            pathEnd[i] = 0;
          }
          
          GLfloat potential_01, potential_02;

          x1 = 0.5f;
          z1 = -0.5f;

          potential = 0.0f;
          for(i=0; i < 3; i++){
            totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(z1-particles[i][2], 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
            }
            potential += k1 * particles[i][4] * 0.000001f / totalDistance;
          }//for(i=0; i < potentialEnd; i++){
          potential_01 = potential * scalePlaneHeigth;

          path[0][0][0] = x1;
          path[0][0][1] = potential_01 + 0.03f;
          path[0][0][2] = z1;

          x1 = 4.0f;
          z1 = -0.5f;

          potential = 0.0f;
          for(i=0; i < 3; i++){
            totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(z1-particles[i][2], 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
            }
            potential += k1 * particles[i][4] * 0.000001f / totalDistance;
          }//for(i=0; i < potentialEnd; i++){
          potential_02 = potential * scalePlaneHeigth;

          path[1][0][0] = x1;
          path[1][0][1] = potential_02 + 0.03f;
          path[1][0][2] = z1;


          for(x1 = 0.6; x1 < 8.0; x1 += 0.0001){//Find staring points of equipotential lines
            potential = 0.0f;
            for(i=0; i < 3; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(z1-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * scalePlaneHeigth;
            //printf("potential=%lf\n", potential);
            if(potential < potential_01 && path[2][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[2][0][0] = x1;
              path[2][0][1] = potential + 0.03f;
              path[2][0][2] = z1;
            }
            if(potential < potential_02 && path[3][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[3][0][0] = x1;
              path[3][0][1] = potential + 0.03f;
              path[3][0][2] = z1;
            }
            if(potential < potential_01 - (potential_02 - potential_01) && path[4][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[4][0][0] = x1;
              path[4][0][1] = potential + 0.03f;
              path[4][0][2] = z1;
            }
            if(potential < potential_01 - 2.0f * (potential_02 - potential_01)  && path[5][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[5][0][0] = x1;
              path[5][0][1] = potential + 0.03f;
              path[5][0][2] = z1;
            }
            if(potential < potential_01 - 3.0f * (potential_02 - potential_01)  && path[6][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[6][0][0] = x1;
              path[6][0][1] = potential + 0.03f;
              path[6][0][2] = z1;
            }
          }
        }
      }
      
      
    }//(pauseFrames > 10){

      

//####################################################################################
//############# Field lines of three positive charges ################################
//####################################################################################

    if(animation >= 1600 && animation < 1700 && fieldLineEndUp > 0.0f){
      GLfloat fieldLineCoords[10000][4];
      GLfloat fieldLineLength = 40.0;
      double alpha, beta;
      int fieldLineNumber = 36 * 3;
      
      arrowRadius = 2.5f;
      fieldLineStep = 0.05f;
      
      glEnable(GL_LIGHTING);
      glPolygonMode(GL_BACK, GL_FILL);
      glPolygonMode(GL_FRONT, GL_FILL);


      glLightfv(GL_LIGHT1, GL_AMBIENT, lightGreen1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightGreen1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightGreen1);

      for(i = 0; i < fieldLineNumber / 3; i++){
        alpha = (GLfloat)(i) * 2.0 * PI / ((GLfloat)(fieldLineNumber) / 3.0f);// + 0.0560
        fieldLineCoords[i][0] = particles[0][0] + cos(alpha) * particles[0][3];
        fieldLineCoords[i][1] = particles[0][1] + sin(alpha) * particles[0][3];
        fieldLineCoords[i][2] = 0.0;
        fieldLineCoords[i][3] = 0.0;
      }

      for(i = 0; i < fieldLineNumber / 3; i++){
        alpha = (GLfloat)(i) * 2.0 * PI / ((GLfloat)(fieldLineNumber) / 3.0f);
        fieldLineCoords[i + fieldLineNumber / 3][0] = particles[1][0] + cos(alpha) * particles[1][3];
        fieldLineCoords[i + fieldLineNumber / 3][1] = particles[1][1] + sin(alpha) * particles[1][3];
        fieldLineCoords[i + fieldLineNumber / 3][2] = 0.0;
        fieldLineCoords[i + fieldLineNumber / 3][3] = 0.0;
      }

      for(i = 0; i < fieldLineNumber / 3; i++){
        alpha = (GLfloat)(i) * 2.0 * PI / ((GLfloat)(fieldLineNumber) / 3.0f);
        fieldLineCoords[i + 2 * fieldLineNumber / 3][0] = particles[2][0] + cos(alpha) * particles[2][3];
        fieldLineCoords[i + 2 * fieldLineNumber / 3][1] = particles[2][1] + sin(alpha) * particles[2][3];
        fieldLineCoords[i + 2 * fieldLineNumber / 3][2] = 0.0;
        fieldLineCoords[i + 2 * fieldLineNumber / 3][3] = 0.0;
      }

      
      alpha = 27.0f * 2.0f * PI / 36.0f + PI / 180.0f * 5.0f;
      fieldLineCoords[fieldLineNumber][0] = particles[2][0] + cos(alpha) * particles[2][3];
      fieldLineCoords[fieldLineNumber][1] = particles[2][1] + sin(alpha) * particles[2][3];
      fieldLineCoords[fieldLineNumber][2] = 0.0;
      fieldLineCoords[fieldLineNumber][3] = 0.0;
      fieldLineNumber++;


      for(i2 = 0; i2 < fieldLineNumber; i2++){
        for(i = 0; i < fieldLineEndUp; i++){
          totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[0][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[0][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[0][2], 2.0));
          totalForce = k * particles[0][4] / pow(totalDistance, 2.0);
          forceX = (fieldLineCoords[i2][0] - particles[0][0]) / totalDistance * totalForce;
          forceY = (fieldLineCoords[i2][1] - particles[0][1]) / totalDistance * totalForce;
          forceZ = (fieldLineCoords[i2][2] - particles[0][2]) / totalDistance * totalForce;

          totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
          totalForce = k * particles[1][4] / pow(totalDistance, 2.0);
          forceX += (fieldLineCoords[i2][0] - particles[1][0]) / totalDistance * totalForce;
          forceY += (fieldLineCoords[i2][1] - particles[1][1]) / totalDistance * totalForce;
          forceZ += (fieldLineCoords[i2][2] - particles[1][2]) / totalDistance * totalForce;

          totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[2][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[2][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[2][2], 2.0));
          totalForce = k * particles[2][4] / pow(totalDistance, 2.0);
          forceX += (fieldLineCoords[i2][0] - particles[2][0]) / totalDistance * totalForce;
          forceY += (fieldLineCoords[i2][1] - particles[2][1]) / totalDistance * totalForce;
          forceZ += (fieldLineCoords[i2][2] - particles[2][2]) / totalDistance * totalForce;

          totalForce = sqrt(pow(forceX, 2.0) + pow(forceY, 2.0) + pow(forceZ, 2.0));
          x2 = fieldLineCoords[i2][0] + forceX / totalForce * fieldLineStep;
          y2 = fieldLineCoords[i2][1] + forceY / totalForce * fieldLineStep;
          z2 = fieldLineCoords[i2][2] + forceZ / totalForce * fieldLineStep;
          
          

          if(i2 != fieldLineNumber - 1){
            drawCylinder(fieldLineCoords[i2][0], fieldLineCoords[i2][1], fieldLineCoords[i2][2], x2, y2, z2, 0.02, 0.02, 32);
            
            if(fieldLineCoords[i2][3] == 0.0){
              //Half of field lines at both particles
              if(i2 < fieldLineNumber / 3){
                totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[0][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[0][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[0][2], 2.0));
              }
              if(i2 >= fieldLineNumber / 3 && i2 < 2 * fieldLineNumber / 3){
                totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
              }
              if(i2 >= 2 * fieldLineNumber / 3){
                totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[2][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[2][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[2][2], 2.0));
              }
              if(totalDistance > arrowRadius){
                drawCylinder(x1, y1, z1, x1 + 2.0f * (x1 - x2), y1 + 2.0f * (y1 - y2), z1 + 2.0f * (z1 - z2), 0.02, 0.1, 32);
                fieldLineCoords[i2][3] = 1.0;
              }
            }
          }
          
          //Stop field line if center of arrangement is reached
          y1 = -2.598076211 + 3.0f * tan(3.1415927 / 180.0 * 30.0);
          if(!(fabs(x2) < 0.05f && fabs(y2-y1) < 0.05f && fabs(z2) < 0.05f)){
            x1 = fieldLineCoords[i2][0];
            y1 = fieldLineCoords[i2][1];
            z1 = fieldLineCoords[i2][2];

            fieldLineCoords[i2][0] = x2;
            fieldLineCoords[i2][1] = y2;
            fieldLineCoords[i2][2] = z2;
          }
          
          if(i2 == fieldLineNumber - 1){
            path[0][i][0] = x2;
            path[0][i][1] = y2;
            path[0][i][2] = 0.0;
          }
          
          
          totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
        }//for(i = 0; i < fieldLineEndUp; i++){
      }//for(i2 = 0; i2 < 2; i2++){
      
    }



    if(pauseFrames > 10 && animationPause == 0){
      
      
      
      if(animation == 1603){//Test charge moving around
        
        if(moveAlpha < 3.0f * PI){
          particles[3][0] = cos(moveAlpha) * 0.5f;
          particles[3][1] = sin(moveAlpha) - 0.5f;
          moveAlpha += 0.01f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);

          savePics = 0;
          printf("...un ferddisch!\n");
          done = True;         // Exit programm !!!
        }
      }

      if(animation == 1602){//Test charge moving around
        
        if(particles[3][1] < 0.5f){
          particles[3][1] += 0.01f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          
          moveAlpha = PI / 2.0f;
        }
      }


      if(animation == 1601){//Test charge on path to center
        if(pathEnd[0] < 70){
          pathEnd[0]++;
          particles[3][0] = path[0][pathEnd[0]][0];
          particles[3][1] = path[0][pathEnd[0]][1];
          particles[3][2] = path[0][pathEnd[0]][2];
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          enableFieldVectors = 10;
          enableLablesFieldVectors = 0;
        }
      }

      if(animation == 1600){//Paint field lines
        
        if(fieldLineEndUp < 200){
          fieldLineEndUp++;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          fieldLineEndUp = 200;
          
          particles[3][0] = path[0][pathEnd[0]][0];
          particles[3][1] = path[0][pathEnd[0]][1];
          particles[3][2] = path[0][pathEnd[0]][2];
          particles[3][3] = 0.15f;
          particles[3][4] = 0.000051f;
          particleNumber++;
          pathColor[0] = 3;

        }
      }


      
      
    }//if(pauseFrames > 10){








//####################################################################################
//############# Potential plane of two positive charges ##############################
//####################################################################################


    if(pauseFrames > 10 && animationPause == 0){


      if(animation == 1504){
        particleNumber = 3;
        enablePotentialSphere = 1;

        if(particles[2][2] < 4.25f){
          particles[2][2] += 0.01f;
          path[9][pathEnd[9]][0] = potentialSphereX;
          path[9][pathEnd[9]][1] = potentialSphereY;
          path[9][pathEnd[9]][2] = potentialSphereZ + 0.03f;
          pathEnd[9]++;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          
          pauseFrames = 0;

          savePics = 0;
          printf("...un ferddisch!\n");
          done = True;         // Exit programm !!!
        }
      }

      if(animation == 1503){
        animation++;
        printf("Animation set to %d\n", animation);
        particles[2][0] = 0.0f;
        particles[2][2] = 0.5f;
        path[9][pathEnd[9]][0] = potentialSphereX;
        path[9][pathEnd[9]][1] = potentialSphereY;
        path[9][pathEnd[9]][2] = potentialSphereZ + 0.03f;
        pathEnd[9]++;
        pathColor[9] = 1;
        pauseFrames = 0;          
      }

      if(animation == 1502){
        particleNumber = 3;
        enablePotentialSphere = 1;

        if(particles[2][0] < 0.0f){
          particles[2][0] += 0.01f;
          path[8][pathEnd[8]][0] = potentialSphereX;
          path[8][pathEnd[8]][1] = potentialSphereY;
          path[8][pathEnd[8]][2] = potentialSphereZ + 0.03f;
          pathEnd[8]++;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          
          pauseFrames = 0;          

        }
      }



      if(animation == 1501){//Draw equipotential lines
        GLfloat pathStep = 0.1;
        GLfloat startPotential, newPotential;
        GLfloat alphaStart, alphaEnd;
        GLfloat beta;
        int pathNumber;
        
        for(pathNumber = 0; pathNumber < 8; pathNumber++){
          potential = 0.0;
          for(i=0; i < 2; i++){
            //printf("path[1][0][0] = %lf, path[1][0][2] = %lf, particles[i][0] = %lf, particles[i][2] = %lf\n", path[1][0][0], path[1][0][2], particles[i][0], particles[i][2]);
            totalDistance = sqrt(pow(path[pathNumber][0][0]-particles[i][0], 2.0) + pow(path[pathNumber][0][2]-particles[i][2], 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
              potential = k1 * particles[i][4] * 0.000001f / totalDistance;
              break;
            }
            potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            //printf("totalDistance = %lf, potential = %lf\n", totalDistance, potential);
          }//for(i=0; i < potentialEnd; i++){
          startPotential = potential * 200.0f;
          
          if(pathEnd[pathNumber] < 1){
            alphaStart = 0.0;
            alphaEnd = 3.1415927;
          }
          else{
            beta = atan2(path[pathNumber][pathEnd[pathNumber]][2] - path[pathNumber][pathEnd[pathNumber]-1][2], path[pathNumber][pathEnd[pathNumber]][0] - path[pathNumber][pathEnd[pathNumber]-1][0]);
            //printf("%lf, %lf\n", path[1][pathEnd[1]][2] - path[1][pathEnd[1]-1][2], path[1][pathEnd[1]][0] - path[1][pathEnd[1]-1][0]);
            alphaStart = beta - 3.1415927 / 2.0;
            alphaEnd = beta + 3.1415927 / 2.0;
            //printf("startPotential = %lf, path[1][pathEnd[1]][0] = %lf, path[1][pathEnd[1]][2] = %lf, beta = %lf, alphaStart = %lf, alphaEnd = %lf\n", startPotential, path[1][pathEnd[1]][0], path[1][pathEnd[1]][2], beta, alphaStart, alphaEnd);
          }
          newPotential = 1000000;
          for(alpha = alphaStart; alpha < alphaEnd; alpha += 0.1){
            x1 = cos(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][0];
            z1 = sin(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][2];
            potential = 0.0;
            for(i=0; i < 2; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(z1-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / minDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * 200.0f;
            //printf("x1 = %lf, z1 = %lf, totalDistance = %lf, potential = %lf\n", x1, z1, totalDistance, potential);
            //printf("abs(startPotential - potential) < abs(startPotential - newPotential): %lf < %lf\n", fabs(startPotential - potential), fabs(startPotential - newPotential));
            if(fabs(startPotential - potential) < fabs(startPotential - newPotential)){
              newPotential = potential;
              path[pathNumber][pathEnd[pathNumber]+1][0] = x1;
              path[pathNumber][pathEnd[pathNumber]+1][1] = potential;
              path[pathNumber][pathEnd[pathNumber]+1][2] = z1;
              //printf("newPotential = %lf\n", newPotential);
            }    
          }
          //path[pathNumber][0][1] = path[pathNumber][1][1];//Approximation for initial potential
        }//for(pathNumber = 1, pathNumber < 3; pathNumber++){

        if(pathEnd[1] < 300){
          for(pathNumber = 0; pathNumber < 8; pathNumber++){
            pathEnd[pathNumber]++;
          }
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);

          path[8][pathEnd[8]][0] = potentialSphereX;
          path[8][pathEnd[8]][1] = potentialSphereY;
          path[8][pathEnd[8]][2] = potentialSphereZ + 0.01f;
          pathEnd[8]++;
          pathColor[8] = 2;


        }
      }
      



      if(animation == 1500){

        if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
          animation = 1501;
        }
        if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
          animation = 1501;
        }
        if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
          animation = 1501;
        }
        if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
          animation = 1501;
        }
        if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
          animation = 1501;
        }
        if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
          animation = 1501;
        }
        if(animation == 1501){
          printf("Animation set to %d\n", animation);
          
          translateXdiff = 0.0f;
          translateYdiff = 0.0f;
          translateZdiff = 0.0f;
          rotXdiff = 0.0f;
          rotYdiff = 0.0f;
          rotZdiff = 0.0f;
                    
          pauseFrames = 0;

          for(i=0; i<10; i++){
            path[i][0][1] = -1000.0f;
            pathEnd[i] = 0;
          }

          for(x1 = -2.0; x1 > -8.0; x1 -= 0.0001){//Find staring points of equipotential lines
            potential = 0.0f;
            for(i=0; i < 2; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / totalDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * scalePlaneHeigth;
            //printf("potential=%lf\n", potential);
            if(potential < 3.00 && path[0][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[0][0][0] = x1;
              path[0][0][1] = potential;
              path[0][0][2] = 0.0;
            }
            if(potential < 2.50 && path[1][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[1][0][0] = x1;
              path[1][0][1] = potential;
              path[1][0][2] = 0.0;
            }
            if(potential < 2.00 && path[2][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[2][0][0] = x1;
              path[2][0][1] = potential;
              path[2][0][2] = 0.0;
            }
            if(potential < 1.50 && path[3][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[3][0][0] = x1;
              path[3][0][1] = potential;
              path[3][0][2] = 0.0;
            }
            if(potential < 1.00 && path[4][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[4][0][0] = x1;
              path[4][0][1] = potential;
              path[4][0][2] = 0.0;
            }
          }

          for(x1 = 2.0; x1 < 8.0; x1 += 0.0001){//Find staring points of equipotential lines
            potential = 0.0f;
            for(i=0; i < 2; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / totalDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * scalePlaneHeigth;
            //printf("potential=%lf\n", potential);
            if(potential < 3.00 && path[5][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[5][0][0] = x1;
              path[5][0][1] = potential;
              path[5][0][2] = 0.0;
            }
            if(potential < 2.50 && path[6][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[6][0][0] = x1;
              path[6][0][1] = potential;
              path[6][0][2] = 0.0;
            }
            if(potential < 2.00 && path[7][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[7][0][0] = x1;
              path[7][0][1] = potential;
              path[7][0][2] = 0.0;
            }
          }


        }
      }
    
      if(animation >= 1500 && animation < 1600){//Calculate height of potential sphere / path

        potential = 0.0;
        
        for(i=0; i < 2; i++){
          totalDistance = sqrt(pow( particles[2][0]-particles[i][0], 2.0) + pow( particles[2][2]-particles[i][2], 2.0));
          if(totalDistance < minDistance){
            totalDistance = minDistance;
            potential = k1 * particles[i][4] * particles[potentialEnd][4] / minDistance;
            break;
          }
          potential += k1 * particles[i][4] * particles[potentialEnd][4] / totalDistance;
        }//for(i=0; i < potentialEnd; i++){
        potentialSphereY = potential * scalePlaneHeigth;
        //potentialSphereY = 1.0f;// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        
        potentialSphereX = particles[2][0];
        potentialSphereZ = particles[2][2];
        
      }
    
    
  }//if(pauseFrames > 10){

//####################################################################################
//############# Field lines of two positive charges ##################################
//####################################################################################

    if(animation >= 1400 && animation < 1500 && fieldLineEndUp > 0.0f){
      GLfloat fieldLineCoords[10000][4];
      GLfloat fieldLineLength = 40.0;
      double alpha, beta;
      int fieldLineNumber = 36;
      
      glEnable(GL_LIGHTING);
      glPolygonMode(GL_BACK, GL_FILL);
      glPolygonMode(GL_FRONT, GL_FILL);


      glLightfv(GL_LIGHT1, GL_AMBIENT, lightGreen1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightGreen1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightGreen1);

      if(animation < 1405){
        for(i = 1; i < 18; i++){
          alpha = (GLfloat)(i) * 2.0 * PI / 18.0f;
          fieldLineCoords[i][0] = particles[0][0] + cos(alpha) * particles[0][3];
          fieldLineCoords[i][1] = particles[0][1] + sin(alpha) * particles[0][3];
          fieldLineCoords[i][2] = 0.0;
          fieldLineCoords[i][3] = 0.0;
        }

        for(i = 1; i < 18; i++){
          alpha = (GLfloat)(i) * 2.0 * PI / 18.0f + PI;
          fieldLineCoords[i + 18][0] = particles[1][0] + cos(alpha) * particles[1][3];
          fieldLineCoords[i + 18][1] = particles[1][1] + sin(alpha) * particles[1][3];
          fieldLineCoords[i + 18][2] = 0.0;
          fieldLineCoords[i + 18][3] = 0.0;
        }
      }
      if((animation > 1404 && animation < 1408) || animation > 1409){
        fieldLineNumber = 54;
        for(i = 0; i < 27; i++){
          alpha = (GLfloat)(i) * 2.0 * PI / 27.0f;
          fieldLineCoords[i][0] = particles[0][0] + cos(alpha) * particles[0][3];
          fieldLineCoords[i][1] = particles[0][1] + sin(alpha) * particles[0][3];
          fieldLineCoords[i][2] = 0.0;
          fieldLineCoords[i][3] = 0.0;
        }

        for(i = 0; i < 27; i++){
          alpha = (GLfloat)(i) * 2.0 * PI / 27.0f + PI;
          fieldLineCoords[i + 27][0] = particles[1][0] + cos(alpha) * particles[1][3];
          fieldLineCoords[i + 27][1] = particles[1][1] + sin(alpha) * particles[1][3];
          fieldLineCoords[i + 27][2] = 0.0;
          fieldLineCoords[i + 27][3] = 0.0;
        }
      }

      if(animation > 1407 && animation < 1410){
        fieldLineNumber = 54;
        for(i = 0; i < 36; i++){
          alpha = (GLfloat)(i) * 2.0 * PI / 36.0f;
          fieldLineCoords[i][0] = particles[0][0] + cos(alpha) * particles[0][3];
          fieldLineCoords[i][1] = particles[0][1] + sin(alpha) * particles[0][3];
          fieldLineCoords[i][2] = 0.0;
          fieldLineCoords[i][3] = 0.0;
        }

        for(i = 0; i < 18; i++){
          alpha = (GLfloat)(i) * 2.0 * PI / 18.0f + PI;
          fieldLineCoords[i + 36][0] = particles[1][0] + cos(alpha) * particles[1][3];
          fieldLineCoords[i + 36][1] = particles[1][1] + sin(alpha) * particles[1][3];
          fieldLineCoords[i + 36][2] = 0.0;
          fieldLineCoords[i + 36][3] = 0.0;
        }
      }



      for(i2 = 0; i2 < fieldLineNumber; i2++){
        for(i = 0; i < fieldLineEndUp; i++){
          totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[0][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[0][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[0][2], 2.0));
          totalForce = k * particles[0][4] / pow(totalDistance, 2.0);
          forceX = (fieldLineCoords[i2][0] - particles[0][0]) / totalDistance * totalForce;
          forceY = (fieldLineCoords[i2][1] - particles[0][1]) / totalDistance * totalForce;
          forceZ = (fieldLineCoords[i2][2] - particles[0][2]) / totalDistance * totalForce;

          totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
          totalForce = k * particles[1][4] / pow(totalDistance, 2.0);
          forceX += (fieldLineCoords[i2][0] - particles[1][0]) / totalDistance * totalForce;
          forceY += (fieldLineCoords[i2][1] - particles[1][1]) / totalDistance * totalForce;
          forceZ += (fieldLineCoords[i2][2] - particles[1][2]) / totalDistance * totalForce;

          totalForce = sqrt(pow(forceX, 2.0) + pow(forceY, 2.0) + pow(forceZ, 2.0));
          x2 = fieldLineCoords[i2][0] + forceX / totalForce * fieldLineStep;
          y2 = fieldLineCoords[i2][1] + forceY / totalForce * fieldLineStep;
          z2 = fieldLineCoords[i2][2] + forceZ / totalForce * fieldLineStep;
          
          
          if(animation == 1405 || animation > 1409){//Field lines ending at 0/0
            if(fieldLineEndUp > 25){//Arrows of middle field lines
              drawCylinder(0.5f, 0.0f, 0.0f, 0.3f, 0.0f, 0.0f, 0.10, 0.03, 32);
              drawCylinder(-0.5f, 0.0f, 0.0f, -0.3f, 0.0f, 0.0f, 0.10, 0.03, 32);
            }
            if(i2 == 0 || i2 == 27){
              y2 = 0.0f;
              z2 = 0.0f;
            }
          }

          if(animation > 1407 && animation < 1410){//Field lines ending at 0/0
            if(fieldLineEndUp > 25){//Arrows of middle field lines
              drawCylinder(0.7f, 0.0f, 0.0f, 0.5f, 0.0f, 0.0f, 0.10, 0.03, 32);
              drawCylinder(0.0f, 0.0f, 0.0f, 0.2f, 0.0f, 0.0f, 0.10, 0.03, 32);
            }
            if(i2 == 0 || i2 == 36){
              y2 = 0.0f;
              z2 = 0.0f;
            }
          }

          if(animation > 1405 && animation < 1408){//Cancel some field lines
             fieldLineCoords[0][3] = -10.0;
             fieldLineCoords[1][3] = -10.0;
             fieldLineCoords[26][3] = -10.0;
             fieldLineCoords[27][3] = -10.0;
             fieldLineCoords[28][3] = -10.0;
             fieldLineCoords[53][3] = -10.0;
          }
          if(fieldLineCoords[i2][3] != -10.0){
            drawCylinder(fieldLineCoords[i2][0], fieldLineCoords[i2][1], fieldLineCoords[i2][2], x2, y2, z2, 0.03, 0.03, 32);
            
            if(fieldLineCoords[i2][3] == 0.0){
              if(animation > 1407 && animation < 1410){
                if(i2 < 36){//Double number of field lines at left charge
                  totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[0][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[0][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[0][2], 2.0));
                }
                else{
                  totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
                }
              }
              else{//Half of field lines at both particles
                if(i2 < fieldLineNumber / 2){
                  totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[0][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[0][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[0][2], 2.0));
                }
                else{
                  totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
                }
              }
              if(totalDistance > arrowRadius){
                drawCylinder(x1, y1, z1, x2, y2, z2, 0.1, 0.03, 32);
                fieldLineCoords[i2][3] = 1.0;
              }
            }
          }

          x1 = fieldLineCoords[i2][0];
          y1 = fieldLineCoords[i2][1];
          z1 = fieldLineCoords[i2][2];

          fieldLineCoords[i2][0] = x2;
          fieldLineCoords[i2][1] = y2;
          fieldLineCoords[i2][2] = z2;
          
          totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
        }//for(i = 0; i < fieldLineEndUp; i++){
      }//for(i2 = 0; i2 < 2; i2++){
      
      if(animation > 1402 && animation < 1405){
        drawCylinder(-1.75f, 0.0f, 0.0f, -0.1f, 0.0f, 0.0f, 0.03, 0.03, 32);
        drawCylinder(-0.1f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.03, 0.0001, 32);
        drawCylinder(-1.0f, 0.0f, 0.0f, -0.8f, 0.0f, 0.0f, 0.1, 0.03, 32);
      }
      if(animation > 1403 && animation < 1405){
        drawCylinder(1.75f, 0.0f, 0.0f, 0.1f, 0.0f, 0.0f, 0.03, 0.03, 32);
        drawCylinder(0.1f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.03, 0.0001, 32);
        drawCylinder(1.0f, 0.0f, 0.0f, 0.8f, 0.0f, 0.0f, 0.1, 0.03, 32);
      }
    }



    if(pauseFrames > 10 && animationPause == 0){
      
      
      
      
      if(animation == 1411){//Draw equipotential lines
        GLfloat pathStep = 0.1;
        GLfloat startPotential, newPotential;
        GLfloat alphaStart, alphaEnd;
        GLfloat beta;
        int pathNumber;
        
        for(pathNumber = 0; pathNumber < PATHMAX; pathNumber++){
          potential = 0.0;
          for(i=0; i < 2; i++){
            totalDistance = sqrt(pow(path[pathNumber][0][0]-particles[i][0], 2.0) + pow(path[pathNumber][0][1]-particles[i][1], 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
              potential = k1 * particles[i][4] * 0.000001f / totalDistance;
              break;
            }
            potential += k1 * particles[i][4] * 0.000001f / totalDistance;
          }//for(i=0; i < potentialEnd; i++){
          startPotential = potential * 200.0f;
          
          if(pathEnd[pathNumber] < 1){
            alphaStart = 0.0;
            alphaEnd = 3.1415927;
          }
          else{
            beta = atan2(path[pathNumber][pathEnd[pathNumber]][1] - path[pathNumber][pathEnd[pathNumber]-1][1], path[pathNumber][pathEnd[pathNumber]][0] - path[pathNumber][pathEnd[pathNumber]-1][0]);
            alphaStart = beta - 3.1415927 / 2.0;
            alphaEnd = beta + 3.1415927 / 2.0;
          }
          newPotential = 1000000;
          for(alpha = alphaStart; alpha < alphaEnd; alpha += 0.1){
            x1 = cos(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][0];
            z1 = sin(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][1];
            potential = 0.0;
            for(i=0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(z1-particles[i][1], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / minDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * 200.0f;
            if(fabs(startPotential - potential) < fabs(startPotential - newPotential)){
              newPotential = potential;
              path[pathNumber][pathEnd[pathNumber]+1][0] = x1;
              path[pathNumber][pathEnd[pathNumber]+1][1] = z1;
              path[pathNumber][pathEnd[pathNumber]+1][2] = 0.0;
              //printf("newPotential = %lf\n", newPotential);
            }    
          }
          //path[pathNumber][0][1] = path[pathNumber][1][1];//Approximation for initial potential
        }//for(pathNumber = 1, pathNumber < 3; pathNumber++){

        if(pathEnd[1] < 470){
          for(pathNumber = 0; pathNumber < PATHMAX; pathNumber++){
            pathEnd[pathNumber]++;
          }
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          
          savePics = 0;
          printf("...un ferddisch!\n");
          done = True;         // Exit programm !!!
        }
      }

      
      
      
      
      
      
      
      if(animation == 1410){
        if(fieldLineEndUp < 70){
          fieldLineEndUp++;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);


          for(i=0; i<10;i++){
            path[i][0][1] = -1000.0f;
            pathEnd[i] = 0;
            pathColor[i] = 3;
          }
          for(x1 = -2.0; x1 > -8.0; x1 -= 0.0001){//Find staring points of equipotential lines
            potential = 0.0;
            for(i = 0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / totalDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * scalePlaneHeigth;
            //printf("potential=%lf\n", potential);
            if(potential < 3.00 && path[0][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[0][0][0] = x1;
              path[0][0][1] = 0.0;
              path[0][0][2] = 0.0;
            }
            if(potential < 2.50 && path[1][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[1][0][0] = x1;
              path[1][0][1] = 0.0;
              path[1][0][2] = 0.0;
            }
            if(potential < 2.00 && path[2][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[2][0][0] = x1;
              path[2][0][1] = 0.0;
              path[2][0][2] = 0.0;
            }
            if(potential < 1.50 && path[3][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[3][0][0] = x1;
              path[3][0][1] = 0.0;
              path[3][0][2] = 0.0;
            }
            if(potential < 1.00 && path[4][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[4][0][0] = x1;
              path[4][0][1] = 0.0;
              path[4][0][2] = 0.0;
            }
            if(potential < 0.50 && path[5][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[5][0][0] = x1;
              path[5][0][1] = 0.0;
              path[5][0][2] = 0.0;
            }
          }
          
          for(x1 = 2.0; x1 < 4.0; x1 += 0.0001){//Find staring points of equipotential lines
            potential = 0.0;
            for(i = 0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / totalDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * scalePlaneHeigth;
            //printf("potential=%lf\n", potential);
            if(potential < 3.00 && path[6][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[6][0][0] = x1;
              path[6][0][1] = 0.0;
              path[6][0][2] = 0.0;
            }
            if(potential < 2.50 && path[7][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[7][0][0] = x1;
              path[7][0][1] = 0.0;
              path[7][0][2] = 0.0;
            }
            if(potential < 2.00 && path[8][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[8][0][0] = x1;
              path[8][0][1] = 0.0;
              path[8][0][2] = 0.0;
            }
          }

        }
      }


      if(animation == 1409){//Draw equipotential lines
        GLfloat pathStep = 0.1;
        GLfloat startPotential, newPotential;
        GLfloat alphaStart, alphaEnd;
        GLfloat beta;
        int pathNumber;
        
        for(pathNumber = 0; pathNumber < PATHMAX; pathNumber++){
          potential = 0.0;
          for(i=0; i < 2; i++){
            totalDistance = sqrt(pow(path[pathNumber][0][0]-particles[i][0], 2.0) + pow(path[pathNumber][0][1]-particles[i][1], 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
              potential = k1 * particles[i][4] * 0.000001f / totalDistance;
              break;
            }
            potential += k1 * particles[i][4] * 0.000001f / totalDistance;
          }//for(i=0; i < potentialEnd; i++){
          startPotential = potential * 200.0f;
          
          if(pathEnd[pathNumber] < 1){
            alphaStart = 0.0;
            alphaEnd = 3.1415927;
          }
          else{
            beta = atan2(path[pathNumber][pathEnd[pathNumber]][1] - path[pathNumber][pathEnd[pathNumber]-1][1], path[pathNumber][pathEnd[pathNumber]][0] - path[pathNumber][pathEnd[pathNumber]-1][0]);
            alphaStart = beta - 3.1415927 / 2.0;
            alphaEnd = beta + 3.1415927 / 2.0;
          }
          newPotential = 1000000;
          for(alpha = alphaStart; alpha < alphaEnd; alpha += 0.1){
            x1 = cos(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][0];
            z1 = sin(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][1];
            potential = 0.0;
            for(i=0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(z1-particles[i][1], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / minDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * 200.0f;
            if(fabs(startPotential - potential) < fabs(startPotential - newPotential)){
              newPotential = potential;
              path[pathNumber][pathEnd[pathNumber]+1][0] = x1;
              path[pathNumber][pathEnd[pathNumber]+1][1] = z1;
              path[pathNumber][pathEnd[pathNumber]+1][2] = 0.0;
              //printf("newPotential = %lf\n", newPotential);
            }    
          }
          //path[pathNumber][0][1] = path[pathNumber][1][1];//Approximation for initial potential
        }//for(pathNumber = 1, pathNumber < 3; pathNumber++){

        if(pathEnd[1] < 360){
          for(pathNumber = 0; pathNumber < PATHMAX; pathNumber++){
            pathEnd[pathNumber]++;
          }
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          particles[0][3] = 0.25f;
          particles[0][4] = 0.000001f;
          fieldLineEndUp = 0;
          for(i=0; i < PATHMAX; i++){
            pathEnd[i] = 0;
          }
        }
      }



      
      if(animation == 1408){//Field lines with left charge being twice as much as the right one
        if(fieldLineEndUp < 70){
          fieldLineEndUp++;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          fieldLineEndUp = 100;
          pauseFrames = 0;


          potentialEnd = 2;
          for(i=0; i<10;i++){
            path[i][0][1] = -1000.0f;
            pathEnd[i] = 0;
            pathColor[i] = 3;
          }
          for(x1 = -2.0; x1 > -8.0; x1 -= 0.0001){//Find staring points of equipotential lines
            potential = 0.0;
            for(i = 0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / totalDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * scalePlaneHeigth;
            //printf("potential=%lf\n", potential);
            if(potential < 3.00 && path[0][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[0][0][0] = x1;
              path[0][0][1] = 0.0;
              path[0][0][2] = 0.0;
            }
            if(potential < 2.50 && path[1][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[1][0][0] = x1;
              path[1][0][1] = 0.0;
              path[1][0][2] = 0.0;
            }
            if(potential < 2.00 && path[2][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[2][0][0] = x1;
              path[2][0][1] = 0.0;
              path[2][0][2] = 0.0;
            }
            if(potential < 1.50 && path[3][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[3][0][0] = x1;
              path[3][0][1] = 0.0;
              path[3][0][2] = 0.0;
            }
            if(potential < 1.00 && path[4][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[4][0][0] = x1;
              path[4][0][1] = 0.0;
              path[4][0][2] = 0.0;
            }
          }
          
          for(x1 = 2.0; x1 < 4.0; x1 += 0.0001){//Find staring points of equipotential lines
            potential = 0.0;
            for(i = 0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * 0.000001f / totalDistance;
                break;
              }
              potential += k1 * particles[i][4] * 0.000001f / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * scalePlaneHeigth;
            //printf("potential=%lf\n", potential);
            if(potential < 3.00 && path[5][0][1] == -1000.0f){
              printf("potential %lf at %lf\n", potential, x1);
              path[5][0][0] = x1;
              path[5][0][1] = 0.0;
              path[5][0][2] = 0.0;
            }
          }

        }
      }

      if(animation == 1407){
        animation++;
        printf("Animation set to %d\n", animation);
        fieldLineEndUp = 0;
        particles[0][3] = 0.5f;
        particles[0][4] = 0.000002f;
        pauseFrames = 0;
      }

      if(animation == 1406){//Chancel some field lines
        animation++;
        printf("Animation set to %d\n", animation);
        pauseFrames = 0;        
      }

      if(animation == 1405){//Paint more field lines
        if(fieldLineEndUp < 70){
          fieldLineEndUp++;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          fieldLineEndUp = 100;
          pauseFrames = 0;
        }        
      }
      
      if(animation == 1404){
        if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
          animation = 1405;
        }
        if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
          animation = 1405;
        }
        if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
          animation = 1405;
        }
        if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
          animation = 1405;
        }
        if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
          animation = 1405;
        }
        if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
          animation = 1405;
        }
        if(animation == 1405){
          printf("Animation set to %d\n", animation);

          translateXdiff = 0.0f;
          translateYdiff = 0.0f;
          translateZdiff = 0.0f;
          rotXdiff = 0.0f;
          rotYdiff = 0.0f;
          rotZdiff = 0.0f;
          
          doubleTemp = -1.75f;
          fieldLineEndUp = 0;
          
          pauseFrames = 0;
        }
      }
      
      
      if(animation == 1403){//Paint right field line
        if(doubleTemp > 0.0f){
          x1 = 1.75f;
          y1 = 0.0f;
          z1 = 0.0f;
          x2 = doubleTemp;
          y2 = 0.0f;
          z2 = 0.0f;
          drawCylinder(x1, y1, z1, x2, y2, z2, 0.03, 0.03, 32);
          if(doubleTemp < 0.8f){
            drawCylinder(1.0f, 0.0f, 0.0f, 0.8f, 0.0f, 0.0f, 0.1, 0.03, 32);
          }
          doubleTemp -= 0.05f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);

          pauseFrames = 0;
          
          drawCylinder(1.75f, 0.0f, 0.0f, 0.1f, 0.0f, 0.0f, 0.03, 0.03, 32);
          drawCylinder(0.1f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.03, 0.0001, 32);
          drawCylinder(1.0f, 0.0f, 0.0f, 0.8f, 0.0f, 0.0f, 0.1, 0.03, 32);

          translateXtarget = 0.0f;
          translateYtarget = 0.0f;
          translateZtarget = -10.0f;
          
          rotXtarget = 0.0f;
          rotYtarget = 0.0f;
          rotZtarget = 0.0f;

          translateXdiff = (translateXtarget - translateX) / steps * 2.0;
          translateYdiff = (translateYtarget - translateY) / steps * 2.0;
          translateZdiff = (translateZtarget - translateZ) / steps * 2.0;
          rotXdiff = (rotXtarget - rotX) / steps * 2.0;
          rotYdiff = (rotYtarget - rotY) / steps * 2.0;
          rotZdiff = (rotZtarget - rotZ) / steps * 2.0;

        }
      }


      if(animation == 1402){//Paint left field line
        if(doubleTemp < 0.0f){
          x1 = -1.75f;
          y1 = 0.0f;
          z1 = 0.0f;
          x2 = doubleTemp;
          y2 = 0.0f;
          z2 = 0.0f;
          drawCylinder(x1, y1, z1, x2, y2, z2, 0.03, 0.03, 32);
          if(doubleTemp > -0.8f){
            drawCylinder(-1.0f, 0.0f, 0.0f, -0.8f, 0.0f, 0.0f, 0.1, 0.03, 32);
          }
          doubleTemp += 0.05f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          doubleTemp = 1.75f;

          drawCylinder(-1.75f, 0.0f, 0.0f, -0.1f, 0.0f, 0.0f, 0.03, 0.03, 32);
          drawCylinder(-0.1f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.03, 0.0001, 32);
          drawCylinder(-1.0f, 0.0f, 0.0f, -0.8f, 0.0f, 0.0f, 0.1, 0.03, 32);
        }
      }
      

      if(animation == 1401){
        if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
          animation = 1402;
        }
        if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
          animation = 1402;
        }
        if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
          animation = 1402;
        }
        if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
          animation = 1402;
        }
        if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
          animation = 1402;
        }
        if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
          animation = 1402;
        }
        if(animation == 1402){
          printf("Animation set to %d\n", animation);

          translateXdiff = 0.0f;
          translateYdiff = 0.0f;
          translateZdiff = 0.0f;
          rotXdiff = 0.0f;
          rotYdiff = 0.0f;
          rotZdiff = 0.0f;
          
          doubleTemp = -1.75f;
        }
      }

      if(animation == 1400){//Paint field lines
        
        if(fieldLineEndUp < 70){
          fieldLineEndUp++;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          fieldLineEndUp = 100;

          translateXtarget = 0.0f;
          translateYtarget = 0.0f;
          translateZtarget = -4.3f;
          
          rotXtarget = 0.0f;
          rotYtarget = 0.0f;
          rotZtarget = 0.0f;

          translateXdiff = (translateXtarget - translateX) / steps * 2.0;
          translateYdiff = (translateYtarget - translateY) / steps * 2.0;
          translateZdiff = (translateZtarget - translateZ) / steps * 2.0;
          rotXdiff = (rotXtarget - rotX) / steps * 2.0;
          rotYdiff = (rotYtarget - rotY) / steps * 2.0;
          rotZdiff = (rotZtarget - rotZ) / steps * 2.0;
        }
      }//if(animation == 1400){
      
      
    }//if(pauseFrames > 10){


//####################################################################################
//############# Animation sequence relative permitivity with permanent dipoles #######
//####################################################################################


/*
    if(animation >= 1300 && animation < 1400){

      glLightfv(GL_LIGHT1, GL_AMBIENT, lightOrange1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightOrange1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightOrange1);
      
      for(i = 0; i < particleNumber; i += 2){
        x1 = particles[i][0];
        x2 = particles[i+1][0];
        y1 = particles[i][1];
        y2 = particles[i+1][1];
        z1 = 0.0f;
        z2 = 0.0f;
        glBegin(GL_LINES);
        glVertex3f(x1, y1, z1);
        glVertex3f(x2, y2, z2);
        glEnd();
      }

    }
*/

    if(pauseFrames > 10 && animationPause == 0){

      if(animation == 1304){
        if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
          animation = 1305;
        }
        if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
          animation = 1305;
        }
        if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
          animation = 1305;
        }
        if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
          animation = 1305;
        }
        if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
          animation = 1305;
        }
        if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
          animation = 1305;
        }
        if(animation == 1305){
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;

          translateXdiff = 0.0f;
          translateYdiff = 0.0f;
          translateZdiff = 0.0f;
          rotXdiff = 0.0f;
          rotYdiff = 0.0f;
          rotZdiff = 0.0f;

          printf("...un ferddisch!\n");
          done = True;         // Exit programm !!!
        }
      }

      if(animation == 1303){
        GLfloat x, y;
        
        i = 0;
        for(x = -2.13; x <= 1.87; x += 0.5){
          for(y = -4.0; y <= 4.0; y += 0.5){
            moveAlpha = particles[i][6];
            particles[i][0] = (GLfloat)(x) + 0.1f * cos(moveAlpha);
            particles[i][1] = (GLfloat)(y) + 0.1f * sin(moveAlpha);
            particles[i][4] = 0.000001f;
            particles[i][5] = 0.0f;
            i++;
            particles[i][0] = (GLfloat)(x) - 0.1f * cos(moveAlpha);
            particles[i][1] = (GLfloat)(y) - 0.1f * sin(moveAlpha);
            particles[i][4] = -0.000001f;
            particles[i][5] = 0.0f;
            i++;
          }
        }

        translateXtarget = 0.0f;
        translateYtarget = 0.0f;
        translateZtarget = -26.0f;
        
        rotXtarget = 0.0f;
        rotYtarget = 0.0f;
        rotZtarget = 0.0f;

        translateXdiff = (translateXtarget - translateX) / steps * 2.0;
        translateYdiff = (translateYtarget - translateY) / steps * 2.0;
        translateZdiff = (translateZtarget - translateZ) / steps * 2.0;
        rotXdiff = (rotXtarget - rotX) / steps * 2.0;
        rotYdiff = (rotYtarget - rotY) / steps * 2.0;
        rotZdiff = (rotZtarget - rotZ) / steps * 2.0;
        
        pauseFrames = 0;
      }

      if(animation == 1302){
        animation++;
        printf("Animation set to %d\n", animation);
        pauseFrames = 0;
        particleNumber += 242;
        for(i=0; i< 306; i++){
          particles[i][0] = 20.0f;
          particles[i][4] = 0.0f;
          particles[i][5] = -1.0f;
        }
      }


      if(animation == 1301){
        animation++;
        printf("Animation set to %d\n", animation);
        pauseFrames = 0;
        particleNumber = 306;
      }

      if(animation == 1300){
        GLfloat x, y;
        int rotationFinished = 1;

        if(particles[0][4] < 0.000003f){
          for(i2 = 0; i2 < 11; i2++){
            for(i=0; i < 11; i++){
              particles[i2 * 11 + i + 306][4] += 0.0000000023f;
              particles[i2 * 11 + i + 306][3] = 0.25f / pow(0.000001f, 1.0f / 3.0f) * pow(fabs(particles[i2 * 11 + i][4]), 1.0f / 3.0f);//Radius accordant to charge
            }
          }
          
          for(i2 = 0; i2 < 11; i2++){
            for(i=0; i < 11; i++){
              particles[i2 * 11 + i + 306 + 121][4] -= 0.0000000023f;
              particles[i2 * 11 + i + 306 + 121][3] = 0.25f / pow(0.000001f, 1.0f / 3.0f) * pow(fabs(particles[i2 * 11 + i + 121][4]), 1.0f / 3.0f);//Radius accordant to charge
            }
          }
        }


        i = 0;
        for(x = -2.13; x <= 1.87; x += 0.5){
          for(y = -4.0; y <= 4.0; y += 0.5){
            if(particles[i][6] > PI && particles[i][6] < 2.0f * PI){
              particles[i][6] += 0.0051f;
              particles[i+1][6] += 0.0051f;
              rotationFinished = 0;
            }
            else{
              if(particles[i][6] > 0.0f && particles[i][6] < PI){
                particles[i][6] -= 0.0051f;
                particles[i+1][6] -= 0.0051f;
                rotationFinished = 0;
              }
            }
            moveAlpha = particles[i][6];
            particles[i][0] = (GLfloat)(x) + 0.1f * cos(moveAlpha);
            particles[i][1] = (GLfloat)(y) + 0.1f * sin(moveAlpha);
            i++;
            particles[i][0] = (GLfloat)(x) - 0.1f * cos(moveAlpha);
            particles[i][1] = (GLfloat)(y) - 0.1f * sin(moveAlpha);
            i++;
          }
        }
        if(rotationFinished == 1){
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;
          
          i = 0;
          for(x = -2.13; x <= 1.87; x += 0.5){
            for(y = -4.0; y <= 4.0; y += 0.5){
              particles[i][6] = 0.0f;
              particles[i+1][6] = 0.0f;
              moveAlpha = particles[i][6];
              particles[i][0] = (GLfloat)(x) + 0.1f * cos(moveAlpha);
              particles[i][1] = (GLfloat)(y) + 0.1f * sin(moveAlpha);
              i++;
              particles[i][0] = (GLfloat)(x) - 0.1f * cos(moveAlpha);
              particles[i][1] = (GLfloat)(y) - 0.1f * sin(moveAlpha);
              i++;
            }
          }
        }
      }
      

    }//if(pauseFrames > 10){


//####################################################################################
//############# Animation sequence relative permitivity ##############################
//####################################################################################

    if(animation > 1204 && animation < 1300 && animation != 1207){
      glLightfv(GL_LIGHT1, GL_AMBIENT, lightOrange1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightOrange1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightOrange1);

      if(animation < 1208){
        for(i = 0; i < 14; i += 2){
          drawCylinder(particles[i + 242][0], particles[i + 242][1], -0.1f, particles[i + 1 + 242][0], particles[i + 1 + 242][1], -0.1f, 0.02f, 0.02f, 32);
        }
      }
      else{
        for(i = 0; i < 14; i += 2){
          drawCylinder(particles[i][0], particles[i][1], -0.1f, particles[i + 1][0], particles[i + 1][1], -0.1f, 0.02f, 0.02f, 32);
        }
      }
    }




    if(pauseFrames > 10 && animationPause == 0){

      if(animation == 1208){
        printf("...un ferddisch!\n");
        done = True;         // Exit programm !!!
      }


      if(animation == 1207){//Insert particles and remove homogeneous field
        
        for(i=0; i < 14; i++){
          GLfloatTemp = particles[i][0];
          particles[i][0] = particles[i + 242][0];
          particles[i + 242][0] = GLfloatTemp;

          GLfloatTemp = particles[i][1];
          particles[i][1] = particles[i + 242][1];
          particles[i + 242][1] = GLfloatTemp;

          GLfloatTemp = particles[i][2];
          particles[i][2] = particles[i + 242][2];
          particles[i + 242][2] = GLfloatTemp;

          GLfloatTemp = particles[i][3];
          particles[i][3] = particles[i + 242][3];
          particles[i + 242][3] = GLfloatTemp;

          GLfloatTemp = particles[i][4];
          particles[i][4] = particles[i + 242][4];
          particles[i + 242][4] = GLfloatTemp;

          GLfloatTemp = particles[i][5];
          particles[i][5] = particles[i + 242][5];
          particles[i + 242][5] = GLfloatTemp;
        }
        
        particleNumber += 14;
        particleNumber -= 242;
        animation++;
        printf("Animation set to %d\n", animation);
        pauseFrames = 0;
      }

      if(animation == 1206){//Remove particles
        particleNumber -= 14;
        animation++;
        printf("Animation set to %d\n", animation);
        pauseFrames = 0;
      }

      if(animation == 1205){//Fade in external field
        if(particles[0][4] < 0.000003f){
          for(i2 = 0; i2 < 11; i2++){
            for(i=0; i < 11; i++){
              particles[i2 * 11 + i][4] += 0.000000025f;
              particles[i2 * 11 + i][3] = 0.25f / pow(0.000001f, 1.0f / 3.0f) * pow(fabs(particles[i2 * 11 + i][4]), 1.0f / 3.0f);//Radius accordant to charge
            }
          }
          
          for(i2 = 0; i2 < 11; i2++){
            for(i=0; i < 11; i++){
              particles[i2 * 11 + i + 121][4] -= 0.000000025f;
              particles[i2 * 11 + i + 121][3] = 0.25f / pow(0.000001f, 1.0f / 3.0f) * pow(fabs(particles[i2 * 11 + i + 121][4]), 1.0f / 3.0f);//Radius accordant to charge
            }
          }
          
          for(i = 0; i < 14; i += 2){
            particles[i + 242][0] += 0.0151f;
            particles[i + 242][5] = 0.0f;
            particles[i + 242 + 1][0] -= 0.0151f;
            particles[i + 242 + 1][5] = 0.0f;
          }
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;
        }
      }



      if(animation == 1204){//Create particles inside electric field
        for(i = 0; i < 14; i += 2){
          particles[i + 242][0] = 0.25f;
          particles[i + 242][1] = (GLfloat)(i) / 2.0f - 3.0f;
          particles[i + 242][2] = 0.0f;
          particles[i + 242][3] = 0.25f;
          particles[i + 242][4] = 0.000001f;
          particles[i + 242][5] = 1.0f;

          particles[i + 242 + 1][0] = 0.25f;
          particles[i + 242 + 1][1] = (GLfloat)(i) / 2.0f - 3.0f;
          particles[i + 242 + 1][2] = 0.0f;
          particles[i + 242 + 1][3] = 0.25f;
          particles[i + 242 + 1][4] = -0.000001f;
          particles[i + 242 + 1][5] = 1.0f;
        }
        particleNumber += 14;
        
        animation++;
        printf("Animation set to %d\n", animation);
        pauseFrames = 0;
      }


      if(animation == 1203){
        if(particles[0][4] > 0.0f){
          for(i2 = 0; i2 < 11; i2++){
            for(i=0; i < 11; i++){
              particles[i2 * 11 + i][4] -= 0.00000005f;
              particles[i2 * 11 + i][3] = 0.25f / pow(0.000001f, 1.0f / 3.0f) * pow(fabs(particles[i2 * 11 + i][4]), 1.0f / 3.0f);//Radius accordant to charge
            }
          }
          
          for(i2 = 0; i2 < 11; i2++){
            for(i=0; i < 11; i++){
              particles[i2 * 11 + i + 121][4] += 0.00000005f;
              particles[i2 * 11 + i + 121][3] = 0.25f / pow(0.000001f, 1.0f / 3.0f) * pow(fabs(particles[i2 * 11 + i + 121][4]), 1.0f / 3.0f);//Radius accordant to charge
            }
          }
          
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;

          for(i2 = 0; i2 < 11; i2++){
            for(i=0; i < 11; i++){
              particles[i2 * 11 + i][4] = 0.0f;
              particles[i2 * 11 + i][3] = 0.0f;
            }
          }
          for(i2 = 0; i2 < 11; i2++){
            for(i=0; i < 11; i++){
              particles[i2 * 11 + i + 121][4] = 0.0f;
              particles[i2 * 11 + i + 121][3] = 0.0f;
            }
          }
        }
      }
          


      if(animation == 1201){
        if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
          animation = 1202;
        }
        if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
          animation = 1202;
        }
        if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
          animation = 1202;
        }
        if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
          animation = 1202;
        }
        if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
          animation = 1202;
        }
        if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
          animation = 1202;
        }
        if(animation == 1202){
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;

          translateXdiff = 0.0f;
          translateYdiff = 0.0f;
          translateZdiff = 0.0f;
          rotXdiff = 0.0f;
          rotYdiff = 0.0f;
          rotZdiff = 0.0f;
        }
      }



      if(animation == 1200){
        if(particles[0][4] < 0.000003f){
          for(i2 = 0; i2 < 11; i2++){
            for(i=0; i < 11; i++){
              particles[i2 * 11 + i][4] += 0.00000005f;
              particles[i2 * 11 + i][3] = 0.25f / pow(0.000001f, 1.0f / 3.0f) * pow(fabs(particles[i2 * 11 + i][4]), 1.0f / 3.0f);//Radius accordant to charge
            }
          }
          
          for(i2 = 0; i2 < 11; i2++){
            for(i=0; i < 11; i++){
              particles[i2 * 11 + i + 121][4] -= 0.00000005f;
              particles[i2 * 11 + i + 121][3] = 0.25f / pow(0.000001f, 1.0f / 3.0f) * pow(fabs(particles[i2 * 11 + i + 121][4]), 1.0f / 3.0f);//Radius accordant to charge
            }
          }
          
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;

          translateXtarget = 0.0f;
          translateYtarget = 0.0f;
          translateZtarget = -10.0f;
          
          rotXtarget = 0.0f;
          rotYtarget = 0.0f;
          rotZtarget = 0.0f;

          translateXdiff = (translateXtarget - translateX) / steps * 2.0;
          translateYdiff = (translateYtarget - translateY) / steps * 2.0;
          translateZdiff = (translateZtarget - translateZ) / steps * 2.0;
          rotXdiff = (rotXtarget - rotX) / steps * 2.0;
          rotYdiff = (rotYtarget - rotY) / steps * 2.0;
          rotZdiff = (rotZtarget - rotZ) / steps * 2.0;
        }
      }
      
    }//if(pauseFrames > 10){



//####################################################################################
//############# Animation sequence induced polarisation ##############################
//####################################################################################


    if(animation > 1100 && animation < 1200){
      glDisable(GL_DEPTH_TEST);
      glEnable(GL_LIGHTING);
      glEnable(GL_LIGHT1);
      glEnable(GL_LIGHT2);
      glEnable(GL_LIGHT3);
      if(animation < 1103){//Cross at center of negative charge
        glLightfv(GL_LIGHT1, GL_AMBIENT, lightBlue3);
        glLightfv(GL_LIGHT1, GL_DIFFUSE, lightBlue3);
        glLightfv(GL_LIGHT1, GL_SPECULAR, lightBlue3);
        
        x1 = (1.0f - doubleTemp) * doubleTemp * 1.2f + 0.25f + 1.5f;
        x2 = x1 - 0.5f;
        y1 = 0.25f;
        y2 = -0.25f;
        z1 = 0.0f;
        z2 = 0.0f;
        
        drawCylinder(x2, y2, z1, x1, y1, z2, 0.05f, 0.05f, 32);
        drawCylinder(x2, y1, z1, x1, y2, z2, 0.05f, 0.05f, 32);
        
      }

      glLightfv(GL_LIGHT1, GL_AMBIENT, lightOrange1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightOrange1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightOrange1);

      if(animation < 1103){
        x1 = (1.0f - doubleTemp) * doubleTemp * 1.2f + 1.5f;
      }
      else{
        x1 = particles[1][0];
      }
      x2 = particles[0][0];
      y1 = 0.0f;
      y2 = 0.0f;
      z1 = 0.0f;
      z2 = 0.0f;
      
      drawCylinder(x1, y1, z1, x2, y2, z2, 0.05f, 0.05f, 32);

      glEnable(GL_DEPTH_TEST);
      glDisable(GL_LIGHT2);
      glDisable(GL_LIGHT3);

      sprintf(textLine, "   ");
      printGLf(-4.3, 2.0, textLine, 0, -3, 0.42f);//Print black background

      sprintf(textLine, "Q1 = %+4.1f^C", particles[0][4] * 1000000.0f);
      printGLf(-4.3, 2.3, textLine, 0, -3, 0.07f);
      if(animation < 1102){
        sprintf(textLine, "Sum Q2 = %+4.1f^C", -particles[0][4] * 1000000.0f);
      }
      else{
        sprintf(textLine, "Q2 = %+4.1f^C", -particles[0][4] * 1000000.0f);
      }
      printGLf(-4.3, 2.0, textLine, 0, -6, 0.07f);
      sprintf(textLine, "d = %3.1fm", fabs(x2 - x1));
      printGLf(-4.3, 1.7, textLine, 0, -2, 0.07f);
      sprintf(textLine, "p = %3.1f^Cm", fabs(particles[0][4]) * 1000000.0f * fabs(x2 - x1));
      printGLf(-4.3, 1.4, textLine, 0, -1, 0.07f);
      sprintf(textLine, " ");
      textLine[0]=123; //Character 123 ist vector arrow (font_123.bmp)
      printGLf(-4.3, 1.4, textLine, 0, 1, 0.07f);

    }

    if(pauseFrames > 10 && animationPause == 0){

      if(animation == 1105){
        printf("...un ferddisch!\n");
        done = True;         // Exit programm !!!
      }

      if(animation == 1104){
        if(particles[3][4] > 0.0f + 0.00000005f){
          particles[1][0] = (doubleTemp - 1.5f) * particles[3][4] / 0.000005f + 1.5f;
          for(i=2; i< particleNumber; i++){
            if(particles[i][4] > 0.0f){
              particles[i][4] -= 0.00000005f;
            }
            else{
              particles[i][4] += 0.00000005f;
            }
            particles[i][3] = 0.25f / pow(0.000001f, 1.0f / 3.0f) * pow(fabs(particles[i][4]), 1.0f / 3.0f);//Radius accordant to charge
          }
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;

          particles[1][0] = 1.5f;
          particles[1][5] = 1.0f;
          particles[0][5] = 1.0f;
          for(i=2; i< particleNumber; i++){
            particles[i][4] = 0.0f;
          }          
        }
      }
            
      if(animation == 1103){
        if(particles[3][4] < 0.000005f){
          particles[1][0] -= 0.01f;
          for(i=2; i< particleNumber; i++){
            if(particles[i][4] > 0.0f){
              particles[i][4] += 0.00000005f;
            }
            else{
              particles[i][4] -= 0.00000005f;
            }
            particles[i][3] = 0.25f / pow(0.000001f, 1.0f / 3.0f) * pow(fabs(particles[i][4]), 1.0f / 3.0f);//Radius accordant to charge
          }
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;
          doubleTemp = particles[1][0];
        }
      }

      if(animation == 1102){//Replace electron shell by single negative charge
        particles[1][0] = (1.0f - doubleTemp) * doubleTemp * 1.2f + 1.5f;
        particles[1][1] = 0.0f;
        particles[1][2] = 0.0f;
        particles[1][3] = 0.5f;
        particles[1][4] = -0.000003f;

        for(i = 2; i < particleNumber-179; i++){
          for(i2 = 0; i2 < 6; i2++){
            particles[i][i2] = particles[i+179][i2];
          }
        }

        particleNumber -= 179;
        animation++;
        printf("Animation set to %d\n", animation);
        pauseFrames = 0;
      }

      if(animation == 1101){
        if(doubleTemp < 2.0f){
          moveAlpha = 0.0f;
          for(i = 1; i < 181; i++){
            particles[i][0] = 2.0 * cos(moveAlpha) * doubleTemp + (1.0f - doubleTemp) * doubleTemp * 1.2f + 1.5f;
            particles[i][1] = 2.0 * sin(moveAlpha);
            particles[i][2] = 0.0;
            moveAlpha += 2.0f * PI / 180;
          }
          doubleTemp += 0.01f;
          
          for(i=181; i < 181 + 242; i++){
            if(particles[i][4] > 0.0f){
              particles[i][4] += 0.00000005f;
            }
            else{
              particles[i][4] -= 0.00000005f;
            }
          }
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;
        }
      }


      if(animation == 1100){
        animation++;
        printf("Animation set to %d\n", animation);
        pauseFrames = 0;

        for(i2 = 0; i2 < 11; i2++){
          for(i=0; i < 11; i++){
            particles[i2 * 11 + i + 181][0] = -14.0f;
            particles[i2 * 11 + i + 181][1] = (GLfloat)(i) - 5.0f;
            particles[i2 * 11 + i + 181][2] = (GLfloat)(i2) - 5.0f;
            particles[i2 * 11 + i + 181][3] = 0.25f;
            particles[i2 * 11 + i + 181][4] = 0.00000005f;
            
            particles[i2 * 11 + i + 181][5] = 0.0f;
          }
        }
        
        for(i2 = 0; i2 < 11; i2++){
          for(i=0; i < 11; i++){
            particles[i2 * 11 + i + 302][0] = 14.0f;
            particles[i2 * 11 + i + 302][1] = (GLfloat)(i) - 5.0f;
            particles[i2 * 11 + i + 302][2] = (GLfloat)(i2) - 5.0f;
            particles[i2 * 11 + i + 302][3] = 0.25f;
            particles[i2 * 11 + i + 302][4] = -0.00000005f;

            particles[i2 * 11 + i + 302][5] = 0.0f;
          }
        }
        
        doubleTemp = 1.0f;
        particleNumber += 242;
        enableFieldVectors = 1;
      }
      
      
    }//if(pauseFrames > 10){


//####################################################################################
//############# Animation sequence attracion between Dipoles #########################
//####################################################################################


    if(pauseFrames > 10 && animationPause == 0){
            
      if(animation == 1002){
        printf("...un ferddisch!\n");
        done = True;         // Exit programm !!!
      }

      if(animation == 1001){
        if(particles[3][0] > 2.0f){
          for(i=3; i<6; i++){
            particles[i][0] -= 0.01f;
          }
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;
        }
      }


      if(animation == 1000){
        if(moveBeta < PI * 2.0f){
          x1 = -4.0f;
          y1 = 2.0f;
          
          particles[4][0] = 1.0 * cos(moveAlpha + moveBeta) + x1;
          particles[4][1] = 1.0 * sin(moveAlpha + moveBeta) + y1;
          particles[4][2] = 0.0;
          particles[4][3] = 0.25;
          particles[4][4] = 0.000001;
          
          particles[5][0] = 1.0 * cos(moveAlpha + PI / 180.0f * 105.0f + moveBeta) + x1;
          particles[5][1] = 1.0 * sin(moveAlpha + PI / 180.0f * 105.0f + moveBeta) + y1;
          particles[5][2] = 0.0;
          particles[5][3] = 0.25;
          particles[5][4] = 0.000001;
          
          x1 = 4.0f;
          y1 = 2.0f;

          particles[10][0] = 1.0 * cos(PI / 2.0f + moveBeta) + x1;
          particles[10][1] = 1.0 * sin(PI / 2.0f + moveBeta) + y1;
          particles[10][2] = 0.0;
          particles[10][3] = 0.25;
          particles[10][4] = 0.000001;
          
          particles[11][0] = 1.0 * cos(PI * 1.5 + moveBeta) + x1;
          particles[11][1] = 1.0 * sin(PI * 1.5 + moveBeta) + y1;
          particles[11][2] = 0.0;
          particles[11][3] = 0.25;
          particles[11][4] = 0.000001;

          moveBeta += 0.01f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;
        }        
      }
      
      
    }//if(pauseFrames > 10){



//####################################################################################
//############# Animation sequence Water molecule ####################################
//####################################################################################
    if(animation >= 900 && animation < 1000){
      glLightfv(GL_LIGHT1, GL_AMBIENT, lightWhite1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightWhite1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightWhite1);
      
      drawCylinder(particles[0][0], particles[0][1], -0.051f, particles[1][0], particles[1][1], -0.051f, 0.05f, 0.05f, 32);//"Bar" between charges
      drawCylinder(particles[0][0], particles[0][1], -0.051f, particles[2][0], particles[2][1], -0.051f, 0.05f, 0.05f, 32);//"Bar" between charges
      
      glDisable(GL_DEPTH_TEST);
      
      glEnable(GL_LIGHT2);
      glEnable(GL_LIGHT3);
      
      glDisable(GL_LIGHTING);
      
      glLightfv(GL_LIGHT1, GL_AMBIENT, lightOrange1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightOrange1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightOrange1);
      
      glColor3f(1.0f, 0.7f, 0.0f);
      
      x1 = particles[1][0] + (particles[2][0] - particles[1][0]) / 2.0f;
      x2 = particles[0][0];
      y1 = particles[1][1];
      y2 = particles[0][1];
      z1 = 0.0f;
      z2 = 0.0f;
      drawCylinder(x2, y2, z1, x1, y1, z2, 0.04, 0.04, 32);
      
      glEnable(GL_LIGHTING);

      glDisable(GL_LIGHT2);
      glDisable(GL_LIGHT3);


      glLightfv(GL_LIGHT1, GL_AMBIENT, lightBlue1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightBlue1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightBlue1);
            
      x1 = particles[0][0] - 0.3f;
      x2 = particles[0][0] + 0.3f;
      y1 = particles[0][1] - 0.3f;
      y2 = particles[0][1] + 0.3f;
      z1 = 0.0f;
      z2 = 0.0f;
      
      drawCylinder(x1, y1, z1, x2, y2, z2, 0.05, 0.05, 32);
      drawCylinder(x1, y2, z1, x2, y1, z2, 0.05, 0.05, 32);


      glLightfv(GL_LIGHT1, GL_AMBIENT, lightRed1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightRed1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightOrange1);
            
      x1 = particles[1][0] + (particles[2][0] - particles[1][0]) / 2.0f - 0.3f;
      x2 = particles[1][0] + (particles[2][0] - particles[1][0]) / 2.0f + 0.3f;
      y1 = particles[1][1] - 0.3f;
      y2 = particles[1][1] + 0.3f;
      z1 = 0.0f;
      z2 = 0.0f;
      
      drawCylinder(x1, y1, z1, x2, y2, z2, 0.05, 0.05, 32);
      drawCylinder(x1, y2, z1, x2, y1, z2, 0.05, 0.05, 32);


    }

    if(animation >= 900 && animation < 904){      
      sprintf(textLine, "Q1 = %+4.1f^C", particles[0][4] * 1000000.0f);
      printGLf(-4.3, 2.3, textLine, 0, -6, 0.07f);
      sprintf(textLine, "Q2 = Q3 = %+4.1f^C", particles[1][4] * 1000000.0f);
      printGLf(-4.3, 2.0, textLine, 0, -3, 0.07f);
      sprintf(textLine, "d = %3.1fm", fabs(particles[1][1] - particles[0][1]));
      printGLf(-4.3, 1.7, textLine, 0, -2, 0.07f);
      sprintf(textLine, "p = %3.1f^Cm", fabs(particles[0][4]) * 1000000.0f * fabs(particles[1][1] - particles[0][1]));
      printGLf(-4.3, 1.4, textLine, 0, -1, 0.07f);
      sprintf(textLine, " ");
      textLine[0]=123; //Character 123 ist vector arrow (font_123.bmp)
      printGLf(-4.3, 1.4, textLine, 0, 1, 0.07f);
    }



    if(pauseFrames > 10 && animationPause == 0){
            
      if(animation == 902){
        printf("...un ferddisch!\n");
        done = True;         // Exit programm !!!        
      }

      if(animation == 901){
        if(moveAlpha > -PI / 180.0f * 80.0f){
          particles[1][0] = 4.0f * cos(moveAlpha);
          particles[1][1] = 4.0f * sin(moveAlpha) + 1.5f;
          
          particles[2][0] = -4.0f * cos(moveAlpha);
          particles[2][1] = 4.0f * sin(moveAlpha) + 1.5f;
          
          moveAlpha -= 0.01f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          
          pauseFrames = 0;
        }
      }

      if(animation == 900){
        if(moveAlpha < 0.0f){
          particles[1][0] = 4.0f * cos(moveAlpha);
          particles[1][1] = 4.0f * sin(moveAlpha) + 1.5f;
          
          particles[2][0] = -4.0f * cos(moveAlpha);
          particles[2][1] = 4.0f * sin(moveAlpha) + 1.5f;
          
          moveAlpha += 0.01f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          
          particles[1][0] = 4.0f;
          particles[1][1] = 1.5f;
          
          particles[2][0] = -4.0f;
          particles[2][1] = 1.5f;

          moveAlpha = 0.0f;
          
          pauseFrames = 0;
        }
      }
      
    }//if(pauseFrames > 10){
    
    

//####################################################################################
//############# Animation sequence Dipoles ###########################################
//####################################################################################
    if(animation >= 800 && animation < 900){
      glLightfv(GL_LIGHT1, GL_AMBIENT, lightOrange1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightOrange1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightOrange1);
      
      drawCylinder(particles[0][0], particles[0][1], -0.051, particles[1][0], particles[1][1], -0.051, 0.05, 0.05, 32);//"Bar" between charges
    }
    
    if(animation >= 800 && animation < 804){      
      sprintf(textLine, "Q1 = %+4.1f^C", particles[0][4] * 1000000.0f);
      printGLf(-4.3, 2.3, textLine, 0, -6, 0.07f);
      sprintf(textLine, "Q2 = %+4.1f^C", particles[1][4] * 1000000.0f);
      printGLf(-4.3, 2.0, textLine, 0, -3, 0.07f);
      sprintf(textLine, "d = %3.1fm", fabs(particles[1][0] - particles[0][0]));
      printGLf(-4.3, 1.7, textLine, 0, -2, 0.07f);
      sprintf(textLine, "p = %3.1f^Cm", fabs(particles[0][4]) * 1000000.0f * fabs(particles[1][0] - particles[0][0]));
      printGLf(-4.3, 1.4, textLine, 0, -1, 0.07f);
      sprintf(textLine, " ");
      textLine[0]=123; //Character 123 ist vector arrow (font_123.bmp)
      printGLf(-4.3, 1.4, textLine, 0, 1, 0.07f);
    }

    if(pauseFrames > 10 && animationPause == 0){
            
      if(animation == 809){
        printf("...un ferddisch!\n");
        done = True;         // Exit programm !!!        
      }

      if(animation == 808){
        if(moveAlpha < PI){
          particles[0][0] = -2.0f * cos(moveAlpha);
          particles[0][1] = -2.0f * sin(moveAlpha);
          particles[1][0] = 2.0f * cos(moveAlpha);
          particles[1][1] = 2.0f * sin(moveAlpha);
          moveAlpha += 0.01f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);

          particles[0][0] = 2.0f;
          particles[0][1] = 0.0f;
          particles[1][0] = -2.0f;
          particles[1][1] = 0.0f;

          pauseFrames = 0;
        }
      }

      if(animation == 807){
        if(particles[3][3] > 0.005f){
          for(i = 2; i < 244; i++){
            particles[i][3] -= 0.005f;
          }
        }
        else{
          for(i = 2; i < 244; i++){
            particles[i][3] = 0.000f;
          }
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;
        }
      }

      if(animation == 806){
        if(translateZ < -8.0f){
          translateZ += 0.5f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;
        }
      }

      if(animation == 805){
        animation++;
        printf("Animation set to %d\n", animation);

        for(i=0; i<11; i++){
          for(i2=0; i2<11; i2++){
            particles[i * 11 + i2 + 2][0] = -7.0f;
            particles[i * 11 + i2 + 2][1] = (GLfloat)(i2 - 5) * 1.8f;
            particles[i * 11 + i2 + 2][2] = (GLfloat)(i - 5) * 1.8f;
            particles[i * 11 + i2 + 2][3] = 0.25f;
            particles[i * 11 + i2 + 2][4] = -0.000001f;
            if(i == 5){
              particles[i * 11 + i2 + 2][5] = 0.0f;
            }
            else{
              //particles[i * 11 + i2 + 2][5] = -1.0f;
            }
          }
        }

        for(i=0; i<11; i++){
          for(i2=0; i2<11; i2++){
            particles[i * 11 + i2 + 123][0] = 7.0f;
            particles[i * 11 + i2 + 123][1] = (GLfloat)(i2 - 5) * 1.8f;
            particles[i * 11 + i2 + 123][2] = (GLfloat)(i - 5) * 1.8f;
            particles[i * 11 + i2 + 123][3] = 0.25f;
            particles[i * 11 + i2 + 123][4] = 0.000001f;
            if(i == 5){
              particles[i * 11 + i2 + 123][5] = 0.0f;
            }
            else{
              //particles[i * 11 + i2 + 123][5] = -1.0f;
            }
          }
        }
        
        particleNumber += 246;
        pauseFrames = 0;
        moveAlpha = 0.0f;
        enableFieldVectors = 20;

      }

      if(animation == 804){
        if(translateZ > -20.0f){
          translateZ -= 0.5f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;
        }
      }


      if(animation == 803){
        if(particles[0][0] > -2.0f){
          particles[0][0] -= 0.01f;
          particles[1][0] += 0.01f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);

          particles[0][0] = -2.0f;
          particles[1][0] = 2.0f;

          pauseFrames = 0;
        }
      }

      if(animation == 802){
        if(particles[0][0] < -0.5f){
          particles[0][0] += 0.01f;
          particles[1][0] -= 0.01f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;
        }
      }

      if(animation == 801){
        if(particles[0][4] < -0.000001f){
          particles[0][3] -= 0.01f;
          particles[0][4] += 0.0000001f;
          particles[1][3] -= 0.01f;
          particles[1][4] -= 0.0000001f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;
        }
      }


      if(animation == 800){
        if(particles[0][4] > -0.000005f){
          particles[0][3] += 0.01f;
          particles[0][4] -= 0.0000001f;
          particles[1][3] += 0.01f;
          particles[1][4] += 0.0000001f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;
        }        
      }
      
      
    }//if(pauseFrames > 10){



//####################################################################################
//############# Animation sequence Molecule ##########################################
//####################################################################################
    if(pauseFrames > 10 && animationPause == 0){
            

      if(animation == 704){
        printf("...un ferddisch!\n");
        done = True;         // Exit programm !!!        
      }
            
      if(animation == 703){
        if(moveAlpha < 2.0f * PI){

          particles[particleNumber][0] = 2.5f * cos(moveAlpha) * 1.3f - 0.7;
          particles[particleNumber][1] = 2.5f * sin(moveAlpha);
          particles[particleNumber][2] = 0.0f;
          particles[particleNumber][3] = 0.08f;
          particles[particleNumber][4] = -0.000001f * 9.0f / 180.0f;
                    
          particleNumber++;
        
          //moveAlpha += 2.0f * PI / 180.0f;
          moveAlpha += doubleTemp;
          if(moveAlpha < PI){
            doubleTemp *= 0.94f;
          }
          else{
            doubleTemp /= 0.94f;
          }
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
        }
      }
            
      if(animation == 702){
        if(particles[7][0] > 1.3f){          
          particles[7][0] -= 0.1f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          moveAlpha = 0.0f;
          doubleTemp = 0.2;
        }
      }
            
      if(animation == 701){
        if(particles[0][0] > -1.3f){
          for(i=0; i< particleNumber; i++){
            particles[i][0] -= 0.05f;
          }
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
        }
      }
      
      if(animation == 700){
        if(doubleTemp > 0.5f){
          doubleTemp -= 0.1f;
          
          particles[1][0] = doubleTemp * cos(2.0f * PI / 6.0f * 1.0f);
          particles[1][1] = doubleTemp * sin(2.0f * PI / 6.0f * 1.0f);

          particles[2][0] = doubleTemp * cos(2.0f * PI / 6.0f * 2.0f);
          particles[2][1] = doubleTemp * sin(2.0f * PI / 6.0f * 2.0f);

          particles[3][0] = doubleTemp * cos(2.0f * PI / 6.0f * 3.0f);
          particles[3][1] = doubleTemp * sin(2.0f * PI / 6.0f * 3.0f);

          particles[4][0] = doubleTemp * cos(2.0f * PI / 6.0f * 4.0f);
          particles[4][1] = doubleTemp * sin(2.0f * PI / 6.0f * 4.0f);

          particles[5][0] = doubleTemp * cos(2.0f * PI / 6.0f * 5.0f);
          particles[5][1] = doubleTemp * sin(2.0f * PI / 6.0f * 5.0f);

          particles[6][0] = doubleTemp * cos(2.0f * PI / 6.0f * 6.0f);
          particles[6][1] = doubleTemp * sin(2.0f * PI / 6.0f * 6.0f);
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
        }
        
      }
    }//if(pauseFrames > 10){


//####################################################################################
//############# Animation sequence Atom ##############################################
//####################################################################################

    if(animation >= 600 && animation < 700){
      if(totalCharge != 0.0f){
        if(animation < 602){
          sprintf(textLine, "Q = %+12.9fE-19C", totalCharge);
        }
        else{
          sprintf(textLine, "  = %5.3fE-19C", totalCharge * 1.602176565);
          printGLf(-4.3, 2.0, textLine, 0, -5, 0.09f);
          sprintf(textLine, "Q = %2.0f x -1.602176565E-19C", fabs(totalCharge));
        }
        printGLf(-4.3, 2.3, textLine, 0, -5, 0.09f);
      }
    }

    if(pauseFrames > 10 && animationPause == 0){
            
            
      if(animation == 614){
        printf("...un ferddisch!\n");
        done = True;         // Exit programm !!!
      }
            
      if(animation == 613){
        if(particleNumber < 182){
          particles[particleNumber][0] = particles[0][0];
          particles[particleNumber][1] = particles[0][1];
          particles[particleNumber][2] = 0.0f;
          particles[particleNumber][3] = 0.07f;
          particles[particleNumber][4] = -0.000001f / 180.0f;
          
          particles[0][4] += 0.000001f / 180.0f;
          
          particles[0][3] -= (0.35f - 0.05f) / 180.0f;
          
          particleNumber++;
        
          particles[0][0] = 3.0f * cos(moveAlpha);
          particles[0][1] = 3.0f * sin(moveAlpha);
          moveAlpha += 2.0f * PI / 180.0f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          
          pauseFrames = 0;
        }
      }

      if(animation == 612){
        if(moveAlpha < 4.0f * PI){
          particles[0][0] = 3.0f * cos(moveAlpha);
          particles[0][1] = 3.0f * sin(moveAlpha);
          moveAlpha += 0.025f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);

          moveAlpha = 0.0f;
          particles[0][0] = 3.0f * cos(moveAlpha);
          particles[0][1] = 3.0f * sin(moveAlpha);
        }
      }

      if(animation == 611){
        if(particles[0][0] < 0.2){
          particles[0][5] = 0.0f;
          particles[1][5] = 0.0f;
        }
        if(particles[0][0] > -3.0f){
          particles[0][0] -= 0.025f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          particles[0][0] = -3.0f;
          particles[1][0] = 0.0f;
          moveAlpha = PI;
        }
      }

      if(animation == 610){
        if(particles[0][0] < 0.0f){
          particles[0][0] += 0.025f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          particles[0][0] = 0.0f;
          particles[1][0] = 0.0f;
          particles[0][5] = 1.0f;
          particles[1][5] = 1.0f;
          pauseFrames = 0;
        }
      }

      if(animation == 609){
        totalCharge = 0.0f;
        pathEnd[0] = 0;
        if(particles[1][0] > 0.0f){
          particles[1][0] -= 0.025f;
          particles[0][0] -= 0.025f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          particles[0][0] = -2.0f;
          particles[1][0] = 0.0f;
        }
      }

      if(animation == 608){
        totalCharge = 0.0f;
        pathEnd[0] = 0;
        if(particles[1][0] > 2.0f){
          particles[1][0] -= 0.025f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          particles[1][0] = 2.0f;          
        }
      }

      if(animation == 607){
        totalCharge = 0.0f;
        pathEnd[0] = 0;
        if(particles[4][0] < 20.0f){
          particles[4][0] += 0.1f;
          particles[3][0] -= 0.1f;
          particles[2][1] += 0.1f;
          particles[1][1] -= 0.1f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          particles[1][0] = 10.0f;
          particles[1][1] = 0.0f;
          particles[1][2] = 0.0f;
          particles[1][3] = 0.35f;
          particles[1][4] = 0.000001f;
          
          particleNumber = 2;
        }
      }

      if(animation == 606){
        if(particles[4][0] < 1.0f){
          totalCharge = -5.0f;
        }
        if(particles[4][0] > 9.0f){
          particles[4][0] -= 1.1f;
        }
        else{
          if(particles[4][0] > 0.7f){
            particles[4][0] -= 0.025f;
          }
          else{
            animation++;
            printf("Animation set to %d\n", animation);
            particles[4][0] = 0.7f;
            pauseFrames = 0;
          }
        }
      }

      if(animation == 605){
        if(particles[3][0] > -1.0f){
          totalCharge = -4.0f;
        }
        if(particles[3][0] < -9.0f){
          particles[3][0] += 1.1f;
        }
        else{
          if(particles[3][0] < -0.7f){
            particles[3][0] += 0.025f;
          }
          else{
            animation++;
            printf("Animation set to %d\n", animation);
            particles[3][0] = -0.7f;
          }
        }
      }

      if(animation == 604){
        if(particles[2][1] < 1.0f){
          totalCharge = -3.0f;
        }
        if(particles[2][1] > 5.0f){
          particles[2][1] -= 1.1f;
        }
        else{
          if(particles[2][1] > 0.7f){
            particles[2][1] -= 0.025f;
          }
          else{
            animation++;
            printf("Animation set to %d\n", animation);
            particles[2][1] = 0.7f;
          }
        }
      }

      if(animation == 603){
        if(particles[1][1] > -1.0f){
          totalCharge = -2.0f;
        }
        if(particles[1][1] < -5.0f){
          particles[1][1] += 1.1f;
        }
        else{
          if(particles[1][1] < -0.7f){
            particles[1][1] += 0.025f;
          }
          else{
            animation++;
            printf("Animation set to %d\n", animation);
            particles[1][1] = -0.7f;
          }
        }
      }

      if(animation == 602){
        particles[1][0] = 0.0;
        particles[1][1] = -20.0;
        particles[1][2] = 0.0;
        particles[1][3] = 0.35;
        particles[1][4] = -0.000001;

        particles[2][0] = 0.0;
        particles[2][1] = 20.0;
        particles[2][2] = 0.0;
        particles[2][3] = 0.35;
        particles[2][4] = -0.000001;

        particles[3][0] = -20.0;
        particles[3][1] = 0.0;
        particles[3][2] = 0.0;
        particles[3][3] = 0.35;
        particles[3][4] = -0.000001;

        particles[4][0] = 20.0;
        particles[4][1] = 0.0;
        particles[4][2] = 0.0;
        particles[4][3] = 0.35;
        particles[4][4] = -0.000001;
        
        alpha = 0.0f;
        for(i = 0; i < 210; i++){
          path[0][i][0] = cos(alpha) * 1.2f;
          path[0][i][1] = sin(alpha) * 1.2f;
          alpha += 2.0f * PI / 200.0f;
        }
        pathEnd[0] = 210;
        pathColor[0] = 2;
        particleNumber += 4;
        
        animation++;
        printf("Animation set to %d\n", animation);
      }

      if(animation == 601){
        if(particles[0][4] < -0.000001f){
          particles[0][4] += 0.0000001f;
          particles[0][3] -= (0.35f -0.035f) / 50.0f;
          totalCharge = 1.602176565f * particles[0][4] * 1000000.0f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          particles[0][3] = 0.35;
          particles[0][4] = -0.000001;
          pauseFrames = 0;
          totalCharge = -1.0f;
        }
      }

      if(animation == 600){
        if(particles[0][4] > -0.000005f){
          particles[0][4] -= 0.0000001f;
          particles[0][3] += (0.35f - 0.035f) / 50.0f;
          totalCharge = 1.602176565f * particles[0][4] * 1000000.0f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);        
        }
      }

      
    }//if(pauseFrames > 10){
    else{
      pauseFrames++;
    }


//####################################################################################
//############# Animation sequence potential plane of two charges ####################
//####################################################################################

    if(animation == 500){
      for(i=0; i < particleNumber; i++){//Particle color white if close to each other
        if(fabs(particles[i][0]) < 0.05f){
          particles[i][5] = 1.0f;
        }
        else{
          particles[i][5] = 0.0f;
        }
      }
      if(particles[0][0] < 0.0f - 0.05f){
        particles[0][0] += 0.025f;
        particles[1][0] -= 0.025f;
      }
      else{
        animation++;
        printf("Animation set to %d\n", animation);
        pauseFrames = 0;
        
        particles[0][0] = 0.0f;
        particles[1][0] = 0.0f;
      }
    }

    if(animation == 501){
      if(pauseFrames < 10){
        pauseFrames++;
      }
      else{
        animation++;
        printf("Animation set to %d\n", animation);
      }
    }

    if(animation == 502){
      for(i=0; i < particleNumber; i++){//Particle color white if close to each other
        if(fabs(particles[i][0]) < 0.05f){
          particles[i][5] = 1.0f;
        }
        else{
          particles[i][5] = 0.0f;
        }
      }
      if(particles[0][0] > -2.0f + 0.05f){
        particles[0][0] -= 0.025f;
        particles[1][0] += 0.025f;
      }
      else{
        animation++;
        printf("Animation set to %d\n", animation);
        pauseFrames = 0;
        
        particles[0][0] = -2.0f;
        particles[1][0] = 2.0f;
      }
    }

    if(animation == 503){
      if(pauseFrames < 10){
        pauseFrames++;
      }
      else{
        animation++;
        printf("Animation set to %d\n", animation);
      }
    }

    if(animation == 504){
      if(pauseFrames < 10){
        pauseFrames++;
      }
      else{
        animation++;
        printf("Animation set to %d\n", animation);
        
        particleNumber += 8;
        
        particles[2][0] = 0.0;
        particles[2][1] = 2.0;
        particles[2][2] = 0.0;
        particles[2][3] = 0.25;
        particles[2][4] = 0.000001;
        particles[2][5] = 1.0f;

        particles[3][0] = 0.0;
        particles[3][1] = 2.0;
        particles[3][2] = 0.0;
        particles[3][3] = 0.25;
        particles[3][4] = -0.000001;
        particles[3][5] = 1.0f;

        particles[4][0] = 0.0;
        particles[4][1] = -2.0;
        particles[4][2] = 0.0;
        particles[4][3] = 0.25;
        particles[4][4] = 0.000001;
        particles[4][5] = 1.0f;

        particles[5][0] = 0.0;
        particles[5][1] = -2.0;
        particles[5][2] = 0.0;
        particles[5][3] = 0.25;
        particles[5][4] = -0.000001;
        particles[5][5] = 1.0f;

        particles[6][0] = 0.0;
        particles[6][1] = 4.0;
        particles[6][2] = 0.0;
        particles[6][3] = 0.25;
        particles[6][4] = 0.000001;
        particles[6][5] = 1.0f;

        particles[7][0] = 0.0;
        particles[7][1] = 4.0;
        particles[7][2] = 0.0;
        particles[7][3] = 0.25;
        particles[7][4] = -0.000001;
        particles[7][5] = 1.0f;

        particles[8][0] = 0.0;
        particles[8][1] = -4.0;
        particles[8][2] = 0.0;
        particles[8][3] = 0.25;
        particles[8][4] = 0.000001;
        particles[8][5] = 1.0f;

        particles[9][0] = 0.0;
        particles[9][1] = -4.0;
        particles[9][2] = 0.0;
        particles[9][3] = 0.25;
        particles[9][4] = -0.000001;
        particles[9][5] = 1.0f;
      }
    }

    if(animation == 505){
      for(i=0; i < particleNumber; i++){//Particle color white if close to each other
        if(fabs(particles[i][0]) < 0.05f){
          particles[i][5] = 1.0f;
        }
        else{
          particles[i][5] = 0.0f;
        }
      }
      if(particles[2][0] > -2.0f + 0.025f){
        particles[2][0] -= 0.025f;
        particles[3][0] += 0.025f;

        particles[4][0] -= 0.025f;
        particles[5][0] += 0.025f;

        particles[6][0] -= 0.025f;
        particles[7][0] += 0.025f;

        particles[8][0] -= 0.025f;
        particles[9][0] += 0.025f;
      }
      else{
        animation++;
        printf("Animation set to %d\n", animation);
        pauseFrames = 0;
        
        //particles[0][0] = -2.0f;
        //particles[1][0] = 2.0f;
      }
    }

    if(animation == 506){
      if(particles[2][0] > -2.5f + 0.01f){
        particles[2][0] -= 0.01f;
        particles[2][1] -= 2.0f / 0.5f * 0.01f;

        particles[3][0] += 0.01f;
        particles[3][1] -= 2.0f / 0.5f * 0.01f;

        particles[4][0] -= 0.01f;
        particles[4][1] += 1.5f / 0.5f * 0.01f;

        particles[5][0] += 0.01f;
        particles[5][1] += 1.5f / 0.5f * 0.01f;

        particles[6][0] -= 0.01f;
        particles[6][1] -= 3.5f / 0.5f * 0.01f;

        particles[7][0] += 0.01f;
        particles[7][1] -= 3.5f / 0.5f * 0.01f;

        particles[8][0] -= 1.0f / 0.5f * 0.01f;
        particles[8][1] += 4.0f / 0.5f * 0.01f;

        particles[9][0] += 1.0f / 0.5f * 0.01f;
        particles[9][1] += 4.0f / 0.5f * 0.01f;
      }
      else{
        animation++;
        printf("Animation set to %d\n", animation);
        pauseFrames = 0;
      }
    }

    if(animation == 507){
      if(pauseFrames < 10){
        pauseFrames++;
      }
      else{
        animation++;
        printf("Animation set to %d\n", animation);
      }
    }

    if(animation == 508){//Arrange particles to chains
      if(particles[0][1] < 1.0f - 0.025f){
        particles[0][1] += 0.025f;
        particles[1][1] += 0.025f;
        
        particles[8][1] -= 0.025f;
        particles[9][1] -= 0.025f;
      }
      else{
        animation++;
        printf("Animation set to %d\n", animation);
      }
    }

    if(animation == 509){
      if(particles[0][0] > -2.5f + 0.025f){
        particles[0][0] -= 0.025f;
        particles[1][0] += 0.025f;
        
        particles[8][0] += 0.025f;
        particles[9][0] -= 0.025f;
      }
      else{
        animation++;
        printf("Animation set to %d\n", animation);
        for(i = 0; i < particleNumber - 5; i++){
          particles[i][1] = 0.5f * (double)(i) - 1.0f;
          particles[i + 5][1] = 0.5f * (double)(i) - 1.0f;
        }
      }
    }

    if(animation == 510){
      if(particles[0][0] < -2.3f + 0.01f){
        for(i=0; i<particleNumber; i += 2){
          particles[i][0] += 0.01f;
          particles[i+1][0] -= 0.01f;
        }        
      }
      else{
        animation++;
        printf("Animation set to %d\n", animation);

        pauseFrames = 0;
        for(i=0; i<particleNumber; i += 2){
          particles[i][0] = -2.3f;
          particles[i+1][0] = 2.3f;
        }        
      }
    }

    if(animation == 511){
      if(pauseFrames < 10){
        pauseFrames++;
      }
      else{
        animation++;
        printf("Animation set to %d\n", animation);
        
        particleNumber += 4;
        
        particles[10][0] = 0.0f;
        particles[10][1] = 1.5f;
        particles[10][2] = 0.0f;
        particles[10][3] = 0.25;
        particles[10][4] = 0.000001;

        particles[11][0] = 0.0f;
        particles[11][1] = 1.5f;
        particles[11][2] = 0.0f;
        particles[11][3] = 0.25;
        particles[11][4] = -0.000001;

        particles[12][0] = 0.0f;
        particles[12][1] = -1.5f;
        particles[12][2] = 0.0f;
        particles[12][3] = 0.25;
        particles[12][4] = 0.000001;

        particles[13][0] = 0.0f;
        particles[13][1] = -1.5f;
        particles[13][2] = 0.0f;
        particles[13][3] = 0.25;
        particles[13][4] = -0.000001;
        
      }
    }

    for(i=512; i<520; i+=2){
      if(animation == i){
        if(fabs(particles[particleNumber-4][0]) < 0.05f){//Particles color is white if close to each other
          particles[particleNumber-1][5] = 1.0f;
          particles[particleNumber-2][5] = 1.0f;
          particles[particleNumber-3][5] = 1.0f;
          particles[particleNumber-4][5] = 1.0f;
        }
        else{
          particles[particleNumber-1][5] = 0.0f;
          particles[particleNumber-2][5] = 0.0f;
          particles[particleNumber-3][5] = 0.0f;
          particles[particleNumber-4][5] = 0.0f;
        }
        if(particles[particleNumber-4][0] > -2.3f + 0.025f){
          particles[particleNumber-4][0] -= 0.025f;
          particles[particleNumber-3][0] += 0.025f;
          particles[particleNumber-2][0] -= 0.025f;
          particles[particleNumber-1][0] += 0.025f;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          pauseFrames = 0;

          particles[particleNumber-4][0] = -2.3f;
          particles[particleNumber-3][0] = 2.3f;
          particles[particleNumber-2][0] = -2.3f;
          particles[particleNumber-1][0] = 2.3f;
        }
      }
    }
    for(i=513; i<519; i+=2){
      if(animation == i){
        if(pauseFrames < 10){
          pauseFrames++;
        }
        else{
          animation++;
          printf("Animation set to %d\n", animation);
          
          particleNumber+=4;

          particles[particleNumber-4][0] = 0.0f;
          particles[particleNumber-4][1] = particles[particleNumber-8][1]+0.5f;
          particles[particleNumber-4][2] = 0.0f;
          particles[particleNumber-4][3] = 0.25;
          particles[particleNumber-4][4] = 0.000001;

          particles[particleNumber-3][0] = 0.0f;
          particles[particleNumber-3][1] = particles[particleNumber-8][1]+0.5f;
          particles[particleNumber-3][2] = 0.0f;
          particles[particleNumber-3][3] = 0.25;
          particles[particleNumber-3][4] = -0.000001;

          particles[particleNumber-2][0] = 0.0f;
          particles[particleNumber-2][1] = particles[particleNumber-6][1]-0.5f;
          particles[particleNumber-2][2] = 0.0f;
          particles[particleNumber-2][3] = 0.25;
          particles[particleNumber-2][4] = 0.000001;

          particles[particleNumber-1][0] = 0.0f;
          particles[particleNumber-1][1] = particles[particleNumber-6][1]-0.5f;
          particles[particleNumber-1][2] = 0.0f;
          particles[particleNumber-1][3] = 0.25;
          particles[particleNumber-1][4] = -0.000001;

        }
      }
    }
    
    if(animation == 519){//Test charge moving at inner area of charges
      moveAlpha = 0.0f;
      particles[particleNumber][0] = 1.5f;
      particles[particleNumber][1] = 0.0f;
      particles[particleNumber][2] = 0.0f;
      particles[particleNumber][3] = 0.20f;
      particles[particleNumber][4] = 0.000001;
      particleNumber++;
      
      animation++;
      printf("Animation set to %d\n", animation);
      enableFieldVectors = 10;
      scalePlaneHeigth = 200.0f;
      pathEnd[0] = 0;
      pathEnd[1] = 0;
      pathColor[0] = 1;
      pathColor[1] = 2;

      minValue1=10000.0f;
      maxValue1=-10000.0f;
      minValue2=10000.0f;
      maxValue2=-10000.0f;
    }

    if(animation == 520){
      if(moveAlpha < 4.0f * PI - 0.01f){
        moveAlpha += 0.01f;
        particles[particleNumber-1][0] = 1.5f * cos(moveAlpha);
        if(moveAlpha < 2.0f * PI){
          particles[particleNumber-1][1] = 1.5f * sin(moveAlpha) * 0.7f;
          path[0][pathEnd[0]][0] = 1.5f * cos(moveAlpha);
          path[0][pathEnd[0]][1] = 1.5f * sin(moveAlpha) * 0.7f;
          path[0][pathEnd[0]][2] = 0.0f;
          pathEnd[0]++;
        }
        else{
          if(minValue2 == 10000.0f){
            minValue2 = 9999.9f;
          }
          particles[particleNumber-1][1] = 1.5f * sin(moveAlpha) * 2.0f;
          path[1][pathEnd[1]][0] = 1.5f * cos(moveAlpha);
          path[1][pathEnd[1]][1] = 1.5f * sin(moveAlpha) * 2.0f;
          path[1][pathEnd[1]][2] = 0.0f;
          pathEnd[1]++;
        }
      }
      else{
        animation++;
        printf("Animation set to %d\n", animation);
        pauseFrames = 0;
        pathEnd[0] = 0;
        pathEnd[1] = 0;
        pathColor[0] = 1;
        pathColor[1] = 2;
      }
    }

    if(animation == 521){
      if(pauseFrames < 10){
        pauseFrames++;
      }
      else{
        animation++;
        printf("Animation set to %d\n", animation);
        enableFieldVectors = 2; 
        particleNumber--;

        translateXtarget = 0.0f;
        translateYtarget = 0.0f;
        translateZtarget = -10.0f;
        
        rotXtarget = -72.75f;
        rotYtarget = 0.0f;
        rotZtarget = 10.75f;

        translateXdiff = (translateXtarget - translateX) / steps * 2.0;
        translateYdiff = (translateYtarget - translateY) / steps * 2.0;
        translateZdiff = (translateZtarget - translateZ) / steps * 2.0;
        rotXdiff = (rotXtarget - rotX) / steps * 2.0;
        rotYdiff = (rotYtarget - rotY) / steps * 2.0;
        rotZdiff = (rotZtarget - rotZ) / steps * 2.0;

      }
    }


    if(animation == 522){
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 523;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 523;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 523;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 523;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 523;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 523;
      }
      if(animation == 523){
        printf("Animation set to %d\n", animation);
        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
        rotXdiff = 0.0f;
        rotYdiff = 0.0f;
        rotZdiff = 0.0f;

        for(i2 = 0; i2 < 6; i2++){
          for(i = particleNumber; i < particleNumber + 13 * 2; i += 2){
            particles[i][0] = -2.3f;
            particles[i][1] = 0.5f * (i - particleNumber) / 2.0f - 3.0f;
            particles[i][2] = -10.0f - 0.5f * (GLfloat)(i2);
            particles[i][3] = 0.25f;
            particles[i][4] = 0.000001f;

            particles[i+1][0] = 2.3f;
            particles[i+1][1] = 0.5f * (i - particleNumber) / 2.0f - 3.0f;
            particles[i+1][2] = -10.0f - 0.5f * (GLfloat)(i2);
            particles[i+1][3] = 0.25f;
            particles[i+1][4] = -0.000001f;
          }
          particleNumber += 13 * 2;

          for(i = particleNumber; i < particleNumber + 13 * 2; i += 2){
            particles[i][0] = -2.3f;
            particles[i][1] = 0.5f * (i - particleNumber) / 2.0f - 3.0f;
            particles[i][2] = 10.0f + 0.5f * (GLfloat)(i2);
            particles[i][3] = 0.25f;
            particles[i][4] = 0.000001f;

            particles[i+1][0] = 2.3f;
            particles[i+1][1] = 0.5f * (i - particleNumber) / 2.0f - 3.0f;
            particles[i+1][2] = 10.0f + 0.5f * (GLfloat)(i2);
            particles[i+1][3] = 0.25f;
            particles[i+1][4] = -0.000001f;
          }
          particleNumber += 13 * 2;
        }

      }
    }
    
    if(animation == 523){//Add rows of charged particles
      if(particles[particleNumber-1][2] > 3.0){
        for(i = particleNumber - 13 * 2 * 2 * 6; i < particleNumber; i++){
          if(particles[i][2] > 0.0f){
            particles[i][2] -= 0.025f;
          }
          else{
            particles[i][2] += 0.025f;
          }
        }
      }
      else{
        animation++;
        printf("Animation set to %d\n", animation);        

        translateXtarget = 0.0f;
        translateYtarget = 0.0f;
        translateZtarget = -10.0f;
        
        rotXtarget = 0.0f;
        rotYtarget = 0.0f;
        rotZtarget = 0.0f;

        translateXdiff = (translateXtarget - translateX) / steps * 2.0;
        translateYdiff = (translateYtarget - translateY) / steps * 2.0;
        translateZdiff = (translateZtarget - translateZ) / steps * 2.0;
        rotXdiff = (rotXtarget - rotX) / steps * 2.0;
        rotYdiff = (rotYtarget - rotY) / steps * 2.0;
        rotZdiff = (rotZtarget - rotZ) / steps * 2.0;
      }
    }

    if(animation == 524){
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 525;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 525;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 525;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 525;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 525;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 525;
      }
      if(animation == 525){
        printf("Animation set to %d\n", animation);
        pauseFrames = 0;
        
        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
        rotXdiff = 0.0f;
        rotYdiff = 0.0f;
        rotZdiff = 0.0f;
      }
    }

    if(animation == 525){
      if(pauseFrames < 10){
        pauseFrames++;
      }
      else{
        animation++;
        printf("Animation set to %d\n", animation);
        
        moveAlpha = 0.0f;
        particles[particleNumber][0] = 1.5f;
        particles[particleNumber][1] = 0.0f;
        particles[particleNumber][2] = 0.0f;
        particles[particleNumber][3] = 0.20f;
        particles[particleNumber][4] = 0.000001;
        particleNumber++;
        enableFieldVectors = 11;

        pathEnd[0] = 0;
        pathEnd[1] = 0;
        pathColor[0] = 1;
        pathColor[1] = 2;

        minValue1=10000.0f;
        maxValue1=-10000.0f;
        minValue2=10000.0f;
        maxValue2=-10000.0f;

      }
    }


    if(animation == 526){
      if(moveAlpha < 4.0f * PI - 0.01f){
        moveAlpha += 0.05f;
        particles[particleNumber-1][0] = 1.5f * cos(moveAlpha);
        if(moveAlpha < 2.0f * PI){
          particles[particleNumber-1][1] = 1.5f * sin(moveAlpha) * 0.7f;
          path[0][pathEnd[0]][0] = 1.5f * cos(moveAlpha);
          path[0][pathEnd[0]][1] = 1.5f * sin(moveAlpha) * 0.7f;
          path[0][pathEnd[0]][2] = 0.0f;
          pathEnd[0]++;
        }
        else{
          if(minValue2 == 10000.0f){
            minValue2 = 9999.9f;
          }
          particles[particleNumber-1][1] = 1.5f * sin(moveAlpha) * 2.0f;
          path[1][pathEnd[1]][0] = 1.5f * cos(moveAlpha);
          path[1][pathEnd[1]][1] = 1.5f * sin(moveAlpha) * 2.0f;
          path[1][pathEnd[1]][2] = 0.0f;
          pathEnd[1]++;
        }
      }
      else{
        animation++;
        printf("Animation set to %d\n", animation);
        pauseFrames = 0;
        enableFieldVectors = 0;
        particleNumber--;

        pathEnd[0] = 0;
        pathEnd[1] = 0;
        pathColor[0] = 1;
        pathColor[1] = 2;

        minValue1=10000.0f;
        maxValue1=-10000.0f;
        minValue2=10000.0f;
        maxValue2=-10000.0f;
      }
    }


    if(animation == 527){
      if(particles[0][0] < -1.5){
        for(i = 0; i < particleNumber; i++){
          if(particles[i][0] > 0.0f){
            particles[i][0] -= 0.01f;
          }
          else{
            particles[i][0] += 0.01f;
          }
        }
      }
      else{
        animation++;
        printf("Animation set to %d\n", animation);
        pauseFrames = 0;
        moveAlpha = 0.0f;
        enableFieldVectors = 11;
        particleNumber++;
      }
    }

    if(animation == 528){
      if(moveAlpha < 4.0f * PI - 0.01f){
        moveAlpha += 0.01f;
        particles[particleNumber-1][0] = 1.0f * cos(moveAlpha);
        if(moveAlpha < 2.0f * PI){
          particles[particleNumber-1][1] = 1.0f * sin(moveAlpha) * 0.7f;
          path[0][pathEnd[0]][0] = 1.0f * cos(moveAlpha);
          path[0][pathEnd[0]][1] = 1.0f * sin(moveAlpha) * 0.7f;
          path[0][pathEnd[0]][2] = 0.0f;
          pathEnd[0]++;
        }
        else{
          if(minValue2 == 10000.0f){
            minValue2 = 9999.9f;
          }
          particles[particleNumber-1][1] = 1.0f * sin(moveAlpha) * 2.0f;
          path[1][pathEnd[1]][0] = 1.0f * cos(moveAlpha);
          path[1][pathEnd[1]][1] = 1.0f * sin(moveAlpha) * 2.0f;
          path[1][pathEnd[1]][2] = 0.0f;
          pathEnd[1]++;
        }
      }
      else{
        animation++;
        printf("Animation set to %d\n", animation);
        
        printf("...un ferddisch!\n");
        done = True;         // Exit programm !!!
      }
    }






//####################################################################################
//############# Animation sequence potential plane of two charges ####################
//####################################################################################

    if(animation == 400){
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 401;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 401;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 401;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 401;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 401;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 401;
      }
      if(animation == 401){
        printf("Animation set to 401.\n");
        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
        rotXdiff = 0.0f;
        rotYdiff = 0.0f;
        rotZdiff = 0.0f;
      }
    }


    if(animation == 401){      
      if(particles[0][0] > -3.0){
        particles[0][0] -= 0.01;
      }
      else{
        animation = 402;
        printf("Animation set to 402.\n");
        particles[0][0] = -3.0;
        potentialEnd = 2;
        particleNumber = 2;
        particles[1][4] = 0.0f;
      }
    }

    if(animation == 402){//Fade in negative charge
      if(particles[1][4] > -0.000001){
        particles[1][4] -= 0.00000001;
      }
      else{
        animation = 403;
        printf("Animation set to 403.\n");
        particles[1][4] = -0.000001;
        
        rotYtarget = rotY - 360;;
        rotYdiff = (rotYtarget - rotY) / steps / 2.0;
      }
    }

    if(animation == 403){
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 404;
      }
      if(animation == 404){
        printf("Animation set to 404.\n");
        rotYdiff = 0.0f;

        translateXtarget = 0.0f;
        translateYtarget = -0.59f;
        translateZtarget = -14.8f;
        
        rotXtarget = -2.25f;
        rotYtarget = 0.9f;
        rotZtarget = 0.0f;

        translateXdiff = (translateXtarget - translateX) / steps * 2.0;
        translateYdiff = (translateYtarget - translateY) / steps * 2.0;
        translateZdiff = (translateZtarget - translateZ) / steps * 2.0;
        rotXdiff = (rotXtarget - rotX) / steps * 2.0;
        rotYdiff = (rotYtarget - rotY) / steps * 2.0;
        rotZdiff = (rotZtarget - rotZ) / steps * 2.0;
      }
    }

    if(animation == 404){
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 405;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 405;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 405;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 405;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 405;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 405;
      }
      if(animation == 405){
        printf("Animation set to 405.\n");

        translateXtarget = 0.0f;
        translateYtarget = -0.59f;
        translateZtarget = -14.8f;
        
        rotXtarget = 40.25f;
        rotYtarget = 0.9f;
        rotZtarget = 0.0f;

        translateXdiff = (translateXtarget - translateX) / steps * 2.0;
        translateYdiff = (translateYtarget - translateY) / steps * 2.0;
        translateZdiff = (translateZtarget - translateZ) / steps * 2.0;
        rotXdiff = (rotXtarget - rotX) / steps * 2.0;
        rotYdiff = (rotYtarget - rotY) / steps * 2.0;
        rotZdiff = (rotZtarget - rotZ) / steps * 2.0;
      }
    }

    if(animation == 405){
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 406;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 406;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 406;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 406;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 406;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 406;
      }
      if(animation == 406){
        printf("Animation set to 406.\n");

        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
        rotXdiff = 0.0f;
        rotYdiff = 0.0f;
        rotZdiff = 0.0f;
        
        path[1][0][0] = 1.0;
        path[1][0][1] = 0.0;
        path[1][0][2] = 0.0;
        pathEnd[1] = 0;

        path[2][0][0] = -1.0;
        path[2][0][1] = 0.0;
        path[2][0][2] = 0.0;
        pathEnd[2] = 0;
        

        path[0][0][1] = -1000.0f;
        path[1][0][1] = -1000.0f;
        path[2][0][1] = -1000.0f;
        path[3][0][1] = -1000.0f;
        path[4][0][1] = -1000.0f;
        path[5][0][1] = -1000.0f;
        path[6][0][1] = -1000.0f;
        path[7][0][1] = -1000.0f;
        path[8][0][1] = -1000.0f;
        path[9][0][1] = -1000.0f;
        pathEnd[0] = 0;
        pathEnd[1] = 0;
        pathEnd[2] = 0;
        pathEnd[3] = 0;
        pathEnd[4] = 0;
        pathEnd[5] = 0;
        pathEnd[6] = 0;
        pathEnd[7] = 0;
        pathEnd[8] = 0;
        pathEnd[9] = 0;
        for(x1 = -3.0; x1 < 3.0; x1 += 0.0001){//Find staring points of equipotential lines
          potential = 0.0;
          for(i=0; i < potentialEnd; i++){
            totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
              potential = k1 * particles[i][4] * particles[potentialEnd][4] / totalDistance;
              break;
            }
            potential += k1 * particles[i][4] * particles[potentialEnd][4] / totalDistance;
          }//for(i=0; i < potentialEnd; i++){
          potential = potential * scalePlaneHeigth;
          //printf("potential=%lf\n", potential);
          if(potential < 2.25 && path[0][0][1] == -1000.0f){
            printf("potential %lf at %lf\n", potential, x1);
            path[0][0][0] = x1;
            path[0][0][1] = 2.25;
            path[0][0][2] = 0.0;
          }
          if(potential < 1.75 && path[1][0][1] == -1000.0f){
            printf("potential %lf at %lf\n", potential, x1);
            path[1][0][0] = x1;
            path[1][0][1] = 1.75;
            path[1][0][2] = 0.0;
          }
          if(potential < 1.25 && path[2][0][1] == -1000.0f){
            printf("potential %lf at %lf\n", potential, x1);
            path[2][0][0] = x1;
            path[2][0][1] = 1.25;
            path[2][0][2] = 0.0;
          }
          if(potential < 0.75 && path[3][0][1] == -1000.0f){
            printf("potential %lf at %lf\n", potential, x1);
            path[3][0][0] = x1;
            path[3][0][1] = 0.75;
            path[3][0][2] = 0.0;
          }
          if(potential < 0.25 && path[4][0][1] == -1000.0f){
            printf("potential %lf at %lf\n", potential, x1);
            path[4][0][0] = x1;
            path[4][0][1] = 0.25;
            path[4][0][2] = 0.0;
          }
          if(potential < -0.25 && path[5][0][1] == -1000.0f){
            printf("potential %lf at %lf\n", potential, x1);
            path[5][0][0] = x1;
            path[5][0][1] = -0.25;
            path[5][0][2] = 0.0;
          }
          if(potential < -0.75 && path[6][0][1] == -1000.0f){
            printf("potential %lf at %lf\n", potential, x1);
            path[6][0][0] = x1;
            path[6][0][1] = -0.75;
            path[6][0][2] = 0.0;
          }
          if(potential < -1.25 && path[7][0][1] == -1000.0f){
            printf("potential %lf at %lf\n", potential, x1);
            path[7][0][0] = x1;
            path[7][0][1] = -1.25;
            path[7][0][2] = 0.0;
          }
          if(potential < -1.75 && path[8][0][1] == -1000.0f){
            printf("potential %lf at %lf\n", potential, x1);
            path[8][0][0] = x1;
            path[8][0][1] = -1.75;
            path[8][0][2] = 0.0;
          }
          if(potential < -2.25 && path[9][0][1] == -1000.0f){
            printf("potential %lf at %lf\n", potential, x1);
            path[9][0][0] = x1;
            path[9][0][1] = -2.25;
            path[9][0][2] = 0.0;
          }
        }
        
        
        
      }
    }

    if(animation == 406){//Draw equipotential lines
      GLfloat pathStep = 0.1;
      GLfloat startPotential, newPotential;
      GLfloat alphaStart, alphaEnd;
      GLfloat beta;
      int pathNumber;
      
      for(pathNumber = 0; pathNumber < 10; pathNumber++){
        potential = 0.0;
        for(i=0; i < potentialEnd; i++){
          //printf("path[1][0][0] = %lf, path[1][0][2] = %lf, particles[i][0] = %lf, particles[i][2] = %lf\n", path[1][0][0], path[1][0][2], particles[i][0], particles[i][2]);
          totalDistance = sqrt(pow(path[pathNumber][0][0]-particles[i][0], 2.0) + pow(path[pathNumber][0][2]-particles[i][2], 2.0));
          if(totalDistance < minDistance){
            totalDistance = minDistance;
            potential = k1 * particles[i][4] * particles[potentialEnd][4] / totalDistance;
            break;
          }
          potential += k1 * particles[i][4] * particles[potentialEnd][4] / totalDistance;
          //printf("totalDistance = %lf, potential = %lf\n", totalDistance, potential);
        }//for(i=0; i < potentialEnd; i++){
        startPotential = potential * 200.0f;
        
        if(pathEnd[pathNumber] < 1){
          alphaStart = 0.0;
          alphaEnd = 3.1415927;
        }
        else{
          beta = atan2(path[pathNumber][pathEnd[1]][2] - path[pathNumber][pathEnd[pathNumber]-1][2], path[pathNumber][pathEnd[pathNumber]][0] - path[pathNumber][pathEnd[pathNumber]-1][0]);
          //printf("%lf, %lf\n", path[1][pathEnd[1]][2] - path[1][pathEnd[1]-1][2], path[1][pathEnd[1]][0] - path[1][pathEnd[1]-1][0]);
          alphaStart = beta - 3.1415927 / 2.0;
          alphaEnd = beta + 3.1415927 / 2.0;
          //printf("startPotential = %lf, path[1][pathEnd[1]][0] = %lf, path[1][pathEnd[1]][2] = %lf, beta = %lf, alphaStart = %lf, alphaEnd = %lf\n", startPotential, path[1][pathEnd[1]][0], path[1][pathEnd[1]][2], beta, alphaStart, alphaEnd);
        }
        newPotential = 1000000;
        for(alpha = alphaStart; alpha < alphaEnd; alpha += 0.1){
          x1 = cos(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][0];
          z1 = sin(alpha) * pathStep + path[pathNumber][pathEnd[pathNumber]][2];
          potential = 0.0;
          for(i=0; i < potentialEnd; i++){
            totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(z1-particles[i][2], 2.0));
            if(totalDistance < minDistance){
              totalDistance = minDistance;
              potential = k1 * particles[i][4] * particles[potentialEnd][4] / minDistance;
              break;
            }
            potential += k1 * particles[i][4] * particles[potentialEnd][4] / totalDistance;
          }//for(i=0; i < potentialEnd; i++){
          potential = potential * 200.0f;
          //printf("x1 = %lf, z1 = %lf, totalDistance = %lf, potential = %lf\n", x1, z1, totalDistance, potential);
          //printf("abs(startPotential - potential) < abs(startPotential - newPotential): %lf < %lf\n", fabs(startPotential - potential), fabs(startPotential - newPotential));
          if(fabs(startPotential - potential) < fabs(startPotential - newPotential)){
            newPotential = potential;
            path[pathNumber][pathEnd[pathNumber]+1][0] = x1;
            path[pathNumber][pathEnd[pathNumber]+1][1] = potential;
            path[pathNumber][pathEnd[pathNumber]+1][2] = z1;
            //printf("newPotential = %lf\n", newPotential);
          }    
        }
        //path[pathNumber][0][1] = path[pathNumber][1][1];//Approximation for initial potential
      }//for(pathNumber = 1, pathNumber < 3; pathNumber++){

      if(pathEnd[1] < 300){
        for(pathNumber = 0; pathNumber < 10; pathNumber++){
          pathEnd[pathNumber]++;
        }
      }
      else{
        animation = 407;
        printf("Animation set to 407.\n");
      }
    }
    
    if(animation == 407){//Flatten plane to 2D
      if(minDistance > 0.30){
        minDistance -= 0.002;
      }

      if(lightAmbient1[0] > 0.4){
        lightAmbient1[0] -= 0.01f;
        lightAmbient1[1] -= 0.01f;
        lightAmbient1[2] -= 0.01f;

        lightDiffuse1[0] -= 0.01f;
        lightDiffuse1[1] -= 0.01f;
        lightDiffuse1[2] -= 0.01f;
        
        if(specular1[0] > 0.0){
          specular1[0] -= 0.02f;
          specular1[1] -= 0.02f;
          specular1[2] -= 0.02f;
        }
        else{
          specular1[0] = 0.0f;
          specular1[1] = 0.0f;
          specular1[2] = 0.0f;
        }
      }
      
      if(planeOffset > -0.1){
        planeOffset -= 0.01f;
      }
      if(axeYColor[1] > 0.0){
        axeYColor[1] -= 0.02f;
        axeYColor[2] -= 0.02f;
      }
      else{
        axeYColor[1] = 0.0f;
        axeYColor[2] = 0.0f;
      }
      
      if(scalePlaneHeigth > 0.0){
        scalePlaneHeigth -= 1.0; 
      }
      else{
        scalePlaneHeigth = 0.0f;
        animation = 408;
        printf("Animation set to 408.\n");
        
        translateXtarget = 0.08f;
        translateYtarget = -0.09f;
        translateZtarget = -10.65f;
        
        rotXtarget = 90.0f;
        rotYtarget = 0.0f;
        rotZtarget = 0.0f;

        translateXdiff = (translateXtarget - translateX) / steps;
        translateYdiff = (translateYtarget - translateY) / steps;
        translateZdiff = (translateZtarget - translateZ) / steps;
        rotXdiff = (rotXtarget - rotX) / steps;
        rotYdiff = (rotYtarget - rotY) / steps;
        rotZdiff = (rotZtarget - rotZ) / steps;
      }
    }

    if(animation == 408){
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 409;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 409;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 409;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 409;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 409;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 409;
      }
      if(animation == 409){
        printf("Animation set to 409.\n");
        translateXtarget = -0.39f;
        translateYtarget = -1.23f;
        translateZtarget = -14.75f;
        
        rotXtarget = 28.16f;
        rotYtarget = -11.75f;
        rotZtarget = 0.0f;

        translateXdiff = (translateXtarget - translateX) / steps;
        translateYdiff = (translateYtarget - translateY) / steps;
        translateZdiff = (translateZtarget - translateZ) / steps;
        rotXdiff = (rotXtarget - rotX) / steps;
        rotYdiff = (rotYtarget - rotY) / steps;
        rotZdiff = (rotZtarget - rotZ) / steps;


      }
    }

    if(animation == 409){
      if(minDistance < 0.39f){
        minDistance += 0.01;
      }
      else{
        minDistance = 0.39f;
      }
      
      if(lightAmbient1[0] < 0.7){
        lightAmbient1[0] += 0.01f;
        lightAmbient1[1] += 0.01f;
        lightAmbient1[2] += 0.01f;

        lightDiffuse1[0] += 0.01f;
        lightDiffuse1[1] += 0.01f;
        lightDiffuse1[2] += 0.01f;
        
        if(specular1[0] < 0.9){
          specular1[0] += 0.02f;
          specular1[1] += 0.02f;
          specular1[2] += 0.02f;
        }
        else{
          specular1[0] = 0.9f;
          specular1[1] = 0.9f;
          specular1[2] = 0.9f;
        }
      }
      else{
        lightAmbient1[0] = 0.7f;
        lightAmbient1[1] = 0.7f;
        lightAmbient1[2] = 0.7f;

        lightDiffuse1[0] = 0.7f;
        lightDiffuse1[1] = 0.7f;
        lightDiffuse1[2] = 0.7f;
      }
      
      if(planeOffset < 0.0){
        planeOffset += 0.01f;
      }
      else{
        planeOffset = 0.0f;
      }
      if(axeYColor[1] < 1.0){
        axeYColor[1] += 0.02f;
        axeYColor[2] += 0.02f;
      }
      else{
        axeYColor[1] = 1.0f;
        axeYColor[2] = 1.0f;
      }
      
      if(scalePlaneHeigth < 200.0){
        scalePlaneHeigth += 1.0; 
      }
      else{
        scalePlaneHeigth = 200.0f;
        animation = 410;
        printf("Animation set to 410.\n");        
      }
    }

    if(animation == 410){
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 411;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 411;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 411;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 411;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 411;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 411;
      }
      if(animation == 411){
        printf("Animation set to 411.\n");
        enablePotentialSphere = 1;
        particleNumber = 3;
        
        particles[2][0] = -2.9f;
        particles[2][1] = 0.0f;
        particles[2][2] = 0.5f;
        
        path[10][0][0] = particles[2][0];
        path[10][0][1] = particles[2][1];
        path[10][0][2] = particles[2][2];
        pathEnd[10] = 0;
        pathColor[10] = 1;

        pathEnd[11] = 0;
        pathColor[11] = 2;
        
        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
        rotXdiff = 0.0f;
        rotYdiff = 0.0f;
        rotZdiff = 0.0f;
      }
    }
    
    if(animation == 411){//Potential sphere rolling on plane from positive to negative charge

      fieldLineStep = 0.01f;
      if(particles[2][0] < 2.9f){
        totalDistance = sqrt(pow(particles[2][0] - particles[0][0], 2.0) + pow(particles[2][1] - particles[0][1], 2.0) + pow(particles[2][2] - particles[0][2], 2.0));
        totalForce = k * particles[0][4] / pow(totalDistance, 2.0);
        forceX = (particles[2][0] - particles[0][0]) / totalDistance * totalForce;
        forceY = (particles[2][1] - particles[0][1]) / totalDistance * totalForce;
        forceZ = (particles[2][2] - particles[0][2]) / totalDistance * totalForce;

        totalDistance = sqrt(pow(particles[2][0] - particles[1][0], 2.0) + pow(particles[2][1] - particles[1][1], 2.0) + pow(particles[2][2] - particles[1][2], 2.0));
        totalForce = k * particles[1][4] / pow(totalDistance, 2.0);
        forceX += (particles[2][0] - particles[1][0]) / totalDistance * totalForce;
        forceY += (particles[2][1] - particles[1][1]) / totalDistance * totalForce;
        forceZ += (particles[2][2] - particles[1][2]) / totalDistance * totalForce;

        totalForce = sqrt(pow(forceX, 2.0) + pow(forceY, 2.0) + pow(forceZ, 2.0));
        particles[2][0] = particles[2][0] + forceX / totalForce * fieldLineStep;
        particles[2][1] = particles[2][1] + forceY / totalForce * fieldLineStep;
        particles[2][2] = particles[2][2] + forceZ / totalForce * fieldLineStep;

        pathEnd[10]++;
        path[10][pathEnd[10]][0] = particles[2][0];
        path[10][pathEnd[10]][1] = particles[2][1];
        path[10][pathEnd[10]][2] = particles[2][2];
        
        //printf("totalDistance=%lf, totalForce=%lf\n", totalDistance, totalForce);
      }
      else{
        animation = 412;
        printf("Animation set to 412.\n");

        translateXtarget = -0.41f;
        translateYtarget = -1.23f;
        translateZtarget = -14.75f;
        
        rotXtarget = -5.59f;
        rotYtarget = -11.75f;
        rotZtarget = 0.0f;

        translateXdiff = (translateXtarget - translateX) / steps;
        translateYdiff = (translateYtarget - translateY) / steps;
        translateZdiff = (translateZtarget - translateZ) / steps;
        rotXdiff = (rotXtarget - rotX) / steps;
        rotYdiff = (rotYtarget - rotY) / steps;
        rotZdiff = (rotZtarget - rotZ) / steps;
      }
    }

    if(animation == 412){
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 413;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 413;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 413;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 413;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 413;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 413;
      }
      if(animation == 413){
        printf("Animation set to 413.\n");
        
        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
        rotXdiff = 0.0f;
        rotYdiff = 0.0f;
        rotZdiff = 0.0f;
      }
    }


    if(animation == 413){//Potential sphere rolling on plane from negative to positive charge

      fieldLineStep = 0.01f;
      if(particles[2][0] > -2.7f){
        totalDistance = sqrt(pow(particles[2][0] - particles[0][0], 2.0) + pow(particles[2][1] - particles[0][1], 2.0) + pow(particles[2][2] - particles[0][2], 2.0));
        totalForce = k * particles[0][4] / pow(totalDistance, 2.0);
        forceX = (particles[2][0] - particles[0][0]) / totalDistance * totalForce;
        forceY = (particles[2][1] - particles[0][1]) / totalDistance * totalForce;
        forceZ = (particles[2][2] - particles[0][2]) / totalDistance * totalForce;

        totalDistance = sqrt(pow(particles[2][0] - particles[1][0], 2.0) + pow(particles[2][1] - particles[1][1], 2.0) + pow(particles[2][2] - particles[1][2], 2.0));
        totalForce = k * particles[1][4] / pow(totalDistance, 2.0);
        forceX += (particles[2][0] - particles[1][0]) / totalDistance * totalForce;
        forceY += (particles[2][1] - particles[1][1]) / totalDistance * totalForce;
        forceZ += (particles[2][2] - particles[1][2]) / totalDistance * totalForce;

        totalForce = sqrt(pow(forceX, 2.0) + pow(forceY, 2.0) + pow(forceZ, 2.0));
        particles[2][0] = particles[2][0] - forceX / totalForce * fieldLineStep;
        particles[2][1] = particles[2][1] - forceY / totalForce * fieldLineStep;
        particles[2][2] = particles[2][2] - forceZ / totalForce * fieldLineStep;        
        //printf("totalDistance=%lf, totalForce=%lf\n", totalDistance, totalForce);
      }
      else{
        animation = 414;
        printf("Animation set to 414.\n");

        translateXtarget = 1.06f;
        translateYtarget = -0.87f;
        translateZtarget = -16.70f;
        
        rotXtarget = 17.16f;
        rotYtarget = 42.00f;
        rotZtarget = 0.0f;

        translateXdiff = (translateXtarget - translateX) / steps;
        translateYdiff = (translateYtarget - translateY) / steps;
        translateZdiff = (translateZtarget - translateZ) / steps;
        rotXdiff = (rotXtarget - rotX) / steps;
        rotYdiff = (rotYtarget - rotY) / steps;
        rotZdiff = (rotZtarget - rotZ) / steps;
      }
    }

    if(animation == 414){
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 415;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 415;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 415;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 415;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 415;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 415;
      }
      if(animation == 415){
        printf("Animation set to 415.\n");

        particles[2][0] = -3.35f;
        particles[2][1] = 0.0f;
        particles[2][2] = 0.0f;
        
        path[12][0][0] = particles[2][0];
        path[12][0][1] = particles[2][1];
        path[12][0][2] = particles[2][2];
        pathEnd[12] = 0;
        pathColor[12] = 1;

        pathEnd[13] = 0;
        pathColor[13] = 2;
        
        
        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
        rotXdiff = 0.0f;
        rotYdiff = 0.0f;
        rotZdiff = 0.0f;
      }
    }

    if(animation == 415){//Potential sphere rolling on plane from positive 

      fieldLineStep = 0.03f;
      if(particles[2][0] > -15.0f){
        totalDistance = sqrt(pow(particles[2][0] - particles[0][0], 2.0) + pow(particles[2][1] - particles[0][1], 2.0) + pow(particles[2][2] - particles[0][2], 2.0));
        totalForce = k * particles[0][4] / pow(totalDistance, 2.0);
        forceX = (particles[2][0] - particles[0][0]) / totalDistance * totalForce;
        forceY = (particles[2][1] - particles[0][1]) / totalDistance * totalForce;
        forceZ = (particles[2][2] - particles[0][2]) / totalDistance * totalForce;

        totalDistance = sqrt(pow(particles[2][0] - particles[1][0], 2.0) + pow(particles[2][1] - particles[1][1], 2.0) + pow(particles[2][2] - particles[1][2], 2.0));
        totalForce = k * particles[1][4] / pow(totalDistance, 2.0);
        forceX += (particles[2][0] - particles[1][0]) / totalDistance * totalForce;
        forceY += (particles[2][1] - particles[1][1]) / totalDistance * totalForce;
        forceZ += (particles[2][2] - particles[1][2]) / totalDistance * totalForce;

        totalForce = sqrt(pow(forceX, 2.0) + pow(forceY, 2.0) + pow(forceZ, 2.0));
        particles[2][0] = particles[2][0] + forceX / totalForce * fieldLineStep;
        particles[2][1] = particles[2][1] + forceY / totalForce * fieldLineStep;
        particles[2][2] = particles[2][2] + forceZ / totalForce * fieldLineStep;

        pathEnd[12]++;
        path[12][pathEnd[12]][0] = particles[2][0];
        path[12][pathEnd[12]][1] = particles[2][1];
        path[12][pathEnd[12]][2] = particles[2][2];
        
        //printf("totalDistance=%lf, totalForce=%lf\n", totalDistance, totalForce);
      }
      else{
        animation = 416;
        printf("Animation set to 416.\n");
        
        translateXtarget = 0.08f;
        translateYtarget = -0.87f;
        translateZtarget = -13.9f;
        
        rotXtarget = -3.84f;
        rotYtarget = 0.75f;
        rotZtarget = 0.0f;

        translateXdiff = (translateXtarget - translateX) / steps;
        translateYdiff = (translateYtarget - translateY) / steps;
        translateZdiff = (translateZtarget - translateZ) / steps;
        rotXdiff = (rotXtarget - rotX) / steps;
        rotYdiff = (rotYtarget - rotY) / steps;
        rotZdiff = (rotZtarget - rotZ) / steps;

      }
    }

    if(animation == 416){
      for(i=0; i< 20; i++){
        if(pathEnd[i] > 0){
          pathEnd[i] -= 5;
        }
      }

      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 417;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 417;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 417;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 417;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 417;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 417;
      }
      if(animation == 417){
        printf("Animation set to 417.\n");        
        
        for(i=0; i< 20; i++){
          pathEnd[i] = 0;
        }
                
        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
        rotXdiff = 0.0f;
        rotYdiff = 0.0f;
        rotZdiff = 0.0f;            
      }
    }

    if(animation == 417){
      if(particles[1][4] < 0.0f){
        particles[1][4] += 0.00000001f;
      }
      else{
        animation = 418;
        printf("Animation set to 418.\n");        
        particles[1][4] = 0.000001;
        potentialEnd = 1;
        particleNumber = 1;
      }
    }
    
    if(animation == 418){
      if(lightAmbient1[0] < 1.0){
        lightAmbient1[0] += 0.01f;
        lightAmbient1[1] += 0.01f;
        lightAmbient1[2] += 0.01f;

        lightDiffuse1[0] += 0.01f;
        lightDiffuse1[1] += 0.01f;
        lightDiffuse1[2] += 0.01f;
        
      }
      else{
        animation = 419;
        printf("Animation set to 419.\n");        

        lightAmbient1[0] = 1.0f;
        lightAmbient1[1] = 1.0f;
        lightAmbient1[2] = 1.0f;

        lightDiffuse1[0] = 1.0f;
        lightDiffuse1[1] = 1.0f;
        lightDiffuse1[2] = 1.0f;

        particles[1][4] = 0.0;
        potentialEnd = 2;
        particleNumber = 2;

      }
    }

    if(animation == 419){
      if(particles[1][4] > -0.000001f){
        particles[1][4] -= 0.00000001f;
      }
      else{
        animation = 420;
        printf("Animation set to 420.\n");        
        particles[1][4] = -0.000001;
      }
    }

    if(animation == 420){//Reducing minDistance while potential at side view
      if(minDistance > 0.1f){
        minDistance -= 0.005f;
      }
      else{
        animation = 421;
        printf("Animation set to 421.\n");        
        particles[1][4] = -0.000001;
        enablePotentialPoints = 1;
      }
    }


    if(animation == 421){
      potentialPoints[0][0] = particles[0][0] - 4.0f;
      potentialPoints[1][0] = particles[0][0] - 2.0f;
      potentialPoints[2][0] = particles[0][0] - 1.0f;
      potentialPoints[3][0] = particles[0][0] - 0.5f;
      potentialPoints[4][0] = particles[0][0] + 0.49995f;
      potentialPoints[5][0] = particles[1][0] - 0.49995f;
      potentialPoints[6][0] = particles[1][0] + 0.5f;
      potentialPoints[7][0] = particles[1][0] + 1.0f;
      potentialPoints[8][0] = particles[1][0] + 2.0f;
      potentialPoints[9][0] = particles[1][0] + 4.0f;
      if(particles[0][0] < -0.5f){
        particles[0][0] += 0.005f;
        particles[1][0] -= 0.005f;
      }
      else{
        animation = 422;
        printf("Animation set to 422.\n");
        particles[0][0] = -0.5f;
        particles[1][0] = 0.5f;
        pauseFrames = 0;
      }
    }
    if(animation == 422){
      if(pauseFrames < 10){
        pauseFrames++;
      }
      else{
        animation = 423;
        printf("Animation set to 423.\n");
      }
    }

    if(animation == 423){
      potentialPoints[0][0] = particles[0][0] - 4.0f;
      potentialPoints[1][0] = particles[0][0] - 2.0f;
      potentialPoints[2][0] = particles[0][0] - 1.0f;
      potentialPoints[3][0] = particles[0][0] - 0.5f;
      potentialPoints[4][0] = particles[0][0] + 0.5f;
      potentialPoints[5][0] = particles[1][0] - 0.5f;
      potentialPoints[6][0] = particles[1][0] + 0.5f;
      potentialPoints[7][0] = particles[1][0] + 1.0f;
      potentialPoints[8][0] = particles[1][0] + 2.0f;
      potentialPoints[9][0] = particles[1][0] + 4.0f;
      if(particles[0][0] > -3.0f){
        particles[0][0] -= 0.005f;
        particles[1][0] += 0.005f;
      }
      else{
        animation = 424;
        printf("Animation set to 424.\n");
        particles[0][0] = -3.0f;
        particles[1][0] = 3.0f;
        pauseFrames = 0;
      }
    }

    if(animation == 424){
      if(pauseFrames < 10){
        pauseFrames++;
      }
      else{
        animation = 425;
        printf("Animation set to 425.\n");        
        enablePotentialPoints = 0;
        enablePotentialDiff = 1;
        potentialPoints[0][0] = -7.0f;
        potentialPoints[1][0] = -6.5f;
        potentialPoints[2][0] = -5.0f;
        potentialPoints[3][0] = -4.5f;
        potentialPoints[4][0] = -0.25f;
        potentialPoints[5][0] = 0.25f;
        potentialPoints[6][0] = 4.5f;
        potentialPoints[7][0] = 5.0f;
        potentialPoints[8][0] = 6.5f;
        potentialPoints[9][0] = 7.0f;
      }
    }

    if(animation == 425){//Particles approaching, calculating potential difference
      if(particles[0][0] < -0.1f){
        particles[0][0] += 0.005f;
        particles[1][0] -= 0.005f;
      }
      else{
        animation = 426;
        printf("Animation set to 426.\n");
        particles[0][0] = -0.1f;
        particles[1][0] = 0.1f;
        pauseFrames = 0;
      }
    }

    if(animation == 426){
      if(pauseFrames < 10){
        pauseFrames++;
      }
      else{
        animation = 427;
        printf("Animation set to 427.\n");        
      }
    }

    if(animation == 427){
      if(particles[0][0] > -3.0f){
        particles[0][0] -= 0.005f;
        particles[1][0] += 0.005f;
      }
      else{
        animation = 428;
        printf("Animation set to 428.\n");
        particles[0][0] = -3.0f;
        particles[1][0] = 3.0f;
        pauseFrames = 0;
      }
    }
    
    if(animation == 428){
      if(pauseFrames < 10){
        pauseFrames++;
      }
      else{
        animation = 429;
        printf("Animation set to 429.\n");        
        enablePotentialPoints = 0;
        enablePotentialDiff = 0;

        translateXtarget = -0.60f;
        translateYtarget = -0.45f;
        translateZtarget = -13.9f;
        
        rotXtarget = 44.16f;
        rotYtarget = -10.75f;
        rotZtarget = 0.0f;

        translateXdiff = (translateXtarget - translateX) / steps;
        translateYdiff = (translateYtarget - translateY) / steps;
        translateZdiff = (translateZtarget - translateZ) / steps;
        rotXdiff = (rotXtarget - rotX) / steps;
        rotYdiff = (rotYtarget - rotY) / steps;
        rotZdiff = (rotZtarget - rotZ) / steps;
      }
    }
    
    if(animation == 429){
      if(lightAmbient1[0] > 0.5){
        lightAmbient1[0] -= 0.01f;
        lightAmbient1[1] -= 0.01f;
        lightAmbient1[2] -= 0.01f;

        lightDiffuse1[0] -= 0.01f;
        lightDiffuse1[1] -= 0.01f;
        lightDiffuse1[2] -= 0.01f;
        
        if(specular1[0] > 0.1){
          specular1[0] -= 0.02f;
          specular1[1] -= 0.02f;
          specular1[2] -= 0.02f;
        }
      }
      if(minDistance < 0.39f){
        minDistance += 0.01;
      }
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 430;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 430;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 430;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 430;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 430;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 430;
      }
      if(animation == 430){
        printf("Animation set to 430.\n");        
        minDistance = 0.39f;

        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
        rotXdiff = 0.0f;
        rotYdiff = 0.0f;
        rotZdiff = 0.0f;            
      }
    }
    
    if(animation == 430){//Particles approaching, equipotential lines
      if(particles[0][0] < -1.0f){
        particles[0][0] += 0.01f;
        particles[1][0] -= 0.01f;
      }
      else{
        animation = 431;
        printf("Animation set to 431.\n");
        particles[0][0] = -1.0f;
        particles[1][0] = 1.0f;
        pauseFrames = 0;
      }
    }    
    
    if(animation == 431){
      if(pauseFrames < 10){
        pauseFrames++;
      }
      else{
        animation = 432;
        printf("Animation set to 432.\n");
      }
    }
    
    if(animation == 432){//Particles daparting, equipotential lines
      if(particles[0][0] > -3.0f){
        particles[0][0] -= 0.01f;
        particles[1][0] += 0.01f;
      }
      else{
        animation = 433;
        printf("Animation set to 433.\n");
        particles[0][0] = -3.0f;
        particles[1][0] = 3.0f;
        pauseFrames = 0;
      }
    }    

    if(animation == 433){
      if(pauseFrames < 10){
        pauseFrames++;
      }
      else{
        animation = 434;
        printf("Animation set to 433.\n");
      }
    }

    if(animation == 434){//Flatten plane to 2D
      if(minDistance > 0.30){
        minDistance -= 0.002;
      }

      if(lightAmbient1[0] > 0.4){
        lightAmbient1[0] -= 0.01f;
        lightAmbient1[1] -= 0.01f;
        lightAmbient1[2] -= 0.01f;

        lightDiffuse1[0] -= 0.01f;
        lightDiffuse1[1] -= 0.01f;
        lightDiffuse1[2] -= 0.01f;
        
        if(specular1[0] > 0.0){
          specular1[0] -= 0.02f;
          specular1[1] -= 0.02f;
          specular1[2] -= 0.02f;
        }
        else{
          specular1[0] = 0.0f;
          specular1[1] = 0.0f;
          specular1[2] = 0.0f;
        }
      }
      
      if(planeOffset > -0.1){
        planeOffset -= 0.01f;
      }
      if(axeYColor[1] > 0.0){
        axeYColor[1] -= 0.02f;
        axeYColor[2] -= 0.02f;
      }
      else{
        axeYColor[1] = 0.0f;
        axeYColor[2] = 0.0f;
      }
      
      if(scalePlaneHeigth > 0.0){
        scalePlaneHeigth -= 1.0; 
      }
      else{
        scalePlaneHeigth = 0.0f;
        animation = 435;
        printf("Animation set to 435.\n");
      
        translateXtarget = 0.08f;
        translateYtarget = -0.09f;
        translateZtarget = -10.65f;
        
        rotXtarget = 90.0f;
        rotYtarget = 0.0f;
        rotZtarget = 0.0f;

        translateXdiff = (translateXtarget - translateX) / steps;
        translateYdiff = (translateYtarget - translateY) / steps;
        translateZdiff = (translateZtarget - translateZ) / steps;
        rotXdiff = (rotXtarget - rotX) / steps;
        rotYdiff = (rotYtarget - rotY) / steps;
        rotZdiff = (rotZtarget - rotZ) / steps;

      }
    }
    
    if(animation == 435){
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 436;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 436;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 436;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 436;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 436;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 436;
      }
      if(animation == 436){
        printf("Animation set to 436.\n");

        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
        rotXdiff = 0.0f;
        rotYdiff = 0.0f;
        rotZdiff = 0.0f;

      }
    }
    
    
    if(animation == 436){//Particles approaching, equipotential lines
      if(particles[0][0] < -1.0f){
        particles[0][0] += 0.01f;
        particles[1][0] -= 0.01f;
      }
      else{
        animation = 437;
        printf("Animation set to 437.\n");
        particles[0][0] = -1.0f;
        particles[1][0] = 1.0f;
        pauseFrames = 0;
      }
    }    
    
    if(animation == 437){
      if(pauseFrames < 10){
        pauseFrames++;
      }
      else{
        animation = 438;
        printf("Animation set to 438.\n");
      }
    }
    
    if(animation == 438){//Particles approaching, equipotential lines
      if(particles[0][0] > -3.0f){
        particles[0][0] -= 0.01f;
        particles[1][0] += 0.01f;
      }
      else{
        animation = 439;
        printf("Animation set to 439.\n");
        particles[0][0] = -3.0f;
        particles[1][0] = 3.0f;
        pauseFrames = 0;
      }
    }    
    
    if(animation == 439){
      if(pauseFrames < 10){
        pauseFrames++;
      }
      else{
        animation = 440;
        printf("Animation set to 440.\n");

        printf("...un ferddisch!\n");
        done = True;         // Exit programm !!!
      }
    }
    
    
    
    
    
    
    
    //---------------------------------------------
    
    if(animation > 410 && animation < 429){//Calculate height of potential sphere / path

      potential = 0.0;
      
      for(i=0; i < potentialEnd; i++){
        totalDistance = sqrt(pow( particles[2][0]-particles[i][0], 2.0) + pow( particles[2][2]-particles[i][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
          potential = k1 * particles[i][4] * particles[potentialEnd][4] / minDistance;
          break;
        }
        potential += k1 * particles[i][4] * particles[potentialEnd][4] / totalDistance;
      }//for(i=0; i < potentialEnd; i++){
      potentialSphereY = potential * scalePlaneHeigth;
      //potentialSphereY = 1.0f;// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      
      potentialSphereX = particles[2][0];
      potentialSphereZ = particles[2][2];
      
      if(animation == 411){
        path[11][pathEnd[11]][0] = particles[2][0];
        path[11][pathEnd[11]][1] = potentialSphereY;
        path[11][pathEnd[11]][2] = particles[2][2]; 
        pathEnd[11]++;       
      }
      if(animation == 415){
        path[13][pathEnd[13]][0] = particles[2][0];
        path[13][pathEnd[13]][1] = potentialSphereY;
        path[13][pathEnd[13]][2] = particles[2][2]; 
        pathEnd[13]++;       
      }
    }
      
    if(animation > 419 && animation < 429){
      for(i2 = 0; i2 < 10; i2++){
        potential = 0.0;
        for(i=0; i < potentialEnd; i++){
          totalDistance = sqrt(pow( potentialPoints[i2][0]-particles[i][0], 2.0) + pow( potentialPoints[i2][2]-particles[i][2], 2.0));
          if(totalDistance < minDistance){
            totalDistance = minDistance;
            potential = k1 * particles[i][4] * particles[potentialEnd][4] / minDistance;
            break;
          }
          potential += k1 * particles[i][4] * particles[potentialEnd][4] / totalDistance;
        }//for(i=0; i < potentialEnd; i++){
        potentialPoints[i2][1] = potential * scalePlaneHeigth;
      }
    }




    if(animation > 429 && animation < 500){//Calculate equipotential paths
      GLfloat startPotential;
      int pathCount;
      int pathNumber;
      GLfloat alphaStart, alphaEnd, beta;
      GLfloat pathStep = 0.08f;
      GLfloat newPotential;
      
      for(i=0; i<10; i++){
        path[i][0][1] = -1000.0f;
        path[i][0][2] = 0.0f;
        pathEnd[i] = 300;
      }
      for(x1 = particles[0][0]; x1 < particles[1][0]; x1 += 0.005){//Find starting points of equipotential lines
        potential = 0.0;
        for(i=0; i < potentialEnd; i++){
          totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(0.0-particles[i][2], 2.0));
          if(totalDistance < minDistance){
            totalDistance = minDistance;
            potential = k1 * particles[i][4] * particles[potentialEnd][4] / totalDistance;
            break;
          }
          potential += k1 * particles[i][4] * particles[potentialEnd][4] / totalDistance;
        }//for(i=0; i < potentialEnd; i++){
        potential = potential * 200.0f;
        //printf("potential=%lf\n", potential);
        if(potential < 2.25 && path[0][0][1] == -1000.0f){
          //printf("potential %lf at %lf\n", potential, x1);
          path[0][0][0] = x1;
          path[0][0][1] = 2.25;
          path[0][0][2] = 0.0;
        }
        if(potential < 1.75 && path[1][0][1] == -1000.0f){
          //printf("potential %lf at %lf\n", potential, x1);
          path[1][0][0] = x1;
          path[1][0][1] = 1.75;
          path[1][0][2] = 0.0;
        }
        if(potential < 1.25 && path[2][0][1] == -1000.0f){
          //printf("potential %lf at %lf\n", potential, x1);
          path[2][0][0] = x1;
          path[2][0][1] = 1.25;
          path[2][0][2] = 0.0;
        }
        if(potential < 0.75 && path[3][0][1] == -1000.0f){
          //printf("potential %lf at %lf\n", potential, x1);
          path[3][0][0] = x1;
          path[3][0][1] = 0.75;
          path[3][0][2] = 0.0;
        }
        if(potential < 0.25 && path[4][0][1] == -1000.0f){
          //printf("potential %lf at %lf\n", potential, x1);
          path[4][0][0] = x1;
          path[4][0][1] = 0.25;
          path[4][0][2] = 0.0;
        }
        if(potential < -0.25 && path[5][0][1] == -1000.0f){
          //printf("potential %lf at %lf\n", potential, x1);
          path[5][0][0] = x1;
          path[5][0][1] = -0.25;
          path[5][0][2] = 0.0;
        }
        if(potential < -0.75 && path[6][0][1] == -1000.0f){
          //printf("potential %lf at %lf\n", potential, x1);
          path[6][0][0] = x1;
          path[6][0][1] = -0.75;
          path[6][0][2] = 0.0;
        }
        if(potential < -1.25 && path[7][0][1] == -1000.0f){
          //printf("potential %lf at %lf\n", potential, x1);
          path[7][0][0] = x1;
          path[7][0][1] = -1.25;
          path[7][0][2] = 0.0;
        }
        if(potential < -1.75 && path[8][0][1] == -1000.0f){
          //printf("potential %lf at %lf\n", potential, x1);
          path[8][0][0] = x1;
          path[8][0][1] = -1.75;
          path[8][0][2] = 0.0;
        }
        if(potential < -2.25 && path[9][0][1] == -1000.0f){
          //printf("potential %lf at %lf\n", potential, x1);
          path[9][0][0] = x1;
          path[9][0][1] = -2.25;
          path[9][0][2] = 0.0;
        }
      }//for(x1 = particles[0][0]; x1 < particles[1][0]; x1 += 0.005){//Find staring points of equipotential lines



      for(pathCount = 0; pathCount < 999; pathCount++){
        for(pathNumber = 0; pathNumber < PATHMAX; pathNumber++){

          startPotential = path[pathNumber][0][1];
          
          if(pathCount < 1){
            alphaStart = 0.0;
            alphaEnd = 3.1415927;
          }
          else{
            beta = atan2(path[pathNumber][pathCount][2] - path[pathNumber][pathCount-1][2], path[pathNumber][pathCount][0] - path[pathNumber][pathCount-1][0]);
            alphaStart = beta - 3.1415927 / 2.0;
            alphaEnd = beta + 3.1415927 / 2.0;
          }
          newPotential = 1000000;
          for(alpha = alphaStart; alpha < alphaEnd; alpha += 0.1){
            x1 = cos(alpha) * pathStep + path[pathNumber][pathCount][0];
            z1 = sin(alpha) * pathStep + path[pathNumber][pathCount][2];
            potential = 0.0;
            for(i=0; i < potentialEnd; i++){
              totalDistance = sqrt(pow(x1-particles[i][0], 2.0) + pow(z1-particles[i][2], 2.0));
              if(totalDistance < minDistance){
                totalDistance = minDistance;
                potential = k1 * particles[i][4] * particles[potentialEnd][4] / minDistance;
                break;
              }
              potential += k1 * particles[i][4] * particles[potentialEnd][4] / totalDistance;
            }//for(i=0; i < potentialEnd; i++){
            potential = potential * 200.0f;
            if(fabs(startPotential - potential) < fabs(startPotential - newPotential)){
              newPotential = potential;
              path[pathNumber][pathCount+1][0] = x1;
              path[pathNumber][pathCount+1][1] = potential;
              path[pathNumber][pathCount+1][2] = z1;
            }    
          }
          //path[pathNumber][0][1] = path[pathNumber][1][1];//Approximation for initial potential
        }//for(pathNumber = 1, pathNumber < 3; pathNumber++){
      }//for(pathCount = 1; pathCount < 300; pathCount++){
        
    }//if(animation > 429 && animation < 500){


//####################################################################################
//############# Animation sequence potential plane  of a single charge ###############
//####################################################################################


    
    if(animation == 333){
      if(alpha < 6 * 3.1415927){
        particles[1][0] = cos(alpha) * doubleTemp;
        particles[1][2] = sin(alpha) * doubleTemp;
        alpha += 0.05;
      }
      else{
        animation = 334;
        printf("Animation set to 334.\n");
        savePics = 0;
        printf("...un ferddisch!\n");
        done = True;         // Exit programm !!!
      }
    }
    if(animation == 332){
        particles[1][0] = cos(alpha) * doubleTemp;
        particles[1][2] = sin(alpha) * doubleTemp;
        alpha += 0.05;
        if(minDistance > 0.002){
          minDistance -= 0.01;
        }
        else{
          animation = 333;
          printf("Animation set to 333.\n");
          while(alpha > 2 * 3.1415927){
            alpha -= 2 * 3.1415927;
          }
        }
    }
    if(animation == 331){
      particles[1][0] = cos(alpha) * doubleTemp;
      particles[1][2] = sin(alpha) * doubleTemp;
      alpha += 0.05;
      if(minDistance > 0.002){
        minDistance -= 0.01;
      }
      if(doubleTemp < 0.7){
        //printf("alpha=%lf.\n", alpha);
        doubleTemp += 0.01;
      }
      else{
        animation = 332;
        printf("Animation set to 332.\n");
      }
    }
    if(animation == 330){
      particles[1][0] = cos(alpha) * doubleTemp;
      particles[1][2] = sin(alpha) * doubleTemp;
      alpha += 0.05;
      if(doubleTemp > 0.5){
        //printf("alpha=%lf.\n", alpha);
        doubleTemp -= 0.01;
      }
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 331;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 331;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 331;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 331;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 331;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 331;
      }
      if(animation == 331){
        animation = 331;
        printf("Animation set to 331.\n");
        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
        rotXdiff = 0.0f;
        rotYdiff = 0.0f;
        rotZdiff = 0.0f;
        //glEnable(GL_LIGHT0);
      }
    }
    if(animation == 329){
      if(alpha < 2 * 3.1415927){
        //printf("alpha=%lf.\n", alpha);
        particles[1][0] = cos(alpha) * 2.246887f;
        particles[1][2] = sin(alpha) * 2.246887f;
        alpha += 0.05;
      }
      else{
        animation = 330;
        printf("Animation set to 330.\n");
        doubleTemp = 2.246887;

        translateXtarget = 0.0f;
        translateYtarget = 1.33f;
        translateZtarget = -29.46f;
        
        rotXtarget = 2.25f;
        rotYtarget = 0.0f;
        rotZtarget = 0.0f;

        translateXdiff = (translateXtarget - translateX) / steps;
        translateYdiff = (translateYtarget - translateY) / steps;
        translateZdiff = (translateZtarget - translateZ) / steps;
        rotXdiff = (rotXtarget - rotX) / steps;
        rotYdiff = (rotYtarget - rotY) / steps;
        rotZdiff = (rotZtarget - rotZ) / steps;
      }
    }
    if(animation == 328){
      if(particles[1][0] < 2.246887f){
        particles[1][0] += 0.01f;
      }
      else{
        animation = 329;
        particles[1][0] = 2.246887f;
        alpha = 0.0f;
      }
    }
    if(animation == 327){
      if(particles[1][0] > 0.6f){
        particles[1][0] -= 0.01f;
      }
      else{
        animation = 328;
      }
    }
    if(animation == 326){
      particles[1][0] -= 0.01f;
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 327;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 327;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 327;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 327;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 327;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 327;
      }
      if(animation == 327){
        printf("Animation set to 327.\n");
        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
        rotXdiff = 0.0f;
        rotYdiff = 0.0f;
        rotZdiff = 0.0f;
        alpha = 1.0;
      }
    }
    if(animation == 325){//Positive particle turned into negative particle
      if(alpha < 0.25){
        alpha += 0.01;
        lightTemp[0]=alpha;
        lightTemp[1]=alpha;
        lightTemp[2]=alpha;
        lightTemp[3]=alpha;
      }
      else{
        animation = 326;
        printf("Animation set to 326.\n");
        //alpha = 1.0;
        
        translateXtarget = 0.0f;
        translateYtarget = 1.33f;
        translateZtarget = -7.96f;
        
        rotXtarget = 52.25f;
        rotYtarget = 0.0f;
        rotZtarget = 0.0f;

        translateXdiff = (translateXtarget - translateX) / steps;
        translateYdiff = (translateYtarget - translateY) / steps;
        translateZdiff = (translateZtarget - translateZ) / steps;
        rotXdiff = (rotXtarget - rotX) / steps;
        rotYdiff = (rotYtarget - rotY) / steps;
        rotZdiff = (rotZtarget - rotZ) / steps;
      }
    }
    if(animation == 324){
      if(alpha > 0.1){
        alpha -= 0.01;
        lightTemp[0]=alpha;
        lightTemp[1]=alpha;
        lightTemp[2]=alpha;
        lightTemp[3]=alpha;
      }
      else{
        animation = 325;
        printf("Animation set to 325.\n");
        particles[0][4] = -0.000001;
        for(i=0; i < pathEnd[1]; i++){
          path[1][i][1]=-path[1][i][1];
        }
        for(i=0; i < pathEnd[2]; i++){
          path[2][i][1]=-path[2][i][1];
        }
        for(i=0; i < pathEnd[3]; i++){
          path[3][i][1]=-path[3][i][1];
        }
        for(i=0; i < pathEnd[4]; i++){
          path[4][i][1]=-path[4][i][1];
        }
        for(i=0; i < pathEnd[5]; i++){
          path[5][i][1]=-path[5][i][1];
        }
      }
    }
    if(animation == 323){
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 324;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 324;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 324;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 324;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 324;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 324;
      }
      if(animation == 324){
        printf("Animation set to 324.\n");
        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
        rotXdiff = 0.0f;
        rotYdiff = 0.0f;
        rotZdiff = 0.0f;
        alpha = 1.0;
      }
    }
    if(animation == 322){
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 323;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 323;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 323;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 323;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 323;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 323;
      }
      if(animation == 323){
        printf("Animation set to 323.\n");
        translateXtarget = 0.0f;
        translateYtarget = 0.0f;
        translateZtarget = -7.96f;
        
        rotXtarget = 90.0f;
        rotYtarget = 0.0f;
        rotZtarget = 0.0f;

        translateXdiff = (translateXtarget - translateX) / steps;
        translateYdiff = (translateYtarget - translateY) / steps;
        translateZdiff = (translateZtarget - translateZ) / steps;
        rotXdiff = (rotXtarget - rotX) / steps;
        rotYdiff = (rotYtarget - rotY) / steps;
        rotZdiff = (rotZtarget - rotZ) / steps;
      }
    }
    if(animation == 321){
      if(alpha < 6 * 3.1415927){
        particles[1][0] = cos(alpha) * 4.493775f;
        particles[1][2] = sin(alpha) * 4.493775f;
        alpha += 0.05;
      }
      else{
        printf("Animation set to 322.\n");
        animation = 322;
        alpha = 0;
        particles[1][0] = cos(alpha) * 4.493775f;
        particles[1][2] = sin(alpha) * 4.493775f;


        translateXtarget = -1.33f;
        translateYtarget = -0.6f;
        translateZtarget = -7.0f;
        
        rotXtarget = 3.66f;
        rotYtarget = -1.5f;
        rotZtarget = -0.005f;

        translateXdiff = (translateXtarget - translateX) / steps;
        translateYdiff = (translateYtarget - translateY) / steps;
        translateZdiff = (translateZtarget - translateZ) / steps;
        rotXdiff = (rotXtarget - rotX) / steps;
        rotYdiff = (rotYtarget - rotY) / steps;
        rotZdiff = (rotZtarget - rotZ) / steps;
      }
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        rotX = rotXtarget;
        rotXdiff = 0.0f;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        rotY = rotYtarget;
        rotYdiff = 0.0f;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        rotZ = rotZtarget;
        rotZdiff = 0.0f;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        translateXdiff = translateXtarget;
        translateXdiff = 0.0f;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        translateYdiff = translateYtarget;
        translateYdiff = 0.0f;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        translateZdiff = translateZtarget;
        translateZdiff = 0.0f;
      }
    }
    if(animation == 320){//Fifth ring
      if(alpha < 2 * 3.1415927){
        //printf("alpha=%lf.\n", alpha);
        particles[1][0] = cos(alpha) * 4.493775f;
        particles[1][2] = sin(alpha) * 4.493775f;

        path[5][pathEnd[5]][0] = particles[1][0];
        path[5][pathEnd[5]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[5][pathEnd[5]][0]-particles[0][0], 2.0) + pow(path[5][pathEnd[5]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.01 + y1 * 0.02;
        path[5][pathEnd[5]][1] = y1;

        pathEnd[5]++;
        alpha += 0.05;
      }
      else{
        printf("Animation set to 320.\n");
        animation = 321;
        particles[1][0] = 4.493775f;
        particles[1][2] = 0.0f;

        path[5][pathEnd[5]][0] = particles[1][0];
        path[5][pathEnd[5]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[5][pathEnd[5]][0]-particles[0][0], 2.0) + pow(path[5][pathEnd[5]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.01 + y1 * 0.02;
        path[5][pathEnd[5]][1] = y1;

        pathEnd[5]++;

        path[5][pathEnd[5]][0] = particles[1][0];
        path[5][pathEnd[5]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[5][pathEnd[5]][0]-particles[0][0], 2.0) + pow(path[5][pathEnd[5]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.01 + y1 * 0.02;
        path[5][pathEnd[5]][1] = y1;

        pathEnd[5]++;

        translateXtarget = -1.33f;
        translateYtarget = -2.13f;
        translateZtarget = -10.05f;
        
        rotXtarget = -10.84f;
        rotYtarget = -1.5f;
        rotZtarget = -0.005f;

        translateXdiff = (translateXtarget - translateX) / steps;
        translateYdiff = (translateYtarget - translateY) / steps;
        translateZdiff = (translateZtarget - translateZ) / steps;
        rotXdiff = (rotXtarget - rotX) / steps;
        rotYdiff = (rotYtarget - rotY) / steps;
        rotZdiff = (rotZtarget - rotZ) / steps;
        while(alpha > 2 * 3.1415927){
          alpha -= 2 * 3.1415927;
        }
      }
    }
    if(animation == 319){//Fourth ring
      if(alpha < 2 * 3.1415927){
        //printf("alpha=%lf.\n", alpha);
        particles[1][0] = cos(alpha) * 2.246887f;
        particles[1][2] = sin(alpha) * 2.246887f;

        path[4][pathEnd[4]][0] = particles[1][0];
        path[4][pathEnd[4]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[4][pathEnd[4]][0]-particles[0][0], 2.0) + pow(path[4][pathEnd[4]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.01 + y1 * 0.02;
        path[4][pathEnd[4]][1] = y1;

        pathEnd[4]++;
        alpha += 0.05;
      }
      else{
        printf("Animation set to 320.\n");
        animation = 320;
        particles[1][0] = 2.246887f;
        particles[1][2] = 0.0f;

        path[4][pathEnd[4]][0] = particles[1][0];
        path[4][pathEnd[4]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[4][pathEnd[4]][0]-particles[0][0], 2.0) + pow(path[4][pathEnd[4]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.01 + y1 * 0.02;
        path[4][pathEnd[4]][1] = y1;

        pathEnd[4]++;

        path[4][pathEnd[4]][0] = particles[1][0];
        path[4][pathEnd[4]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[4][pathEnd[4]][0]-particles[0][0], 2.0) + pow(path[4][pathEnd[4]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.01 + y1 * 0.02;
        path[4][pathEnd[4]][1] = y1;

        pathEnd[4]++;
        alpha = 0.0f;
      }
    }
    if(animation == 318){//Third ring
      if(alpha < 2 * 3.1415927){
        //printf("alpha=%lf.\n", alpha);
        particles[1][0] = cos(alpha) * 0.898755f;
        particles[1][2] = sin(alpha) * 0.898755f;

        path[3][pathEnd[3]][0] = particles[1][0];
        path[3][pathEnd[3]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[3][pathEnd[3]][0]-particles[0][0], 2.0) + pow(path[3][pathEnd[3]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.01 + y1 * 0.02;
        path[3][pathEnd[3]][1] = y1;

        pathEnd[3]++;
        alpha += 0.05;
      }
      else{
        printf("Animation set to 319.\n");
        animation = 319;
        particles[1][0] = 0.898755f;
        particles[1][2] = 0.0f;

        path[3][pathEnd[3]][0] = particles[1][0];
        path[3][pathEnd[3]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[3][pathEnd[3]][0]-particles[0][0], 2.0) + pow(path[3][pathEnd[3]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.01 + y1 * 0.02;
        path[3][pathEnd[3]][1] = y1;

        pathEnd[3]++;

        path[3][pathEnd[3]][0] = particles[1][0];
        path[3][pathEnd[3]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[3][pathEnd[3]][0]-particles[0][0], 2.0) + pow(path[3][pathEnd[3]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.01 + y1 * 0.02;
        path[3][pathEnd[3]][1] = y1;

        pathEnd[3]++;
        pathEnd[4] = 0;
        alpha = 0.0f;
      }
    }
    if(animation == 317){//Second ring
      if(alpha < 2 * 3.1415927){
        //printf("alpha=%lf.\n", alpha);
        particles[1][0] = cos(alpha) * 1.123444f;
        particles[1][2] = sin(alpha) * 1.123444f;

        path[2][pathEnd[2]][0] = particles[1][0];
        path[2][pathEnd[2]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[2][pathEnd[2]][0]-particles[0][0], 2.0) + pow(path[2][pathEnd[2]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.01 + y1 * 0.02;
        path[2][pathEnd[2]][1] = y1;

        pathEnd[2]++;
        alpha += 0.05;
      }
      else{
        printf("Animation set to 318.\n");
        animation = 318;
        particles[1][0] = 1.123444f;
        particles[1][2] = 0.0f;
        alpha = 0.0f;

        path[2][pathEnd[2]][0] = particles[1][0];
        path[2][pathEnd[2]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[2][pathEnd[2]][0]-particles[0][0], 2.0) + pow(path[2][pathEnd[2]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.01 + y1 * 0.02;
        path[2][pathEnd[2]][1] = y1;

        pathEnd[2]++;

        path[2][pathEnd[2]][0] = particles[1][0];
        path[2][pathEnd[2]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[2][pathEnd[2]][0]-particles[0][0], 2.0) + pow(path[2][pathEnd[2]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.01 + y1 * 0.02;
        path[2][pathEnd[2]][1] = y1;

        pathEnd[2]++;
        pathEnd[3] = 0;
      }
    }
    if(animation == 316){//First ring
      if(alpha < 4 * 3.1415927){
        //printf("alpha=%lf.\n", alpha);
        alpha += 0.05;
        particles[1][0] = cos(alpha) * 1.497925f;
        particles[1][2] = sin(alpha) * 1.497925f;

        path[1][pathEnd[1]][0] = particles[1][0];
        path[1][pathEnd[1]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[1][pathEnd[1]][0]-particles[0][0], 2.0) + pow(path[1][pathEnd[1]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.01 + y1 * 0.02;
        path[1][pathEnd[1]][1] = y1;

        pathEnd[1]++;
      }
      else{
        printf("Animation set to 317.\n");
        animation = 317;
        particles[1][0] = 1.497925f;
        particles[1][2] = 0.0f;
        alpha = 0.0f;
      }
    }
    if(animation == 315){//Particle on circular trajectory (first ring)
      if(alpha < 6 * 3.1415927){
        alpha += 0.05;
        particles[1][0] = cos(alpha) * 1.497925f;
        particles[1][2] = sin(alpha) * 1.497925f;

        path[1][pathEnd[1]][0] = particles[1][0];
        path[1][pathEnd[1]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[1][pathEnd[1]][0]-particles[0][0], 2.0) + pow(path[1][pathEnd[1]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.01 + y1 * 0.02;
        path[1][pathEnd[1]][1] = y1;

        pathEnd[1]++;
      }    
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 316;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 316;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 316;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 316;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 316;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 316;
      }
      if(animation == 316){
        printf("Animation set to 316.\n");
        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
        rotXdiff = 0.0f;
        rotYdiff = 0.0f;
        rotZdiff = 0.0f;
      }
    }
    if(animation == 314){
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 315;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 315;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 315;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 315;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 315;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 315;
      }
      if(animation == 315){
        printf("Animation set to 315.\n");
        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
        rotXdiff = 0.0f;
        rotYdiff = 0.0f;
        rotZdiff = 0.0f;
        pathEnd[1] = 0;
        pathEnd[2] = 0;
        pathColor[1] = 0;
        pathColor[2] = 0;
        alpha = 0.0f;

        translateXtarget = -1.33f;
        translateYtarget = -0.83f;
        translateZtarget = -8.75f;
        
        rotXtarget = 13.66f;
        rotYtarget = -17.75f;
        rotZtarget = -0.005f;

        translateXdiff = (translateXtarget - translateX) / steps;
        translateYdiff = (translateYtarget - translateY) / steps;
        translateZdiff = (translateZtarget - translateZ) / steps;
        rotXdiff = (rotXtarget - rotX) / steps;
        rotYdiff = (rotYtarget - rotY) / steps;
        rotZdiff = (rotZtarget - rotZ) / steps;
      }
    }
    if(animation == 313){//Particle approaches indirectly
      if(particles[1][0] < 1.497925 - 0.025f){
        particles[1][0] += 0.025f;
        particles[1][2] -= 0.0f;

        path[2][pathEnd[2]][0] = particles[1][0];
        path[2][pathEnd[2]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[2][pathEnd[2]][0]-particles[0][0], 2.0) + pow(path[2][pathEnd[2]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.02 + y1 * 0.02;
        path[2][pathEnd[2]][1] = y1;

        pathEnd[2]++;
      }
      else{
        particles[1][0] = 1.497925f;
        particles[1][2] = 0.0f;        

        path[2][pathEnd[2]][0] = particles[1][0];
        path[2][pathEnd[2]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[2][pathEnd[2]][0]-particles[0][0], 2.0) + pow(path[2][pathEnd[2]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.02 + y1 * 0.02;
        path[2][pathEnd[2]][1] = y1;

        pathEnd[2]++;

        animation = 314;
        printf("Animation set to 314.\n");

        translateXtarget = -1.33f;
        translateYtarget = -0.83f;
        translateZtarget = -5.05f;
        
        rotXtarget = 0.66f;
        rotYtarget = -17.75f;
        rotZtarget = -0.005f;

        translateXdiff = (translateXtarget - translateX) / steps;
        translateYdiff = (translateYtarget - translateY) / steps;
        translateZdiff = (translateZtarget - translateZ) / steps;
        rotXdiff = (rotXtarget - rotX) / steps;
        rotYdiff = (rotYtarget - rotY) / steps;
        rotZdiff = (rotZtarget - rotZ) / steps;
      }
    }
    if(animation == 312){//Particle approaches indirectly
      if(particles[1][0] < 1.00 - 0.025){
        particles[1][0] += 0.025f;
        particles[1][2] = sqrt(1.0f - pow(particles[1][0], 2.0));

        path[2][pathEnd[2]][0] = particles[1][0];
        path[2][pathEnd[2]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[2][pathEnd[2]][0]-particles[0][0], 2.0) + pow(path[2][pathEnd[2]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.02 + y1 * 0.02;
        path[2][pathEnd[2]][1] = y1;

        pathEnd[2]++;
      }
      else{
        animation = 313;
        printf("Animation set to 313.\n");
        particles[1][0] = 1.0f;
        particles[1][2] = 0.0f;

        path[2][pathEnd[2]][0] = particles[1][0];
        path[2][pathEnd[2]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[2][pathEnd[2]][0]-particles[0][0], 2.0) + pow(path[2][pathEnd[2]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.02 + y1 * 0.02;
        path[2][pathEnd[2]][1] = y1;

        pathEnd[2]++;
      }
    }
    if(animation == 311){//Particle approaches indirectly
      if(particles[1][2] > 1.00 + 0.025f){
        particles[1][0] += 0.0f;
        particles[1][2] -= 0.025f;

        path[2][pathEnd[2]][0] = particles[1][0];
        path[2][pathEnd[2]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[2][pathEnd[2]][0]-particles[0][0], 2.0) + pow(path[2][pathEnd[2]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.02 + y1 * 0.02;
        path[2][pathEnd[2]][1] = y1;

        pathEnd[2]++;
      }
      else{
        particles[1][0] = 0.0f;
        particles[1][2] = 1.0f;        
        animation = 312;
        printf("Animation set to %d.\n", animation);

        path[2][pathEnd[2]][0] = particles[1][0];
        path[2][pathEnd[2]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[2][pathEnd[2]][0]-particles[0][0], 2.0) + pow(path[2][pathEnd[2]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.02 + y1 * 0.02;
        path[2][pathEnd[2]][1] = y1;

        pathEnd[2]++;
      }
    }
    if(animation == 310){//Particle approaches indirectly
      if(particles[1][0] < 0.00 - 0.05f){
        particles[1][0] += 0.05f;
        particles[1][2] += 0.0f;

        path[2][pathEnd[2]][0] = particles[1][0];
        path[2][pathEnd[2]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[2][pathEnd[2]][0]-particles[0][0], 2.0) + pow(path[2][pathEnd[2]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.02 + y1 * 0.02;
        path[2][pathEnd[2]][1] = y1;

        pathEnd[2]++;
      }
      else{
        particles[1][0] = 0.0f;
        particles[1][2] = 2.0f;        
        animation = 311;
        printf("Animation set to %d.\n", animation);

        path[2][pathEnd[2]][0] = particles[1][0];
        path[2][pathEnd[2]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[2][pathEnd[2]][0]-particles[0][0], 2.0) + pow(path[2][pathEnd[2]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.02 + y1 * 0.02;
        path[2][pathEnd[2]][1] = y1;

        pathEnd[2]++;
      }
    }
    if(animation == 309){//Particle approaches indirectly
      if(particles[1][0] < -2.00 - 0.025f){
        particles[1][0] += 0.025f;
        particles[1][2] += 0.075f;

        path[2][pathEnd[2]][0] = particles[1][0];
        path[2][pathEnd[2]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[2][pathEnd[2]][0]-particles[0][0], 2.0) + pow(path[2][pathEnd[2]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.02 + y1 * 0.02;
        path[2][pathEnd[2]][1] = y1;

        pathEnd[2]++;
      }
      else{
        particles[1][0] = -2.0f;
        particles[1][2] = 2.0f;        
        animation = 310;
        printf("Animation set to %d.\n", animation);

        path[2][pathEnd[2]][0] = particles[1][0];
        path[2][pathEnd[2]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[2][pathEnd[2]][0]-particles[0][0], 2.0) + pow(path[2][pathEnd[2]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.02 + y1 * 0.02;
        path[2][pathEnd[2]][1] = y1;

        pathEnd[2]++;
      }
    }
    if(animation == 308){//Particle approaches indirectly      
      if(particles[1][0] > -4.00){
        particles[1][0] -= 0.10f;

        path[2][pathEnd[2]][0] = particles[1][0];
        path[2][pathEnd[2]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[2][pathEnd[2]][0]-particles[0][0], 2.0) + pow(path[2][pathEnd[2]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.02 + y1 * 0.02;
        path[2][pathEnd[2]][1] = y1;

        pathEnd[2]++;
      }
      else{
        particles[1][0] = -4.0;
        animation = 309;
        printf("Animation set to %d.\n", animation);

        path[2][pathEnd[2]][0] = particles[1][0];
        path[2][pathEnd[2]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[2][pathEnd[2]][0]-particles[0][0], 2.0) + pow(path[2][pathEnd[2]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.02 + y1 * 0.02;
        path[2][pathEnd[2]][1] = y1;

        pathEnd[2]++;
      }
    }
    if(animation == 307){//Particle approaches directly
      if(particles[1][0] > 1.497925){

        path[1][pathEnd[1]][0] = particles[1][0];
        path[1][pathEnd[1]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[1][pathEnd[1]][0]-particles[0][0], 2.0) + pow(path[1][pathEnd[1]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.02 + y1 * 0.02;
        path[1][pathEnd[1]][1] = y1;

        pathEnd[1]++;
        particles[1][0] -= 0.05f;
      }
      else{
        path[1][pathEnd[1]][0] = 1.497925;
        path[1][pathEnd[1]][2] = particles[1][2];
        totalDistance = sqrt(pow(path[1][pathEnd[1]][0]-particles[0][0], 2.0) + pow(path[1][pathEnd[1]][2]-particles[0][2], 2.0));
        if(totalDistance < minDistance){
          totalDistance = minDistance;
        }
        potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
        y1 = potential * 200.0f + 0.02 + y1 * 0.02;
        path[1][pathEnd[1]][1] = y1;
        
        pathEnd[1]++;
        
        animation = 308;
        printf("Animation set to %d.\n", animation);
        particles[1][0] = 12.0f;
        particles[1][2] = -4.0f;
        
        pathEnd[2] = 0;
        pathColor[2] = 2;
      }
    }
    if(animation == 306){
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 307;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 307;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 307;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 307;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 307;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 307;
      }
      if(animation == 307){
        printf("Animation set to 307.\n");
        
        enablePotentialSphere = 1;
        
        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
        rotXdiff = 0.0f;
        rotYdiff = 0.0f;
        rotZdiff = 0.0f;
        particles[1][0] = 10.0f;
        pathEnd[1] = 0;
        pathColor[1] = 1;
      }
    }
    if(animation == 305){
      if(minDistance < 0.39){
        minDistance += 0.01;
      }
      else{
        animation = 306;
        minDistance = 0.39;

        translateXtarget = -2.61f;
        translateYtarget = -0.53f;
        translateZtarget = -8.6f;
        
        rotXtarget = 21.41f;
        rotYtarget = -9.5f;
        rotZtarget = -3.005f;

        translateXdiff = (translateXtarget - translateX) / steps;
        translateYdiff = (translateYtarget - translateY) / steps;
        translateZdiff = (translateZtarget - translateZ) / steps;
        rotXdiff = (rotXtarget - rotX) / steps;
        rotYdiff = (rotYtarget - rotY) / steps;
        rotZdiff = (rotZtarget - rotZ) / steps;
      }
    }
    if(animation == 304){
      if(minDistance > 0.1){
        minDistance -= 0.01;
      }
      else{
        animation = 305;
      }
    }
    if(animation == 303){
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 304;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 304;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 304;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 304;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 304;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 304;
      }
      if(animation == 304){
        printf("Animation set to 303.\n");
        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
        rotXdiff = 0.0f;
        rotYdiff = 0.0f;
        rotZdiff = 0.0f;
      }
    }
    if(animation == 302){
      if(xMax < 16){
        xMax += 0.1;
        yMax = xMax / 16.0f * 9.0f;
      }
      else{
        animation = 303;
        
        xMax = 16.0;
        yMax = 9.0;
                
        translateXtarget = 0.0f;
        translateYtarget = -1.24f;
        translateZtarget = -29.7f;
        
        rotXtarget = -1.95f;
        rotYtarget = 0.0f;
        rotZtarget = 0.0f;

        translateXdiff = (translateXtarget - translateX) / steps * 2.0;
        translateYdiff = (translateYtarget - translateY) / steps * 2.0;
        translateZdiff = (translateZtarget - translateZ) / steps * 2.0;
        rotXdiff = (rotXtarget - rotX) / steps * 2.0;
        rotYdiff = (rotYtarget - rotY) / steps * 2.0;
        rotZdiff = (rotZtarget - rotZ) / steps * 2.0;
      }
        
    }
    if(animation == 301){
      if(gridX < 128){
        gridX += 2;
        gridY += 1;
      }
      else{
        animation = 302;                
      }
    }
    if(animation == 300){
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 301;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 301;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 301;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 301;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 301;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 301;
      }
      if(animation == 301){
        printf("Animation set to 301.\n");
        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
        rotXdiff = 0.0f;
        rotYdiff = 0.0f;
        rotZdiff = 0.0f;
      }
    }

    
    
    if(animation > 306 && animation < 400){//Calculate height of potential sphere
      totalDistance = sqrt(pow(particles[0][0]-particles[1][0], 2.0) + pow(particles[0][2]-particles[1][2], 2.0));
      if(totalDistance < minDistance){
        totalDistance = minDistance;
      }
      potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
      potentialSphereY = potential * scalePlaneHeigth;

      potentialSphereX = particles[1][0];
      potentialSphereZ = particles[1][2];
      
    }
    //printf("E = 4mj, r=%lf\n", k * particles[1][4] * particles[0][4] / 0.004);
    //printf("E = 6mj, r=%lf\n", k * particles[1][4] * particles[0][4] / 0.006);
    //printf("E = 8mj, r=%lf\n", k * particles[1][4] * particles[0][4] / 0.008);
    //printf("E = 10mj, r=%lf\n", k * particles[1][4] * particles[0][4] / 0.010);
    //printf("E = 02mj, r=%lf\n", k * particles[1][4] * particles[0][4] / 0.002);
    //printf("\n");
    

    
//####################################################################################
//############# Paint force plane  ###################################################
//####################################################################################
    
    if(animation == 1){      
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 2;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 2;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 2;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 2;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 2;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 2;
      }
      if(animation == 2){
        rotYtarget -= 540.0f;
        printf("Animation set to 2.\n");
      }
    }
    if(animation == 2){
      rotXdiff = 0.0f;
      rotZdiff = 0.0f;
      rotYdiff = -0.5f;
      translateXdiff = 0.0f;
      translateYdiff = 0.0f;
      translateZdiff = 0.0f;
      if(rotY < rotYtarget){
        printf("Animation set to 3.\n");
        animation = 3;
        rotYdiff = 0.0f;
        translateYdiff = -0.01f;
      }
    }

    if(animation == 3){
      if(minDistance < 0.41f){
        minDistance += 0.0005f;
      }
      else{
        //printf("minDistance = %lf\n", minDistance);
        printf("Animation set to 4.\n");
        minDistance = 0.41f;
        animation = 4;
        steps = 400.0f;
        translateXtarget = 0.0f;
        translateYtarget = -2.896f;
        translateZtarget = -10.1924f;
        
        rotXtarget = 57.4762f;
        rotZtarget = 0.0f;
        rotYtarget = rotY - 360.0f;
        
        translateXdiff = (translateXtarget - translateX) / steps;
        translateYdiff = (translateYtarget - translateY) / steps;
        translateZdiff = (translateZtarget - translateZ) / steps;
        rotXdiff = (rotXtarget - rotX) / steps;
        rotYdiff = (rotYtarget - rotY) / steps;
        rotZdiff = (rotZtarget - rotZ) / steps;
      }
    }

    if(animation == 4){
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 5;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 5;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 5;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 5;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 5;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 5;
      }
      if(animation == 5){
        printf("Animation set to 5.\n");
        steps = 400.0f;
        rotYtarget = rotY - 360.0f;
        
        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
        rotXdiff = 0.0f;
        rotZdiff = 0.0f;
        rotYdiff = (rotYtarget - rotY) / steps;
      }
      
    }

    if(animation == 5){
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
//        animation = 7;
        rotXdiff = 0.0f;
        rotZdiff = 0.0f;
        rotYdiff = -0.5f;
        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
//        if(rotY < rotYtarget){
          printf("Animation set to 6.\n");
          animation = 6;
          steps = 400.0f;
          
          translateXtarget = 0.0f;
          translateYtarget = -1.77378f;
          translateZtarget = -15.4f;
          
          rotXtarget = -6.563946f;
          rotZtarget = 0.0f;
          rotYtarget = -1440.5f;
          
          translateXdiff = (translateXtarget - translateX) / steps;
          translateYdiff = (translateYtarget - translateY) / steps;
          translateZdiff = (translateZtarget - translateZ) / steps;
          rotXdiff = (rotXtarget - rotX) / steps;
          rotYdiff = (rotYtarget - rotY) / steps;
          rotZdiff = (rotZtarget - rotZ) / steps;
          
//        }
      }
    }


    if(animation == 6){
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 7;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 7;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 7;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 7;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 7;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 7;
      }
      if(animation == 7){
        printf("Animation set to 7.\n");
        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
        rotXdiff = 0.0f;
        rotYdiff = 0.0f;
        rotZdiff = 0.0f;
        pauseFrames = 0;
      }
    }
    
    
    if(animation == 7){
      if(pauseFrames > 10){
        animation++;
        printf("Animation set to %d.\n", animation);

        translateXtarget = 0.0f;
        translateYtarget = 0.0f;
        translateZtarget = -15.4f;

        rotXtarget = 90.0f;
        rotZtarget = 0.0f;
        while(rotY < 0.0f){
          rotY += 360.0f;
        }
        rotYtarget = 360.0f;
        
        translateXdiff = (translateXtarget - translateX) / steps;
        translateYdiff = (translateYtarget - translateY) / steps;
        translateZdiff = (translateZtarget - translateZ) / steps;
        rotXdiff = (rotXtarget - rotX) / steps;
        rotYdiff = (rotYtarget - rotY) / steps;
        rotZdiff = (rotZtarget - rotZ) / steps;
      }
    }

    if(animation == 8){
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 9;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 9;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 9;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 9;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 9;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 9;
      }
      if(animation == 9){
        pauseFrames = 0;
        printf("Animation set to 9.\n");
        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
        rotXdiff = 0.0f;
        rotYdiff = 0.0f;
        rotZdiff = 0.0f;
      }
    }

    if(animation == 9){
      if(pauseFrames > 10){
        savePics = 0;
        printf("...un ferddisch!\n");
        done = True;         // Exit programm !!!
      }
      paintLastPic++;
    }
    

   

    //Change the rotation angles and translation    
    if(firstStart == 0 && animationPause == 0){
      translateX += translateXdiff;
      translateY += translateYdiff;
      translateZ += translateZdiff;
      rotX += rotXdiff;
      rotY += rotYdiff;
      rotZ += rotZdiff;
    }

    
    glDisable(GL_TEXTURE_2D);       /* Enable Texture Mapping */
    glDisable(GL_BLEND);



//####################################################################################
//############# Paint field lines of a single positive charge ########################
//####################################################################################
    if(animation == 100){
      GLfloat fieldLineLength = 40.0;
      double alpha, beta;
      
      glEnable(GL_LIGHTING);
      glPolygonMode(GL_BACK, GL_FILL);
      glPolygonMode(GL_FRONT, GL_FILL);


      glLightfv(GL_LIGHT1, GL_AMBIENT, lightGreen1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightGreen1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightGreen1);


      for(alpha = 0.0; alpha < 2.0 * 3.1415927; alpha += 2.0 * 3.1415927 / 18.0){
        drawCylinder(0.0, 0.0, 0.0, fieldLineLength * cos(alpha), fieldLineLength * sin(alpha), 0.0, 0.03, 0.03, 32);//Field line
        drawCylinder( 3.5 * cos(alpha), 3.5 * sin(alpha), 0.0, 4.0 * cos(alpha), 4.0 * sin(alpha), 0.0, 0.14, 0.0, 32);//Field line pointer
      }
      glColor3f(1.0f, 0.0f, 0.0f);

      glLightfv(GL_LIGHT1, GL_AMBIENT, lightRed1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightRed1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightOrange1);
      drawSphere(0.0, 0.0, 0.0, 0.5, 40.0, 40.0, 0);      
      glLightfv(GL_LIGHT1, GL_AMBIENT, lightAmbient1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightDiffuse1);
      glLightfv(GL_LIGHT1,GL_SPECULAR,specular1);
    }


    if(animation == 100){
      if((rotX > rotXtarget && rotXdiff > 0.0f) || (rotX < rotXtarget && rotXdiff < 0.0f)){
        animation = 101;
      }
      if((rotY > rotYtarget && rotYdiff > 0.0f) || (rotY < rotYtarget && rotYdiff < 0.0f)){
        animation = 101;
      }
      if((rotZ > rotZtarget && rotZdiff > 0.0f) || (rotZ < rotZtarget && rotZdiff < 0.0f)){
        animation = 101;
      }
      if((translateX > translateXtarget && translateXdiff > 0.0f) || (translateX < translateXtarget && translateXdiff < 0.0f)){
        animation = 101;
      }
      if((translateY > translateYtarget && translateYdiff > 0.0f) || (translateY < translateYtarget && translateYdiff < 0.0f)){
        animation = 101;
      }
      if((translateZ > translateZtarget && translateZdiff > 0.0f) || (translateZ < translateZtarget && translateZdiff < 0.0f)){
        animation = 101;
      }
      if(animation == 101){
        printf("Animation set to 101.\n");
        translateXdiff = 0.0f;
        translateYdiff = 0.0f;
        translateZdiff = 0.0f;
        rotXdiff = 0.0f;
        rotYdiff = 0.0f;
        rotZdiff = 0.0f;
      }
    }
    
    if(animation == 101){
      GLfloat fieldLineLength = 40.0;
      double alpha, beta;
      
      glEnable(GL_LIGHTING);
      glPolygonMode(GL_BACK, GL_FILL);
      glPolygonMode(GL_FRONT, GL_FILL);


      glLightfv(GL_LIGHT1, GL_AMBIENT, lightGreen1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightGreen1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightGreen1);


      for(alpha = 0.0; alpha < 2.0 * 3.1415927; alpha += 2.0 * 3.1415927 / 18.0){
        drawCylinder(0.0, 0.0, 0.0, fieldLineLength * cos(alpha), fieldLineLength * sin(alpha), 0.0, 0.03, 0.03, 32);//Field line
        drawCylinder( 3.5 * cos(alpha), 3.5 * sin(alpha), 0.0, 4.0 * cos(alpha), 4.0 * sin(alpha), 0.0, 0.14, 0.0, 32);//Field line pointer
      }
      
      for(alpha = 0.0; alpha < 2.0 * 3.1415927; alpha += 2.0 * 3.1415927 / 9.0){
        
        drawCylinder(0.0, 0.0, 0.0, fieldLineEndUp * cos(alpha), fieldLineEndUp * sin(alpha), -fieldLineEndUp * sin(2.0 * 3.1415927 / 9), 0.03, 0.03, 32);//Field line
        drawCylinder( 3.5 * fieldLineEndUp / fieldLineLength * cos(alpha),
                      3.5 * fieldLineEndUp / fieldLineLength * sin(alpha),
                      -3.5 * fieldLineEndUp / fieldLineLength * sin(2.0 * 3.1415927 / 9),
                      4.0 * fieldLineEndUp / fieldLineLength * cos(alpha),
                      4.0 * fieldLineEndUp / fieldLineLength * sin(alpha),
                      -4.0 * fieldLineEndUp / fieldLineLength * sin(2.0 * 3.1415927 / 9), 0.14, 0.0, 32);//Field line pointer
      }

      drawCylinder(0.0, 0.0, 0.0, 0.001, 0.001, -fieldLineEndUp, 0.03, 0.03, 32);//Field line
      drawCylinder( 0.0, 0.0, -3.5 * fieldLineEndUp / fieldLineLength, 0.001, 0.001, -4.0 * fieldLineEndUp / fieldLineLength, 0.14, 0.0, 32);//Field line pointer

      glColor3f(1.0f, 0.0f, 0.0f);

      glLightfv(GL_LIGHT1, GL_AMBIENT, lightRed1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightRed1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightOrange1);
      drawSphere(0.0, 0.0, 0.0, 0.5, 40.0, 40.0, 0);      
      glLightfv(GL_LIGHT1, GL_AMBIENT, lightAmbient1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightDiffuse1);
      glLightfv(GL_LIGHT1,GL_SPECULAR,specular1);      
      
      fieldLineEndUp += 0.3;
      if(fieldLineEndUp > fieldLineLength){
        printf("Animation set to 102.\n");
        animation = 102;
      }
    }    

    if(animation == 102){
      GLfloat fieldLineLength = 40.0;
      double alpha, beta;
      
      glEnable(GL_LIGHTING);
      glPolygonMode(GL_BACK, GL_FILL);
      glPolygonMode(GL_FRONT, GL_FILL);


      glLightfv(GL_LIGHT1, GL_AMBIENT, lightGreen1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightGreen1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightGreen1);


      for(alpha = 0.0; alpha < 2.0 * 3.1415927; alpha += 2.0 * 3.1415927 / 18.0){
        drawCylinder(0.0, 0.0, 0.0, fieldLineLength * cos(alpha), fieldLineLength * sin(alpha), 0.0, 0.03, 0.03, 32);//Field line
        drawCylinder( 3.5 * cos(alpha), 3.5 * sin(alpha), 0.0, 4.0 * cos(alpha), 4.0 * sin(alpha), 0.0, 0.14, 0.0, 32);//Field line pointer
      }
      
      for(alpha = 0.0; alpha < 2.0 * 3.1415927; alpha += 2.0 * 3.1415927 / 9.0){
        
        drawCylinder(0.0, 0.0, 0.0, fieldLineEndUp * cos(alpha), fieldLineEndUp * sin(alpha), -fieldLineEndUp * sin(2.0 * 3.1415927 / 9), 0.03, 0.03, 32);//Field line
        drawCylinder( 3.5 * fieldLineEndUp / fieldLineLength * cos(alpha),
                      3.5 * fieldLineEndUp / fieldLineLength * sin(alpha),
                      -3.5 * fieldLineEndUp / fieldLineLength * sin(2.0 * 3.1415927 / 9),
                      4.0 * fieldLineEndUp / fieldLineLength * cos(alpha),
                      4.0 * fieldLineEndUp / fieldLineLength * sin(alpha),
                      -4.0 * fieldLineEndUp / fieldLineLength * sin(2.0 * 3.1415927 / 9), 0.14, 0.0, 32);//Field line pointer
      }

      drawCylinder(0.0, 0.0, 0.0, 0.001, 0.001, -fieldLineEndUp, 0.03, 0.03, 32);//Field line
      drawCylinder( 0.0, 0.0, -3.5 * fieldLineEndUp / fieldLineLength, 0.001, 0.001, -4.0 * fieldLineEndUp / fieldLineLength, 0.14, 0.0, 32);//Field line pointer


      for(alpha = 0.0; alpha < 2.0 * 3.1415927; alpha += 2.0 * 3.1415927 / 9.0){
        
        drawCylinder(0.0, 0.0, 0.0, fieldLineEndDown * cos(alpha), fieldLineEndDown * sin(alpha), fieldLineEndDown * sin(2.0 * 3.1415927 / 9), 0.03, 0.03, 32);//Field line
        drawCylinder( 3.5 * fieldLineEndDown / fieldLineLength * cos(alpha),
                      3.5 * fieldLineEndDown / fieldLineLength * sin(alpha),
                      3.5 * fieldLineEndDown / fieldLineLength * sin(2.0 * 3.1415927 / 9),
                      4.0 * fieldLineEndDown / fieldLineLength * cos(alpha),
                      4.0 * fieldLineEndDown / fieldLineLength * sin(alpha),
                      4.0 * fieldLineEndDown / fieldLineLength * sin(2.0 * 3.1415927 / 9), 0.14, 0.0, 32);//Field line pointer
      }

      drawCylinder(0.0, 0.0, 0.0, 0.001, 0.001, fieldLineEndDown, 0.03, 0.03, 32);//Field line
      drawCylinder( 0.0, 0.0, 3.5 * fieldLineEndDown / fieldLineLength, 0.001, 0.001, 4.0 * fieldLineEndDown / fieldLineLength, 0.14, 0.0, 32);//Field line pointer


      glColor3f(1.0f, 0.0f, 0.0f);

      glLightfv(GL_LIGHT1, GL_AMBIENT, lightRed1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightRed1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightOrange1);
      drawSphere(0.0, 0.0, 0.0, 0.5, 40.0, 40.0, 0);      
      glLightfv(GL_LIGHT1, GL_AMBIENT, lightAmbient1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightDiffuse1);
      glLightfv(GL_LIGHT1,GL_SPECULAR,specular1);      
      
      if(fieldLineEndDown > fieldLineLength){
        animation = 102;
        if(rotX < 360.0){
          rotXdiff = 0.3;
        }
        else{
          savePics = 0;
          printf("...un ferddisch!\n");
          done = True;         // Exit programm !!!
        }
      }
      else{
        fieldLineEndDown += 0.3;
      }
    }    


//####################################################################################
//############# Paint field lines of a positive and a negative charge ################
//####################################################################################
    if(animation == 200 && firstStart < 2){
      GLfloat fieldLineCoords[10000][4];
      GLfloat fieldLineLength = 40.0;
      double alpha, beta;
      
      glEnable(GL_LIGHTING);
      glPolygonMode(GL_BACK, GL_FILL);
      glPolygonMode(GL_FRONT, GL_FILL);


      glLightfv(GL_LIGHT1, GL_AMBIENT, lightGreen1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightGreen1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightGreen1);


      for(i = 0; i < 8; i++){
        alpha = i * 2.0 * 3.1415927 / 8;
        fieldLineCoords[i][0] = particles[0][0] + sin(alpha) * particles[0][3];
        fieldLineCoords[i][1] = particles[0][1] + cos(alpha) * particles[0][3];
        fieldLineCoords[i][2] = 0.0;
      }
      for(i = 8; i < 12; i++){
        alpha = i * 2.0 * 3.1415927 / 4;
        fieldLineCoords[i][0] = particles[0][0] + sin(alpha) * particles[0][3] * 0.5;
        fieldLineCoords[i][1] = particles[0][1] + cos(alpha) * particles[0][3] * 0.5;
        fieldLineCoords[i][2] = particles[0][3] * 0.5;
      }
      for(i = 12; i < 16; i++){
        alpha = i * 2.0 * 3.1415927 / 4;
        fieldLineCoords[i][0] = particles[0][0] + sin(alpha) * particles[0][3] * 0.5;
        fieldLineCoords[i][1] = particles[0][1] + cos(alpha) * particles[0][3] * 0.5;
        fieldLineCoords[i][2] = -particles[0][3] * 0.5;
      }

      fieldLineCoords[16][0] = particles[0][0];
      fieldLineCoords[16][1] = particles[0][1];
      fieldLineCoords[16][2] = -particles[0][3];

      fieldLineCoords[17][0] = particles[0][0];
      fieldLineCoords[17][1] = particles[0][1];
      fieldLineCoords[17][2] = particles[0][3];

      fieldLineCoords[18][0] = 40.0;
      fieldLineCoords[18][1] = 0.0;
      fieldLineCoords[18][2] = 0.0;
      
      for(i=0; i<18; i++){
        fieldLineCoords[i][3] = 0.0;
        if(fieldLineCoords[i][0] > particles[0][0]){
          fieldLineCoords[i][3] = 3.0;
        }
      }
      fieldLineCoords[18][3] = 1.0;

      for(i2 = 0; i2 < 19; i2++){
        for(i = 0; i < fieldLineEndUp; i++){
//        for(i = 0; i < 44; i++){// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
          totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[0][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[0][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[0][2], 2.0));
          totalForce = k * particles[0][4] / pow(totalDistance, 2.0);
          forceX = (fieldLineCoords[i2][0] - particles[0][0]) / totalDistance * totalForce;
          forceY = (fieldLineCoords[i2][1] - particles[0][1]) / totalDistance * totalForce;
          forceZ = (fieldLineCoords[i2][2] - particles[0][2]) / totalDistance * totalForce;

          totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
          totalForce = k * particles[1][4] / pow(totalDistance, 2.0);
          forceX += (fieldLineCoords[i2][0] - particles[1][0]) / totalDistance * totalForce;
          forceY += (fieldLineCoords[i2][1] - particles[1][1]) / totalDistance * totalForce;
          forceZ += (fieldLineCoords[i2][2] - particles[1][2]) / totalDistance * totalForce;

          totalForce = sqrt(pow(forceX, 2.0) + pow(forceY, 2.0) + pow(forceZ, 2.0));
          x2 = fieldLineCoords[i2][0] + forceX / totalForce * fieldLineStep;
          y2 = fieldLineCoords[i2][1] + forceY / totalForce * fieldLineStep;
          z2 = fieldLineCoords[i2][2] + forceZ / totalForce * fieldLineStep;
          
          drawCylinder(fieldLineCoords[i2][0], fieldLineCoords[i2][1], fieldLineCoords[i2][2], x2, y2, z2, 0.03, 0.03, 32);
          
          if(fieldLineCoords[i2][3] == 3.0){
            if(x2 > 0.0){
              drawCylinder(x1, y1, z1, x2, y2, z2, 0.1, 0.03, 32);
              fieldLineCoords[i2][3] = 2.0;
            }
          }
          else{
            if(fieldLineCoords[i2][3] == 0.0){
              totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[0][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[0][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[0][2], 2.0));
              if(totalDistance > arrowRadius){
                drawCylinder(x1, y1, z1, x2, y2, z2, 0.1, 0.03, 32);
                fieldLineCoords[i2][3] = 1.0;
              }
            }
            if(fieldLineCoords[i2][3] == 1.0){
              totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
              if(totalDistance < arrowRadius){
                drawCylinder(x1, y1, z1, x2, y2, z2, 0.1, 0.03, 32);
                fieldLineCoords[i2][3] = 2.0;
              }
            }
          }

          x1 = fieldLineCoords[i2][0];
          y1 = fieldLineCoords[i2][1];
          z1 = fieldLineCoords[i2][2];

          fieldLineCoords[i2][0] = x2;
          fieldLineCoords[i2][1] = y2;
          fieldLineCoords[i2][2] = z2;
          
          totalDistance = sqrt(pow(fieldLineCoords[i2][0] - particles[1][0], 2.0) + pow(fieldLineCoords[i2][1] - particles[1][1], 2.0) + pow(fieldLineCoords[i2][2] - particles[1][2], 2.0));
          if(totalDistance < particles[1][3]){//Field line has reached negative charge
            break;
          }
        }//for(i = 0; i < fieldLineEndUp; i++){
      }//for(i2 = 0; i2 < 2; i2++){
      
//      for(alpha = 0.0; alpha < 2.0 * 3.1415927; alpha += 2.0 * 3.1415927 / 18.0){
//        drawCylinder(0.0, 0.0, 0.0, fieldLineLength * cos(alpha), fieldLineLength * sin(alpha), 0.0, 0.03, 0.03, 32);//Field line
//        drawCylinder( 3.5 * cos(alpha), 3.5 * sin(alpha), 0.0, 4.0 * cos(alpha), 4.0 * sin(alpha), 0.0, 0.14, 0.0, 32);//Field line pointer
//      }
      glColor3f(1.0f, 0.0f, 0.0f);

      glLightfv(GL_LIGHT1, GL_AMBIENT, lightRed1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightRed1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightOrange1);
      drawSphere(particles[0][0], particles[0][1], particles[0][2], particles[0][3], 40.0, 40.0, 0);      

      glLightfv(GL_LIGHT1, GL_AMBIENT, lightBlue1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightBlue1);
      glLightfv(GL_LIGHT1, GL_SPECULAR, lightBlue1);
      drawSphere(particles[1][0], particles[1][1], particles[1][2], particles[1][3], 40.0, 40.0, 0);      

      glLightfv(GL_LIGHT1, GL_AMBIENT, lightAmbient1);
      glLightfv(GL_LIGHT1, GL_DIFFUSE, lightDiffuse1);
      glLightfv(GL_LIGHT1,GL_SPECULAR,specular1);
      
      if(fieldLineEndUp < 1000){
        fieldLineEndUp++;
      }
      if(fieldLineEndUp > 40){
        rotXdiff = 0.5;
        if(translateZ > -27.0){
          translateZdiff = -0.05;
        }
        else{
          translateZdiff = 0.0;
        }
      }
      if(rotX > 370.0){
        savePics = 0;
        printf("...un ferddisch!\n");
        done = True;         // Exit programm !!!
      }
    }




//###########################################################################################
//############### Print text messages #######################################################
//###########################################################################################

    glEnable(GL_TEXTURE_2D);       /* Enable Texture Mapping */
    glEnable(GL_BLEND);
    glDisable(GL_LIGHTING);
    glEnable(GL_DEPTH_TEST);
    //glDisable(GL_BLEND);
    glLoadIdentity();
    glColor3f(1.0f, 1.0f, 0.0f);
    if(animation < 100){
      sprintf(textLine, "Force over X/Z plane");
      printGLf(-4.3, 2.3, textLine, 0, 1, 0.07f);
      sprintf(textLine, "%+0.002lf^C at 0/0", particleCharge);//Character ^ ist transformed to µ
      if(particles[0][4] > 0.0f){
        printGLf(-4.3, 2.0, textLine, 0, 3, 0.07f);
      }
      else{
        printGLf(-4.3, 2.0, textLine, 0, 6, 0.07f);
      }
    }
    

    if(animation >= 400 && animation < 435){
      sprintf(textLine, "Potential over X/Z plane", particleCharge);
      printGLf(-4.35, 2.35, textLine, 0, -5, 0.075f);
      sprintf(textLine, "+%0.01lf^C at %0.01lf/%0.01lf", particles[0][4] * 1000000, particles[0][0], particles[0][2]);
      printGLf(-4.35, 2.05, textLine, 0, -5, 0.075f);
      if(particleNumber > 1){
        sprintf(textLine, "%0.01lf^C at  %0.01lf/%0.01lf", particles[1][4] * 1000000, particles[1][0], particles[1][2]);
        printGLf(-4.35, 1.75, textLine, 0, -5, 0.075f);
      }
    }

    if(animation >= 435 && animation < 500){
      sprintf(textLine, "+%0.01lf^C at %0.01lf/%0.01lf", particles[0][4] * 1000000, particles[0][0], particles[0][2]);
      printGLf(-4.35, 2.35, textLine, 0, -5, 0.075f);
      if(particleNumber > 1){
        sprintf(textLine, "%0.01lf^C at  %0.01lf/%0.01lf", particles[1][4] * 1000000, particles[1][0], particles[1][2]);
        printGLf(-4.35, 2.05, textLine, 0, -5, 0.075f);
      }
    }


    if(enablePotentialPoints == 1){
      for(i = 0; i < 10; i++){
        //printf("potentialPoints[i][1] / scalePlaneHeigth = %lf\n", potentialPoints[i][1] / scalePlaneHeigth);
        if(potentialPoints[i][1] < 0.0f){
          sprintf(textLine, "P(%2d)=%+6.2lfkJ/C", i+1, potentialPoints[i][1] * 1000.0f / scalePlaneHeigth);
          printGLf(1.5, 2.4 - (i * 0.25f), textLine, 0, -6, 0.075f);
        }
        else{
          sprintf(textLine, "P(%2d)=%+6.2lfkJ/C", i+1, potentialPoints[i][1] * 1000.0f / scalePlaneHeigth);
          printGLf(1.5, 2.4 - (i * 0.25f), textLine, 0, -3, 0.075f);
        }        
      }
    }

    if(enablePotentialDiff == 1){
      for(i = 0; i < 10; i+=2){
        sprintf(textLine, "P(%2d)-P(%2d)=%+6.2lfkJ/C", i+2, i+1, potentialPoints[i+1][1] * 1000.0f / scalePlaneHeigth - potentialPoints[i][1] * 1000.0f / scalePlaneHeigth);
        if(i == 4){
          if(particles[0][0] > -0.25f || fabs(potentialPoints[i+1][1] * 1000.0f / scalePlaneHeigth - potentialPoints[i][1] * 1000.0f / scalePlaneHeigth) > 99.99f){
            sprintf(textLine, "P(%2d)-P(%2d)<-99.99kJ/C", i+2, i+1);
            printGLf(1.1, 2.4 - (i * 0.25f), textLine, 0, -6, 0.075f);
          }
          else{
            printGLf(1.1, 2.4 - (i * 0.25f), textLine, 0, -6, 0.075f);
          }
        }
        else{
          printGLf(1.1, 2.4 - (i * 0.25f), textLine, 0, -3, 0.075f);
        }
      }
    }


    if(animation >= 300 && animation < 400){
      sprintf(textLine, "Potential over", particleCharge);
      printGLf(-4.3, 2.4, textLine, 0, -5, 0.06f);
      sprintf(textLine, "X/Z plane", particleCharge);
      printGLf(-4.3, 2.15, textLine, 0, -5, 0.06f);
      if(animation > 324){
        sprintf(textLine, "-1^C at 0/0");
      }
      else{
        sprintf(textLine, "+1^C at 0/0");
      }
      printGLf(-4.3, 1.9, textLine, 0, -5, 0.06f);

      totalDistance = sqrt(pow(particles[1][0], 2.0) + pow(particles[1][1], 2.0) + pow(particles[1][2], 2.0));
      if(totalDistance < minDistance){
        totalDistance = minDistance;
      }
      potential = k1 * particles[0][4] * particles[1][4] / totalDistance;
      y1 = potential * 1000.0f;
      
      if(animation == 307){
        sprintf(textLine, "+1^C at %0.02lf/%0.02lf", particles[1][0], particles[1][2]);//Character ^ ist transformed to µ
        printGLf(1.2, 2.2, textLine, 0, 1, 0.08f);
        
        sprintf(textLine, "E = %0.002lfmJ", y1);//Character ^ ist transformed to µ
        printGLf(1.2, 1.9, textLine, 0, 1, 0.08f);
      }
      if(animation > 307 && animation < 315){
        sprintf(textLine, "+1^C at 1.50/0.00");//Character ^ ist transformed to µ
        printGLf(1.2, 2.2, textLine, 0, 1, 0.08f);

        sprintf(textLine, "E = 6.00mJ");//Character ^ ist transformed to µ
        printGLf(1.2, 1.9, textLine, 0, 1, 0.08f);
        

        sprintf(textLine, "+1^C at %0.02lf/%0.02lf", particles[1][0], particles[1][2]);//Character ^ ist transformed to µ
        printGLf(1.2, 1.6, textLine, 0, 2, 0.08f);
        
        sprintf(textLine, "E = %0.002lfmJ", y1);//Character ^ ist transformed to µ
        printGLf(1.2, 1.3, textLine, 0, 2, 0.08f);
      }
      if(animation > 314 && animation < 321){

        sprintf(textLine, "+1^C at %0.02lf/%0.02lf", particles[1][0], particles[1][2]);//Character ^ ist transformed to µ
        printGLf(1.2, 2.2, textLine, 0, 3, 0.08f);
        
        sprintf(textLine, "E = %0.002lfmJ", y1);//Character ^ ist transformed to µ
        printGLf(1.2, 1.9, textLine, 0, 3, 0.08f);
      }
      if(animation > 325 && animation < 330){
        sprintf(textLine, "+1^C at %0.02lf/%0.02lf", particles[1][0], particles[1][2]);//Character ^ ist transformed to µ
        printGLf(1.2, 2.2, textLine, 0, -4, 0.08f);
        
        sprintf(textLine, "E = %0.002lfmJ", y1);//Character ^ ist transformed to µ
        printGLf(1.2, 1.9, textLine, 0, -4, 0.08f);
      }

    }
    
    glDisable(GL_TEXTURE_2D);       /* Enable Texture Mapping */
    glDisable(GL_BLEND);
    
    
    
    glFlush();
    glXSwapBuffers(GLWin.dpy, GLWin.win);

    if(firstStart > 0){
      firstStart--;
    }
    else{
      if(savePics == 1){
        if(picNumber >= startPicNumber){// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
          ScreenShot();
        }
        else{
          picNumber++;
        }
      }
    }

    return True;    
}

/* function to release/destroy our resources and restoring the old desktop */
GLvoid killGLWindow(GLvoid){
  if (GLWin.ctx){
    if (!glXMakeCurrent(GLWin.dpy, None, NULL)){
      printf("Could not release drawing context.\n");
    }
    glXDestroyContext(GLWin.dpy, GLWin.ctx);
    GLWin.ctx = NULL;
  }
  /* switch back to original desktop resolution if we were in fs */
  if (GLWin.fs){
    XF86VidModeSwitchToMode(GLWin.dpy, GLWin.screen, &GLWin.deskMode);
    XF86VidModeSetViewPort(GLWin.dpy, GLWin.screen, 0, 0);
  }
  XCloseDisplay(GLWin.dpy);
}

/* this function creates our window and sets it up properly */
/* FIXME: bits is currently unused */
Bool createGLWindow(char* title, int width, int height, int bits, Bool fullscreenflag){
    XVisualInfo *vi;
    Colormap cmap;
    int dpyWidth, dpyHeight;
    int i;
    int glxMajorVersion, glxMinorVersion;
    int vidModeMajorVersion, vidModeMinorVersion;
    XF86VidModeModeInfo **modes;
    int modeNum;
    int bestMode;
    Atom wmDelete;
    Window winDummy;
    unsigned int borderDummy;
    
    GLWin.fs = fullscreenflag;
    /* set best mode to current */
    bestMode = 0;
    /* get a connection */
    GLWin.dpy = XOpenDisplay(0);
    GLWin.screen = DefaultScreen(GLWin.dpy);
    XF86VidModeQueryVersion(GLWin.dpy, &vidModeMajorVersion,
        &vidModeMinorVersion);
    printf("XF86VidModeExtension-Version %d.%d\n", vidModeMajorVersion,
        vidModeMinorVersion);
    XF86VidModeGetAllModeLines(GLWin.dpy, GLWin.screen, &modeNum, &modes);
    /* save desktop-resolution before switching modes */
    GLWin.deskMode = *modes[0];
    /* look for mode with requested resolution */
    for (i = 0; i < modeNum; i++)
    {
        if ((modes[i]->hdisplay == width) && (modes[i]->vdisplay == height))
        {
            bestMode = i;
        }
    }
    /* get an appropriate visual */
    vi = glXChooseVisual(GLWin.dpy, GLWin.screen, attrListDbl);
    if (vi == NULL)
    {
        vi = glXChooseVisual(GLWin.dpy, GLWin.screen, attrListSgl);
        printf("Only Singlebuffered Visual!\n");
    }
    else
    {
        printf("Got Doublebuffered Visual!\n");
    }
    glXQueryVersion(GLWin.dpy, &glxMajorVersion, &glxMinorVersion);
    printf("glX-Version %d.%d\n", glxMajorVersion, glxMinorVersion);
    /* create a GLX context */
    GLWin.ctx = glXCreateContext(GLWin.dpy, vi, 0, GL_TRUE);
    /* create a color map */
    cmap = XCreateColormap(GLWin.dpy, RootWindow(GLWin.dpy, vi->screen),
        vi->visual, AllocNone);
    GLWin.attr.colormap = cmap;
    GLWin.attr.border_pixel = 0;

    if (GLWin.fs)
    {
        XF86VidModeSwitchToMode(GLWin.dpy, GLWin.screen, modes[bestMode]);
        XF86VidModeSetViewPort(GLWin.dpy, GLWin.screen, 0, 0);
        dpyWidth = modes[bestMode]->hdisplay;
        dpyHeight = modes[bestMode]->vdisplay;
        printf("Resolution %dx%d\n", dpyWidth, dpyHeight);
        XFree(modes);
    
        /* create a fullscreen window */
        GLWin.attr.override_redirect = True;
        GLWin.attr.event_mask = ExposureMask | KeyPressMask | ButtonPressMask |
            StructureNotifyMask;
        GLWin.win = XCreateWindow(GLWin.dpy, RootWindow(GLWin.dpy, vi->screen),
            0, 0, dpyWidth, dpyHeight, 0, vi->depth, InputOutput, vi->visual,
            CWBorderPixel | CWColormap | CWEventMask | CWOverrideRedirect,
            &GLWin.attr);
        XWarpPointer(GLWin.dpy, None, GLWin.win, 0, 0, 0, 0, 0, 0);
		XMapRaised(GLWin.dpy, GLWin.win);
        XGrabKeyboard(GLWin.dpy, GLWin.win, True, GrabModeAsync,
            GrabModeAsync, CurrentTime);
        XGrabPointer(GLWin.dpy, GLWin.win, True, ButtonPressMask,
            GrabModeAsync, GrabModeAsync, GLWin.win, None, CurrentTime);
    }
    else
    {
        /* create a window in window mode*/
        GLWin.attr.event_mask = ExposureMask | KeyPressMask | ButtonPressMask |
            StructureNotifyMask;
        GLWin.win = XCreateWindow(GLWin.dpy, RootWindow(GLWin.dpy, vi->screen),
            0, 0, width, height, 0, vi->depth, InputOutput, vi->visual,
            CWBorderPixel | CWColormap | CWEventMask, &GLWin.attr);
        /* only set window title and handle wm_delete_events if in windowed mode */
        wmDelete = XInternAtom(GLWin.dpy, "WM_DELETE_WINDOW", True);
        XSetWMProtocols(GLWin.dpy, GLWin.win, &wmDelete, 1);
        XSetStandardProperties(GLWin.dpy, GLWin.win, title,
            title, None, NULL, 0, NULL);
        XMapRaised(GLWin.dpy, GLWin.win);
    }       
    /* connect the glx-context to the window */
    glXMakeCurrent(GLWin.dpy, GLWin.win, GLWin.ctx);
    XGetGeometry(GLWin.dpy, GLWin.win, &winDummy, &GLWin.x, &GLWin.y,
        &GLWin.width, &GLWin.height, &borderDummy, &GLWin.depth);
    printf("Depth %d\n", GLWin.depth);
    if (glXIsDirect(GLWin.dpy, GLWin.ctx)) 
        printf("Congrats, you have Direct Rendering!\n");
    else
        printf("Sorry, no Direct Rendering possible!\n");
    if (!initGL())
    {
        printf("Could not initialize OpenGL.\nAborting...\n");
        return False;
    }        
    return True;    
}
void keyPressed(KeySym key){
    switch (key){
        case XK_Escape:
            done = True;
            break;
        case XK_F1:
            killGLWindow();
            GLWin.fs = !GLWin.fs;
            createGLWindow("Electric field", 480, 270, 24, GLWin.fs);
            break;
        case XK_F2:
            ScreenShot();
            break;
        case XK_space:
            if(animationPause == 0){
              animationPause = 1;
            }
            else{
              animationPause = 0;
            }
            break;
        case XK_u:
            translateY += 0.01f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
        case XK_d:
            translateY -= 0.01f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
        case XK_l:
            translateX -= 0.01f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
        case XK_r:
            translateX += 0.01f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
        case XK_plus:
            translateZ += 0.05f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
        case XK_minus:
            translateZ -= 0.05f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
        case XK_Left:
            rotY += 0.25f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
        case XK_Right:
            rotY -= 0.25f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
        case XK_Up:
            rotX -= 0.25f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
        case XK_Down:
            rotX += 0.25f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
        case XK_Page_Up:
            rotZ += 0.25f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
        case XK_Page_Down:
            rotZ -= 0.25f;
            printf("translateX=%lf, translateY=%lf, translateZ=%lf, rotX=%lf, rotY=%lf, rotZ=%lf\n", translateX, translateY, translateZ, rotX, rotY, rotZ);
            break;
    }
}
int main(int argc, char **argv){
    XEvent event;
    KeySym key;
    GLfloat steps = 200.0;
    double x, y = 1;
    int i;
    int wrongArgs = 1;
    
    animation = -1;
    //savePics = 1;
    startPicNumber = 1000000;
    slowDownMS = 0;

    
    if (argc > 3){
      for (i = 1; i < argc; i++){
	      printf("argv[%d] = %s\n", i, argv[i]);
	    }
      if(strcmp(argv[1], "1") == 0){
        animation = 1;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "2") == 0){
        animation = 100;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "3") == 0){
        animation = 200;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "4") == 0){
        animation = 300;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "5") == 0){
        animation = 400;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "6") == 0){
        animation = 500;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "7") == 0){
        animation = 600;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "8") == 0){
        animation = 700;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "9") == 0){
        animation = 800;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "10") == 0){
        animation = 900;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "11") == 0){
        animation = 1000;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "12") == 0){
        animation = 1100;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "13") == 0){
        animation = 1200;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "14") == 0){
        animation = 1300;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "15") == 0){
        animation = 1400;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "16") == 0){
        animation = 1500;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "17") == 0){
        animation = 1600;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "18") == 0){
        animation = 1700;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "19") == 0){
        animation = 1800;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "20") == 0){
        animation = 1900;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "21") == 0){
        animation = 2000;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "22") == 0){
        animation = 2100;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "23") == 0){
        animation = 2200;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "24") == 0){
        animation = 2300;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "25") == 0){
        animation = 2400;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "26") == 0){
        animation = 2500;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "27") == 0){
        animation = 2600;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "28") == 0){
        animation = 2700;
        wrongArgs = 0;
      }
      if(strcmp(argv[1], "29") == 0){
        animation = 2800;
        wrongArgs = 0;
      }


      slowDownMS = strtol(argv[2], NULL, 10);
      startPicNumber = strtol(argv[3], NULL, 10);
      if(startPicNumber == 0){
        startPicNumber = 1000000;
      }
      

    }
    else{
      wrongArgs = 1;
    }
        
    if(wrongArgs > 0){
      printf("The command had no or too few arguments.\n");
      printf("Example of usage: Electrostatics 21 10 0\n");
      printf("First integer dials the animation sequence (1-29)\n");
      printf("Second integer sets slow down factor of animation in milliseconds(0-10000)\n");
      printf("If third integer is greater than 0, pictures are stored in folder PicSequence, starting with picture number given.\n");
      printf("\n");
      printf("When animation is running:\n");
      printf("Space: Stop/Start\n");
      printf("+: Zoom in\n");
      printf("-: Zoom out\n");
      printf("u: move up\n");
      printf("d: move down\n");
      printf("cursor: rotate\n");
      printf("up/down key: rotate\n");
      printf("F2: take snapshot\n");
      printf("F1: change between window mode and fullscreen\n");
      printf("\n");
      printf("www.HomoFaciens.de\n");
      exit(1);
    }
    
    
    printf("animation set to %d\n",animation);
    for(x=0; x<10;x++){
      printf("atan2(%lf, %lf)=%lf,    atan2(%lf, %lf)=%lf\n", y, x, atan2(y, x), x, y, atan2(x, y));
    }
        
    done = False;
    /* default to fullscreen */
    GLWin.fs = True;
    if (!createGLWindow("Electric field", 1920, 1080, 24, GLWin.fs)){
        done = True;
    }
    
    
    printf("Force = %lf\n", 8.988f * 1000000000.0f * 0.000001f * 0.000001f / (8.0f * 8.0f));

    
    for(i=0; i<PATHMAX; i++){
      pathWidth[i] = 4.5f;
    }
    
    if(animation == 1){

      enableMinDisc = 1;
      enableParticles = 1;

      particles[0][0] = 0.0f;
      particles[0][1] = 0.0f;
      particles[0][2] = 0.0f;
      particles[0][3] = 0.5f;
      particles[0][4] = 0.000001f;

      
      
      translateX = 0.0f;
      translateY = -2.0f;
      translateZ = -6.9f;
      rotX = -15.75f;
      rotY = 0.0f;
      rotZ = 0.0f;

      translateXtarget = 0.0f;
      translateYtarget = -1.14f;
      translateZtarget = -15.4f;
      
      rotXtarget = 29.5f;
      rotYtarget = 0.0f;
      rotZtarget = 0.0f;
      
      translateXdiff = (translateXtarget - translateX) / steps;
      translateYdiff = (translateYtarget - translateY) / steps;
      translateZdiff = (translateZtarget - translateZ) / steps;
      rotXdiff = (rotXtarget - rotX) / steps;
      rotYdiff = (rotYtarget - rotY) / steps;
      rotZdiff = (rotZtarget - rotZ) / steps;
      
    }

    if(animation == 100){
      
      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -13.3f;
      rotX = 0.0f;
      rotZ = 0.0f;
      rotY = 0.0f;
/*      
      translateXtarget = 0.0f;
      translateYtarget = -0.94f;
      translateZtarget = -13.3f;
      
      rotXtarget = 113.0f;
      rotZtarget = 0.0f;
      rotYtarget = 0.0f;
*/      
      translateXtarget = 0.0f;
      translateYtarget = -0.94f;
      translateZtarget = -13.3f;
      
      rotXtarget = -94.00f;
      rotYtarget = 0.0f;
      rotZtarget = -3.5f;
      
      translateXdiff = (translateXtarget - translateX) / steps;
      translateYdiff = (translateYtarget - translateY) / steps;
      translateZdiff = (translateZtarget - translateZ) / steps;
      rotXdiff = (rotXtarget - rotX) / steps;
      rotYdiff = (rotYtarget - rotY) / steps;
      rotZdiff = (rotZtarget - rotZ) / steps;
      
    }
    
    if(animation == 200){
      fieldLineEndUp = 0.0f;

      particles[0][0] = -2.2;
      particles[0][1] = 0.0;
      particles[0][2] = 0.0;
      particles[0][3] = 0.5;
      particles[0][4] = 0.000001;
      
      particles[1][0] = 2.2;
      particles[1][1] = 0.0;
      particles[1][2] = 0.0;
      particles[1][3] = 0.5;
      particles[1][4] = -0.000001;
      
      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -13.3f;
      rotX = 0.0f;
      rotZ = 0.0f;
      rotY = 0.0f;
      
      translateXtarget = 0.0f;
      translateYtarget = 0.0f;
      translateZtarget = -13.3f;
      
      rotXtarget = 0.0f;
      rotZtarget = 0.0f;
      rotYtarget = 0.0f;
/*      
      translateXtarget = 0.0f;
      translateYtarget = -0.94f;
      translateZtarget = -13.3f;
      
      rotXtarget = 85.75f;
      rotYtarget = 0.0f;
      rotZtarget = -3.5f;
*/      
      translateXdiff = (translateXtarget - translateX) / steps;
      translateYdiff = (translateYtarget - translateY) / steps;
      translateZdiff = (translateZtarget - translateZ) / steps;
      rotXdiff = (rotXtarget - rotX) / steps;
      rotYdiff = (rotYtarget - rotY) / steps;
      rotZdiff = (rotZtarget - rotZ) / steps;
      
    }

    if(animation == 300){
      
      enablePotentialPlane = 1;
      enableMinDisc = 1;
      minDistance = 0.39;
      
      particleCharge = 1.0;
      
      particleNumber = 2;
      
      particles[0][0] = 0.0;
      particles[0][1] = 0.0;
      particles[0][2] = 0.0;
      particles[0][3] = 0.3;
      particles[0][4] = 0.000001;
      
      particles[1][0] = 100.0;
      particles[1][1] = 0.0;
      particles[1][2] = 0.0;
      particles[1][3] = 0.15;
      particles[1][4] = 0.000001;
            
      translateX = 0.0f;
      translateY = -1.53f;
      translateZ = -5.4f;
      rotX = -11.45f;
      rotY = 0.0f;
      rotZ = 1.0f;


      translateXtarget = 0.0f;
      translateYtarget = -1.24f;
      translateZtarget = -29.7f;
      
      rotXtarget = 25.3f;
      rotYtarget = 0.0f;
      rotZtarget = 1.0f;

      translateXdiff = (translateXtarget - translateX) / steps / 2.0;
      translateYdiff = (translateYtarget - translateY) / steps / 2.0;
      translateZdiff = (translateZtarget - translateZ) / steps / 2.0;
      rotXdiff = (rotXtarget - rotX) / steps / 2.0;
      rotYdiff = (rotYtarget - rotY) / steps / 2.0;
      rotZdiff = (rotZtarget - rotZ) / steps / 2.0;

    }

    if(animation == 400){
      static GLboolean s_usePerspective = GL_TRUE;
//      glMatrixMode(GL_PROJECTION);
//      glLoadIdentity();
//      glOrtho(-1.0, 1.0, -1.0, 1.0, 5, 100);
//      glFrustum(-1.0, 1.0, -1.0, 1.0, 5, 100);

      
      //gridX = gridX * 4;
      //gridY = gridY * 4;
      
      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 1;
      
      minDistance = 0.39;
      enableMinDisc = 1;
      
      particleCharge = 1.0;
      
      particleNumber = 1;
      potentialEnd = 1;

      lightAmbient1[0] = 0.8f;
      lightAmbient1[1] = 0.8f;
      lightAmbient1[2] = 0.8f;

      lightDiffuse1[0] = 0.8f;
      lightDiffuse1[1] = 0.8f;
      lightDiffuse1[2] = 0.8f;
      
      particles[0][0] = 0.0;
      particles[0][1] = 0.0;
      particles[0][2] = 0.0;
      particles[0][3] = 0.3;
      particles[0][4] = 0.000001;
      
      particles[1][0] = 3.0;
      particles[1][1] = 0.0;
      particles[1][2] = 0.0;
      particles[1][3] = 0.3;
      particles[1][4] = 0.000001;
            
      //test charge
      particles[2][0] = 200.0;
      particles[2][1] = 0.0;
      particles[2][2] = 0.0;
      particles[2][3] = 0.15;
      particles[2][4] = 0.000001;
            
      translateX = 0.0f;
      translateY = -1.25f;
      translateZ = -14.8f;
      rotX = 26.05f;
      rotY = 0.0f;
      rotZ = 1.0f;


      translateXtarget = 0.0f;
      translateYtarget = -1.25f;
      translateZtarget = -14.8f;
      
      rotXtarget = 26.5f;
      rotYtarget = 360.0f;
      rotZtarget = 1.0f;

      translateXdiff = (translateXtarget - translateX) / steps / 2.0;
      translateYdiff = (translateYtarget - translateY) / steps / 2.0;
      translateZdiff = (translateZtarget - translateZ) / steps / 2.0;
      rotXdiff = (rotXtarget - rotX) / steps / 2.0;
      rotYdiff = (rotYtarget - rotY) / steps / 2.0;
      rotZdiff = (rotZtarget - rotZ) / steps / 2.0;

    }

    if(animation == 500){
      
      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 0;
      enableMinDisc = 0;
      enableParticles = 1;
      enableFieldVectors = 1;
      minVectorDistance = 0.07f;
      
      
      particleNumber = 2;

      
      particles[0][0] = -2.0;
      particles[0][1] = 0.0;
      particles[0][2] = 0.0;
      particles[0][3] = 0.25;
      particles[0][4] = 0.000001;
      
      particles[1][0] = 2.0;
      particles[1][1] = 0.0;
      particles[1][2] = 0.0;
      particles[1][3] = 0.25;
      particles[1][4] = -0.000001;
            
            
      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -10.0f;
      rotX = 0.0f;
      rotY = 0.0f;
      rotZ = 0.0f;


      translateXtarget = 0.0f;
      translateYtarget = 0.0f;
      translateZtarget = 0.0f;
      
      rotXtarget = 0.0f;
      rotYtarget = 0.0f;
      rotZtarget = -10.0f;

/*
      translateXdiff = (translateXtarget - translateX) / steps / 2.0;
      translateYdiff = (translateYtarget - translateY) / steps / 2.0;
      translateZdiff = (translateZtarget - translateZ) / steps / 2.0;
      rotXdiff = (rotXtarget - rotX) / steps / 2.0;
      rotYdiff = (rotYtarget - rotY) / steps / 2.0;
      rotZdiff = (rotZtarget - rotZ) / steps / 2.0;
*/
    }

    if(animation == 600){

      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 0;
      enableMinDisc = 0;
      enableParticles = 1;
      enableFieldVectors = 1;

      particles[0][0] = 0.0;
      particles[0][1] = 0.0;
      particles[0][2] = 0.0;
      particles[0][3] = 0.35;
      particles[0][4] = -0.000001;
      
      particleNumber = 1;

      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -10.0f;
      rotX = 0.0f;
      rotY = 0.0f;
      rotZ = 0.0f;
      
    }
    
    if(animation == 700){

      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 0;
      enableMinDisc = 0;
      enableParticles = 1;
      enableFieldVectors = 1;

      particles[0][0] = 0.0;
      particles[0][1] = 0.0;
      particles[0][2] = 0.0;
      particles[0][3] = 0.25;
      particles[0][4] = 0.000001;

      doubleTemp = 10.0f;

      particles[1][0] = doubleTemp * cos(2.0f * PI / 6.0f * 1.0f);
      particles[1][1] = doubleTemp * sin(2.0f * PI / 6.0f * 1.0f);
      particles[1][2] = 0.0;
      particles[1][3] = 0.25;
      particles[1][4] = 0.000001;

      particles[2][0] = doubleTemp * cos(2.0f * PI / 6.0f * 2.0f);
      particles[2][1] = doubleTemp * sin(2.0f * PI / 6.0f * 2.0f);
      particles[2][2] = 0.0;
      particles[2][3] = 0.25;
      particles[2][4] = 0.000001;

      particles[3][0] = doubleTemp * cos(2.0f * PI / 6.0f * 3.0f);
      particles[3][1] = doubleTemp * sin(2.0f * PI / 6.0f * 3.0f);
      particles[3][2] = 0.0;
      particles[3][3] = 0.25;
      particles[3][4] = 0.000001;

      particles[4][0] = doubleTemp * cos(2.0f * PI / 6.0f * 4.0f);
      particles[4][1] = doubleTemp * sin(2.0f * PI / 6.0f * 4.0f);
      particles[4][2] = 0.0;
      particles[4][3] = 0.25;
      particles[4][4] = 0.000001;

      particles[5][0] = doubleTemp * cos(2.0f * PI / 6.0f * 5.0f);
      particles[5][1] = doubleTemp * sin(2.0f * PI / 6.0f * 5.0f);
      particles[5][2] = 0.0;
      particles[5][3] = 0.25;
      particles[5][4] = 0.000001;

      particles[6][0] = doubleTemp * cos(2.0f * PI / 6.0f * 6.0f);
      particles[6][1] = doubleTemp * sin(2.0f * PI / 6.0f * 6.0f);
      particles[6][2] = 0.0;
      particles[6][3] = 0.25;
      particles[6][4] = 0.000001;

      particles[7][0] = 12.0f;
      particles[7][1] = 0.0f;
      particles[7][2] = 0.0;
      particles[7][3] = 0.25;
      particles[7][4] = 0.000001;

      
      particleNumber = 8;

      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -10.0f;
      rotX = 0.0f;
      rotY = 0.0f;
      rotZ = 0.0f;
      
    }
    
    if(animation == 800){

      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 0;
      enableMinDisc = 0;
      enableParticles = 1;
      enableFieldVectors = 1;

      particles[0][0] = -2.0;
      particles[0][1] = 0.0;
      particles[0][2] = 0.0;
      particles[0][3] = 0.25;
      particles[0][4] = -0.000001;
      
      particles[1][0] = 2.0;
      particles[1][1] = 0.0;
      particles[1][2] = 0.0;
      particles[1][3] = 0.25;
      particles[1][4] = 0.000001;
      
      particleNumber = 2;

      doubleTemp = 10.0f;
      
      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -10.0f;
      rotX = 0.0f;
      rotY = 0.0f;
      rotZ = 0.0f;
    }

    if(animation == 900){

      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 0;
      enableMinDisc = 0;
      enableParticles = 1;
      enableFieldVectors = 1;

      particles[0][0] = 0.0;
      particles[0][1] = 1.5;
      particles[0][2] = 0.0;
      particles[0][3] = 0.6;
      particles[0][4] = -0.000002;
      
      moveAlpha = -PI / 180.0f * 37.5f;
      
      particles[1][0] = 4.0 * cos(moveAlpha);
      particles[1][1] = 4.0 * sin(moveAlpha) + 1.5f;
      particles[1][2] = 0.0;
      particles[1][3] = 0.35;
      particles[1][4] = 0.000001;
      
      particles[2][0] = -4.0 * cos(moveAlpha);
      particles[2][1] = 4.0 * sin(moveAlpha) + 1.5f;
      particles[2][2] = 0.0;
      particles[2][3] = 0.35;
      particles[2][4] = 0.000001;
      
      particleNumber = 3;

      doubleTemp = 10.0f;
      
      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -10.0f;
      rotX = 0.0f;
      rotY = 0.0f;
      rotZ = 0.0f;
    }


    if(animation == 1000){
      GLfloat x1, y1;

      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 0;
      enableMinDisc = 0;
      enableParticles = 1;
      enableFieldVectors = 21;

      moveAlpha = -PI / 180.0f * (52.5f - 90.0f);
      moveBeta = 0.0f;
      x1 = -4.0f;
      y1 = -2.0f;

      particles[0][0] = 0.0 + x1;
      particles[0][1] = 0.0 + y1;
      particles[0][2] = 0.0;
      particles[0][3] = 0.5;
      particles[0][4] = -0.000002;
      
      
      particles[1][0] = 1.0 * cos(moveAlpha + moveBeta) + x1;
      particles[1][1] = 1.0 * sin(moveAlpha + moveBeta) + y1;
      particles[1][2] = 0.0;
      particles[1][3] = 0.25;
      particles[1][4] = 0.000001;
      
      particles[2][0] = 1.0 * cos(moveAlpha + PI / 180.0f * 105.0f + moveBeta) + x1;
      particles[2][1] = 1.0 * sin(moveAlpha + PI / 180.0f * 105.0f + moveBeta) + y1;
      particles[2][2] = 0.0;
      particles[2][3] = 0.25;
      particles[2][4] = 0.000001;
      
      moveBeta = -3.145927f / 180.0f * 180.0f;
      x1 = -4.0f;
      y1 = 2.0f;

      particles[3][0] = 0.0 + x1;
      particles[3][1] = 0.0 + y1;
      particles[3][2] = 0.0;
      particles[3][3] = 0.5;
      particles[3][4] = -0.000002;
      
      
      particles[4][0] = 1.0 * cos(moveAlpha + moveBeta) + x1;
      particles[4][1] = 1.0 * sin(moveAlpha + moveBeta) + y1;
      particles[4][2] = 0.0;
      particles[4][3] = 0.25;
      particles[4][4] = 0.000001;
      
      particles[5][0] = 1.0 * cos(moveAlpha + PI / 180.0f * 105.0f + moveBeta) + x1;
      particles[5][1] = 1.0 * sin(moveAlpha + PI / 180.0f * 105.0f + moveBeta) + y1;
      particles[5][2] = 0.0;
      particles[5][3] = 0.25;
      particles[5][4] = 0.000001;
      

      x1 = 4.0f;
      y1 = -2.0f;

      particles[6][0] = 0.0 + x1;
      particles[6][1] = 0.0 + y1;
      particles[6][2] = 0.0;
      particles[6][3] = 0.5;
      particles[6][4] = -0.000002;
      
      
      particles[7][0] = 1.0 * cos(0.0f + moveBeta) + x1;
      particles[7][1] = 1.0 * sin(0.0f + moveBeta) + y1;
      particles[7][2] = 0.0;
      particles[7][3] = 0.25;
      particles[7][4] = 0.000001;
      
      particles[8][0] = 1.0 * cos(PI + moveBeta) + x1;
      particles[8][1] = 1.0 * sin(PI + moveBeta) + y1;
      particles[8][2] = 0.0;
      particles[8][3] = 0.25;
      particles[8][4] = 0.000001;
      
      x1 = 4.0f;
      y1 = 2.0f;

      particles[9][0] = 0.0 + x1;
      particles[9][1] = 0.0 + y1;
      particles[9][2] = 0.0;
      particles[9][3] = 0.5;
      particles[9][4] = -0.000002;
      
      
      particles[10][0] = 1.0 * cos(PI / 2.0f + moveBeta) + x1;
      particles[10][1] = 1.0 * sin(PI / 2.0f + moveBeta) + y1;
      particles[10][2] = 0.0;
      particles[10][3] = 0.25;
      particles[10][4] = 0.000001;
      
      particles[11][0] = 1.0 * cos(PI * 1.5f + moveBeta) + x1;
      particles[11][1] = 1.0 * sin(PI * 1.5f + moveBeta) + y1;
      particles[11][2] = 0.0;
      particles[11][3] = 0.25;
      particles[11][4] = 0.000001;
      


      particleNumber = 12;

      doubleTemp = 10.0f;
      
      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -10.0f;
      rotX = 0.0f;
      rotY = 0.0f;
      rotZ = 0.0f;
    }


    if(animation == 1100){
      int i;

      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 0;
      enableMinDisc = 0;
      enableParticles = 1;
      enableFieldVectors = 1;

      particles[0][0] = 1.5f;
      particles[0][1] = 0.0;
      particles[0][2] = 0.0;
      particles[0][3] = 0.5;
      particles[0][4] = 0.000003;
      
      
      moveAlpha = 0.0f;
      for(i = 1; i < 181; i++){
        particles[i][0] = 2.0 * cos(moveAlpha) + 1.5f;
        particles[i][1] = 2.0 * sin(moveAlpha);
        particles[i][2] = 0.0;
        particles[i][3] = 0.045;
        particles[i][4] = -0.000003 / 180.0f;
        moveAlpha += 2.0f * PI / 180;
      }      


      particleNumber = 181;
      
      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -10.0f;
      rotX = 0.0f;
      rotY = 0.0f;
      rotZ = 0.0f;
    }

    if(animation == 1200){
      int i, i2;

      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 0;
      enableMinDisc = 0;
      enableParticles = 1;
      enableFieldVectors = 1;
      pauseFrames = 0;



      for(i2 = 0; i2 < 11; i2++){
        for(i=0; i < 11; i++){
          particles[i2 * 11 + i][0] = -14.0f;
          particles[i2 * 11 + i][1] = (GLfloat)(i) - 5.0f;
          particles[i2 * 11 + i][2] = (GLfloat)(i2) - 5.0f;
          particles[i2 * 11 + i][3] = 0.0f;
          particles[i2 * 11 + i][4] = 0.0f;
          
          particles[i2 * 11 + i][5] = 0.0f;
        }
      }
      
      for(i2 = 0; i2 < 11; i2++){
        for(i=0; i < 11; i++){
          particles[i2 * 11 + i + 121][0] = 14.0f;
          particles[i2 * 11 + i + 121][1] = (GLfloat)(i) - 5.0f;
          particles[i2 * 11 + i + 121][2] = (GLfloat)(i2) - 5.0f;
          particles[i2 * 11 + i + 121][3] = 0.0f;
          particles[i2 * 11 + i + 121][4] = 0.0f;

          particles[i2 * 11 + i + 121][5] = 0.0f;
        }
      }


      particleNumber = 242;
      
      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -26.0f;
      rotX = 0.0f;
      rotY = 0.0f;
      rotZ = 0.0f;
    }


    if(animation == 1300){//Permanent dipoles in homegeneous field
      int i, i2;
      GLfloat x, y;

      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 0;
      enableMinDisc = 0;
      enableParticles = 1;
      enableFieldVectors = 1;
      pauseFrames = 0;

      moveAlpha = 0.0f;
      i = 0;
      
      int divisor = RAND_MAX/(PI * 10000 + 1);
      
      for(x = -2.13; x <= 1.87; x += 0.5){
        for(y = -4.0; y <= 4.0; y += 0.5){
          moveAlpha += 1.03f;
          moveAlpha = (double)(rand()) * 2.0f * PI / (double)(RAND_MAX);
          while(moveAlpha > PI * 2.0f){
            moveAlpha -= PI * 2.0f;
          }
          //printf("Random number = %lf\n", (double)(rand()) * PI / (double)(RAND_MAX));
          particles[i][0] = (GLfloat)(x) + 0.1f * cos(moveAlpha);
          particles[i][1] = (GLfloat)(y) + 0.1f * sin(moveAlpha);
          particles[i][2] = 0.0f;
          particles[i][3] = 0.15f;
          particles[i][4] = 0.000002f;
          particles[i][5] = 0.0f;
          particles[i][6] = moveAlpha;
          i++;
          particles[i][0] = (GLfloat)(x) - 0.1f * cos(moveAlpha);
          particles[i][1] = (GLfloat)(y) - 0.1f * sin(moveAlpha);
          particles[i][2] = 0.0f;
          particles[i][3] = 0.15f;
          particles[i][4] = -0.000002f;
          particles[i][5] = 0.0f;
          particles[i][6] = moveAlpha;
          i++;
        }
      }
      
      minVectorDistance = 0.17f;

      particleNumber = i;
      
      printf("i=%d\n", i);
      
      for(i2 = 0; i2 < 11; i2++){
        for(i=0; i < 11; i++){
          particles[i2 * 11 + i + particleNumber][0] = -14.0f;
          particles[i2 * 11 + i + particleNumber][1] = (GLfloat)(i) - 5.0f;
          particles[i2 * 11 + i + particleNumber][2] = (GLfloat)(i2) - 5.0f;
          particles[i2 * 11 + i + particleNumber][3] = 0.0f;
          particles[i2 * 11 + i + particleNumber][4] = 0.0f;
          
          particles[i2 * 11 + i + particleNumber][5] = 0.0f;
        }
      }
      
      for(i2 = 0; i2 < 11; i2++){
        for(i=0; i < 11; i++){
          particles[i2 * 11 + i + particleNumber + 121][0] = 14.0f;
          particles[i2 * 11 + i + particleNumber + 121][1] = (GLfloat)(i) - 5.0f;
          particles[i2 * 11 + i + particleNumber + 121][2] = (GLfloat)(i2) - 5.0f;
          particles[i2 * 11 + i + particleNumber + 121][3] = 0.0f;
          particles[i2 * 11 + i + particleNumber + 121][4] = 0.0f;

          particles[i2 * 11 + i + particleNumber + 121][5] = 0.0f;
        }
      }

      particleNumber += 242;
      
      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -10.0f;
      rotX = 0.0f;
      rotY = 0.0f;
      rotZ = 0.0f;
    }

    if(animation == 1400){//Field lines of two positive charges
      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 0;
      enableMinDisc = 0;
      enableParticles = 1;
      enableFieldVectors = 0;
      pauseFrames = 0;

      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -10.0f;
      rotX = 0.0f;
      rotY = 0.0f;
      rotZ = 0.0f;
    
      particles[0][0] = -2.0f;
      particles[0][1] = 0.0f;
      particles[0][2] = 0.0f;
      particles[0][3] = 0.25f;
      particles[0][4] = 0.000001f;
    
      particles[1][0] = 2.0f;
      particles[1][1] = 0.0f;
      particles[1][2] = 0.0f;
      particles[1][3] = 0.25f;
      particles[1][4] = 0.000001f;
      
      fieldLineEndUp = -1.0f;
      
      particleNumber = 2;
    }

    if(animation == 1500){//Potential plane of two positive charges
      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 1;
      enableMinDisc = 1;
      minDistance = 0.345f;
      enableParticles = 1;
      enableFieldVectors = 0;
      pauseFrames = 0;


      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -10.0f;
      rotX = 0.0f;
      rotY = 0.0f;
      rotZ = 0.0f;

      translateXtarget = -0.4f;
      translateYtarget = -0.85f;
      translateZtarget = -14.85f;
      
      rotXtarget = 31.00f;
      rotYtarget = -16.5f;
      rotZtarget = 0.0f;

      translateXdiff = (translateXtarget - translateX) / steps / 2.0;
      translateYdiff = (translateYtarget - translateY) / steps / 2.0;
      translateZdiff = (translateZtarget - translateZ) / steps / 2.0;
      rotXdiff = (rotXtarget - rotX) / steps / 2.0;
      rotYdiff = (rotYtarget - rotY) / steps / 2.0;
      rotZdiff = (rotZtarget - rotZ) / steps / 2.0;
    
      particles[0][0] = -2.0f;
      particles[0][1] = 0.0f;
      particles[0][2] = 0.0f;
      particles[0][3] = 0.25f;
      particles[0][4] = 0.000001f;
    
      particles[1][0] = 2.0f;
      particles[1][1] = 0.0f;
      particles[1][2] = 0.0f;
      particles[1][3] = 0.25f;
      particles[1][4] = 0.000001f;
      
      particles[2][0] = -1.6f;
      particles[2][1] = 0.0f;
      particles[2][2] = 0.0f;
      particles[2][3] = 0.15f;
      particles[2][4] = 0.000001f;
      
      particleNumber = 2;
      potentialEnd = 2;
    }


    if(animation == 1600){//Field lines of three positive charges
      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 0;
      enableMinDisc = 0;
      enableParticles = 1;
      enableFieldVectors = 0;
      pauseFrames = 0;

      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -10.0f;
      rotX = 0.0f;
      rotY = 0.0f;
      rotZ = 0.0f;
    
      particles[0][0] = -3.0f;
      particles[0][1] = -2.598076211f;
      particles[0][2] = 0.0f;
      particles[0][3] = 0.25f;
      particles[0][4] = 0.000251f;
    
      particles[1][0] = 3.0f;
      particles[1][1] = -2.598076211f;
      particles[1][2] = 0.0f;
      particles[1][3] = 0.25f;
      particles[1][4] = 0.000251f;
      
      particles[2][0] = 0.0f;
      particles[2][1] = 2.598076211f;
      particles[2][2] = 0.0f;
      particles[2][3] = 0.25f;
      particles[2][4] = 0.000251f;
      
      fieldLineEndUp = -1.0f;
      
      particleNumber = 3;
    }

    if(animation == 1700){//Potential field of three positive charges
      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 1;
      enableMinDisc = 1;
      enableParticles = 1;
      enableFieldVectors = 0;
      pauseFrames = 0;

      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -10.0f;
      rotX = -90.0f;
      rotY = 0.0f;
      rotZ = 0.0f;
    
      translateXtarget = 0.0f;
      translateYtarget = -0.83f;
      translateZtarget = -9.3f;
      
      rotXtarget = 35.25f;
      rotYtarget = 160.3f;
      rotZtarget = 0.0f;
      
      translateXdiff = (translateXtarget - translateX) / steps;
      translateYdiff = (translateYtarget - translateY) / steps;
      translateZdiff = (translateZtarget - translateZ) / steps;
      rotXdiff = (rotXtarget - rotX) / steps;
      rotYdiff = (rotYtarget - rotY) / steps;
      rotZdiff = (rotZtarget - rotZ) / steps;
    
    
    
      particles[0][0] = -3.0f;
      particles[0][1] = 0.0f;
      particles[0][2] = -2.598076211f;
      particles[0][3] = 0.25f;
      particles[0][4] = 0.000001f;
    
      particles[1][0] = 3.0f;
      particles[1][1] = 0.0f;
      particles[1][2] = -2.598076211f;
      particles[1][3] = 0.25f;
      particles[1][4] = 0.000001f;
      
      particles[2][0] = 0.0f;
      particles[2][1] = 0.0f;
      particles[2][2] = 2.598076211f;
      particles[2][3] = 0.25f;
      particles[2][4] = 0.000001f;
      
      particles[3][0] = 0.0f;
      particles[3][1] = 0.0f;
      particles[3][2] = 0.0f;
      particles[3][3] = 0.15f;
      particles[3][4] = 0.000001f;
      
      fieldLineEndUp = -1.0f;
      
      particleNumber = 3;
      potentialEnd = 3;
    }


    if(animation == 1800){//Potential field of 9 positive charges
      int i;
      
      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 1;
      enableMinDisc = 1;
      enableParticles = 1;
      enableFieldVectors = 0;
      pauseFrames = 0;
      enableGrid = 0;
      
      minDistance = 0.25f;
      
      gridX = gridX * 32;
      gridY = gridY * 32;


      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -10.0f;
      rotX = -90.0f;
//rotX = 35.0f;
      rotY = 0.0f;
      rotZ = 0.0f;
    
    
      translateXtarget = 0.0f;
      translateYtarget = -1.11f;
      translateZtarget = -9.3f;
      
      rotXtarget = 35.25f;
      rotYtarget = 0.0f;
      rotZtarget = 0.0f;
      
      translateXdiff = (translateXtarget - translateX) / steps;
      translateYdiff = (translateYtarget - translateY) / steps;
      translateZdiff = (translateZtarget - translateZ) / steps;
      rotXdiff = (rotXtarget - rotX) / steps;
      rotYdiff = (rotYtarget - rotY) / steps;
      rotZdiff = (rotZtarget - rotZ) / steps;
    
    
      alpha = 0.0f;
      for(i=0; i<9; i++){
        alpha = (GLfloat)(i) * 2.0f * PI / 9.0f;
        particles[i][0] = cos(alpha) * 3.0f;
        particles[i][1] = 0.0f;
        particles[i][2] = sin(alpha) * 3.0f;
        particles[i][3] = 0.25f;
        particles[i][4] = 0.0000003f;
      }
    
      
      particles[9][0] = 0.0f;
      particles[9][1] = 0.0f;
      particles[9][2] = 0.0f;
      particles[9][3] = 0.15f;
      particles[9][4] = 0.000001f;
      
      fieldLineEndUp = -1.0f;
      
      particleNumber = 9;
      potentialEnd = 9;
    }



    if(animation == 1900){//Field lines of a sphere
      int i;
      
      
      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 0;
      enableMinDisc = 0;
      enableParticles = 1;
      enableFieldVectors = 0;
      pauseFrames = 0;

      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -10.0f;
      rotX = 0.0f;
      rotY = 0.0f;
      rotZ = 0.0f;
    
      alpha = 0.0f;
      for(i=0; i<9; i++){
        alpha = (GLfloat)(i) * 2.0f * PI / 9.0f;
        particles[i][0] = cos(alpha) * 3.0f;
        particles[i][1] = sin(alpha) * 3.0f;
        particles[i][2] = 0.0f;
        particles[i][3] = 0.25f;
        particles[i][4] = 0.0000012f;
      }
      
      fieldLineEndUp = -1.0f;
      
      particleNumber = 9;
    }

    if(animation == 2000){//Field lines of two positive charges with distant view
      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 0;
      enableMinDisc = 0;
      enableParticles = 1;
      enableFieldVectors = 0;
      pauseFrames = 0;

      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -10.0f;
      rotX = 0.0f;
      rotY = 0.0f;
      rotZ = 0.0f;
    
      particles[0][0] = -2.0f;
      particles[0][1] = 0.0f;
      particles[0][2] = 0.0f;
      particles[0][3] = 0.25f;
      particles[0][4] = 0.000001f;
    
      particles[1][0] = 2.0f;
      particles[1][1] = 0.0f;
      particles[1][2] = 0.0f;
      particles[1][3] = 0.25f;
      particles[1][4] = 0.000001f;
      
      fieldLineEndUp = -1.0f;
      
      particleNumber = 2;      
    }

    if(animation == 2100){//3D equipotential plane
      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 0;
      enableMinDisc = 0;
      enableParticles = 1;
      enableFieldVectors = 0;
      pauseFrames = 0;

      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -10.0f;
      rotX = 0.0f;
      rotY = 0.0f;
      rotZ = 0.0f;
    
      particles[0][0] = -2.0f;
      particles[0][1] = 0.0f;
      particles[0][2] = 0.0f;
      particles[0][3] = 0.25f;
      particles[0][4] = 0.000001f;
    
      particles[1][0] = 2.0f;
      particles[1][1] = 0.0f;
      particles[1][2] = 0.0f;
      particles[1][3] = 0.25f;
      particles[1][4] = 0.000001f;
      
      fieldLineEndUp = -1.0f;
      
      particleNumber = 2;      
    }

    if(animation == 2200){//Proton composed of Quarks
      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 0;
      enableMinDisc = 0;
      enableParticles = 1;
      enableFieldVectors = 0;
      pauseFrames = 0;

      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -10.0f;
      rotX = 0.0f;
      rotY = 0.0f;
      rotZ = 0.0f;
    
      particles[0][0] = -19.0f;
      particles[0][1] = 0.0f;
      particles[0][2] = 0.0f;
      particles[0][3] = 0.5f;
      particles[0][4] = 0.000001602f * 2.0f / 3.0f;
    
      particles[1][0] = 19.0f;
      particles[1][1] = 0.0f;
      particles[1][2] = 0.0f;
      particles[1][3] = 0.5f;
      particles[1][4] = 0.000001602f * 2.0f / 3.0f;
      
      particles[2][0] = 0.0f;
      particles[2][1] = 0.0f;
      particles[2][2] = 0.0f;
      particles[2][3] = 0.5f;
      particles[2][4] = -0.000001602f * 1.0f / 3.0f;
      
      fieldLineEndUp = 400.0f;
      
      particleNumber = 3;
    }

    if(animation == 2300){//Potential plane of proton composed of quarks
      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 1;
      potentialSphereRadius = 0.32f;
      enableMinDisc = 0;
      enableParticles = 1;
      enableFieldVectors = 0;
      pauseFrames = 0;
      enableGrid = 0;
      for(i=0; i<PATHMAX; i++){
        pathWidth[i] = 7.5f;
      }

      gridX = gridX * 8;
      gridY = gridY * 8;
      
      xMax = 32.0f;
      yMax = xMax / 16.0f * 9.0f;


      translateX = 0.0f;
      translateY = 0.64f;
      translateZ = -29.35f;
      rotX = 42.25f;
      rotY = -10.75f;
      rotZ = 0.0f;
    
      particles[0][0] = -10.0f;
      particles[0][1] = 0.0f;
      particles[0][2] = 0.0f;
      particles[0][3] = 0.51f;
      particles[0][4] = 0.000001602f * 2.0f / 3.0f;
    
      particles[1][0] = 10.0f;
      particles[1][1] = 0.0f;
      particles[1][2] = 0.0f;
      particles[1][3] = 0.51f;
      particles[1][4] = 0.000001602f * 2.0f / 3.0f;
      
      particles[2][0] = 0.0f;
      particles[2][1] = 0.0f;
      particles[2][2] = 0.0f;
      particles[2][3] = 0.5f;
      particles[2][4] = -0.000001602f * 1.0f / 3.0f;
      
      particles[3][0] = 0.0f;
      particles[3][1] = 0.0f;
      particles[3][2] = 0.0f;
      particles[3][3] = 0.25f;
      particles[3][4] = 0.000006f;
      
      
      particleNumber = 3;
      potentialEnd = 3;
      
    }

    if(animation == 2400){//Energy of approaching protons
      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 0;
      enableMinDisc = 0;
      enableParticles = 1;
      enableFieldVectors = 0;
      pauseFrames = 0;
      enableGrid = 0;

      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -10.0f;
      rotX = 0.0f;
      rotY = 0.0f;
      rotZ = 0.0f;
    
      particles[0][0] = -5.5f;
      particles[0][1] = 0.6f;
      particles[0][2] = 0.0f;
      particles[0][3] = 0.25f;
      particles[0][4] = 0.000001602 * 2.0f / 3.0f;
    
      particles[1][0] = -5.5f;
      particles[1][1] = 0.0f;
      particles[1][2] = 0.0f;
      particles[1][3] = 0.25f;
      particles[1][4] = -0.000001602 * 1.0f / 3.0f;
      
      particles[2][0] = -5.5f;
      particles[2][1] = -0.6f;
      particles[2][2] = 0.0f;
      particles[2][3] = 0.25f;
      particles[2][4] = 0.000001602 * 2.0f / 3.0f;
      
      particles[3][0] = 10.0f + 0.6f;
      particles[3][1] = 0.0f;
      particles[3][2] = 0.0f;
      particles[3][3] = 0.25f;
      particles[3][4] = 0.000001602 * 2.0f / 3.0f;
      
      particles[4][0] = 10.0f;
      particles[4][1] = 0.0f;
      particles[4][2] = 0.0f;
      particles[4][3] = 0.25f;
      particles[4][4] = -0.000001602 * 1.0f / 3.0f;
      
      particles[5][0] = 10.0f - 0.6f;
      particles[5][1] = 0.0f;
      particles[5][2] = 0.0f;
      particles[5][3] = 0.25f;
      particles[5][4] = 0.000001602 * 2.0f / 3.0f;
      
      
      particleNumber = 6;
      potentialEnd = 6;
//double k1 = 1.0 / (4.0 * PI * 8.85419E-12);      
      printf("Potential = %lf\n", 1.0f / (4.0f * PI * 8.854e-12) * 0.000001603 * 0.000001603 / 3.4);
      printf("k = %lf\n", 1.0f / (4.0f * PI * 8.854e-12));
      printf("Potential = %lf\n", 1.0f / (4.0f * PI * 8.854e-12) * 0.000001603 * 0.000001603 / 2.0);
    }

    if(animation == 2500){//Moving negative particle
      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 0;
      enableMinDisc = 0;
      enableParticles = 1;
      enableFieldVectors = 0;
      pauseFrames = 0;
      enableGrid = 0;

      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -10.0f;
      rotX = 0.0f;
      rotY = 0.0f;
      rotZ = 0.0f;
    
      particles[0][0] = 0.0f;
      particles[0][1] = 0.0f;
      particles[0][2] = 0.0f;
      particles[0][3] = 0.25f;
      particles[0][4] = -0.00005f;
    
      particleSpeed = 0.0f;
      pauseFrames = 0; 
      particleNumber = 1;
      potentialEnd = 1;
    }

    if(animation == 2600){//Influence on "molecules"
      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 0;
      enableMinDisc = 0;
      enableParticles = 1;
      enableFieldVectors = 0;
      pauseFrames = 0;
      enableGrid = 0;

      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -10.0f;
      rotX = 0.0f;
      rotY = 0.0f;
      rotZ = 0.0f;
    
      particles[0][0] = 10.0f;
      particles[0][1] = 0.0f;
      particles[0][2] = 0.0f;
      particles[0][3] = 1.50f;
      particles[0][4] = 0.00001f;
    

      particles[1][0] = -5.2f;
      particles[1][1] = 0.0f;
      particles[1][2] = 0.0f;
      particles[1][3] = 1.50f;
      particles[1][4] = 0.00003f;
      particles[1][5] = 1.0f;
    
      particles[2][0] = -5.2f;
      particles[2][1] = 0.0f;
      particles[2][2] = 0.0f;
      particles[2][3] = 1.50f;
      particles[2][4] = -0.00003f;
      particles[2][5] = 1.0f;
    

      particleNumber = 3;
      potentialEnd = 3;
      fieldLineEndUp = 100;
      minVectorDistance = 0.2f;
    }


    if(animation == 2700){//Influence on metal sphere
      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 0;
      enableMinDisc = 0;
      enableParticles = 1;
      enableFieldVectors = 0;
      pauseFrames = 0;
      enableGrid = 0;

      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -10.0f;
      rotX = 0.0f;
      rotY = 0.0f;
      rotZ = 0.0f;
    
      particles[0][0] = 10.0f;
      particles[0][1] = 0.0f;
      particles[0][2] = 0.0f;
      particles[0][3] = 1.00f;
      particles[0][4] = 0.00051f;
    

      alpha = 0.0f;
      for(i = 1; i < 25; i+=2){
        alpha = 2.0f * PI / 24.0f * (i - 1);
        particles[i][0] = cos(alpha) * 2.5f - 3.0f;;
        particles[i][1] = sin(alpha) * 2.5f;
        particles[i][2] = 0.0f;
        particles[i][3] = 0.25f;
        particles[i][4] = 0.00000f;
        particles[i][5] = 0.0f;
      }

      alpha = 0.0f;
      for(i = 2; i < 25; i+=2){
        alpha = 2.0f * PI / 24.0f * (i - 1);
        particles[i][0] = cos(alpha) * 2.5f - 3.0f;
        particles[i][1] = sin(alpha) * 2.5f;
        particles[i][2] = 0.0f;
        particles[i][3] = 0.25f;
        particles[i][4] = -0.00001f;
        particles[i][5] = 3.0f;
      }
          
      particleNumber = 25;
      potentialEnd = 25;
    }

    if(animation == 2800){//Coulomb constant
      int i, i2;
      GLfloat x, y;

      enableLight2 = 0;
      enableLight3 = 0;
      enablePotentialPlane = 0;
      enableMinDisc = 0;
      enableParticles = 1;
      enableFieldVectors = 0;
      pauseFrames = 0;

      moveAlpha = 0.0f;
      doubleTemp = 0.0f;      
      i = 0;
      
      int divisor = RAND_MAX/(PI * 10000 + 1);
      
      
      
      particles[0][0] = -5.5f;
      particles[0][1] = 0.0f;
      particles[0][2] = 0.0f;
      particles[0][3] = 1.0f;
      particles[0][4] = 0.00001f;
      particles[0][5] = 0.0f;
      particles[0][6] = 0.0f;

      particles[1][0] = 5.5f;
      particles[1][1] = 0.0f;
      particles[1][2] = 0.0f;
      particles[1][3] = 1.0f;
      particles[1][4] = -0.00001f;
      particles[1][5] = 0.0f;
      particles[1][6] = 0.0f;

      i = 2;
      for(x = -4.0f; x <= 4.0f; x += 0.5){
        for(y = -4.0; y <= 4.0; y += 0.5){
          moveAlpha += 1.03f;
          moveAlpha = (double)(rand()) * 2.0f * PI / (double)(RAND_MAX);
          while(moveAlpha > PI * 2.0f){
            moveAlpha -= PI * 2.0f;
          }
          if(sqrt(pow(particles[0][0] - x, 2.0) + pow(particles[0][1] - y, 2.0)) > 1.15 &&
             sqrt(pow(particles[1][0] - x, 2.0) + pow(particles[1][1] - y, 2.0)) > 1.15){
            particles[i][0] = x + dipoleRadius * cos(moveAlpha);
            particles[i][1] = y + dipoleRadius * sin(moveAlpha);
            particles[i][2] = 0.0f;
            particles[i][3] = 0.15f;
            particles[i][4] = 0.00000015f;
            particles[i][5] = 0.0f;
            particles[i][6] = moveAlpha;
            i++;
            particles[i][0] = x - dipoleRadius * cos(moveAlpha);
            particles[i][1] = y - dipoleRadius * sin(moveAlpha);
            particles[i][2] = 0.0f;
            particles[i][3] = 0.15f;
            particles[i][4] = -0.00000015f;
            particles[i][5] = 0.0f;
            particles[i][6] = moveAlpha;
            i++;
          }
        }
      }
      

      particleNumber = i;
      
      printf("i=%d\n", i);
      
      
      translateX = 0.0f;
      translateY = 0.0f;
      translateZ = -10.0f;
      rotX = 0.0f;
      rotY = 0.0f;
      rotZ = 0.0f;
      
      //animation = 2805;//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    }

    
    /* wait for events*/ 
    while (!done){
        /* handle the events in the queue */
        while (XPending(GLWin.dpy) > 0){
            XNextEvent(GLWin.dpy, &event);
            switch (event.type){
                case Expose:
	                if (event.xexpose.count != 0)
	                    break;
                    drawGLScene();
                    break;
                case ConfigureNotify:
                /* call resizeGLScene only if our window-size changed */
                    if ((event.xconfigure.width != GLWin.width) || (event.xconfigure.height != GLWin.height)){
                        GLWin.width = event.xconfigure.width;
                        GLWin.height = event.xconfigure.height;
                        printf("Resize event\n");
                        resizeGLScene(event.xconfigure.width,
                            event.xconfigure.height);
                    }
                    break;
                /* exit in case of a mouse button press */
                case ButtonPress:     
                    done = True;
                    break;
                case KeyPress:
                    key = XLookupKeysym(&event.xkey, 0);
                    keyPressed(key);
                    break;
                case ClientMessage:    
                    if (*XGetAtomName(GLWin.dpy, event.xclient.message_type) == *"WM_PROTOCOLS"){
                        printf("Exiting sanely...\n");
                        done = True;
                    }
                    break;
                default:
                    break;
            }
        }
        drawGLScene();
    }
    killGLWindow();
    return 0;
}
